<?php
/**
 * Demo import manager
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSAdminUpdater
{
    protected $internalWPDB 		= NULL;
    protected $nrsPrefix 			= "";
    protected $lang 				= NULL;
    protected $debugMode 			= 0;

    // NOTE: The 3.2 version number here is ok, because it defines the case of older plugin versions,
    // when plugin version data was not saved to db
    protected $databaseVersion      = 3.2;
    protected $databaseUpdated      = FALSE;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set database version
        $this->setDatabaseVersion();

        // Set database update status
        $this->setDatabaseUpdateStatus();
    }

    private function setDatabaseVersion()
    {
        $sql = "
				SELECT conf_value AS plugin_version
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_plugin_version'
			";
        $pluginVersionFieldValue = $this->internalWPDB->get_var($sql);

        // NOTE: The 3.2 version number here is ok, because it defines the case of older plugin versions,
        // when plugin version data was not saved to db
        $databaseVersion = 3.2;
        if(!is_null($pluginVersionFieldValue))
        {
            $databaseVersion = floatval($pluginVersionFieldValue);
        }

        $this->databaseVersion = $databaseVersion;

        return $databaseVersion;
    }

    private function setDatabaseUpdateStatus()
    {
        $sql = "
				SELECT conf_value AS updated
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_updated'
			";
        $dbUpdatedValue = $this->internalWPDB->get_var($sql);

        $databaseUpdated = FALSE;
        if(!is_null($dbUpdatedValue))
        {
            $databaseUpdated = intval($dbUpdatedValue) == 1 ? TRUE : FALSE;
        }

        $this->databaseUpdated = $databaseUpdated;
        // DEBUG
        //echo "DB UPDATED: {"; var_dump($databaseUpdated); echo "}<br />";
    }

    public function getDatabaseVersion()
    {
        return $this->databaseVersion;
    }

    public function isDatabaseUpdated()
    {
        return $this->databaseUpdated;
    }

    /**
     * Check if this is a root update, i.e. from V3 to V4 etc., not V4 to V4.1
     * @param $paramCodeVersion
     * @return bool
     */
    public function isMajorUpgrade($paramCodeVersion)
    {
        $majorUpgrade = FALSE;
        if(intval($this->databaseVersion) < intval($paramCodeVersion))
        {
            $majorUpgrade = TRUE;
        }
        return $majorUpgrade;
    }

    public function isPluginUpToDate($paramCodeVersion)
    {
        return $this->databaseVersion == $paramCodeVersion ? TRUE : FALSE;
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function processUpdate($paramCodeVersion, $paramNewUploadPath)
    {
        // DEBUG
        //echo "DB VERSION: {$nrsDBVersion}<br />";
        //echo "CODE VERSION: {$codeVersion}<br />";

        $status = 0;
        if($this->isMajorUpgrade($paramCodeVersion))
        {
            // Major upgrade - we need to check for database external update status
            if ($this->isDatabaseUpdated() && floatval($this->databaseVersion) < floatval($paramCodeVersion))
            {
                $updated = $this->updateCore($paramCodeVersion);
                $copied = NRSValidator::copyFolder(ABSPATH . "wp-content/plugins/car-rental-system/front/gallery", $paramNewUploadPath);

                if ($updated == 2 && $copied)
                {
                    $status = 4;
                } else if ($updated)
                {
                    $status = $updated;
                } else if ($copied)
                {
                    $status = 3;
                }
            }

            if($status >= 2)
            {
                $_SESSION['ok_msg'] = 'Completed: Plugin upgraded successfully!';
            } else
            {
                $_SESSION['msg'] = 'Failed: Plugin was not upgraded!';
            }
        } else
        {
            // Minor update - no need to check for database external update status
            if (floatval($this->databaseVersion) < floatval($paramCodeVersion))
            {
                $status = $this->updateCore($paramCodeVersion);
            }

            if($status >= 2)
            {
                $_SESSION['ok_msg'] = 'Completed: Plugin updated successfully!';
            } else
            {
                $_SESSION['msg'] = 'Failed: Plugin was not updated!';
            }
        }

        return $status;
    }

    /**
     * Update the core
     * @param $paramCodeVersion
     * @return bool
     */
    private function updateCore($paramCodeVersion)
    {
        $ret = 2;
        // Insert NRS tables data
        $arrSQL = $this->getNRSSQLData($paramCodeVersion);
        foreach($arrSQL AS $sql)
        {
            $status = $this->internalWPDB->query($sql);
            if($status === FALSE)
            {
                $ret = 1;
            }
        }

        // DEBUG
        // echo nl2br(print_r($arrSQL, TRUE)); die();

        return $ret;
    }

    /**
     * NRS SQL Data
     * @param $paramCodeVersion
     * @return array
     */
    private function getNRSSQLData($paramCodeVersion)
    {
        $validCodeVersion = number_format(floatval($paramCodeVersion), 1,'.','');
        $arrSQL = array();

        if($this->databaseVersion == 4.0)
        {
            // For updating V4 to V4.1
            $arrSQL[] = "DROP TABLE IF EXISTS `{$this->nrsPrefix}api_log`;";

            $arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}api_log` (
              `log_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
              `email` varchar(128) NOT NULL,
              `year` smallint(4) unsigned NOT NULL DEFAULT '0',
              `year_required` tinyint(1) unsigned NOT NULL DEFAULT '0',
              `ip` varchar(32) NOT NULL DEFAULT '0.0.0.0',
              `real_ip` varchar(32) NOT NULL DEFAULT '0.0.0.0',
              `host` varchar(255) NOT NULL,
              `agent` varchar(255) NOT NULL,
              `browser` varchar(50) NOT NULL,
              `os` varchar(50) NOT NULL,
              `total_requests_left` int(11) NOT NULL DEFAULT '1',
              `failed_requests_left` int(11) NOT NULL DEFAULT '1',
              `email_attempts_left` int(11) NOT NULL DEFAULT '1',
              `is_robot` tinyint(1) unsigned NOT NULL,
              `status` tinyint(1) unsigned NOT NULL DEFAULT '2',
              `log_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
              PRIMARY KEY (`log_id`),
              KEY `email` (`email`),
              KEY `year` (`year`),
              KEY `year_required` (`year_required`),
              KEY `ip` (`ip`),
              KEY `real_ip` (`real_ip`),
              KEY `is_robot` (`is_robot`),
              KEY `status` (`status`),
              KEY `log_timestamp` (`log_timestamp`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}bookings` CHANGE `is_canceled` `is_cancelled` tinyint(1) unsigned NOT NULL DEFAULT '0'";

            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}email_contents` DROP `email_name`";
            $arrSQL[] = "TRUNCATE TABLE `{$this->nrsPrefix}email_contents`";

            $arrSQL[] = "INSERT INTO `{$this->nrsPrefix}email_contents` (`email_id`, `email_subject`, `email_body`) VALUES
                (1, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_DETAILS_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n\n[INVOICE]\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
                (2, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_CONFIRMED_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_PAYMENT_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n\n[INVOICE]\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
                (3, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_CANCELLED_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_YOUR_RESERVATION_DETAILS_TEXT')."\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
                (4, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_DETAILS_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n[INVOICE]'),
                (5, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_CONFIRMED_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_PAID_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n[INVOICE]'),
                (6, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_CANCELLED_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_CANCELLED_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_CANCELLED_RESERVATION_DETAILS_TEXT')."\n[INVOICE]')";

            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}invoices` ADD `fixed_deposit_amount` decimal(10,2) unsigned NOT NULL DEFAULT '0.00' AFTER `grand_total`";

$arrSQL[] = 'INSERT INTO `'.$this->nrsPrefix.'invoices` (`booking_id`, `customer_name`, `customer_email`, `grand_total`, `fixed_deposit_amount`, `total_pay_now`, `pickup_location`, `dropoff_location`, `invoice`) VALUES
(0, \'Mr. John Smith\', \'john.smith@gmail.com\', 189.24, 230.00, 189.24, \'2. San Francisco Intl. Airport (SFO) - Hwy 101, San Francisco, CA 94128\', \'3. Oakland Intl. Airport (OAK) - 1 Airport Dr, Oakland, CA 94621\', \'<table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background-color:#eeeeee;width:840px;border:none" cellpadding="5" cellspacing="1">
    <tbody>
    <tr>
        <td align="left" style="font-weight:bold;background-color:#eeeeee;padding-left:5px" colspan="2">Customer Details</td>
    </tr>
    <tr>
        <td align="left" style="width:160px;background-color:#ffffff;padding-left:5px">Reservation code</td>
        <td align="left" style="background-color:#ffffff;padding-left:5px">R1AHKECN</td>
    </tr>
            <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Customer</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Mr. John Smith</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Date of Birth</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">1980-06-09</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Address</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">625 2nd Street</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">City</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">San Francisco</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">State</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">CA</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Postal Code</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">94107</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Country</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">United States</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Phone</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">+14506004790</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Email</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">john.smith@gmail.com</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Additional Comments</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Please leave car keys by the front door upon delivery.</td>
        </tr>
        </tbody>
</table>
<br />
<table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background:#999999;width:840px;border:none" cellpadding="5" cellspacing="1">
<tbody>
<tr>
    <td align="left" style="font-weight:bold;background-color:#eeeeee;padding-left:5px" colspan="3">Rental Details</td>
</tr>
            <tr style="background-color:#343434;color: white" class="location-headers">
            <td align="left" class="col1" style="padding-left:5px"><strong>Pick-up Location</strong></td>
            <td align="left" class="col2" style="padding-left:5px" colspan="2"><strong>Drop-Off Location</strong></td>
        </tr>

    <tr style="background-color:#FFFFFF" class="location-details">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1">
		2. San Francisco Intl. Airport (SFO) - Hwy 101, San Francisco, CA 94128            </td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="2">
		3. Oakland Intl. Airport (OAK) - 1 Airport Dr, Oakland, CA 94621            </td>
            </tr>


    <tr style="background-color:#343434;color: white" class="duration-headers">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1"><strong>Pick-up Date &amp; Time</strong></td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="1"><strong>Drop-off Date &amp; Time</strong></td>
            <td align="right" class="col3" style="padding-right:5px"><strong>Period</strong></td>
            </tr>

    <tr style="background-color:#FFFFFF" class="duration-details">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1">
		October 16, 2015 &nbsp;&nbsp; 11:00 AM            </td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="1">
		October 19, 2015 &nbsp;&nbsp; 10:00 PM            </td>
            <td align="right" class="col3" style="padding-right:5px">
		4 days            </td>
            </tr>

<!-- ITEMS -->
    <tr class="item-headers" style="background-color:#343434;color: white">
        <td align="left" class="col1" style="padding-left:5px"><strong>Selected Cars</strong></td>
        <td align="left" class="col2" style="padding-left:5px"><strong>Price</strong></td>
        <td align="right" class="col3" style="padding-right:5px"><strong>Total</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="items">
        <td align="left" class="col1" style="padding-left:5px">
		Mazda 6, Intermediate        </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 115.72 w/o Tax + $ 10.41 Tax = 1 x $ 126.14" style="cursor:pointer">
		$ 115.72            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 115.72 w/o Tax + $ 10.41 Tax = $ 126.14" style="cursor:pointer">
		$ 115.72            </span>
        </td>
    </tr>

<!-- PICKUP FEES -->
    <tr style="background-color:#343434;color: white" class="office-fee-headers">
        <td align="left" class="col1" style="padding-left:5px" colspan="3"><strong>Location Fees</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="office-fees">
        <td align="left" class="col1" style="padding-left:5px">Pick-up fee                    </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 7.44 w/o Tax + $ 0.67 Tax = 1 x $ 8.11" style="cursor:pointer">
		$ 7.44            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 7.44 w/o Tax + $ 0.67 Tax = $ 8.11" style="cursor:pointer">
		$ 7.44            </span>
        </td>
    </tr>



<!-- DROPOFF FEES -->
    <tr style="background-color:#FFFFFF" class="office-fees">
        <td align="left" class="col1" style="padding-left:5px">Drop-off fee           (Nightly rate applied)        </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 23.97 w/o Tax + $ 2.16 Tax = 1 x $ 26.13" style="cursor:pointer">
		$ 23.97            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 23.97 w/o Tax + $ 2.16 Tax = $ 26.13" style="cursor:pointer">
		$ 23.97            </span>
        </td>
    </tr>

<!-- EXTRAS -->
    <tr class="extra-headers" style="background-color:#343434;color: white">
       <td align="left" class="col1" colspan="3"><strong>Rental Options</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="extras">
         <td align="left" class="col1" style="padding-left:5px">Baby Seat</td>
         <td align="left" class="col2" style="padding-left:5px">
             <span title="1 extra(s) x $ 13.24 w/o Tax + $ 1.19 Tax = 1 x $ 14.43" style="cursor:pointer">
		$ 13.24             </span>
         </td>
         <td align="right" class="col3" style="padding-right:5px">
             <span title="$ 13.24 w/o Tax + $ 1.19 Tax = $ 14.43" style="cursor:pointer">
		$ 13.24            </span>
         </td>
    </tr>
    <tr style="background-color:#FFFFFF" class="extras">
         <td align="left" class="col1" style="padding-left:5px">GPS</td>
         <td align="left" class="col2" style="padding-left:5px">
             <span title="1 extra(s) x $ 13.24 w/o Tax + $ 1.19 Tax = 1 x $ 14.43" style="cursor:pointer">
		$ 13.24             </span>
         </td>
         <td align="right" class="col3" style="padding-right:5px">
             <span title="$ 13.24 w/o Tax + $ 1.19 Tax = $ 14.43" style="cursor:pointer">
		$ 13.24            </span>
         </td>
    </tr>


<!-- TOTAL -->
<tr style="background-color:#343434;color: white" class="total-headers">
    <td align="left" class="col1" colspan="3" style="padding-left:5px"><strong>Total</strong></td>
</tr>
    <tr style="background-color:#FFFFFF">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
            <strong>Sub Total:</strong>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <strong>$ 173.61</strong>
        </td>
    </tr>
    <tr style="background-color:#f2f2f2">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
		Tax (9.00 %):
        </td>
        <td align="right" class="col3" style="padding-right:5px">
		$ 15.62        </td>
    </tr>
<tr style="background-color:#FFFFFF">
    <td align="right" class="col1" style="padding-right:5px" colspan="2">
        <strong>Grand Total:</strong>
    </td>
    <td align="right" class="col3" style="padding-right:5px">
        <strong>$ 189.24</strong>
    </td>
</tr>
    <tr style="background-color:#f2f2f2">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
		Deposit:
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="Cars Deposit ($ 230.00) + Extras Deposit ($ 0.00) = $ 230.00" style="cursor:pointer">
		$ 230.00            </span>
        </td>
    </tr>
</tbody>
</table>

<!-- PAYMENT METHOD DETAILS -->
    <br />
    <table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background:#999999;width:840px;border:none" cellpadding="4" cellspacing="1">
        <tr>
            <td align="left" colspan="2" style="font-weight:bold;background-color:#eeeeee;padding-left:5px">Payment Details</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Pay By</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Bank Transfer</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Payment Details</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Receiver: NATIVE RENTAL, LTD<br />
Account no.: US27 7300 0204 2870 6432<br />
Bank: Bank of America, Inc.</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Transaction ID</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">N/A</td>
        </tr>
    </table>
\')';

            $arrSQL[] = "INSERT INTO `{$this->nrsPrefix}settings` (`conf_key`,`conf_value`) VALUES
                ('conf_paypal_sandbox_mode', '0'),
                ('conf_recaptcha_enabled', '0'),
			    ('conf_recaptcha_site_key', ''),
			    ('conf_recaptcha_secret_key', ''),
                ('conf_api_max_requests_per_period', '50'),
			    ('conf_api_max_failed_requests_per_period', '3')";
        }
        if(in_array($this->databaseVersion, array(4.0, 4.1)))
        {
            // For updating V4 or V4.1 to V4.2

            // Improve SQL database search speed in items/extras availability table
            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}bookings` ADD INDEX (`is_cancelled`)";

            // Make discounts to support 3 chars after comma.
            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}discounts` CHANGE `discount_percentage` `discount_percentage` decimal(10,3) unsigned NOT NULL DEFAULT '0.000'";

            $WPError = false;

            // Language file already loaded, so we can use translated text
            // Create terms of use page post object
            $arrCancellationPage = array(
                'post_title'    => $this->lang->getText('NRS_CANCELLED_PAYMENT_PAGE_TITLE_TEXT'),
                'post_content'  => $this->lang->getText('NRS_CANCELLED_PAYMENT_PAGE_CONTENT_TEXT'),
                'post_status'   => 'publish',
                'post_type'     => $GLOBALS['NRS_EXTENSION'],
                /*'post_author'   => 1,*/ /*auto assign current user*/
                /*'post_category' => array(8,39)*/ /*no categories needed here*/
            );

            // Create terms of use page post object
            $arrConfirmationPage = array(
                'post_title'    => $this->lang->getText('NRS_CONFIRMATION_PAGE_TITLE_TEXT'),
                'post_content'  => $this->lang->getText('NRS_CONFIRMATION_PAGE_CONTENT_TEXT'),
                'post_status'   => 'publish',
                'post_type'     => $GLOBALS['NRS_EXTENSION'],
                /*'post_author'   => 1,*/ /*auto assign current user*/
                /*'post_category' => array(8,39)*/ /*no categories needed here*/
            );

            // Insert corresponding post as post type 'post'
            $newCancellationPageId = wp_insert_post( $arrCancellationPage, $WPError );
            $newConfirmationPageId = wp_insert_post( $arrConfirmationPage, $WPError );

            // Insert new settings for confirmation and cancellation pages
            $arrSQL[] = "INSERT INTO `{$this->nrsPrefix}settings` (`conf_key`,`conf_value`) VALUES
			  ('conf_cancelled_payment_page_id', '{$newCancellationPageId}'),
			  ('conf_confirmation_page_id', '{$newConfirmationPageId}'),
			  ('conf_noon_time', '12:00:00')";
        }

        if(in_array($this->databaseVersion, array(4.0, 4.1, 4.2)))
        {
            // For updating V4, V4.1 or V4.2 to V4.3

            // Improve SQL database search speed in items/extras availability table
            $arrSQL[] = "ALTER TABLE `{$this->nrsPrefix}discounts` ADD INDEX (`discount_percentage`)";
        }

        // Update plugin version till newest
        $arrSQL[] = "UPDATE `{$this->nrsPrefix}settings` SET `conf_value` = '{$validCodeVersion}' WHERE `conf_key` ='conf_plugin_version'";
        // Set update process back to 0, to be ready for future updates
        $arrSQL[] = "UPDATE `{$this->nrsPrefix}settings` SET `conf_value` = '0' WHERE `conf_key` ='conf_updated'";

        return $arrSQL;
    }
}