<?php
/**
 * Class in the middle for front end search
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSFrontEndSearch extends NRSSearch
{
	/**
	 * @param $paramWPDB
	 * @param string $paramNRSPrefix
	 * @param NRSLanguage &$paramNRSLang
	 * @param array $paramConfig
	 * @param string $clearExpired - "CLEAR_EXPIRED_BOOKINGS" or ""
	 */
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig, $clearExpired = "")
	{
		// Set booking expiration, price calculation type and clear expired bookings
		parent::__construct($paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig, $clearExpired);
	}

	/**
	 * Step no. 1 - Show reservation box. (optional) + show car
	 * Step no. 2 - (optional) Select car, if no car provided
	 * Step no. 3 - Select car extras
	 * Step no. 4 - Show booking details
	 * Step no. 5 - Process booking
	 * Step no. 6 - PayPal payment
	 */
	protected function setRequestParams()
	{
		/*******************************************************************/
		/************** Get required search & user fields list *************/
		/*******************************************************************/
		$objCore = new NRSCore($this->internalWPDB, $this->nrsPrefix, $this->lang);

		// If it is not visible, then if will not be required (function will always return false of 'required+not visible')
		$titleRequired = $objCore->getCustomerFieldStatus("title", "REQUIRED");
		$firstNameRequired = $objCore->getCustomerFieldStatus("first_name", "REQUIRED");
		$lastNameRequired = $objCore->getCustomerFieldStatus("last_name", "REQUIRED");
		$birthdateRequired = $objCore->getCustomerFieldStatus("birthdate", "REQUIRED");
		$streetAddressRequired = $objCore->getCustomerFieldStatus("street_address", "REQUIRED");
		$cityRequired = $objCore->getCustomerFieldStatus("city", "REQUIRED");
		$stateRequired = $objCore->getCustomerFieldStatus("state", "REQUIRED");
		$zipcodeRequired = $objCore->getCustomerFieldStatus("zipcode", "REQUIRED");
		$countryRequired = $objCore->getCustomerFieldStatus("country", "REQUIRED");
		$phoneRequired = $objCore->getCustomerFieldStatus("phone", "REQUIRED");
		$emailRequired = $objCore->getCustomerFieldStatus("email", "REQUIRED");
		$additionalCommentsRequired = $objCore->getCustomerFieldStatus("additional_comments", "REQUIRED");
		$paymentMethodRequired = $this->isPaymentRequired();

		// Search fields visibility settings
		$pickupDateVisible = $objCore->getSearchFieldStatus("pickup_date", "VISIBLE");
		$dropoffDateVisible = $objCore->getSearchFieldStatus("dropoff_date", "VISIBLE");

		// Search fields requirement settings
		$pickupLocationRequired = $objCore->getSearchFieldStatus("pickup_location", "REQUIRED");
		$pickupDateRequired = $objCore->getSearchFieldStatus("pickup_date", "REQUIRED");
		$dropoffLocationRequired = $objCore->getSearchFieldStatus("dropoff_location", "REQUIRED");
		$dropoffDateRequired = $objCore->getSearchFieldStatus("dropoff_date", "REQUIRED");
		$bodyTypeRequired = $objCore->getSearchFieldStatus("body_type", "REQUIRED");
		$transmissionTypeRequired = $objCore->getSearchFieldStatus("transmission_type", "REQUIRED");
		$fuelTypeRequired = $objCore->getSearchFieldStatus("fuel_type", "REQUIRED");
		$existingBookingCodeRequired = $objCore->getSearchFieldStatus("booking_code", "REQUIRED");
		// We don't need it anymore after here
		unset($objCore);

		/*******************************************************************/
		/******************* SET BOOKING CODE IF PASSED ********************/
		/*******************************************************************/
		// Process booking ID (figure out if it's new reservation or existing reservation
		if (
			(isset($_REQUEST['booking_code']) && strlen($_REQUEST['booking_code']) > 0 && $_REQUEST['booking_code'] != $this->lang->getText('NRS_I_HAVE_BOOKING_NUMBER_TEXT')) ||
			(isset($_SESSION['booking_code']) && strlen($_SESSION['booking_code']) > 0 && $_SESSION['booking_code'] != $this->lang->getText('NRS_I_HAVE_BOOKING_NUMBER_TEXT'))
		)
		{
			// If passed, then required
			$this->bookingCode = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'booking_code', '', $existingBookingCodeRequired, 'guest_text_validation'); // ! esc_html is vulnerable, we must use 'intval'
		}

		/*******************************************************************/
		/****************** SET BOOKING DATA IF EXIST **********************/
		/*******************************************************************/
		// If existing booking
		if ($this->bookingCode != "")
		{
			if (isset($_REQUEST['booking_code']) && strlen($_REQUEST['booking_code']) > 0)
			{
				// If we provided a booking code from _POST or _GET, we need to flush all session data, to not allow to overwrite it
				$this->removeSessionVariables();
			}

			// Set variables from booking data first
			$this->setVariablesByBookingCode($this->bookingCode);
		}

        if(!isset($_REQUEST['booking_code']) ||
            (
                isset($_REQUEST['booking_code']) &&
                strlen($_REQUEST['booking_code']) == 0 ||
                $_POST['booking_code'] == $this->lang->getText('NRS_I_HAVE_BOOKING_NUMBER_TEXT')
            )
        ) {
            // Important: Process only if this is NOT _POST or _GET.
            /*******************************************************************/
            /********************** PARAMETERS FROM STEP1 **********************/
            /*******************************************************************/

            // 1 - Pickup Location Id
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['pickup_location_id']) || isset($_SESSION['pickup_location_id']))
            {
                $this->pickupLocationId = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'pickup_location_id', 0, $pickupLocationRequired, 'intval');
            }

            // 2/3 - Pickup Date & Time
            if($pickupDateVisible && (
                    isset($_REQUEST['nrs_do_search4']) ||
                    isset($_REQUEST['pickup_date'], $_REQUEST['pickup_time']) ||
                    isset($_SESSION['pickup_date'], $_SESSION['pickup_time']))
            ) {
                $defaultPickupTimestamp = time() + $this->minPeriodUntilPickup + get_option( 'gmt_offset' ) * 3600;
                $customerInputPickupDate = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'pickup_date', date_i18n($this->shortDateFormat, $defaultPickupTimestamp, TRUE), $pickupDateRequired, $this->shortDateFormat);
                $customerInputPickupTime = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'pickup_time', date_i18n("H:i:s", $defaultPickupTimestamp, TRUE), $pickupDateRequired, 'time_validation');
                $this->pickupTimestamp = NRSValidator::getUTCTimestampFromLocalISODateTime($customerInputPickupDate, $customerInputPickupTime);
            } else if($pickupDateVisible == FALSE)
            {
                // Pickup date & time is not visible, so we set default value, which is current time plus minimum period until pickup
                $this->pickupTimestamp = time() + $this->minPeriodUntilPickup;
            }

            // 4 - Dropoff Location Id
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['dropoff_location_id']) || isset($_SESSION['dropoff_location_id']))
            {
                $this->dropoffLocationId = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'dropoff_location_id', 0, $dropoffLocationRequired, 'intval');
            }

            // 5 - If booking duration is passed, it will override dropoff date and time
            if($dropoffDateVisible && (
                isset($_REQUEST['nrs_do_search4']) ||
                isset($_REQUEST['booking_period']) || isset($_SESSION['booking_period']) ||
                isset($_REQUEST['dropoff_date'], $_REQUEST['dropoff_time']) ||
                isset($_SESSION['dropoff_date'], $_SESSION['dropoff_time'])
            )) {
                if(isset($_REQUEST['booking_period']) || isset($_SESSION['booking_period']))
                {
                    // Dropoff date is a integer duration in seconds, and has to be added to pickup date
                    $validBookingPeriod = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'booking_period', 0, $dropoffDateRequired, 'intval');
                    $this->dropoffTimestamp = $this->pickupTimestamp + $validBookingPeriod;
                } else
                {
                    // 6/7 - Dropoff Date & Time
                    $defaultDropoffTimestamp = $this->pickupTimestamp + $this->minBookingPeriod;
                    $customerInputDropoffDate = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'dropoff_date', date_i18n($this->shortDateFormat, $defaultDropoffTimestamp, TRUE), $dropoffDateRequired, $this->shortDateFormat);
                    $customerInputDropoffTime = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'dropoff_time', date("H:i:s"), $dropoffDateRequired, 'time_validation');
                    $this->dropoffTimestamp = NRSValidator::getUTCTimestampFromLocalISODateTime($customerInputDropoffDate, $customerInputDropoffTime);
                }
            } else if($dropoffDateVisible == FALSE)
            {
                // Dropoff date & time is not visible, so we set default value, which is a pickup timestamp plus minimum booking period
                $this->dropoffTimestamp = $this->pickupTimestamp + $this->minBookingPeriod;
            }

            // 8 - Body Type Id
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['body_type_id']) || isset($_SESSION['body_type_id']))
            {
                $this->bodyTypeId = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'body_type_id', 0, $bodyTypeRequired, 'intval');
            }

            // 9 - Transmission Type Id
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['transmission_type_id']) || isset($_SESSION['transmission_type_id']))
            {
                $this->transmissionTypeId = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'transmission_type_id', 0, $transmissionTypeRequired, 'intval');
            }

            // 10 - Fuel Type Id
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['fuel_type_id']) || isset($_SESSION['fuel_type_id']))
            {
                $this->fuelTypeId = $this->getValidValueInput(array('POST', 'GET', 'SESSION'), 'fuel_type_id', 0, $fuelTypeRequired, 'intval');
            }

            //////////////////////////////////////////////////////////////////
            // FOR STEP3
            /******************* ITEM - IDs, UNITS, OPTIONS *****************/

            // 11 - Items Ids
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['item_ids']) || isset($_SESSION['item_ids']))
            {
                $itemIdsRequired = (isset($_REQUEST['nrs_do_search2']) || isset($_REQUEST['nrs_do_search3']) || isset($_REQUEST['nrs_do_search4'])) ? true : false;
                // came back to step3 from step3->step2->step edit mode
                $this->itemIds = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'item_ids', array(), $itemIdsRequired, 'intval');
            }

            // 12 - Item Units
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['item_units']) || isset($_SESSION['item_units']))
            {
                // came back to step2 from step3
                $this->itemUnits = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'item_units', array(), false, 'intval');
            }

            // 13 - Item Options
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['item_options']) || isset($_SESSION['item_options']))
            {
                // came back to step3 from step3->step2->step edit mode
                $this->itemOptions = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'item_options', array(), false, 'intval');
            }

            /******************* EXTRA - IDs, UNITS, OPTIONS *****************/

            // 14 - Extra Ids
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['extra_ids']) || isset($_SESSION['extra_ids']))
            {
                $this->extraIds = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'extra_ids', array(), false, 'intval');
            }

            // 15 - Extra Units
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['extra_units']) || isset($_SESSION['extra_units']))
            {
                $this->extraUnits = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'extra_units', array(), false, 'intval');
            }

            // 16 - Extra Options
            if(isset($_REQUEST['nrs_do_search4']) || isset($_REQUEST['extra_options']) || isset($_SESSION['extra_options']))
            {
                $this->extraOptions = $this->getValidArrayInput(array('POST', 'GET', 'SESSION'), 'extra_options', array(), false, 'intval');
            }
        }

		// Push alerts on empty required fields on booking process step
		if(isset($_REQUEST['nrs_do_search4']))
		{
			// came from step4
			// ! esc_html is vulnerable, we must use 'guest_text_validation',  'guest_multiline_text_validation' or 'email_validation'
			$this->customerData['title'] 				= $this->getValidValueInput(array('GET', 'POST'), 'title', '', $titleRequired, 'guest_text_validation');
			$this->customerData['first_name'] 			= $this->getValidValueInput(array('GET', 'POST'), 'first_name', '', $firstNameRequired, 'guest_text_validation');
			$this->customerData['last_name'] 			= $this->getValidValueInput(array('GET', 'POST'), 'last_name', '', $lastNameRequired, 'guest_text_validation');
			$birthYear						 			= $this->getValidValueInput(array('GET', 'POST'), 'birth_year', '0000', $birthdateRequired, 'intval');
			$birthMonth						 			= $this->getValidValueInput(array('GET', 'POST'), 'birth_month', '00', $birthdateRequired, 'intval');
			$birthDay						 			= $this->getValidValueInput(array('GET', 'POST'), 'birth_day', '00', $birthdateRequired, 'intval');
			$this->customerData['birthdate'] 			= NRSValidator::getValidISODate($birthYear.'-'.$birthMonth.'-'.$birthDay, 'Y-m-d');
			if($birthdateRequired && NRSValidator::isDate($birthYear.'-'.$birthMonth.'-'.$birthDay, 'Y-m-d') == FALSE)
			{
				// Invalidate the request. Like in try/catch/finally, but without it
				$this->addInputError('birthdate');
			}
			$this->customerData['street_address'] 		= $this->getValidValueInput(array('GET', 'POST'), 'street_address', '', $streetAddressRequired, 'guest_text_validation');
			$this->customerData['city'] 				= $this->getValidValueInput(array('GET', 'POST'), 'city', '', $cityRequired, 'guest_text_validation');
			$this->customerData['state'] 				= $this->getValidValueInput(array('GET', 'POST'), 'state', '', $stateRequired, 'guest_text_validation');
			$this->customerData['zipcode'] 				= $this->getValidValueInput(array('GET', 'POST'), 'zipcode', '', $zipcodeRequired, 'guest_text_validation');
			$this->customerData['country'] 				= $this->getValidValueInput(array('GET', 'POST'), 'country', '', $countryRequired, 'guest_text_validation');
			$this->customerData['phone'] 				= $this->getValidValueInput(array('GET', 'POST'), 'phone', '', $phoneRequired, 'guest_text_validation');
			$this->customerData['email'] 				= $this->getValidValueInput(array('GET', 'POST'), 'email', '', $emailRequired, 'email_validation');
			$this->customerData['additional_comments'] 	= $this->getValidValueInput(array('GET', 'POST'), 'additional_comments', '', $additionalCommentsRequired, 'guest_multiline_text_validation');
			$this->customerData['ip'] 					= $this->getValidValueInput(array('SERVER'), 'REMOTE_ADDR', '', false, 'guest_text_validation');
			if($this->isNewBooking())
			{
				$this->paymentMethodCode 				= $this->getValidValueInput(array('GET', 'POST'), 'payment_method_code', '', $paymentMethodRequired, 'guest_text_validation');
			}

			// ReCaptcha Validation for new Booking
			if($this->useReCaptcha() && $this->isNewBooking())
			{
				$validReCaptchaResponse					= $this->getValidValueInput(array('POST'), 'g-recaptcha-response', '', TRUE, 'guest_text_validation');

				// If the form submission includes the "g-captcha-response" field
				// Create an instance of the service using your secret
				$objReCaptcha 							= new \ReCaptcha\ReCaptcha($this->reCaptchaSecretKey);

				// If file_get_contents() is locked down on your PHP installation to disallow
				// its use with URLs, then you can use the alternative request method instead.
				// This makes use of fsockopen() instead.
				//  $objReCaptcha = new \ReCaptcha\ReCaptcha($this->reCaptchaSecretKey, new \ReCaptcha\RequestMethod\SocketPost());

				// Make the call to verify the response and also pass the user's IP address
				$objReCaptchaResponse = $objReCaptcha->verify($validReCaptchaResponse, $this->customerData['ip']);

				if ($objReCaptchaResponse->isSuccess())
				{
					// If the response is a success, that's it!
					// Do nothing
				} else
				{
					// If it's not successful, then one or more error codes will be returned.
					$this->validSearch = FALSE;
					$this->errorMessages[] = $this->lang->getText('NRS_ERROR_INVALID_SECURITY_CODE_TEXT');
					if($this->debugMode == 1)
					{
						// Error codes reference can be found at:
						// https://developers.google.com/recaptcha/docs/verify#error-code-reference%22
						echo "<br />ReCaptcha validator returned the following errors: ";
						foreach ($objReCaptchaResponse->getErrorCodes() as $code) {
							echo '<br />'.$code.'';
						}
						echo '<br />Check the error code reference at ';
						echo '<a href="https://developers.google.com/recaptcha/docs/verify#error-code-reference">';
						echo 'https://developers.google.com/recaptcha/docs/verify#error-code-reference</a>';
						echo '<br /><strong>Note:</strong> Error code <em>missing-input-response</em> may mean';
						echo ' the user just didn&#39;t complete the reCAPTCHA.';
					}
				}
			}

		}

		if($this->debugMode == 2)
		{
			echo "<br />Final stage of class variables after setRequestParams() was called.";
			echo "<br />".nl2br(print_r($this->getSearchInputDataArray(), TRUE));
		}
	}

	protected function setVariablesByBookingCode($paramBookingCode)
	{
		if($paramBookingCode != "")
		{
			$objBooking = new NRSBooking($this->internalWPDB, $this->nrsPrefix, $this->lang, NULL, $paramBookingCode);
			$bookingDetails = $objBooking->getBookingDetails();
			if(!is_null($bookingDetails))
			{
				$objCustomer = new NRSCustomer($this->internalWPDB, $this->nrsPrefix, $this->lang, "", $objBooking->getCustomerId());
				// Might be NULL as well
				$this->customerData = $objCustomer->getCustomerDetails();
				// These variables are the regular params
				$this->pickupLocationId = $bookingDetails['pickup_location_id']; // used in all steps
				$this->dropoffLocationId = $bookingDetails['dropoff_location_id']; // used in all steps
				$this->pickupTimestamp = $bookingDetails['pickup_timestamp'];
				$this->dropoffTimestamp = $bookingDetails['dropoff_timestamp'];
				$this->bodyTypeId = $bookingDetails['body_type_id']; // used in step 2 (3)
				$this->transmissionTypeId = $bookingDetails['transmission_type_id']; // used in step 2 (3)
				$this->fuelTypeId = $bookingDetails['fuel_type_id']; // used in step 2 (3)
				// These arrays are the regular params
				$this->itemIds = $bookingDetails['item_ids'];
				$this->itemUnits = $bookingDetails['item_units'];
				$this->itemOptions = $bookingDetails['item_options'];
				$this->extraIds = $bookingDetails['extra_ids'];
				$this->extraUnits = $bookingDetails['extra_units'];
				$this->extraOptions = $bookingDetails['extra_options'];
				// Payment method code
				$this->paymentMethodCode = $bookingDetails['payment_method_code'];

				if($this->debugMode)
				{
					echo "<br />Class variables set from booking code. ";
					//echo "<br />Class variables set from booking code. Class variables: ";
					//echo nl2br(print_r($this->getSearchInputDataArray(), TRUE));
				}
			}

			if($objBooking->getBookingId() == 0)
			{
				// Booking code passed, but not exist in database
				$this->validSearch = FALSE;
				$this->errorMessages[] = $this->lang->getText('NRS_ERROR_INVALID_BOOKING_CODE_TEXT');
			} else if($objBooking->isValidBooking() == FALSE)
			{
				$this->validSearch = FALSE;
				if($objBooking->isDepartedBooking())
				{
					$this->errorMessages[] = sprintf($this->lang->getText('NRS_ERROR_DEPARTED_TEXT'), $paramBookingCode);
				}
				if($objBooking->isCancelledBooking())
				{
					$this->errorMessages[] = sprintf($this->lang->getText('NRS_ERROR_CANCELLED_TEXT'), $paramBookingCode);
				}
				if($objBooking->isRefundedBooking())
				{
					$this->errorMessages[] = sprintf($this->lang->getText('NRS_ERROR_REFUNDED_TEXT'), $paramBookingCode);
				}
			}
		}
	}

	/**
	 * @return array
	 */
	public function getPriceSummary()
	{
		// Set default values, to avoid not existing key errors
		$retPrices 											= array();
		$retPrices['items'] 								= array();
		$retPrices['extras'] 								= array();
		$retPrices['pickup']								= array();
		$retPrices['pickup']['nightly_rate_applied']		= FALSE;
		$retPrices['pickup']['fee']   						= 0.00;
		$retPrices['pickup']['fee_with_vat']   				= 0.00;
		$retPrices['pickup']['vat_amount']   				= 0.00;
		$retPrices['dropoff']								= array();
		$retPrices['dropoff']['nightly_rate_applied']		= FALSE;
		$retPrices['dropoff']['fee']   						= 0.00;
		$retPrices['dropoff']['fee_with_vat']   			= 0.00;
		$retPrices['dropoff']['vat_amount']   				= 0.00;
		// Counted defaults from price element
		$retPrices['percentage']							= array();
		$retPrices['percentage']['vat']						= $this->vatPercentage;
		$retPrices['item_totals'] 							= array();
		$retPrices['extra_totals'] 							= array();
		$retPrices['overall'] 								= array();
		// Counted default from price element
		$retPrices['overall']['discounted_total'] 			= 0.00;
		$retPrices['overall']['discounted_vat_amount'] 		= 0.00;
		$retPrices['overall']['discounted_total_with_vat'] 	= 0.00;
        $retPrices['overall']['item_prepayment_amount']             = 0.00;
        $retPrices['overall']['item_prepayment_amount_with_vat']    = 0.00;
        $retPrices['overall']['extra_prepayment_amount']            = 0.00;
        $retPrices['overall']['extra_prepayment_amount_with_vat']   = 0.00;
		$retPrices['overall']['prepayment_amount'] 			= 0.00;
		$retPrices['overall']['prepayment_amount_with_vat'] = 0.00;
		$retPrices['overall']['fixed_item_deposit_amount'] 	= 0.00;
		$retPrices['overall']['fixed_extra_deposit_amount']	= 0.00;
		$retPrices['overall']['fixed_deposit_amount'] 		= 0.00;
		// Counted overalls
		$retPrices['overall']['pickup_fee']   				= 0.00;
		$retPrices['overall']['dropoff_fee']   				= 0.00;
		$retPrices['overall']['gross_total']     			= 0.00;
		$retPrices['overall']['total_vat']     				= 0.00;
		$retPrices['overall']['grand_total']   				= 0.00;
		$retPrices['overall']['total_pay_now']   			= 0.00;
		$retPrices['overall']['total_pay_later']   			= 0.00;

		// Set internal search config for use when creating classes
		$internalSearchConfig = array(
			"conf_currency_symbol" => $this->currencySymbol,
			"conf_currency_code" => $this->currencyCode,
			"conf_vat_percentage" => $this->vatPercentage,
			"conf_show_prices_with_vat" => $this->showPricesWithVAT,
		);

		// 1 - LOAD THE DATA
		$objPrepaymentsObserver = new NRSItemPrepaymentsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$objBooking = new NRSBooking($this->internalWPDB, $this->nrsPrefix, $this->lang, NULL, $this->bookingCode);
		$objSearchManagerForItems = new NRSSearchManagerForItems(
			$this->internalWPDB, $this->nrsPrefix, $this->lang, $internalSearchConfig,
			$objBooking->getBookingId(), $this->pickupLocationId, $this->itemIds, $this->itemUnits, $this->itemOptions
		);
		$objSearchManagerForExtras = new NRSSearchManagerForExtras(
			$this->internalWPDB, $this->nrsPrefix, $this->lang, $internalSearchConfig,
			$objBooking->getBookingId(), $this->pickupLocationId, $this->extraIds, $this->extraUnits, $this->extraOptions
		);
		$objPickupLocation = new NRSLocation(
			$this->internalWPDB, $this->nrsPrefix, $this->lang, $internalSearchConfig, $this->pickupLocationId
		);
		$objDropoffLocation = new NRSLocation(
			$this->internalWPDB, $this->nrsPrefix, $this->lang, $internalSearchConfig, $this->dropoffLocationId
		);

		$pickupLocationData = $objPickupLocation->getLocationDetails($this->getLocalPickupDayOfWeek());
		$dropoffLocationData = $objDropoffLocation->getLocationDetails($this->getLocalDropoffDayOfWeek());
		$retPrices['items'] = $objSearchManagerForItems->getSelectedElements($this->pickupTimestamp, $this->dropoffTimestamp);
		$retPrices['extras'] = $objSearchManagerForExtras->getSelectedElements($this->pickupTimestamp, $this->dropoffTimestamp);

		/************************************************************************************************************/
		// Update the overall prices stack with counted values
		// Update the overall price array by adding current element multiplied prices to overall prices
		foreach($retPrices['items'] AS $item)
		{
			foreach($item['multiplied'] AS $key => $multipliedPrice)
			{
				if(!isset($retPrices['overall'][$key]))
				{
					// Set first price to that specific key
					$retPrices['overall'][$key] = $multipliedPrice;
				} else
				{
					// Add prices one by one to that specific key
					$retPrices['overall'][$key] += $multipliedPrice;
				}

				// We use item_totals only for hover details text
				if(!isset($retPrices['item_totals'][$key]))
				{
					$retPrices['item_totals'][$key] = $multipliedPrice;
				} else
				{
					$retPrices['item_totals'][$key] += $multipliedPrice;
				}
			}
		}

		// Add extras prices
		foreach($retPrices['extras'] AS $extra)
		{
			foreach($extra['multiplied'] AS $key => $multipliedPrice)
			{
				if(!isset($retPrices['overall'][$key]))
				{
					// Set first price to that specific key
					$retPrices['overall'][$key] = $multipliedPrice;
				} else
				{
					// Add prices one by one to that specific key
					$retPrices['overall'][$key] += $multipliedPrice;
				}

				// We use extra_totals only for hover details text
				if(!isset($retPrices['extra_totals'][$key]))
				{
					$retPrices['extra_totals'][$key] = $multipliedPrice;
				} else
				{
					$retPrices['extra_totals'][$key] += $multipliedPrice;
				}
			}
		}

        // Prepayments blank items set for both scenarios - default value if no items/extras selected, and number if exist
        if(isset($retPrices['item_totals']['prepayment_amount']))
        {
            $retPrices['overall']['item_prepayment_amount'] = $retPrices['item_totals']['prepayment_amount'];
        }
        if(isset($retPrices['item_totals']['prepayment_amount_with_vat']))
        {
            $retPrices['overall']['item_prepayment_amount_with_vat'] = $retPrices['item_totals']['prepayment_amount_with_vat'];
        }
        if(isset($retPrices['extra_totals']['prepayment_amount']))
        {
            $retPrices['overall']['extra_prepayment_amount'] = $retPrices['extra_totals']['prepayment_amount'];
        }
        if(isset($retPrices['extra_totals']['prepayment_amount_with_vat']))
        {
            $retPrices['overall']['extra_prepayment_amount_with_vat'] = $retPrices['extra_totals']['prepayment_amount_with_vat'];
        }

        // Deposits blank items set for both scenarios - default value if no items/extras selected, and number if exist
		if(isset($retPrices['item_totals']['fixed_deposit_amount']))
		{
			$retPrices['overall']['fixed_item_deposit_amount'] 	= $retPrices['item_totals']['fixed_deposit_amount'];
		}
		if(isset($retPrices['extra_totals']['fixed_deposit_amount']))
		{
			$retPrices['overall']['fixed_extra_deposit_amount'] = $retPrices['extra_totals']['fixed_deposit_amount'];
		}

		/************************************************************************************************************/
		// Pickup
		if(!is_null($pickupLocationData))
		{
			$retPrices['pickup']['nightly_rate_applied'] = $objPickupLocation->isAfterhoursTime($this->getLocalPickupDayOfWeek(), $this->getLocalPickupTime());
			if($retPrices['pickup']['nightly_rate_applied'] === TRUE)
			{
				// After hours
				$retPrices['pickup']['fee'] 		= $pickupLocationData['afterhours_pickup_fee'];
			} else
			{
				// Working hours
				$retPrices['pickup']['fee'] 		= $pickupLocationData['pickup_fee'];
			}
			$retPrices['pickup']['fee_with_vat']	= $retPrices['pickup']['fee']  * (1 + $this->vatPercentage / 100);
			$retPrices['pickup']['vat_amount']  	= $retPrices['pickup']['fee']  * ($this->vatPercentage / 100);
		}

		// Dropoff
		if(!is_null($dropoffLocationData))
		{
			$retPrices['dropoff']['nightly_rate_applied'] = $objDropoffLocation->isAfterhoursTime(
				$this->getLocalDropoffDayOfWeek(), $this->getLocalDropoffTime()
			);
			if($retPrices['dropoff']['nightly_rate_applied'] === TRUE)
			{
				// After hours
				$retPrices['dropoff']['fee']   		= $dropoffLocationData['afterhours_dropoff_fee'];
			} else
			{
				// Working hours
				$retPrices['dropoff']['fee']   		= $dropoffLocationData['dropoff_fee'];
			}
			$retPrices['dropoff']['fee_with_vat'] 	= $retPrices['dropoff']['fee'] * (1 + $this->vatPercentage / 100);
			$retPrices['dropoff']['vat_amount']   	= $retPrices['dropoff']['fee'] * ($this->vatPercentage / 100);
		}

		// Pickup & Dropoff totals
		$totalItemUnitsSelected 					= $objSearchManagerForItems->getTotalSelectedQuantity();
		$retPrices['overall']['units_selected'] 	= $totalItemUnitsSelected;
		$retPrices['overall']['pickup_fee'] 		= $totalItemUnitsSelected * $retPrices['pickup']['fee'];
		$retPrices['overall']['pickup_fee_with_vat']= $totalItemUnitsSelected * $retPrices['pickup']['fee_with_vat'];
		$retPrices['overall']['pickup_vat_amount'] 	= $totalItemUnitsSelected * $retPrices['pickup']['vat_amount'];
		$retPrices['overall']['dropoff_fee']   		= $totalItemUnitsSelected * $retPrices['dropoff']['fee'];
		$retPrices['overall']['dropoff_fee_with_vat']=$totalItemUnitsSelected * $retPrices['dropoff']['fee_with_vat'];
		$retPrices['overall']['dropoff_vat_amount'] = $totalItemUnitsSelected * $retPrices['dropoff']['vat_amount'];

		/* -------------------------------- Calculate totals ------------------------------------ */
		// Add total item+extras discounted total, pickup and dropoff location fees to gross total
		$retPrices['overall']['gross_total'] = $retPrices['overall']['discounted_total'];
		$retPrices['overall']['gross_total'] += ($retPrices['overall']['pickup_fee'] + $retPrices['overall']['dropoff_fee']);

		$retPrices['overall']['total_vat'] 	 = $retPrices['overall']['discounted_vat_amount'];
		$retPrices['overall']['total_vat'] 	 += ($retPrices['overall']['pickup_vat_amount'] + $retPrices['overall']['dropoff_vat_amount']);

		$retPrices['overall']['grand_total'] = $retPrices['overall']['discounted_total_with_vat'];
		$retPrices['overall']['grand_total'] += ($retPrices['overall']['pickup_fee_with_vat'] + $retPrices['overall']['dropoff_fee_with_vat']);

		if($objPrepaymentsObserver->arePrepaymentsEnabled())
		{
			$retPrices['overall']['total_pay_now'] = $retPrices['overall']['prepayment_amount_with_vat'];
			$retPrices['overall']['total_pay_now'] += ($retPrices['overall']['pickup_fee_with_vat'] + $retPrices['overall']['dropoff_fee_with_vat']);
			$retPrices['overall']['total_pay_later'] = $retPrices['overall']['grand_total'] - $retPrices['overall']['total_pay_now'];
		}

		// Overall pickup prices prints
		$retPrices['pickup_tiny_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "tiny");
		$retPrices['pickup_tiny_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "tiny_without_fraction");
		$retPrices['pickup_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "regular");
		$retPrices['pickup_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "regular_without_fraction");
		$retPrices['pickup_long_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "long");
		$retPrices['pickup_long_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['pickup'], "long_without_fraction");

		// Overall dropoff prices prints
		$retPrices['dropoff_tiny_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "tiny");
		$retPrices['dropoff_tiny_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "tiny_without_fraction");
		$retPrices['dropoff_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "regular");
		$retPrices['dropoff_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "regular_without_fraction");
		$retPrices['dropoff_long_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "long");
		$retPrices['dropoff_long_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['dropoff'], "long_without_fraction");

		// Overall price prints
		$retPrices['overall_tiny_print'] = $this->getFormattedPriceArray($retPrices['overall'], "tiny");
		$retPrices['overall_tiny_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['overall'], "tiny_without_fraction");
		$retPrices['overall_print'] = $this->getFormattedPriceArray($retPrices['overall'], "regular");
		$retPrices['overall_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['overall'], "regular_without_fraction");
		$retPrices['overall_long_print'] = $this->getFormattedPriceArray($retPrices['overall'], "long");
		$retPrices['overall_long_without_fraction_print'] = $this->getFormattedPriceArray($retPrices['overall'], "long_without_fraction");

		// Print percentages
		$retPrices['percentage_print'] = NRSFormatter::getFormattedPercentageArray($retPrices['percentage'], "regular");

		// Print totals (for hover text)
		$retPrices['item_totals_print'] = $this->getFormattedPriceArray($retPrices['item_totals'], "regular");
		$retPrices['extra_totals_print'] = $this->getFormattedPriceArray($retPrices['extra_totals'], "regular");


		//echo nl2br(esc_html(print_r($retPrices, TRUE)));
		return $retPrices;
	}
}
