<?php
/**
 * Elements observer (array)
 * Abstract classes can't be created with new instance. It is only possible if they are extended by childs
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSSearchManager
{
	protected $internalWPDB 		= NULL;
	protected $nrsPrefix 			= "";
	protected $lang 				= NULL;
	protected $debugMode 			= 2;

	protected $bookingId			= 0;
	protected $locationId			= 0;
	protected $elementIds			= array();
	protected $elementUnits			= array();
	protected $elementOptions		= array();

	protected $currencySymbol		= '$';
	protected $currencyCode			= 'USD';
	protected $vatPercentage		= 0.00;
	protected $showPricesWithVAT	= 0;

	/**
	 * @param wpdb $paramWPDB
	 * @param $paramNRSPrefix
	 * @param NRSLanguage &$paramNRSLang - Language
	 * @param array $paramConfig
	 * @param int $paramBookingId = 0 - used because we need to support booking edits
	 * @param int $paramLocationId = 0 - used because we need to support item blocks in specific location as well, not only all location blocks
	 * @param array $paramElementIds
	 * @param array $paramElementUnits
	 * @param array $paramElementOptions
	 */
	public function __construct(
		wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array(),
		$paramBookingId = 0, $paramLocationId = 0, $paramElementIds = array(), $paramElementUnits = array(), $paramElementOptions = array()
	) {
		// Set class settings
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;

		$this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
		$this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
		$this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
		$this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));

		$this->bookingId = NRSValidator::getValidInteger($paramBookingId, 0);
		$this->locationId = NRSValidator::getValidInteger($paramLocationId, 0);
		$this->elementIds = NRSValidator::getValidArray($paramElementIds, "intval", 0);
		$this->elementUnits = NRSValidator::getValidArray($paramElementUnits, "intval", 0);
		$this->elementOptions = NRSValidator::getValidArray($paramElementOptions, "intval", 0);
	}

	public function inDebug()
	{
		return ($this->debugMode >= 1 ? TRUE : FALSE);
	}

	protected function getQuantity($paramElementId, $paramMaxAvailableUnits)
	{
		$validElementId = NRSValidator::getValidInteger($paramElementId, 0);
		$validMaxAvailableUnits = NRSValidator::getValidInteger($paramMaxAvailableUnits, 0);
		$itemUnitsSelected = 0;
		if(is_array($this->elementIds) && sizeof($this->elementIds) > 0 && in_array($validElementId, $this->elementIds))
		{
			if(isset($this->elementUnits[$validElementId]) && $this->elementUnits[$validElementId] > 0)
			{
				if($validMaxAvailableUnits > $this->elementUnits[$validElementId])
				{
					$itemUnitsSelected = $this->elementUnits[$validElementId];
				} else
				{
					$itemUnitsSelected = $validMaxAvailableUnits;
				}
			}
		}

		return $itemUnitsSelected;
	}
	protected function getOption($paramElementId)
	{
		$validElementId = NRSValidator::getValidInteger($paramElementId, 0);
		$selectedOptionId = 0;
		if(is_array($this->elementIds) && sizeof($this->elementIds) > 0 && in_array($validElementId, $this->elementIds))
		{
			if(isset($this->elementOptions[$validElementId]) && $this->elementOptions[$validElementId] > 0)
			{
				$selectedOptionId = $this->elementOptions[$validElementId];
			}
		}

		return $selectedOptionId;
	}

	/**
	 * Makes impact to location fees
	 * @return int
	 */
	public function getTotalSelectedQuantity()
	{
		$totalUnitsSelected = 0;
		foreach($this->elementIds AS $itemId)
		{
			$itemUnitsSelected = isset($this->elementUnits[$itemId]) ? $this->elementUnits[$itemId] : 1;
			// Add current units amount
			$totalUnitsSelected += $itemUnitsSelected;
		}

		return $totalUnitsSelected;
	}
}
