<?php
/**
 * Extras Observer (array)
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSSearchManagerForExtras extends NRSSearchManager implements iNRSSearchManager
{
	public function __construct(
		wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array(),
		$paramBookingId = 0, $paramLocationId = 0, $paramElementIds = array(), $paramElementUnits = array(), $paramElementOptions = array()
	) {
		parent::__construct(
			$paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig, $paramBookingId, $paramLocationId, $paramElementIds, $paramElementUnits, $paramElementOptions
		);
	}

	public function getAvailableElements($paramPickupTimestamp, $paramDropoffTimestamp)
	{
		return $this->getElements("AVAILABLE", $paramPickupTimestamp, $paramDropoffTimestamp);
	}

	public function getSelectedElements($paramPickupTimestamp, $paramDropoffTimestamp)
	{
		return $this->getElements("SELECTED", $paramPickupTimestamp, $paramDropoffTimestamp);
	}

	/**
	 * @param string $paramType - "AVAILABLE" or "SELECTED"
	 * @param int $paramPickupTimestamp
	 * @param int $paramDropoffTimestamp
	 * @return array
	 */
	public function getElements($paramType, $paramPickupTimestamp, $paramDropoffTimestamp)
	{
		$selectedElementsOnly = $paramType == "SELECTED" ? TRUE : FALSE;

		$retElements = array();
		$internalConfig = array(
			"conf_currency_symbol" => $this->currencySymbol,
			"conf_currency_code" => $this->currencyCode,
			"conf_vat_percentage" => $this->vatPercentage,
			"conf_show_prices_with_vat" => $this->showPricesWithVAT,
		);

		foreach($this->elementIds AS $extraId)
		{
			$objUnitsManager = new NRSExtraUnitManager(
				$this->internalWPDB,
				$this->nrsPrefix,
				$this->lang,
				$extraId,
				$paramPickupTimestamp,
				$paramDropoffTimestamp
			);

			$availableUnits = $objUnitsManager->getTotalUnitsAvailable($this->locationId, $this->bookingId);
			$minAllowedUnits = $objUnitsManager->getMinUnitsPerBooking();
			// If there is more items in stock than booked, and more items in stock than min quantity for booking
			if($availableUnits > 0 && $availableUnits >= $minAllowedUnits)
			{
				$objExtra = new NRSExtra($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId);
				$objOptionsManager = new NRSExtraOptionManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId);
				$objPriceManager = new NRSExtraPriceManager(
					$this->internalWPDB, $this->nrsPrefix, $this->lang,
					$internalConfig, $extraId
				);

				// 1 - Process extra details & prices
				$extraDetails = $objExtra->getFullDetails();
				$maxAllowedUnits = $objUnitsManager->getMaxAllowedUnitsForBooking($this->locationId, $this->bookingId);
				$selectedQuantity = $this->getQuantity($extraId, $maxAllowedUnits);
				//$extraPriceDetails = $objPriceManager->getUnitPriceDetailsByInterval($paramPickupTimestamp, $paramDropoffTimestamp);
				$extraPriceDetails = $objPriceManager->getMultipliedPriceDetailsByInterval($paramPickupTimestamp, $paramDropoffTimestamp, $selectedQuantity);

				// 2 - Process extra options
				$totalOptions = $objOptionsManager->getTotalOptions();
				if($totalOptions == 1)
				{
					// Auto-select first option
					$arrOptions = $objOptionsManager->getOptions();
					$selectedOptionId = $arrOptions[0]['option_id'];
				} else
				{
					$selectedOptionId = $this->getOption($extraId);
				}
				$objSelectedOption = new NRSOption($this->internalWPDB, $this->nrsPrefix, $this->lang, $selectedOptionId);
				$selectedOptionDetails = $objSelectedOption->getDetails();
				$selectedOptionName = "";
				$optionsHTML = "";
				if($totalOptions > 1)
				{
					$selectedOptionName = $selectedOptionDetails['option_name'];
					if($extraDetails['options_display_mode'] == 1)
					{
						$optionsHTML = $objOptionsManager->getDropDown($selectedOptionId, $extraDetails['measurement_unit']);
					} else
					{
						//echo nl2br(print_r($extraDetails, TRUE));
						//echo "Extra ID: $extraId, Measurement Unit: {$extraDetails['measurement_unit']}";
						$optionsHTML = $objOptionsManager->getSlider($selectedOptionId, $extraDetails['measurement_unit']);
					}
				} else if($totalOptions == 1)
				{
					$selectedOptionName = $selectedOptionDetails['option_name'];
					$optionsHTML = $selectedOptionDetails['option_name'];
				}

				$printExtraNameWithSelectedOption  = $extraDetails['extra_name'];
				$printExtraNameWithSelectedOption .= $selectedOptionName ? ', '.$selectedOptionName.' '.$extraDetails['measurement_unit'] : '';

				// 3 - Extend the $extra output with new details
				$extraDetails['selected'] = $selectedQuantity > 0 ? true : false;
				$extraDetails['selected_quantity'] = $selectedQuantity;
				$extraDetails['min_allowed_units'] = $minAllowedUnits;
				$extraDetails['max_allowed_units'] = $maxAllowedUnits;
				$extraDetails['selected_option_id'] = $selectedOptionId;
				$extraDetails['selected_option_name'] = $selectedOptionName;
				$extraDetails['options_html'] = $optionsHTML;
				$extraDetails['total_options'] = $objOptionsManager->getTotalOptions();
				$extraDetails['print_checked'] = $selectedQuantity > 0 ? ' checked="checked"' : '';
				$extraDetails['print_selected'] = $selectedQuantity > 0 ? 'selected' : '';
				$extraDetails['print_extra_name_with_selected_option'] = $printExtraNameWithSelectedOption;

				if(($selectedElementsOnly && $selectedQuantity > 0) || $selectedElementsOnly == FALSE)
				{
					// Add to stack only if element is selected or if we return all elements
					$retElements[] = array_merge($extraDetails, $extraPriceDetails);
				}

				if($this->debugMode == 1)
				{
					echo "<br /><br />Extra with ID={$extraId} is <span style='color:green;font-weight:bold;'>AVAILABLE</span> for booking ";
					echo "has {$selectedQuantity} units selected of {$availableUnits} units available, with total {$extraDetails['units_in_stock']} units in stock, ";
					echo "<br />with minimum/maximum {$minAllowedUnits}/{$maxAllowedUnits} units allowed per booking, ";
					echo "and booking min/max unit limits set to: {$extraDetails['min_units_per_booking']}/{$extraDetails['max_units_per_booking']}, ";
					echo "including current booking #{$this->bookingId}";
				}
			} else
			{
				if($this->debugMode == 1)
				{
					echo "<br /><br />Extra with ID={$extraId} is <span style='color:red;font-weight:bold;'>NOT AVAILABLE</span> for booking ";
					echo "and currently has {$availableUnits} units available, ";
					echo "<br />with minimum {$minAllowedUnits} units allowed per booking, ";
					echo "including current booking #{$this->bookingId}";
				}
			}
		}

		return $retElements;
	}
}
