<?php
/**
 * Items Observer (array)
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSSearchManagerForItems extends NRSSearchManager implements iNRSSearchManager
{
	public function __construct(
		wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array(),
		$paramBookingId = 0, $paramLocationId = 0, $paramElementIds = array(), $paramElementUnits = array(), $paramElementOptions = array()
	) {
		parent::__construct(
			$paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig, $paramBookingId, $paramLocationId, $paramElementIds, $paramElementUnits, $paramElementOptions
		);
	}

	public function getAvailableElements($paramPickupTimestamp, $paramDropoffTimestamp)
	{
		return $this->getElements("AVAILABLE", $paramPickupTimestamp, $paramDropoffTimestamp);
	}

	public function getSelectedElements($paramPickupTimestamp, $paramDropoffTimestamp)
	{
		return $this->getElements("SELECTED", $paramPickupTimestamp, $paramDropoffTimestamp);
	}

	/**
	 * @param string $paramType - "AVAILABLE" or "SELECTED"
	 * @param int $paramPickupTimestamp
	 * @param int $paramDropoffTimestamp
	 * @return array
	 */
	private function getElements($paramType, $paramPickupTimestamp, $paramDropoffTimestamp)
	{
		$selectedElementsOnly = $paramType == "SELECTED" ? TRUE : FALSE;
		$retElements = array();
		$internalConfig = array(
			"conf_currency_symbol" => $this->currencySymbol,
			"conf_currency_code" => $this->currencyCode,
			"conf_vat_percentage" => $this->vatPercentage,
			"conf_show_prices_with_vat" => $this->showPricesWithVAT,
		);

		foreach($this->elementIds AS $itemId)
		{
			$objUnitsManager = new NRSItemUnitManager(
				$this->internalWPDB,
				$this->nrsPrefix,
				$this->lang,
				$itemId,
				$paramPickupTimestamp,
				$paramDropoffTimestamp
			);

			$availableUnits = $objUnitsManager->getTotalUnitsAvailable($this->locationId, $this->bookingId);
			$minAllowedUnits = $objUnitsManager->getMinUnitsPerBooking();

			// If there is more items in stock than booked, and more items in stock than min quantity for booking
			if($availableUnits > 0 && $availableUnits >= $minAllowedUnits)
			{
				$objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId);
				$objOptionsManager = new NRSItemOptionManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId);
				$objPriceManager = new NRSItemPriceManager(
					$this->internalWPDB, $this->nrsPrefix, $this->lang,
					$internalConfig, $itemId
				);

				// 1 - Process full item details & prices
				$itemDetails = $objItem->getFullDetails();
				$maxAllowedUnits = $objUnitsManager->getMaxAllowedUnitsForBooking($this->locationId, $this->bookingId);
				$selectedQuantity = $this->getQuantity($itemId, $maxAllowedUnits);
				//$itemPriceDetails = $objPriceManager->getUnitPriceDetailsByInterval($paramPickupTimestamp, $paramDropoffTimestamp);
				$itemPriceDetails = $objPriceManager->getMultipliedPriceDetailsByInterval($paramPickupTimestamp, $paramDropoffTimestamp, $selectedQuantity);

				// 2 - Process item options
				$totalOptions = $objOptionsManager->getTotalOptions();
				if($totalOptions == 1)
				{
					// Auto-select first option
					$arrOptions = $objOptionsManager->getOptions();
					$selectedOptionId = $arrOptions[0]['option_id'];
				} else
				{
					$selectedOptionId = $this->getOption($itemId);
				}
				$objSelectedOption = new NRSOption($this->internalWPDB, $this->nrsPrefix, $this->lang, $selectedOptionId);
				$selectedOptionDetails = $objSelectedOption->getDetails();
				$selectedOptionName = "";
				$optionsHTML = "";
				if($totalOptions > 1)
				{
					$selectedOptionName = $selectedOptionDetails['option_name'];
					if($itemDetails['options_display_mode'] == 1)
					{
						$optionsHTML = $objOptionsManager->getDropDown($selectedOptionId, $itemDetails['measurement_unit']);
					} else
					{
						//echo nl2br(print_r($itemDetails, TRUE));
						//echo "Item ID: $itemId, Measurement Unit: {$itemDetails['measurement_unit']}";
						$optionsHTML = $objOptionsManager->getSlider($selectedOptionId, $itemDetails['measurement_unit']);
					}
				} else if($totalOptions == 1)
				{
					$selectedOptionName = $selectedOptionDetails['option_name'];
					$optionsHTML = $selectedOptionDetails['option_name'];
				}


				$printItemNameWithSelectedOption = $itemDetails['manufacturer_title'].' '.$itemDetails['model_name'];
				$printItemNameWithSelectedOption .= $itemDetails['body_type_title'] ? ', '.$itemDetails['body_type_title'] : '';
				$printItemNameWithSelectedOption .= $selectedOptionName ? ', '.$selectedOptionName.' '.$itemDetails['measurement_unit'] : '';

				// 3 - Extend the $item output with new details
				$itemDetails['selected'] = $selectedQuantity > 0 ? true : false;
				$itemDetails['selected_quantity'] = $selectedQuantity;
				$itemDetails['min_allowed_units'] = $minAllowedUnits;
				$itemDetails['max_allowed_units'] = $maxAllowedUnits;
				$itemDetails['selected_option_id'] = $selectedOptionId;
				$itemDetails['selected_option_name'] = $selectedOptionName;
				$itemDetails['options_html'] = $optionsHTML;
				$itemDetails['total_options'] = $totalOptions;
				$itemDetails['print_checked'] = $selectedQuantity > 0 ? ' checked="checked"' : '';
				$itemDetails['print_selected'] = $selectedQuantity > 0 ? 'selected' : '';
				$itemDetails['print_item_name_with_selected_option'] = $printItemNameWithSelectedOption;

				if(($selectedElementsOnly && $selectedQuantity > 0) || $selectedElementsOnly == FALSE)
				{
					// Add to stack only if element is selected or if we return all elements
					$retElements[] = array_merge($itemDetails, $itemPriceDetails);
				}

				if($this->debugMode == 1)
				{
					echo "<br /><br />Car with ID={$itemId} is <span style='color:green;font-weight:bold;'>AVAILABLE</span> for booking ";
					echo "has {$selectedQuantity} units selected of {$availableUnits} units available, with total {$itemDetails['units_in_stock']} units in stock, ";
					echo "<br />with minimum/maximum {$minAllowedUnits}/{$maxAllowedUnits} units allowed per booking, ";
					echo "and booking min/max unit limits set to: 0/{$itemDetails['max_units_per_booking']}, ";
					echo "including current booking #{$this->bookingId}";
				}
			} else
			{
				if($this->debugMode == 1)
				{
					echo "<br /><br />Car with ID={$itemId} is <span style='color:red;font-weight:bold;'>NOT AVAILABLE</span> for booking ";
					echo "and currently has {$availableUnits} units available, ";
					echo "<br />with minimum {$minAllowedUnits} units allowed per booking, ";
					echo "including current booking #{$this->bookingId}";
				}
			}
		}

		return $retElements;
	}
}
