<?php
/**
 * NRS Discount Manager
 * Abstract class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

class NRSDiscountManager {
    protected $internalWPDB 		    = NULL;
    protected $nrsPrefix 			    = "";
    protected $lang 			        = NULL;
    protected $debugMode 			    = 0;
    /**
     * @var int - either itemId or either extraId;
     */
    protected $elementId                = 0;
    /**
     * @var int - price calculation: 1 - daily, 2 - hourly, 3 - mixed (daily+hourly)
     */
    protected $priceCalculationType     = 1;
    protected $discountEnabled  		= TRUE;

    /**
     * @param &$paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage $paramNRSLang
     * @param int $paramElementId - Item or Extra Id
     * @param array $paramOptionalConfig
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramElementId, $paramOptionalConfig = array())
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;
        $this->elementId = NRSValidator::getValidInteger($paramElementId, 1);

        // Set price calculation type
        $this->setPriceCalculationType($paramOptionalConfig);

        // Set discount status
        $this->setDiscountStatus($paramOptionalConfig);
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    /**
     * Set price calculation type - called in class constructor
     * @param array $paramOptionalConfig
     */
    private function setPriceCalculationType($paramOptionalConfig)
    {
        if(isset($paramOptionalConfig['conf_price_calculation_type']))
        {
            // Fast processing mode (for availability and price table), to reduce sql query amount
            $this->priceCalculationType = NRSValidator::getValidInteger($paramOptionalConfig['conf_price_calculation_type'], 1);
        } else
        {
            $sql = "
				SELECT conf_value AS price_calculation_type
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_price_calculation_type'
			";
            $priceCalculationType = $this->internalWPDB->get_var($sql);

            if(!is_null($priceCalculationType))
            {
                $this->priceCalculationType = NRSValidator::getValidInteger($priceCalculationType, 1);
            } else
            {
                $this->priceCalculationType = 1;
            }
        }
    }

    /**
     * Set discount status - called in class constructor
     * @param array $paramOptionalConfig
     */
    private function setDiscountStatus($paramOptionalConfig)
    {
        if(isset($paramOptionalConfig['conf_discount_enabled']))
        {
            // Fast processing mode (for availability and price table), to reduce sql query amount
            if($this->debugMode >= 1)
            {
                echo "<br />Constructor: Discount settings loaded from cache";
            }
            $this->discountEnabled = NRSValidator::getValidInteger($paramOptionalConfig['conf_discount_enabled'], 1) == 1 ? TRUE : FALSE;
        } else
        {
            if($this->debugMode >= 1)
            {
                echo "<br />Constructor: Discount settings loaded via database call";
            }
            $sql = "
				SELECT conf_value AS discount_enabled
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_discount_enabled'
			";
            $discountEnabled = $this->internalWPDB->get_var($sql);

            if(!is_null($discountEnabled) && intval($discountEnabled) == 1)
            {
                $this->discountEnabled = TRUE;
            } else
            {
                $this->discountEnabled = FALSE;
            }
        }
    }

    /**
     * Are discount differs for each item & extra
     * @return bool
     */
    public function isDiscountEnabled()
    {
        return $this->discountEnabled;
    }

    /**
     * We are making sure that we ALWAYS GET A RESULT, EVENT if there is no exact match for that period
     * Returns discount for specific period
     * @param int $paramItemId
     * @param int $paramExtraId
     * @param int $paramPeriod
     * @param string $paramDiscountType - "ITEM_BOOKING_DURATION", "ITEM_BOOKING_IN_ADVANCE", "EXTRA_BOOKING_DURATION", "EXTRA_BOOKING_IN_ADVANCE"
     * @return float
     */
    protected function getDiscountPercentage($paramItemId, $paramExtraId, $paramDiscountType, $paramPeriod)
    {
        $discountPercentage = 0;

        if($this->discountEnabled)
        {
            $validItemId = NRSValidator::getValidInteger($paramItemId, 0);
            $validExtraId = NRSValidator::getValidInteger($paramExtraId, 0);
            $validPeriod = NRSValidator::getValidInteger($paramPeriod, 0);

            // Defaults
            $validDiscountType = 1;
            // SPEED OPTIMIZATION NOTE: it is IMPORTANT to keep the WHERE columns in period_from, period_till order to speed up SQL search,
            // because we have an INDEX in database called 'period (period_from, period_to)'.
            // And we have to invert the sign, but not the order, if we are looking for discounts in advance
            $validBetweenPeriodSQL = "period_from AND period_till";
            if($paramDiscountType == "ITEM_BOOKING_DURATION")
            {
                $validDiscountType = 1;
                $validBetweenPeriodSQL = "period_from AND period_till";
            } else if($paramDiscountType == "ITEM_BOOKING_IN_ADVANCE")
            {
                $validDiscountType = 2;
                $validBetweenPeriodSQL = "period_till AND period_from";
            } else if($paramDiscountType == "EXTRA_BOOKING_DURATION")
            {
                $validDiscountType = 3;
                $validBetweenPeriodSQL = "period_from AND period_till";
            } else if($paramDiscountType == "EXTRA_BOOKING_IN_ADVANCE")
            {
                $validDiscountType = 4;
                $validBetweenPeriodSQL = "period_till AND period_from";
            }

            // ID=0,1,2...
            $sqlForZeroOrBiggerValue = "
				SELECT discount_percentage
				FROM {$this->nrsPrefix}discounts
				WHERE discount_type='{$validDiscountType}' AND item_id='{$validItemId}' AND extra_id='{$validExtraId}' AND (
					  '{$validPeriod}' BETWEEN {$validBetweenPeriodSQL}
					)
				ORDER BY discount_percentage DESC LIMIT 1
			";
            // ID=0
            $sqlForZeroValueOnly = "
				SELECT discount_percentage
				FROM {$this->nrsPrefix}discounts
				WHERE discount_type='{$validDiscountType}' AND item_id='0' AND extra_id='0' AND (
					  '{$validPeriod}' BETWEEN {$validBetweenPeriodSQL}
					)
				ORDER BY discount_percentage DESC LIMIT 1
			";

            if($this->debugMode == 2)
            {
                // Deep debug:
                echo "<br />";
                echo "<br /><strong>FUNCTION:</strong> getDiscountPercentage(ITEM ID={$paramItemId}, EXTRA ID={$paramExtraId}, DISCOUNT TYPE={$paramDiscountType}";
                echo "<br />SQL (ID=0,1,2...): ".nl2br($sqlForZeroOrBiggerValue);
                echo "<br />SQL (ID=0):".nl2br($sqlForZeroValueOnly);
            }

            // Search for exact or nearest alternative discount
            $dataExactOrNearestZeroOrBigger = $this->internalWPDB->get_row($sqlForZeroOrBiggerValue, ARRAY_A);
            if(!is_null($dataExactOrNearestZeroOrBigger))
            {
                // Same nearest discounts for each item/extra
                $discountPercentage = $dataExactOrNearestZeroOrBigger['discount_percentage'];
            } else
            {
                // SPEED OPTIMIZATION NOTE: For better system speed - do not move the get_row line out of THIS ELSE SCOPE!
                // Search for exact or nearest alternative discount for default (0) discount type
                $dataExactOrNearestZeroOnly = $this->internalWPDB->get_row($sqlForZeroValueOnly, ARRAY_A);
                if(!is_null($dataExactOrNearestZeroOnly) && ($validItemId > 0 || $validExtraId > 0))
                {
                    // Same nearest discounts for each item/extra
                    $discountPercentage = $dataExactOrNearestZeroOnly['discount_percentage'];
                }
            }

            // Protect system from max total discounts bigger than 100%
            if($discountPercentage > 100)
            {
                $discountPercentage = 100;
            }
        }

        return $discountPercentage;
    }

    /**
     * @param int $paramItemId
     * @param int $paramExtraId
     * @param string $paramDiscountType - "ITEM_BOOKING_DURATION", "ITEM_BOOKING_IN_ADVANCE", "EXTRA_BOOKING_DURATION", "EXTRA_BOOKING_IN_ADVANCE"
     * @return int
     */
    protected function getMaxDiscountOnDurationPercent($paramItemId, $paramExtraId, $paramDiscountType)
    {
        $maxDiscountPercentage = 0;

        if($this->discountEnabled)
        {
            // Defaults
            $validDiscountType = 1;
            if($paramDiscountType == "ITEM_BOOKING_DURATION")
            {
                $validDiscountType = 1;
            } else if($paramDiscountType == "ITEM_BOOKING_IN_ADVANCE")
            {
                $validDiscountType = 2;
            } else if($paramDiscountType == "EXTRA_BOOKING_DURATION")
            {
                $validDiscountType = 3;
            } else if($paramDiscountType == "EXTRA_BOOKING_IN_ADVANCE")
            {
                $validDiscountType = 4;
            }
            $validItemId = NRSValidator::getValidInteger($paramItemId, 0);
            $validExtraId = NRSValidator::getValidInteger($paramExtraId, 0);

            $sqlForZeroOrBiggerValue = "
                SELECT discount_percentage
                FROM {$this->nrsPrefix}discounts
                WHERE discount_type='{$validDiscountType}' AND item_id='{$validItemId}' AND extra_id='{$validExtraId}'
                ORDER BY discount_percentage DESC LIMIT 1
            ";
            $sqlForZeroValueOnly = "
                SELECT discount_percentage
                FROM {$this->nrsPrefix}discounts
                WHERE discount_type='{$validDiscountType}' AND item_id='0' AND extra_id='0'
                ORDER BY discount_percentage DESC LIMIT 1
            ";

            // Search for exact discount
            $rowZeroOrBigger = $this->internalWPDB->get_row($sqlForZeroOrBiggerValue, ARRAY_A);
            // Search for exact discount for default (0) discount type
            $rowZeroOnly = $this->internalWPDB->get_row($sqlForZeroValueOnly, ARRAY_A);

            if(!is_null($rowZeroOrBigger))
            {
                // Use different discounts for each item/extra (unless 0 was passed, than it will return default here)
                $maxDiscountPercentage = $rowZeroOrBigger['discount_percentage'];
            } else
            {
                if(!is_null($rowZeroOnly) && ($validItemId > 0 || $validExtraId > 0))
                {
                    // Use same discounts for all items/extras
                    $maxDiscountPercentage = $rowZeroOnly['discount_percentage'];
                }
            }
        }

        return $maxDiscountPercentage;
    }
}