<?php
/**
 * NRS Discounts Observer (no setup for single item)
 * Abstract class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSDiscountsObserver
{
	protected $internalWPDB 		    = NULL;
	protected $nrsPrefix 			    = "";
	protected $lang 			        = NULL;
	protected $debugMode 			    = 0;
	protected $discountsEnabled  		= TRUE;
	// Price calculation type: 1 - daily, 2 - hourly, 3 - mixed (daily+hourly)
	protected $priceCalculationType 	= 1;

	/**
	 * @param &$paramWPDB
	 * @param $paramNRSPrefix
	 * @param NRSLanguage $paramNRSLang
	 */
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
	{
		// Set class settings
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;
		// Set price calculation type
		$this->setPriceCalculationType();
		// Set discount status
		$this->setDiscountsStatus();
	}

	/**
	 * Set discount status - called in class constructor
	 */
	private function setPriceCalculationType()
	{
		$sql = "
				SELECT conf_value AS price_calculation_type
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_price_calculation_type'
			";
		$priceCalculationType = $this->internalWPDB->get_var($sql);

		if(!is_null($priceCalculationType))
		{
			$this->priceCalculationType = NRSValidator::getValidInteger($priceCalculationType, 1);
		} else
		{
			$this->priceCalculationType = 1;
		}
	}

	/**
	 * Set discount status - called in class constructor
	 */
	private function setDiscountsStatus()
	{
		$sql = "
				SELECT conf_value AS discount_enabled
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_discount_enabled'
			";
		$discountEnabled = $this->internalWPDB->get_var($sql);

		if(!is_null($discountEnabled) && intval($discountEnabled) == 1)
		{
			$this->discountsEnabled = TRUE;
		} else
		{
			$this->discountsEnabled = FALSE;
		}
	}

	public function areDiscountsEnabled()
	{
		return $this->discountsEnabled;
	}

	/**
	 * USED IN BOTH - FRONT-END and ADMIN
	 * @param $paramElementType - "ITEM" or "EXTRA"
	 * @param $paramDiscountType - "BOOKING_DURATION" or "BOOKING_IN_ADVANCE"
	 * @return array|mixed
	 */
	protected function getDiscountPeriods($paramElementType, $paramDiscountType)
	{
		$rows = array();
		$sqlWhere = "";

		// Find out element and discount type values
		// item_id=0 with extra_id=0 situation is ok, meaning that default discounts would be applied for both
		if($paramElementType == "ITEM" && $paramDiscountType == "BOOKING_DURATION")
		{
			$sqlWhere = "WHERE discount_type='1' AND extra_id='0'";
		} else if($paramElementType == "ITEM" && $paramDiscountType == "BOOKING_IN_ADVANCE")
		{
			$sqlWhere = "WHERE discount_type='2' AND extra_id='0'";
		} else if($paramElementType == "EXTRA" && $paramDiscountType == "BOOKING_DURATION")
		{
			$sqlWhere = "WHERE discount_type='3' AND item_id='0'";
		} else if($paramElementType == "EXTRA" && $paramDiscountType == "BOOKING_IN_ADVANCE")
		{
			$sqlWhere = "WHERE discount_type='4' AND item_id='0'";
		}

		if($this->discountsEnabled && $sqlWhere != "")
		{
			$rows = $this->internalWPDB->get_results("
                SELECT period_from, period_till
                FROM {$this->nrsPrefix}discounts
                {$sqlWhere}
                GROUP BY period_from, period_till
                ORDER BY period_from ASC, period_till ASC
            ", ARRAY_A);

			$newRows = array();
			foreach ($rows AS $discountRow)
			{
				if ($this->priceCalculationType == "1")
				{
					// Days only
					$durationFrom = NRSValidator::getFloorDaysFromSeconds($discountRow['period_from']);
					$durationTill = NRSValidator::getFloorDaysFromSeconds($discountRow['period_till']);
					$durationTimeExtension = $this->lang->getText('NRS_DAYS_TEXT');
				} else if ($this->priceCalculationType == "2")
				{
					// Hours only
					$durationFrom = NRSValidator::getFloorHoursFromSeconds($discountRow['period_from']);
					$durationTill = NRSValidator::getFloorHoursFromSeconds($discountRow['period_till']);
					$durationTimeExtension = $this->lang->getText('NRS_HOURS_TEXT');
				} else
				{
					// Mixed
					$durationFrom = NRSValidator::getFloorDaysFromSeconds($discountRow['period_from']);
					$durationTill = NRSValidator::getFloorDaysFromSeconds($discountRow['period_till']);
					$durationTimeExtension = $this->lang->getText('NRS_DAYS_TEXT');
				}
				$discountRow['dynamic_duration_from'] = $durationFrom;
				$discountRow['dynamic_duration_till'] = $durationTill;
				$discountRow['dynamic_duration_extension'] = $durationTimeExtension;
				$discountRow['print_dynamic_period_label'] = $durationFrom.'-'.$durationTill.' '.$durationTimeExtension;

				$newRows[] = $discountRow;
			}

			$rows = $newRows;
		}

		return $rows;
	}

	/**
	 * In case if discounts are disabled, for price table and maybe somewhere else, we will need a default discount
	 * @return array
	 */
	public function getDefaultDiscountPeriod()
	{
		$defaultDiscount = array(
			"period_from" => 0,
			"period_till" => $this->priceCalculationType == 2 ? 3600 : 86400, /* Hour vs Day */
			"dynamic_duration_from" => 0,
			"dynamic_duration_till" => 1,
			"dynamic_duration_extension" => $this->priceCalculationType == 2 ? $this->lang->getText('NRS_HOURS_TEXT') : $this->lang->getText('NRS_DAYS_TEXT'),
			"print_dynamic_period_label" => $this->priceCalculationType == 2 ? '1 '.$this->lang->getText('NRS_HOUR_TEXT') : '1 '.$this->lang->getText('NRS_DAY_TEXT'),
		);

		return $defaultDiscount;
	}
}