<?php
/**
 * Element processor
 * Abstract classes can't be created with new instance. It is only possible if they are extended by childs
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSElement
{
    protected $internalWPDB 		= NULL;
    protected $nrsPrefix 			= "";
    protected $lang 				= NULL;
    protected $debugMode 			= 0;
    /**
     * @var int - either itemId or either extraId;
     */
    protected $elementId			= 0;
    protected $measurementUnit      = "";

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - Language
     * @param $paramElementId
     * @param array $paramOptionalConfig
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramElementId, $paramOptionalConfig = array())
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        $this->elementId = NRSValidator::getValidValue($paramElementId, "intval", 0);
        // Set measurement unit
        $this->setMeasurementUnit($paramOptionalConfig);
    }

    /**
     * Set measurement unit - called in class constructor
     * @param array $paramOptionalConfig
     */
    private function setMeasurementUnit($paramOptionalConfig)
    {
        if(isset($paramOptionalConfig['conf_measurement_unit']))
        {
            // Fast processing mode (for availability and price table), to reduce sql query amount
            $this->measurementUnit = sanitize_text_field($paramOptionalConfig['conf_measurement_unit']);
        } else
        {
            $sql = "
				SELECT conf_value AS measurement_unit
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_measurement_unit'
			";
            $measurementUnit = $this->internalWPDB->get_var($sql);

            if(!is_null($measurementUnit))
            {
                $this->measurementUnit = sanitize_text_field($measurementUnit);
            } else
            {
                $this->measurementUnit = "";
            }
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getId()
    {
        return $this->elementId;
    }

    /***************************************************************************************************/
    /***************************************************************************************************/

    /**
     * Get drop-down for car units in garage
     * @param int $selectedUnits - for edit mode
     * @return string passengers drop-down html
     */
    public function getUnitsInStockDropDown($selectedUnits = 1)
    {
        return $this->getAdminNumberDropDown("units_in_stock", 1, 100, $selectedUnits, TRUE, "0");
    }

    /**
     * Get drop-down for car units in garage
     * @param int $selectedUnits - for edit mode
     * @return string passengers drop-down html
     */
    public function getMinUnitsPerBookingDropDown($selectedUnits = 0)
    {
        return $this->getAdminNumberDropDown("min_units_per_booking", 0, 100, $selectedUnits, TRUE, "0");
    }

    /**
     * Get drop-down for car units in garage
     * @param int $selectedUnits - for edit mode
     * @return string passengers drop-down html
     */
    public function getMaxUnitsPerBookingDropDown($selectedUnits = 1)
    {
        return $this->getAdminNumberDropDown("max_units_per_booking", 1, 100, $selectedUnits, TRUE, "0");
    }

    public function getOptionDisplayTypeDropDown($paramName="options_display_mode", $selected = "0", $paramRequired = TRUE)
    {
        $required = $paramRequired == FALSE ? "" : "required";

        $extraPriceTypeHTML = '<select name="'.$paramName.'" id="'.$paramName.'" class="'.$required.'">';
        $extraPriceTypeHTML .= '<option value="1"'.($selected == 1 ? ' selected="selected"' : '').'>'.$this->lang->getText('NRS_ADMIN_DROPDOWN_TEXT').'</option>';
        $extraPriceTypeHTML .= '<option value="2"'.($selected == 2 ? ' selected="selected"' : '').'>'.$this->lang->getText('NRS_ADMIN_SLIDER_TEXT').'</option>';
        $extraPriceTypeHTML .= '</select>';

        return $extraPriceTypeHTML;
    }

    /**
     * Get drop-down for any admin option
     * @param $paramName
     * @param int $paramValueFrom
     * @param int $paramValueTill
     * @param int $paramSelectedValue
     * @param bool $paramRequired
     * @param string $paramZeroText
     * @return string
     */
    protected function getAdminNumberDropDown($paramName, $paramValueFrom = 0, $paramValueTill = 100, $paramSelectedValue = 0, $paramRequired = FALSE, $paramZeroText = "")
    {
        $required = $paramRequired == FALSE ? "" : "required";

        $driverAgeHTML = '<select name="'.$paramName.'" id="'.$paramName.'" class="'.$required.'">';
        for ($i = $paramValueFrom; $i <= $paramValueTill; $i++)
        {
            $selected = $i == $paramSelectedValue ? ' selected="selected"' : '';
            $value = $i;
            $text = $i == 0 ? $paramZeroText : $i;
            $driverAgeHTML .= '<option value="'.$value.'"'.$selected.'>'.$text.'</option>';
        }
        $driverAgeHTML .= '</select>';

        return $driverAgeHTML;
    }
}