<?php
/**
 * Price Manager
 * Abstract classes can't be created with new instance. It is only possible if they are extended by childs
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSPriceManager
{
	protected $internalWPDB 		= NULL;
	protected $nrsPrefix 			= "";
	protected $lang 				= NULL;
	protected $debugMode 			= 0; // 0 - off, 1 - regular, 2 - deep items, 3 - deep extras
    protected $savedConfig          = array();
	/**
	 * @var int - either itemId or either extraId;
	 */
	protected $elementId			= 0;
	/**
	 * @var int - price calculation: 1 - daily, 2 - hourly, 3 - mixed (daily+hourly)
	 */
	protected $priceCalculationType = 1;
	protected $currencySymbol		= '$';
	protected $currencyCode			= 'USD';
	protected $vatPercentage		= 0.00;
	protected $showPricesWithVAT	= 0;

	/**
	 * @param $paramWPDB
	 * @param $paramNRSPrefix
	 * @param NRSLanguage &$paramNRSLang - Language
	 * @param array $paramConfig
	 * @param $paramElementId
	 */
	public function __construct(
        wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig,
        $paramElementId
    ) {
		// Set class settings
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;
        $this->savedConfig = $paramConfig;

		$this->elementId = NRSValidator::getValidValue($paramElementId, "intval", 0);
		$this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
		$this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
		$this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
		$this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));

		// Set price calculation type
		$this->setPriceCalculationType($paramConfig);
	}

	public function inDebug()
	{
		return ($this->debugMode >= 1 ? TRUE : FALSE);
	}

	/**
	 * Set price calculation type - called in class constructor
     * @param array $paramOptionalConfig
     */
	private function setPriceCalculationType($paramOptionalConfig)
	{
        if(isset($paramOptionalConfig['conf_price_calculation_type']))
        {
            // Fast processing mode (for availability and price table), to reduce sql query amount
            if($this->debugMode >= 1)
			{
				echo "<br />Constructor: Price settings loaded from cache";
			}
            $this->priceCalculationType = NRSValidator::getValidInteger($paramOptionalConfig['conf_price_calculation_type'], 1);
        } else
        {
            if($this->debugMode >= 1)
            {
                echo "<br />Constructor: Price settings loaded via database call";
            }
            $sql = "
                    SELECT conf_value AS price_calculation_type
                    FROM {$this->nrsPrefix}settings
                    WHERE conf_key='conf_price_calculation_type'
                ";
            $priceCalculationType = $this->internalWPDB->get_var($sql);

            if(!is_null($priceCalculationType))
            {
                $this->priceCalculationType = NRSValidator::getValidInteger($priceCalculationType, 1);
            } else
            {
                $this->priceCalculationType = 1;
            }
        }
	}

	protected function getFormattedPriceArray($paramArray, $paramFormatType)
	{
		if($paramFormatType == "word")
		{
			$retArray = array();
			foreach($retArray AS $key => $price)
			{
				$floatPrice = round($price, 2);
				if($key == "fixed_deposit_amount")
				{
					$word = $floatPrice == 0.00 ? $this->lang->getText('NRS_NOT_REQUIRED_TEXT') : $this->lang->getText('NRS_REQUIRED_TEXT');
				} else
				{
					$word = $floatPrice == 0.00 ? $this->lang->getText('NRS_EXIST_TEXT') : $this->lang->getText('NRS_NONE_TEXT');
				}
				$retArray[$key] = $word;
			}
		} else
		{
			$retArray = NRSFormatter::getFormattedPriceArray($paramArray, $paramFormatType, $this->currencySymbol, $this->currencyCode);
		}
		return $retArray;
	}

	protected function isPriceDisplayedByDays()
	{
		$ret = false;
		if ($this->priceCalculationType == 1 || $this->priceCalculationType == 3)
		{
			$ret = true;
		}

		return $ret;
	}
}
