<?php
/**
 * Abstract classes can't be created with new instance. It is only possible if they are extended by childs
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
*/

abstract class NRSSearch
{
	protected $internalWPDB	 	= NULL;
	protected $nrsPrefix 		= "";
	protected $lang 			= NULL;
	protected $debugMode 	 	= 0;
	protected $validSearch   	= TRUE;
	protected $errorMessages    = array();

	// settings from DB
	protected $searchEnabled 		= 1;
	/**
	 * @var int - admin setting in seconds - minimum block period between two bookings for same car
	 */
	protected $minBookingPeriod 	= 0;
	/**
	* @var int - admin setting in seconds - maximum reservation period
	*/
	protected $maxBookingPeriod 	= 0;
	/**
	 * @var int - price calculation: 1 - daily, 2 - hourly, 3 - mixed (daily+hourly)
	 */
	protected $blockPeriod 			= 0;
	/**
	 * @var int - admin setting in seconds - minimum period until pickup
	 */
	protected $minPeriodUntilPickup	= 0;
	/**
	 * @var int - admin setting in seconds - minimum reservation period
	 */
	protected $priceCalculationType = 1;
	/**
	 * @var int - multi mode: FALSE - single car model (any quantity) per booking, TRUE - multiple car models (any quantity) per booking
	 */
	protected $multiMode 			= FALSE;

	// Settings, passed to mail class
	protected $portalName 			= "";
	protected $portalEmail 			= "";
	protected $portalPhone 			= "";
	protected $sendEmails			= 0;

	// ReCaptcha settings
	protected $reCaptchaEnabled		= 0;
	protected $reCaptchaSiteKey		= "";
	protected $reCaptchaSecretKey	= "";

	// pre-filled in step 1
	protected $pickupTimestamp  	= 0;
	protected $dropoffTimestamp 	= 0;
	protected $pickupLocationId   	= 0;
	protected $dropoffLocationId  	= 0;
	protected $bodyTypeId         	= 0; // USE ONLY IN STEP1
	protected $transmissionTypeId 	= 0; // USE ONLY IN STEP1
	protected $fuelTypeId     		= 0; // USE ONLY IN STEP1

	// pre-filled in step 2 (or step1 if booked from car page)
	protected $itemIds            	= array();
	protected $itemUnits        	= array();
	protected $itemOptions        	= array();

	// pre-filled in step 3
	protected $extraIds		   		= array();
	protected $extraUnits      		= array();
	protected $extraOptions    		= array();

	// pre-filled in step 4
	protected $customerData			= NULL; // expected - array(), but might be null
	protected $customerEmail		= "";
	protected $paymentMethodCode	= "";

	// pre-filled in step 5
	public $bookingCode				= "";

	// Extra settings used for location manager, not by the class itself
	protected $shortDateFormat		= 'Y-m-d';
	protected $currencySymbol		= '$';
	protected $currencyCode			= 'USD';
	protected $vatPercentage		= 0.00;
	protected $showPricesWithVAT	= 0;

	/**
	 * @param $paramWPDB
	 * @param $paramNRSPrefix
	 * @param NRSLanguage &$paramNRSLang - LANGUAGE
	 * @param array $paramConfig
	 * @param string $clearExpiredBookings
	 */
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig, $clearExpiredBookings = "")
	{
		// Set class settings
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;

		$this->searchEnabled = NRSValidator::getValidSetting($paramConfig['conf_search_enabled'], "intval", 1, array(0, 1));
		$this->minBookingPeriod = NRSValidator::getValidSetting($paramConfig['conf_minimum_booking_period'], "intval", 0);
		$this->maxBookingPeriod = NRSValidator::getValidSetting($paramConfig['conf_maximum_booking_period'], "intval", 0);
		$this->blockPeriod = NRSValidator::getValidSetting($paramConfig['conf_minimum_block_period_between_bookings'], "intval", 0);
		$this->minPeriodUntilPickup = NRSValidator::getValidSetting($paramConfig['conf_minimum_period_until_pickup'], "intval", 0);
		$this->priceCalculationType = NRSValidator::getValidSetting($paramConfig['conf_price_calculation_type'], "intval", 1, array(1, 2, 3));
		$this->multiMode = isset($paramConfig['conf_booking_model']) && $paramConfig['conf_booking_model'] == 2 ? TRUE : FALSE;
		$this->portalName = NRSValidator::getValidSetting($paramConfig['conf_portal_name'], "textval", "");
		$this->portalEmail = NRSValidator::getValidSetting($paramConfig['conf_portal_email'], "textval", "");
		$this->portalPhone = NRSValidator::getValidSetting($paramConfig['conf_portal_phone'], "textval", "");
		$this->sendEmails = NRSValidator::getValidSetting($paramConfig['conf_send_emails'], "intval", 0);

		// ReCaptcha settings
		$this->reCaptchaEnabled = NRSValidator::getValidSetting($paramConfig['conf_recaptcha_enabled'], "intval", 0);
		$this->reCaptchaSiteKey = NRSValidator::getValidSetting($paramConfig['conf_recaptcha_site_key'], "textval", "");
		$this->reCaptchaSecretKey = NRSValidator::getValidSetting($paramConfig['conf_recaptcha_secret_key'], "textval", "");

		// Extra settings used for location manager, not by the class itself
		$this->shortDateFormat = NRSValidator::getValidSetting($paramConfig['conf_short_date_format'], "date_format", "Y-m-d");
		$this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
		$this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
		$this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
		$this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));

		// Clear expired bookings if required
		if($clearExpiredBookings == "CLEAR_EXPIRED_BOOKINGS")
		{
			// clear expired booking before doing anything else
			$this->clearExpiredBookings();
		}

		// DO NOTHING ELSE - the CHILD CLASSES WILL DO THE WORK
	}

	/**
	 * Most modern parameter setup
	 * @param array $varTypes - array where to search for data SESSION, POST, SERVER, ...
	 * @param string $varParam
	 * @param $defaultValue
	 * @param bool $required
	 * @param string $validate
	 * @return bool
	 */
	protected function getValidValueInput($varTypes = array(), $varParam = "", $defaultValue, $required = false, $validate = "guest_text_validation")
	{
		$tmpValue = FALSE;
		$varTypeUsed = "";
		foreach($varTypes AS $varType)
		{
			$stack = array();
			switch($varType)
			{
				case "POST":
					$stack = $_POST;
					break;
				case "GET":
					$stack = $_GET;
					break;
				case "SESSION":
					$stack = $_SESSION;
					break;
				case "REQUEST":
					$stack = $_REQUEST;
					break;
				case "SERVER":
					$stack = $_SERVER;
					break;
			}

			if(isset($stack[$varParam]))
			{
				$varTypeUsed = $varType;
				$tmpValue = $stack[$varParam];
				// break when first success was found - that's why order - POST, GET, SESSION - is important
				break;
			}
		}

		// Make a error note if we still have FALSE after all process and this field is required
		if($required == TRUE && $tmpValue === FALSE)
		{
			// Invalidate the request. Like in try/catch/finally, but without it
			$this->addInputError($varParam);
		}

		if(is_array($tmpValue))
		{
			// NOT ALLOWED! This will happen when we pass an array, when we strict to not arrays only
			// It's ok to pass default value here twice in this situation
			$ret = NRSValidator::getValidValue($defaultValue, $validate, $defaultValue);
		} else
		{
			// OK
			$ret = NRSValidator::getValidValue($tmpValue, $validate, $defaultValue);
		}

		// Make a error note if we do not yet been got it from getRequestedMemberValue(), this means that it didn't returned false
		if($required == TRUE && ($ret === FALSE || $ret == "" || $ret == "0" || $ret == "0000-00-00"))
		{
			// Invalidate the request. Like in try/catch/finally, but without it
			$this->addInputError($varParam);
		}

		if($this->debugMode == 1)
		{
			echo "<br /><strong>[Security]</strong> ";
			//echo "Types: [".implode(", ", $varTypes)."], ";
			echo "Used: {$varTypeUsed}, ";
			echo "Param: $varParam, ";
			echo "Validation: {$validate}, Required: "; var_dump($required); echo ", Value: "; var_dump($ret);
		}

		return $ret;
	}

	/**
	 * Most modern parameter setup
	 * @param array $varTypes - array where to search for data SESSION, POST, SERVER, ...
	 * @param string $varParam
	 * @param $defaultArray
	 * @param bool $required
	 * @param string $validate
	 * @return bool
	 */
	protected function getValidArrayInput($varTypes = array(), $varParam = '', $defaultArray, $required = false, $validate = "guest_text_validation")
	{
		$varTypeUsed = "";
		$tmpArray = array();
		foreach($varTypes AS $varType)
		{
			$stack = array();
			switch($varType)
			{
				case "POST":
					$stack = $_POST;
					break;
				case "GET":
					$stack = $_GET;
					break;
				case "SESSION":
					$stack = $_SESSION;
					break;
				case "REQUEST":
					$stack = $_REQUEST;
					break;
				case "SERVER":
					$stack = $_SERVER;
					break;
			}

			if(isset($stack[$varParam]))
			{
				$varTypeUsed = $varType;
				$tmpArray = $stack[$varParam];
				// break when first success was found - that's why order - POST, GET, SESSION - is important
				break;
			}
		}

		// Make a error note if we still have FALSE after all process and this field is required
		if($required == TRUE && sizeof($tmpArray) == 0)
		{
			// Invalidate the request. Like in try/catch/finally, but without it
			$this->addInputError($varParam);
		}

		if(is_array($tmpArray))
		{
			// OK
			$ret = NRSValidator::getValidArray($tmpArray, $validate, $defaultArray);
		} else
		{
			// NOT ALLOWED! This will happen when we pass not array, when we strict to arrays only
			// It's ok to pass default value here twice in this situation
			$ret = NRSValidator::getValidArray($defaultArray, $validate, $defaultArray);
		}

		// Make a error note if we do not yet been got it from getRequestedMemberValue(), this means that it didn't returned false
		if($required == TRUE && sizeof($ret) == 0)
		{
			// Invalidate the request. Like in try/catch/finally, but without it
			$this->addInputError($varParam);
		}

		if($this->debugMode == 1)
		{
			echo "<br /><strong>[Security]</strong> ";
			//echo "Types: [".implode(", ", $varTypes)."], ";
			echo "Used: {$varTypeUsed}, ";
			echo "Param: $varParam, ";
			echo "Validation: {$validate}, Required: "; var_dump($required); echo ", Array: "; var_dump($ret);
		}

		return $ret;
	}

	protected function addInputError($brokenParam)
	{
		$this->validSearch = FALSE;
		$brokenInputFieldTitle = ucfirst(str_replace("_", " ", $brokenParam));
		$this->errorMessages[] = $this->lang->getText('NRS_ERROR_REQUIRED_FIELD_TEXT')." ({$brokenInputFieldTitle}) ".$this->lang->getText('NRS_ERROR_IS_EMPTY_TEXT');
	}

	protected function setMySessionVars()
	{
		// filled data in step 1, pre-filled in step 2, 3, 4, 5, 6
		$_SESSION['pickup_location_id']	= $this->pickupLocationId;
		$_SESSION['pickup_date']   	  	= $this->getPrintShortPickupDate();
		$_SESSION['pickup_time']   		= $this->getPrintShortPickupTime();
		$_SESSION['dropoff_location_id']= $this->dropoffLocationId;
		$_SESSION['dropoff_date']  	  	= $this->getPrintShortDropoffDate();
		$_SESSION['dropoff_time']  		= $this->getPrintShortDropoffTime();

		// Used only in step 1
		$_SESSION['body_type_id']     	= $this->bodyTypeId;
		$_SESSION['transmission_type_id']= $this->transmissionTypeId;
		$_SESSION['fuel_type_id']  		= $this->fuelTypeId;

		// filled in step 2 (or step1 if booked from car page), pre-filled in step 3, 4, 5, 6
		$_SESSION['item_ids']   		= $this->itemIds;
		$_SESSION['item_units']   	  	= $this->itemUnits;
		$_SESSION['item_options']   	= $this->itemOptions;

		// filled in step 3, pre-filled in step 4, 5, 6
		$_SESSION['extra_ids']   	  	= $this->extraIds;
		$_SESSION['extra_units']   		= $this->extraUnits;
		$_SESSION['extra_options']   	= $this->extraOptions;

		// set in step 5 (booking process) or came from booking edit
		$_SESSION['booking_code']   	  	= $this->bookingCode;
	}


	protected function removeSessionVariables()
	{
		//  USE ONLY IN STEP1. After saving used in edit only
		if(isset($_SESSION['body_type_id']))
		{
			unset($_SESSION['body_type_id']);
		}

		//  USE ONLY IN STEP1. After saving used in edit only
		if(isset($_SESSION['transmission_type_id']))
		{
			unset($_SESSION['transmission_type_id']);
		}

		//  USE ONLY IN STEP1. After saving used in edit only
		if(isset($_SESSION['fuel_type_id']))
		{
			unset($_SESSION['fuel_type_id']);
		}

		// filled data in step1
		if(isset($_SESSION['pickup_location_id']))
		{
			unset($_SESSION['pickup_location_id']);
		}
		if(isset($_SESSION['pickup_date']))
		{
			unset($_SESSION['pickup_date']);
		}
		if(isset($_SESSION['pickup_time']))
		{
			unset($_SESSION['pickup_time']);
		}

		if(isset($_SESSION['dropoff_location_id']))
		{
			unset($_SESSION['dropoff_location_id']);
		}
		if(isset($_SESSION['dropoff_date']))
		{
			unset($_SESSION['dropoff_date']);
		}
		if(isset($_SESSION['dropoff_time']))
		{
			unset($_SESSION['dropoff_time']);
		}


		// filled in step 2 (or step1 if booked from car page)
		if(isset($_SESSION['item_ids']))
		{
			unset($_SESSION['item_ids']);
		}

		// filled in step 3
		if(isset($_SESSION['item_units']))
		{
			unset($_SESSION['item_units']);
		}

		if(isset($_SESSION['item_options']))
		{
			unset($_SESSION['item_options']);
		}

		if(isset($_SESSION['extra_ids']))
		{
			unset($_SESSION['extra_ids']);
		}

		if(isset($_SESSION['extra_units']))
		{
			unset($_SESSION['extra_units']);
		}

		if(isset($_SESSION['extra_options']))
		{
			unset($_SESSION['extra_options']);
		}

		// set in step 5 (booking process) or came from booking edit
		if(isset($_SESSION['booking_code']))
		{
			unset($_SESSION['booking_code']);
		}
	}

	/**************************************************************/
	/************************* SETTINGS ***************************/
	/**************************************************************/

	public function inDebug()
	{
		return ($this->debugMode >= 1 ? TRUE : FALSE);
	}

	public function isValidSearch()
	{
		return $this->validSearch;
	}

	public function getPrintErrors()
	{
		return implode("<br /><br />", $this->errorMessages);
	}

	/**************************************************************/
	/************************** GET PRINTS ************************/
	/**************************************************************/

	public function getPrintPickupDate()
	{
		// WordPress bug
		// BAD: return date_i18n(get_option('date_format'), $this->pickupTimestamp);
		// OK: return date(get_option('date_format'), $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600);

		// WordPress bug WorkAround
		return date_i18n(get_option('date_format'), $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintPickupTime()
	{
		// WordPress bug
		// BAD: return date_i18n(get_option('time_format'), $this->pickupTimestamp);
		// OK: return date(get_option('time_format'), $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600);
		return date_i18n(get_option('time_format'), $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintDropoffDate()
	{
		// WordPress bug
		// BAD: return date_i18n(get_option('date_format'), $this->dropoffTimestamp);
		// OK: return date(get_option('date_format'), $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600);
		return date_i18n(get_option('date_format'), $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintDropoffTime()
	{
		// WordPress bug
		// BAD: return date_i18n(get_option('time_format'), $this->dropoffTimestamp);
		// OK: return date(get_option('time_format'), $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600);
		return date_i18n(get_option('time_format'), $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintShortPickupDate()
	{
		// WordPress workaround
		return date_i18n($this->shortDateFormat, $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintShortPickupTime()
	{
		// WordPress workaround
		return date_i18n("H:i:s", $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintShortDropoffDate()
	{
		// WordPress workaround
		return date_i18n($this->shortDateFormat, $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintShortDropoffTime()
	{
		// WordPress workaround
		return date_i18n("H:i:s", $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);
	}

	public function getPrintBookingDuration()
	{
		return $this->lang->getPrintCeilDurationByPeriod($this->priceCalculationType, NRSValidator::getPeriod($this->pickupTimestamp, $this->dropoffTimestamp, FALSE));
	}

	/**************************************************************/
	/************** Just an methods abbreviation ******************/
	/**************************************************************/
	public function getLocalPickupDate()
	{
		return NRSValidator::getLocalDateByTimestamp($this->pickupTimestamp, "Y-m-d");
	}

	public function getLocalPickupTime()
	{
		return NRSValidator::getLocalDateByTimestamp($this->pickupTimestamp, "H:i:s");
	}

	public function getLocalDropoffDate()
	{
		return NRSValidator::getLocalDateByTimestamp($this->dropoffTimestamp, "Y-m-d");
	}

	public function getLocalDropoffTime()
	{
		return NRSValidator::getLocalDateByTimestamp($this->dropoffTimestamp, "H:i:s");
	}

	public function getLocalPickupDayOfWeek()
	{
		return NRSValidator::getLocalDateByTimestamp($this->pickupTimestamp, "D");
	}

	public function getLocalDropoffDayOfWeek()
	{
		return NRSValidator::getLocalDateByTimestamp($this->dropoffTimestamp, "D");
	}

	public function getBookingPeriod()
	{
		return NRSValidator::getPeriod($this->pickupTimestamp, $this->dropoffTimestamp, FALSE);
	}

	/**********************************************************************************************/
	/************************************* ACTUAL ELEMENTS ****************************************/
	/**********************************************************************************************/

	public function getPickupTimestamp()
	{
		return $this->pickupTimestamp;
	}

	public function getDropoffTimestamp()
	{
		return $this->dropoffTimestamp;
	}

	public function getReCaptchaSiteKey()
	{
		return $this->reCaptchaSiteKey;
	}

	public function getReCaptchaLanguage()
	{
		$retLang = "en";
		$lang = $this->lang->getText('NRS_RECAPTCHA_LANG');
		if($lang != "")
		{
			$retLang = $lang;
		}

		return $retLang;
	}

	/**************************************************************/
	/********* Methods to retrieve booking location, item, ********/
	/********* customer and booking details ***********************/
	/**************************************************************/

	/**
	 * Can we select more than more
	 * @return bool
	 */
	public function areMultiMode()
	{
		return $this->multiMode;
	}

	public function useReCaptcha()
	{
		if($this->reCaptchaEnabled == 1 && $this->reCaptchaSiteKey != "" && $this->reCaptchaSecretKey != "")
		{
			return TRUE;
		}

		return FALSE;
	}

	public function isNewBooking()
	{
		return $this->bookingCode == "" ? TRUE : FALSE;
	}

	public function getBookingCode()
	{
		return $this->bookingCode;
	}

	public function getPickupLocationId()
	{
		return $this->pickupLocationId;
	}

	public function getDropoffLocationId()
	{
		return $this->dropoffLocationId;
	}

	public function getBodyTypeId()
	{
		return $this->bodyTypeId;
	}

	public function getTransmissionTypeId()
	{
		return $this->transmissionTypeId;
	}

	public function getFuelTypeId()
	{
		return $this->fuelTypeId;
	}

	public function getPaymentMethodCode()
	{
		return $this->paymentMethodCode;
	}

	public function getSearchInputDataArray()
	{
		$bookingInput = array(
			"pickup_timestamp" => $this->pickupTimestamp,
			"dropoff_timestamp" => $this->dropoffTimestamp,
			"pickup_location_id" => $this->pickupLocationId,
			"dropoff_location_id" => $this->dropoffLocationId,
			"body_type_id" => $this->bodyTypeId,
			"transmission_type_id" => $this->transmissionTypeId,
			"fuel_type_id" => $this->fuelTypeId,
			"item_ids" => $this->itemIds,
			"item_units" => $this->itemUnits,
			"item_options" => $this->itemOptions,
			"extra_ids" => $this->extraIds,
			"extra_units" => $this->extraUnits,
			"extra_options" => $this->extraOptions,
			"payment_method_code" => $this->paymentMethodCode,
		);

		return $bookingInput;
	}
	public function getCustomerEmail()
	{
		return isset($this->customerData['email']) ? $this->customerData['email'] : "";
	}

	public function getCustomerInputDataArray()
	{
		return $this->customerData;
	}

	/**************************************************************/
	/******************** Advanced methods 1 **********************/
	/**************************************************************/

	/**
	 * @param $limitType
	 * @return array
	 */
	public function getSelectedItemIdsForType($limitType)
	{
		return $this->getSearchItemIds(TRUE, $limitType);
	}

	public function getAvailableItemIdsForType($limitType)
	{
		return $this->getSearchItemIds(FALSE, $limitType);
	}

	public function getSelectedItemIds()
	{
		return $this->getSearchItemIds(TRUE, -1);
	}

	public function getAvailableItemIds()
	{
		return $this->getSearchItemIds(FALSE, -1);
	}

	private function getSearchItemIds($paramSelectedItemsOnly = TRUE, $paramLimitType = -1)
	{
		$addQuery = '';
		$validLimitType = NRSValidator::getValidInteger($paramLimitType, -1);
		$validPickupLocationId = NRSValidator::getValidInteger($this->pickupLocationId, 0);
		$validDropoffLocationId = NRSValidator::getValidInteger($this->dropoffLocationId, 0);
		$validBodyTypeId = NRSValidator::getValidInteger($this->bodyTypeId, 0);
		$validTransmissionTypeId = NRSValidator::getValidInteger($this->transmissionTypeId, 0);
		$validFuelTypeId = NRSValidator::getValidInteger($this->fuelTypeId, 0);

		if($paramSelectedItemsOnly == TRUE && is_array($this->itemIds) && sizeof($this->itemIds) > 0)
		{
			$subQueries = array();
			foreach($this->itemIds AS $itemId)
			{
				$validItemId = NRSValidator::getValidInteger($itemId, 0);
				if($validItemId > 0)
				{
					$subQueries[] = " it.item_id='{$validItemId}' ";
				}
			}
			$addQuery .= " AND (".implode(" OR ", $subQueries).") ";
		}

		if($validBodyTypeId > 0)
		{
			$addQuery .= " AND it.body_type_id='{$validBodyTypeId}' ";
		}

		// This will allow us to show items for classified listing
		if($validLimitType >= 0)
		{
			$addQuery .= " AND it.body_type_id='{$validLimitType}' ";
		}

		if($validTransmissionTypeId > 0)
		{
			$addQuery .= " AND it.transmission_type_id='{$validTransmissionTypeId}' ";
		}

		if($validFuelTypeId > 0)
		{
			$addQuery .= " AND it.fuel_type_id='{$validFuelTypeId}' ";
		}

		if($validPickupLocationId > 0)
		{
			$addQuery .= "
				AND it.item_id IN
				(
					SELECT item_id
					FROM {$this->nrsPrefix}item_locations
					WHERE location_id='{$validPickupLocationId}' AND location_type='1'
				)";
		}

		if($validDropoffLocationId > 0)
		{
			$addQuery .= "AND it.item_id IN
			(
				SELECT item_id
				FROM {$this->nrsPrefix}item_locations
				WHERE location_id='{$validDropoffLocationId}' AND location_type='2'
			)";
		}

		$searchSQL = "
			SELECT it.item_id
			FROM {$this->nrsPrefix}items it
			LEFT JOIN {$this->nrsPrefix}manufacturers mf ON it.manufacturer_id=mf.manufacturer_id
			WHERE it.units_in_stock > 0 AND it.enabled = '1'
			{$addQuery}
			ORDER BY manufacturer_title ASC, model_name ASC
			";

		//echo "<br />".$searchSQL."<br />"; //die;

		if(
			($paramSelectedItemsOnly == TRUE && is_array($this->itemIds) && sizeof($this->itemIds) > 0)
			|| $paramSelectedItemsOnly == FALSE
		) {
			$sqlRows = $this->internalWPDB->get_col($searchSQL);
		} else
		{
			// No results found, because no cars selected
			$sqlRows = array();
		}


		if($this->debugMode == 1)
		{
			echo "<br />TOTAL CANDIDATE ITEMS FOUND: " . sizeof($sqlRows);
			echo "<br /><em>(Note: the candidate number is not final, it does not include blocked items ";
            echo "(for all or specific locations) or the situation when all item units are booked)</em>";
		}

		return $sqlRows;
	}

	public function getSelectedItemUnits()
	{
		return $this->itemUnits;
	}

	public function getSelectedItemOptions()
	{
		return $this->itemOptions;
	}

	public function getSelectedExtraIds()
	{
		return $this->getSearchExtraIds(TRUE);
	}

	public function getAvailableExtraIds()
	{
		return $this->getSearchExtraIds(FALSE);
	}

	/**
	 * @param bool $paramSelectedExtrasOnly - get all available or Selected extras only
	 * @return array
	 */
	private function getSearchExtraIds($paramSelectedExtrasOnly = FALSE)
	{
		$addQuery = "";
		if($paramSelectedExtrasOnly == TRUE && is_array($this->itemIds) && sizeof($this->itemIds) > 0)
		{
			$subQueries = array();
			foreach($this->extraIds AS $extraId)
			{
				$validExtraId = NRSValidator::getValidInteger($extraId, 0);
				if($validExtraId > 0)
				{
					$subQueries[] = " extra_id='{$validExtraId}' ";
				}
			}
			$addQuery .= " AND (".implode(" OR ", $subQueries).") ";
		}

		$searchSQL = "
            SELECT extra_id
            FROM {$this->nrsPrefix}extras
            WHERE units_in_stock > 0 AND max_units_per_booking > 0 AND max_units_per_booking >= min_units_per_booking
            {$addQuery}
			ORDER BY extra_name ASC
		";
		//echo "<br />".$searchSQL."<br />"; //die;

		$sqlRows = $this->internalWPDB->get_col($searchSQL);

		if($this->debugMode == 1)
		{
			echo "<br />TOTAL CANDIDATE EXTRAS FOUND: " . sizeof($sqlRows);
            echo "<br /><em>(Note: the candidate number is not final, it does not include blocked extras ";
            echo "or the situation when all extra units are booked)</em>";
		}

		return $sqlRows;
	}

	public function getSelectedExtraUnits()
	{
		return $this->extraUnits;
	}

	public function getSelectedExtraOptions()
	{
		return $this->extraOptions;
	}

	public function isPaymentRequired()
	{
		$sql = "
				SELECT conf_value AS prepayment_enabled
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_prepayment_enabled'
			";
		$tmpPrepaymentStatus = $this->internalWPDB->get_var($sql);

		if(!is_null($tmpPrepaymentStatus) && intval($tmpPrepaymentStatus) == 1)
		{
			$prepaymentEnabled = TRUE;
		} else
		{
			$prepaymentEnabled = FALSE;
		}

		$enabledMethods = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}payment_methods
            WHERE method_enabled='1'
        ", ARRAY_A);

		$required = ($prepaymentEnabled && sizeof($enabledMethods) > 0) ? TRUE : FALSE;

		return $required;
	}

	/**************************************************************/
	/******************** Advanced methods 2 **********************/
	/**************************************************************/

	/**
	 * @param $paramArray
	 * @param $paramFormatType
	 * @return array
	 */
	protected function getFormattedPriceArray($paramArray, $paramFormatType)
	{
		return NRSFormatter::getFormattedPriceArray($paramArray, $paramFormatType, $this->currencySymbol, $this->currencyCode);
	}

	protected function validateSearchEngine()
	{
		if ($this->searchEnabled == 0)
		{
			$this->validSearch = FALSE;
			$this->errorMessages[] = $this->lang->getText('NRS_ERROR_SEARCH_ENGINE_DISABLED_TEXT');
		}

		return $this->validSearch;
	}

	/**
	 * @param $pickupTimestamp
	 * @param $dropoffTimestamp
	 * @return bool
	 */
	protected function validateTimeInput($pickupTimestamp, $dropoffTimestamp)
	{
		if (NRSValidator::getPeriod($pickupTimestamp, $dropoffTimestamp, TRUE) < 0)
		{
			$this->validSearch = FALSE;
			$this->errorMessages[] = $this->lang->getText('NRS_ERROR_OUT_BEFORE_IN_TEXT');
		} else if (NRSValidator::getPeriod($pickupTimestamp, $dropoffTimestamp, FALSE) < $this->minBookingPeriod)
		{
			$this->validSearch = FALSE;
			$errorMessage = $this->lang->getText('NRS_ERROR_MINIMUM_NUMBER_OF_NIGHT_SHOULD_NOT_BE_LESS_THAN_TEXT').' ';
			$errorMessage .= $this->lang->getPrintCeilDurationByPeriod($this->priceCalculationType, $this->minBookingPeriod).'. '. $this->lang->getText('NRS_ERROR_PLEASE_MODIFY_YOUR_SEARCH_CRITERIA_TEXT');
			$this->errorMessages[] = $errorMessage;
		} else if (NRSValidator::getPeriod($pickupTimestamp, $dropoffTimestamp, FALSE) > $this->maxBookingPeriod)
		{
			$errorMessage = $this->lang->getText('NRS_ERROR_MAXIMUM_BOOKING_LENGTH_CANT_BE_MORE_THAN_TEXT').' ';
			$errorMessage .= $this->lang->getPrintCeilDurationByPeriod($this->priceCalculationType, $this->maxBookingPeriod).'. '. $this->lang->getText('NRS_ERROR_PLEASE_MODIFY_YOUR_SEARCH_CRITERIA_TEXT');
			$this->validSearch = FALSE;
			$this->errorMessages[] = $errorMessage;
		}

		// Both members bellow are in GMT+0
		if ($this->pickupTimestamp < time())
		{
			$this->validSearch = FALSE;
			$currentDateTime = date_i18n(get_option('date_format').' '.get_option('time_format'));
			$errorMessage = $this->lang->getText('NRS_ERROR_PICKUP_IS_NOT_POSSIBLE_ON_TEXT').' ';
			$errorMessage .= $this->getPrintPickupDate().' '.$this->getPrintPickupTime().'. ';
			$errorMessage .= $this->lang->getText('NRS_ERROR_PLEASE_MODIFY_YOUR_PICKUP_TIME_BY_WEBSITE_TIME_TEXT').' <br />';
			$errorMessage .= $this->lang->getText('NRS_ERROR_CURRENT_DATE_TIME_TEXT') .' '.$currentDateTime;

			$this->errorMessages[] = $errorMessage;
		} else if ($this->pickupTimestamp < (time() + $this->minPeriodUntilPickup))
		{
			$this->validSearch = FALSE;
			$earliestPossiblePickupLocalTime = time() + $this->minPeriodUntilPickup + get_option('gmt_offset') * 3600;
			$earliestPossiblePickupDateTime = date_i18n(get_option('date_format').' '.get_option('time_format'), $earliestPossiblePickupLocalTime, TRUE);
			$errorMessage = $this->lang->getText('NRS_ERROR_PICKUP_IS_NOT_POSSIBLE_ON_TEXT').' ';
			$errorMessage .= $this->getPrintPickupDate().' '.$this->getPrintPickupTime().'. <br />';
			$errorMessage .= $this->lang->getText('NRS_ERROR_EARLIEST_POSSIBLE_PICKUP_DATE_TIME_TEXT') .' '.$earliestPossiblePickupDateTime.' ';
			$errorMessage .= $this->lang->getText('NRS_ERROR_OR_NEXT_WORKING_HOURS_OF_PICKUP_LOCATION_TEXT').'.';
			if($this->debugMode)
			{
				$errorMessage .= ' ('.$earliestPossiblePickupLocalTime.')';
			}

			$this->errorMessages[] = $errorMessage;
		}

		return $this->validSearch;
	}

	protected function validatePickupInput($paramLocationId, $paramTimestamp)
	{
		return $this->validateLocationInput("PICKUP", $paramLocationId, $paramTimestamp);
	}

	protected function validateDropoffInput($paramLocationId, $paramTimestamp)
	{
		return $this->validateLocationInput("DROPOFF", $paramLocationId, $paramTimestamp);
	}

	/**
	 * Checks the location
	 * @param string $paramType - "PICKUP" or "DROPOFF"
	 * @param $paramLocationId
	 * @param $paramTimestamp
	 * @return bool
	 */
	private function validateLocationInput($paramType = "PICKUP", $paramLocationId, $paramTimestamp)
	{
		$validType = ($paramType == "DROPOFF") ? "DROPOFF" : "PICKUP";
		$validTimestamp = NRSValidator::getValidInteger($paramTimestamp);
		$lowercaseType = strtolower($validType);
		$internalSearchConfig = array(
			"conf_currency_symbol" => $this->currencySymbol,
			"conf_currency_code" => $this->currencyCode,
			"conf_vat_percentage" => $this->vatPercentage,
			"conf_show_prices_with_vat" => $this->showPricesWithVAT,
		);

		/********************************************************************/
		$objCore = new NRSCore($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$locationRequired = $objCore->getSearchFieldStatus($validType == "DROPOFF" ? "dropoff_location" : "pickup_location", "REQUIRED");
		unset($objCore);
		/*******************************************************************/

		// Location validation
		$objLocation = new NRSLocation($this->internalWPDB, $this->nrsPrefix, $this->lang, $internalSearchConfig, $paramLocationId);

		if(($paramLocationId > 0 || $locationRequired) && $objLocation->isLocationOpenAtDate(NRSValidator::getLocalDateByTimestamp($validTimestamp, "Y-m-d")) === FALSE)
		{
			// Location is closed at selected date, because this date is marked as holidays 'Holidays day'
			$locationData = $objLocation->getLocationDetails(NRSValidator::getLocalDateByTimestamp($validTimestamp, "D"));
			$this->validSearch = FALSE;
			$this->errorMessages[]  = sprintf(
				$this->lang->getText('NRS_ERROR_'.$validType.'_LOCATION_IS_CLOSED_AT_THIS_DATE_TEXT'),
				$locationData['location_title'], $locationData['location_address'],
				NRSValidator::getLocalDateByTimestamp($validTimestamp, "Y-m-d")
			);
		} else if(($paramLocationId > 0 || $locationRequired) && $objLocation->isLocationOpenAtTime(
				NRSValidator::getLocalDateByTimestamp($validTimestamp, "H:i:s"),
				$validType,
				NRSValidator::getLocalDateByTimestamp($validTimestamp, "D")
			) == 0)
		{
			// Pickup/Dropoff Location doesn't work at this time - it's working hours
			$locationData =  $objLocation->getLocationDetails(NRSValidator::getLocalDateByTimestamp($validTimestamp, "D"));
			$errorMessage = "";
			$errorMessage .= sprintf(
				$this->lang->getText('NRS_ERROR_'.$validType.'_LOCATION_IS_CLOSED_AT_THIS_TIME_TEXT'),
				$locationData['location_title'],
				$locationData['location_address'], NRSValidator::getLocalDateByTimestamp($validTimestamp, "H:i:s")
			);

            // WordPress bug WorkAround
            $printDate = date_i18n(get_option('date_format'), $validTimestamp + get_option( 'gmt_offset' ) * 3600, TRUE);

			$errorMessage .= "<br />".sprintf(
				$this->lang->getText('NRS_ERROR_BRANCH_OPEN_HOURS_ARE_TEXT'),
                NRSValidator::getLocalDateByTimestamp($validTimestamp, "D"),
                $printDate,
				$locationData['print_open_hours']
			);
			$errorMessage .= "<br />".$this->lang->getText('NRS_ERROR_BRANCH_WEEKLY_OPEN_HOURS_ARE_TEXT')."<br />";
			$errorMessage .= $objLocation->getPrintWeekWorkingHoursWithDayName();

			if($locationData['afterhours_'.$lowercaseType.'_location_id'] == 0)
			{
				$errorMessage .= "<br />".$this->lang->getText('NRS_ERROR_AFTERHOURS_PICKUP_IS_NOT_ALLOWED_AT_LOCATION_TEXT');
			} else
			{
				if($locationData['print_afterhours_'.$lowercaseType.'_open_hours'])
				{
					$errorMessage  = "<br />".sprintf(
						$this->lang->getText('NRS_ERROR_AFTERHOURS_'.$validType.'_LOCATION_IS_CLOSED_AT_THIS_TIME_TEXT'),
						$locationData['afterhours_'.$lowercaseType.'_location_title'],
						$locationData['afterhours_'.$lowercaseType.'_location_address']
					);
					$errorMessage .= "<br />".sprintf(
						$this->lang->getText('NRS_ERROR_AFTERHOURS_'.$validType.'_BRANCH_OPEN_HOURS_ARE_TEXT'),
						$locationData['print_afterhours_'.$lowercaseType.'_open_hours']
					);
				}
			}
			$this->validSearch = FALSE;
			$this->errorMessages[] = $errorMessage;
		}

		if($this->debugMode)
		{
			echo "<br />{$validType} Validation - Date: ".NRSValidator::getLocalDateByTimestamp($validTimestamp, "Y-m-d").", ";
			echo "Time: ".NRSValidator::getLocalDateByTimestamp($validTimestamp, "H:i:s").", ";
			echo "Day of Week: ".NRSValidator::getLocalDateByTimestamp($validTimestamp, "D").", ";
            echo "Unix Timestamp: ".$validTimestamp.", ";
			echo "Location Id: ".$objLocation->getId();
			//echo nl2br(print_r($locationData, TRUE));
		}

		return $this->validSearch;
	}

	// Clearing must be called in constructor if needed
	private function clearExpiredBookings()
	{
		// OPTIMIZED - we don't use UNIX_TIMESTAMP() here is SQL WHERE, to save resource and not to recalculate time every time
		// so we use PHP's time() function instead, which does the work pretty well,
		// and do not needs to be calculated on every SQL query
		$sqlRows = $this->internalWPDB->get_results("
			SELECT b.booking_id
			FROM {$this->nrsPrefix}bookings b
			JOIN {$this->nrsPrefix}payment_methods pm ON b.payment_method_code=pm.payment_method_code
			WHERE b.payment_success='0' AND pm.unpaid_booking_expiration_time!='0'
			AND (
				(b.booking_timestamp < (".time()."-pm.unpaid_booking_expiration_time))
				OR (b.pickup_timestamp < (".(time()-86400)."))
			)
		", ARRAY_A);

		foreach ($sqlRows AS $currentRow)
		{
			$this->internalWPDB->query("
				DELETE FROM {$this->nrsPrefix}bookings
				WHERE booking_id='".$currentRow['booking_id']."'
			");
			$this->internalWPDB->query("
				DELETE FROM {$this->nrsPrefix}booking_options
				WHERE booking_id='".$currentRow['booking_id']."'
			");
			$this->internalWPDB->query("
				DELETE FROM {$this->nrsPrefix}invoices
				WHERE booking_id='".$currentRow['booking_id']."'
			");
		}
	}
}