<?php
/**
 * NRS Units Manager
 * Abstract class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

abstract class NRSUnitManager
{
    protected $internalWPDB 		    = NULL;
    protected $nrsPrefix 			    = "";
    protected $lang 			        = NULL;
    protected $debugMode 			    = 0; // 1 - items (3 - deep), 2 - extras (4 - deep), 0 - disabled
    protected $debugPickupTimestamp     = -1; // timestamp for exact pickup, or -1 - for all
    protected $elementId                = 0;
    protected $pickupTimestamp          = 0;
    protected $dropoffTimestamp         = 0;
    protected $blockPeriod              = 0;

    public function __construct(
        wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang,
        $paramElementId, $paramTimestampFrom, $paramTimestampTo, $paramOptionalConfig = array()
    ) {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;
        $this->elementId = NRSValidator::getValidInteger($paramElementId, 0);
        $this->pickupTimestamp = NRSValidator::getValidInteger($paramTimestampFrom, 0);
        $this->dropoffTimestamp = NRSValidator::getValidInteger($paramTimestampTo, 0);
        // Set block interval between two booking
        $this->setBlockPeriod($paramOptionalConfig);
    }

    /**
     * Set block interval between two booking - called in class constructor
     * @param array $paramOptionalConfig
     */
    private function setBlockPeriod($paramOptionalConfig = array())
    {
        if(isset($paramOptionalConfig['conf_minimum_block_period_between_bookings']))
        {
            // Fast processing mode (for availability and price table), to reduce sql query amount
            $this->blockPeriod = NRSValidator::getValidInteger($paramOptionalConfig['conf_minimum_block_period_between_bookings'], 0);
        } else
        {
            $sql = "
				SELECT conf_value AS minimum_block_period_between_bookings
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_minimum_block_period_between_bookings'
			";
            $blockPeriod = $this->internalWPDB->get_var($sql);

            if(!is_null($blockPeriod))
            {
                $this->blockPeriod = NRSValidator::getValidInteger($blockPeriod, 0);
            } else
            {
                $this->blockPeriod = 0;
            }
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    /**
     * Supports units
     * @param string $type - "ITEM" or "EXTRA"
     * @param int $paramLocationId (DEFAULT = 0, when it is applied to any location)
     * @param int $paramIgnoreFromBookingId (DEFAULT = 0, when it will not exclude any booking from calculation any booking)
     * @return int
     */
    protected function getTotalUnitsBookedByType($type = "ITEM", $paramLocationId = 0, $paramIgnoreFromBookingId = 0)
    {
        $arrDebugCodes = $type == "EXTRA" ? array(2, 4) :  array(1, 3);
        $deepDebugCode = $type == "EXTRA" ? 4 : 3;
        $validDBIDField = $type == "EXTRA" ? "extra_id" : "item_id";
        $validLocationId = NRSValidator::getValidInteger($paramLocationId, 0);
        $validIgnoreFromBookingId = NRSValidator::getValidInteger($paramIgnoreFromBookingId, 0);
        $startTimestampMinusBlockPeriod = $this->pickupTimestamp - $this->blockPeriod;
        $startTimestamp = $this->pickupTimestamp;
        $endTimestamp = $this->dropoffTimestamp;
        $endTimestampPlusBlockPeriod = $this->dropoffTimestamp + $this->blockPeriod;

        if(in_array($this->debugMode, $arrDebugCodes) && in_array($this->debugPickupTimestamp, array(-1,$this->pickupTimestamp)))
        {
            echo "<br />----------------------------------------------";
        }

        // ------- Intersection Algorithm -------
        //  (C) 2016, Kestutis Matuliauskas
        // All plugin's code, including this complex algorithm described bellow
        // is COPYRIGHTED BY GNU AFFERO GENERAL PUBLIC LICENSE (AGPL)
        // And is STRICTLY PROHIBITED TO REPLICATE, COPY, SELL OR USE anywhere else, EXCEPT IN THIS PLUGIN!
        //
        // ---------- New Algorithm description ----------
        // 1. Make intervals list
        // 2. For each interval - sum and save the number of totally booked units
        // 3. Get the max number
        //
        // NEW BOOKING:
        // 2016-01-09 - 2016-01-31
        //
        // EXISTING BOOKINGS:
        // 2016-01-04 - 2016-01-10
        // 2016-01-01 - 2016-01-10
        // 2016-01-11 - 2016-01-13
        // 2016-01-09 - 2016-01-11
        // 2016-01-14 - 2016-01-20
        //
        // WHERE LINE 1. SEARCH FOR OTHER BOOKINGS BETWEEN:    15th -> (30+1)th    [NEW BOOKING FROM/TO: 10th -> 25th]
        // WHERE LINE 2. SEARCH FOR OTHER BOOKINGS BETWEEN: (5+1)th ->  15th       [NEW BOOKING FROM/TO: 10th -> 25th]
        // WHERE LINE 3. SEARCH FOR OTHER BOOKINGS BETWEEN:    5th  ->  30th       [NEW BOOKING FROM/TO: 10th -> 25th]
        // WHERE LINE 4. SEARCH FOR OTHER BOOKINGS BETWEEN: 5th->15h, 20th->30th   [NEW BOOKING FROM/TO: 10th -> 25th]
        // And sometimes there is 2 or 3 bookings in between, i.e. 6th-8th, 9th-10th, 10th-12th, 11th-13th,
        // which only partly intersects the 5th-30th booking
        // -------------------------------------------------------------------
        // -------------------------------------------------------------------
        // -------------------------------------------------------------------

        // 1. We need to run a SQL query to find all previous bookings, which:
        //   1.1. has pickup date between 10th and (25+BLOCK_PERIOD)th, or
        //   1.2. has dropoff date between (10-BLOCK_PERIOD)th and 25th
        //   1.3. has pickup date before 10th and dropoff date after (25+BLOCK_PERIOD)th
        $searchSQL = "
			SELECT bopt.booking_id, bopt.{$validDBIDField} AS element_id, bopt.units_booked,
			bb.booking_code, bb.pickup_timestamp, bb.dropoff_timestamp, bb.pickup_location_id AS location_id
			FROM {$this->nrsPrefix}booking_options bopt
			JOIN {$this->nrsPrefix}bookings bb ON bopt.booking_id=bb.booking_id
			WHERE bopt.{$validDBIDField}='{$this->elementId}' AND bb.booking_id!='{$validIgnoreFromBookingId}' AND bb.is_cancelled='0'
			AND (
                (
                    pickup_timestamp BETWEEN '{$startTimestamp}' AND '{$endTimestampPlusBlockPeriod}'
                ) OR (
                    dropoff_timestamp BETWEEN '{$startTimestampMinusBlockPeriod}' AND '{$endTimestamp}'
                ) OR (
                    pickup_timestamp < '{$startTimestamp}' AND dropoff_timestamp > '{$endTimestampPlusBlockPeriod}'
                )
            )
		";

        $bookedData = $this->internalWPDB->get_results($searchSQL, ARRAY_A);

        $allUnitsBlocked = FALSE;
        $arrPickupTimestamps = array();
        $arrDropoffTimestamps = array();
        $arrDebugDropoffTimestampsWithoutBlockPeriod = array();

        foreach ($bookedData AS $bookedRow)
        {
            // 3. Check if all units are blocked
            if(($bookedRow['location_id'] == 0 || $bookedRow['location_id'] == $validLocationId) &&
               $bookedRow['units_booked'] == -1
            ) {
                // This block is applied for this location, or if the block is applied to all locations

                $allUnitsBlocked = TRUE;
                // Exit foreach
                break;
            } else if($bookedRow['units_booked'] == -1)
            {
                // This block is applied for other location
                // Skip to the next booking
                continue;
            }

            if(
                !in_array($bookedRow['pickup_timestamp'], $arrPickupTimestamps) &&
                $bookedRow['pickup_timestamp'] >= $startTimestamp &&
                $bookedRow['pickup_timestamp'] <= $endTimestampPlusBlockPeriod
            ) {
                // 4. Add current pickup date to the list
                $arrPickupTimestamps[] = $bookedRow['pickup_timestamp'];
            }

            if(
                !in_array($bookedRow['dropoff_timestamp']+$this->blockPeriod, $arrDropoffTimestamps) &&
                $bookedRow['dropoff_timestamp'] >= $startTimestamp &&
                $bookedRow['dropoff_timestamp'] <= $endTimestampPlusBlockPeriod
            ) {
                // 5. Add current pickup date to the list
                $arrDropoffTimestamps[] = $bookedRow['dropoff_timestamp']+$this->blockPeriod;
                $arrDebugDropoffTimestampsWithoutBlockPeriod[] = $bookedRow['dropoff_timestamp'];
            }
        }

        $arrPeriods = array();
        $maxUnitsBooked = 0;
        if($allUnitsBlocked === TRUE)
        {
            $maxUnitsBooked = -1;
        } else
        {
            // Now we are sure, that there is no blocks in the list, and no non-valid blocks

            // 6. Sort the arrays from lowest to biggest
            sort($arrPickupTimestamps, SORT_NUMERIC);
            sort($arrDropoffTimestamps, SORT_NUMERIC);
            sort($arrDebugDropoffTimestampsWithoutBlockPeriod, SORT_NUMERIC);

            // 7. Get first pickup timestamps
            if(sizeof($arrPickupTimestamps) >= 1)
            {
                $firstPickupTimestamp = $arrPickupTimestamps[0];

                // 7.1. If search start time is earlier than earliest matching pickup
                if($startTimestamp < $firstPickupTimestamp)
                {
                    // Then add search start time to array
                    array_unshift($arrPickupTimestamps, $startTimestamp);

                    if($this->debugMode == $deepDebugCode && in_array($this->debugPickupTimestamp, array(-1, $this->pickupTimestamp)))
                    {
                        echo "<br />Search start timestamp {$startTimestamp} added to pickup timestamps array.";
                    }
                }
            } else
            {
                $firstPickupTimestamp = $startTimestamp;
                array_unshift($arrPickupTimestamps, $startTimestamp);

                if($this->debugMode == $deepDebugCode && in_array($this->debugPickupTimestamp, array(-1, $this->pickupTimestamp)))
                {
                    echo "<br />Search start timestamp {$startTimestamp} added to pickup timestamps array.";
                }
            }

            // 8. Get last dropoff timestamps
            if(sizeof($arrDropoffTimestamps) >= 1)
            {
                $lastDropoffTimestamp = $arrDropoffTimestamps[sizeof($arrDropoffTimestamps)-1];

                // 8.1. If search end time plus block period is later than latest matching dropoff
                if($endTimestampPlusBlockPeriod > $lastDropoffTimestamp)
                {
                    // Then add search end time + block period to array
                    $arrDropoffTimestamps[] = $endTimestampPlusBlockPeriod;
                    $arrDebugDropoffTimestampsWithoutBlockPeriod[] = $endTimestamp;

                    if($this->debugMode == $deepDebugCode && in_array($this->debugPickupTimestamp, array(-1, $this->pickupTimestamp)))
                    {
                        echo "<br />Search end timestamp ({$endTimestamp}) plus block period ({$this->blockPeriod}) ";
                        echo "- {$endTimestampPlusBlockPeriod} added to dropoff timestamps array.";
                    }
                }
            } else
            {
                $lastDropoffTimestamp = $endTimestampPlusBlockPeriod;
                $arrDropoffTimestamps[] = $endTimestampPlusBlockPeriod;
                $arrDebugDropoffTimestampsWithoutBlockPeriod[] = $endTimestamp;
            }

            if($this->debugMode == $deepDebugCode && in_array($this->debugPickupTimestamp, array(-1, $this->pickupTimestamp)))
            {
                echo "<br />First Pickup Timestamp: {$firstPickupTimestamp}";
                echo "<br />Last Dropoff Timestamp: {$lastDropoffTimestamp}";
            }

            // 10. Make timestamp periods for each pickup
            foreach($arrPickupTimestamps AS $pickupTimestamp)
            {
                // 10.1 By default - set closest pickup to search start timestamp
                $closestDropoffTimestamp = $endTimestampPlusBlockPeriod;
                $closestDropoffTimeDistance = -1;

                foreach($arrDropoffTimestamps AS $dropoffTimestampWithBlockPeriod)
                {
                    // 10.1. Get only those dropoffs, which is later or equal to pickup
                    if($pickupTimestamp <= $dropoffTimestampWithBlockPeriod)
                    {
                        if(
                            $closestDropoffTimeDistance == -1 ||
                            (
                                $closestDropoffTimeDistance >= 0 &&
                                $closestDropoffTimeDistance > $dropoffTimestampWithBlockPeriod - $pickupTimestamp
                            )
                        ) {
                            // If previous closest dropoff distance was longer, then make it sorter
                            $closestDropoffTimeDistance = $dropoffTimestampWithBlockPeriod - $pickupTimestamp;
                            $closestDropoffTimestamp = $dropoffTimestampWithBlockPeriod;
                        }
                    }
                }

                // 10.2. Add new period
                $arrPeriods[] = array(
                    "from" => $pickupTimestamp,
                    "till" => $closestDropoffTimestamp,
                );
            }

            // 11. Get total size of periods
            $totalPeriods = sizeof($arrPeriods);

            // 12. Loop over all timestamp periods
            for($i = 0; $i < $totalPeriods; $i++)
            {
                $totalUnitsBooked = 0;
                $periodStart = $arrPeriods[$i]['from'];
                $periodEnd = $arrPeriods[$i]['till'];
                foreach ($bookedData AS $bookedRow)
                {
                    if(
                        (
                            $bookedRow['pickup_timestamp'] >= $periodStart && $bookedRow['pickup_timestamp'] <= $periodEnd
                        ) || (
                            $bookedRow['dropoff_timestamp']+$this->blockPeriod >= $periodStart &&
                            $bookedRow['dropoff_timestamp']+$this->blockPeriod <= $periodEnd
                        ) || (
                            $bookedRow['pickup_timestamp'] < $periodStart && $bookedRow['dropoff_timestamp'] > $periodEnd
                        )
                    ) {
                        $totalUnitsBooked += $bookedRow['units_booked'];
                    }
                }
                $arrPeriods[$i]['total_units_booked'] = $totalUnitsBooked;
            }

            // 13. Find the period with maximum total units booked
            // 13.1. Iterate through bookings and their intersections
            foreach($arrPeriods AS $periodDetails)
            {
                // 13.2. And assign to MAX_UNITS_BOOKED the maximum value of TOTAL_UNITS_WITH_INTERSECTIONS
                if($periodDetails['total_units_booked'] > $maxUnitsBooked)
                {
                    $maxUnitsBooked = $periodDetails['total_units_booked'];
                }
            }
        }

        if(in_array($this->debugMode, $arrDebugCodes) && in_array($this->debugPickupTimestamp, array(-1, $this->pickupTimestamp)))
        {
            $pickupTMZStamp = $this->pickupTimestamp + get_option( 'gmt_offset' ) * 3600;
            $dropoffTMZStamp = $this->dropoffTimestamp + get_option( 'gmt_offset' ) * 3600;
            $printGMTPickupDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $this->pickupTimestamp, TRUE);
            $printGMTDropoffDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $this->dropoffTimestamp, TRUE);
            $printPickupDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $pickupTMZStamp, TRUE);
            $printDropoffDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $dropoffTMZStamp, TRUE);

            echo "<br /><strong>OVERALL SUMMARY: </strong>";
            echo "<br />UNIX local FROM-TO timestamps:{$this->pickupTimestamp} - {$this->dropoffTimestamp}";
            echo "<br />Server GMT dates &amp; times: {$printGMTPickupDate} - {$printGMTDropoffDate}";
            echo "<br />Website&#39;s dates &amp; times: {$printPickupDate} - {$printDropoffDate}";
            echo "<br /><strong>Location Id</strong> (to count element units, 0 - all)<strong>:</strong> {$paramLocationId}";
            echo "<br /><strong>Booking Id</strong> (to exclude element units from counting)<strong>:</strong> {$paramIgnoreFromBookingId}";
            if($this->debugMode == $deepDebugCode)
            {
                // Deep debug
                echo "<br /><strong>Search SQL query:</strong>";
                echo nl2br($searchSQL);
                echo "<br /><strong>SQL search result:</strong>";
                echo nl2br(print_r($bookedData, TRUE));
            }
            echo "<br /><strong>All pickups:</strong>";
            echo nl2br(print_r($arrPickupTimestamps, TRUE));
            echo "<br /><strong>All debug dropoffs (w/o block period):</strong>";
            echo nl2br(print_r($arrDebugDropoffTimestampsWithoutBlockPeriod, TRUE));
            echo "<br /><strong>All dropoffs:</strong>";
            echo nl2br(print_r($arrDropoffTimestamps, TRUE));
            echo "<br /><strong>All periods with total bookings in them:</strong>";
            echo nl2br(print_r($arrPeriods, TRUE));
            echo "<br />Returned Maximum Booked Unit (for {$type} ID={$this->elementId}): {$maxUnitsBooked}";
            echo "<br />----------------------------------------------";
            echo "<br />";
        }

        return $maxUnitsBooked;
    }
}