<?php
/**
 * NRS Control Root Class - we use it in initializer, so it cant be abstract
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

class NRSCore
{
    protected $internalWPDB     = NULL;
    protected $nrsPrefix        = "";
    protected $lang 			= NULL;
    protected $config           = array();
    protected $debugMode        = 0;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;
        $this->setConfig();
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getLangText($paramName)
    {
        return $this->lang->getText($paramName);
    }

    private function setConfig()
    {
        $sqlRows = $this->internalWPDB->get_results("
			SELECT conf_key, conf_value
			FROM {$this->nrsPrefix}settings
		", ARRAY_A);

        foreach ($sqlRows AS $currentRow)
        {
            if($currentRow['conf_key'])
            {
                $this->config[sanitize_key($currentRow['conf_key'])] = trim($currentRow['conf_value']);
            }
        }

        if(isset($this->config['conf_short_date_format']))
        {
            // Add datepicker format
            $datePickerFormat = "yy/mm/dd";
            if($this->config['conf_short_date_format'] == "Y-m-d")
            {
                $datePickerFormat = "yy-mm-dd";
            } else if($this->config['conf_short_date_format'] == "d/m/Y")
            {
                $datePickerFormat = "dd/mm/yy";
            } else if($this->config['conf_short_date_format'] == "m/d/Y")
            {
                $datePickerFormat = "mm/dd/yy";
            }
            $this->config['conf_datepicker_date_format'] = $datePickerFormat;
        }
    }

    /**
     * Returns with CONF_ prefix
     * @return array
     */
    public function getAllConfigArray()
    {
        return $this->config;
    }

    public function getConfig($key)
    {
        $ret = "";
        $sanitizedKey = sanitize_key($key);
        if($sanitizedKey != "")
        {
            $ret = isset($this->config['conf_'.$sanitizedKey]) ? $this->config['conf_'.$sanitizedKey] : "";
        }

        return $ret;
    }

    /**
     * Visibility/requirement check for customers data fields
     * @param $fieldName
     * @param $type
     * @return bool
     */
    public function getCustomerFieldStatus($fieldName, $type)
    {
        return $this->getSettingsFieldStatus("customer_", $fieldName, $type);
    }

    /**
     * Visibility/requirement check for search fields
     * @param $fieldName
     * @param $type
     * @return bool
     */
    public function getSearchFieldStatus($fieldName, $type)
    {
        return $this->getSettingsFieldStatus("search_", $fieldName, $type);
    }

    // Pull up settings
    /**
     * @param $prefix - "search_" or "customer_"
     * @param $fieldName - i.e. "first_name"
     * @param $type - REQUIRED || VISIBLE
     * @return bool
     */
    private function getSettingsFieldStatus($prefix, $fieldName, $type)
    {
        $fieldVisible = false;
        $fieldRequired = false;

        $sanitizedPrefix = sanitize_text_field($prefix);
        $sanitizedFieldName = sanitize_text_field($fieldName);
        $fullFieldNameVisible = $sanitizedPrefix.$sanitizedFieldName."_visible";
        $fullFieldNameRequired = $sanitizedPrefix.$sanitizedFieldName."_required";

        if($this->getConfig($fullFieldNameVisible) == "1")
        {
            $fieldVisible = true;
        }
        if($fieldVisible && $this->getConfig($fullFieldNameRequired) == "1")
        {
            $fieldRequired = true;
        }

        if($this->debugMode)
        {
            echo "<br /><strong>[Method: getCustomerFieldStatus]:</strong> Visible Field Name: {$fullFieldNameVisible} "; var_dump($fieldVisible);
            echo ", Required Field Name: {$fullFieldNameRequired} "; var_dump($fieldRequired);
        }

        return $type == "REQUIRED" ? $fieldRequired : $fieldVisible;
    }

    public function getRTLSuffixIfRTL()
    {
        return $this->lang->isRTL() ? "-rtl" : "";
    }

    public function getWeekdays()
    {
        if(get_option('start_of_week') == 1)
        {
            $weeksDays = array(
                'mon' => $this->lang->getText('NRS_MON_TEXT'),
                'tue' => $this->lang->getText('NRS_TUE_TEXT'),
                'wed' => $this->lang->getText('NRS_WED_TEXT'),
                'thu' => $this->lang->getText('NRS_THU_TEXT'),
                'fri' => $this->lang->getText('NRS_FRI_TEXT'),
                'sat' => $this->lang->getText('NRS_SAT_TEXT'),
                'sun' => $this->lang->getText('NRS_SUN_TEXT'),
            );
        } else
        {
            $weeksDays = array(
                'sun' => $this->lang->getText('NRS_SUN_TEXT'),
                'mon' => $this->lang->getText('NRS_MON_TEXT'),
                'tue' => $this->lang->getText('NRS_TUE_TEXT'),
                'wed' => $this->lang->getText('NRS_WED_TEXT'),
                'thu' => $this->lang->getText('NRS_THU_TEXT'),
                'fri' => $this->lang->getText('NRS_FRI_TEXT'),
                'sat' => $this->lang->getText('NRS_SAT_TEXT'),
            );
        }

        return $weeksDays;
    }

    /**************************************************************************************/
    /***************************** START OF DROP-DOWNS ************************************/
    /**************************************************************************************/

    /**
     * @param int $paramSelectPageId
     * @param string $name
     * @param null $id
     * @return string
     */
    public function getWPNRSPagesDropDown($paramSelectPageId = 0, $name = "item_description_page_id", $id = null)
    {
        $nrsDescriptionArgs = array(
            'depth' => 1,
            'child_of' => 0,
            'selected' => $paramSelectPageId,
            'echo' => 0,
            'name' => $name,
            'id' => $id, // string
            'show_option_none' => ' - Choose description page - ', // string
            'sort_order' => 'ASC',
            'sort_column' => 'post_title',
            'post_type' => $GLOBALS['NRS_EXTENSION'],
        );
        $itemDescriptionDropDownHTML = wp_dropdown_pages($nrsDescriptionArgs);

        // DEBUG
        //echo "RESULT: $itemDescriptionDropDownHTML";

        return $itemDescriptionDropDownHTML;
    }

    /**
     * @param string $timeFieldId
     * @param string $name
     * @param string $selectedTime
     * @param array $excludedTimes - times to exclude, i.e. (09:00:00)
     * @param bool $show_24_7_Time
     * @return string
     */
    public function getTimeDropDown(
        $timeFieldId = "time", $name = "time", $selectedTime = "09:00:00",
        $show_24_7_Time = false, $excludedTimes = array()
    ) {
        $timeDropDown = '<select id="'.$timeFieldId.'" name="'.$name.'">';

        for($hour = 0; $hour < 24; $hour++)
        {
            for($min = 0; $min < 60; $min = $min+30)
            {
                $currentHour = sprintf("%02d", $hour);
                $currentMin = sprintf("%02d", $min);

                $currentTime = $currentHour.':'.$currentMin.':00';
                $selected = ($currentTime == $selectedTime ? ' selected="selected"' : '');

                $UTCUnixTime = strtotime(date("Y-m-d")." ".$currentTime);
                $printCurrentTime = date_i18n(get_option('time_format'), $UTCUnixTime, TRUE);

                // Show time only it is is not in exclude list
                if(!in_array($currentTime, $excludedTimes))
                {
                    $timeDropDown .= '<option value="'.$currentTime.'"'.$selected.'>'.$printCurrentTime.'</option>';
                }
            }
        }

        // Special 23:59:59
        if($show_24_7_Time)
        {
            $selected = ($selectedTime == "23:59:59" ? ' selected="selected"' : '');

            $UTCUnixTime = strtotime(date("Y-m-d")." 23:59:59");
            $printCurrentTime = date_i18n(get_option('time_format'), $UTCUnixTime, TRUE);

            $timeDropDown .= '<option value="23:59:59"'.$selected.'>'.$printCurrentTime.'</option>';
        }

        $timeDropDown .= '</select>';

        return $timeDropDown;
    }

    /**************************************************************************************/
    /****************************** START OF EXTENDED METHODS *****************************/
    /**************************************************************************************/

    public function getPaymentMethodCodes()
    {
        $paymentGatewayCodes = array();
        $sqlRows = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}payment_methods
            WHERE method_enabled='1'
            ORDER BY method_order ASC
        ", ARRAY_A);

        foreach($sqlRows AS $currentRow)
        {
            $paymentGatewayCodes[] = $currentRow["payment_method_code"];
        }

        return $paymentGatewayCodes;
    }

    public function getPaymentMethods($selectedGatewayCode = "")
    {
        $paymentGateways = array();
        $sqlRows = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}payment_methods
            WHERE method_enabled='1'
            ORDER BY method_order ASC
        ", ARRAY_A);

        // DEBUG
        //echo "SEL: {$selectedGatewayCode}, DEFAULT: {$sqlRows[0]['payment_method_code']}<br />";

        // Set default to first item to be selected, if nothing is selected
        if(isset($sqlRows[0]['payment_method_code']))
        {
            $selectedGatewayCode = $selectedGatewayCode == "" ? $sqlRows[0]['payment_method_code'] : $selectedGatewayCode;
        }


        foreach($sqlRows AS $currentRow)
        {
            $selected = $selectedGatewayCode == $currentRow['payment_method_code'] ? true : false;
            $printChecked = $selected ? ' checked="checked"' : '';
            $printSelected = $selected ? ' selected="selected"' : '';

            $printMethodDetails = $currentRow['payment_method_code'] == "pp" ? $this->lang->getText('NRS_SECURE_INSTANT_PAYMENT_TEXT') : nl2br($currentRow['method_details']);

            $paymentGateways[$currentRow['payment_method_code']] = array(
                "method_name" => $currentRow['method_name'],
                "method_details" => $currentRow['method_details'],
                "selected" => $selected,
                "print_method_details" => $printMethodDetails,
                "print_checked" => $printChecked,
                "print_selected" => $printSelected,
            );
        }

        return $paymentGateways;
    }

    public function getPaymentDataByCode($gatewayCode)
    {
        $row = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}payment_methods
            WHERE payment_method_code='".sanitize_text_field($gatewayCode)."'
        ", ARRAY_A);

        if(!is_null($row))
        {
            $printMethodDetails = $row['payment_method_code'] == "pp" ? $this->lang->getText('NRS_SECURE_INSTANT_PAYMENT_TEXT') : nl2br($row['method_details']);
            $row['print_method_details'] = $printMethodDetails;
        }

        return $row;
    }

    public function getClosedDates()
    {
        $sqlRows = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}closed_dates", ARRAY_A);
        $closedDates = array();
        foreach ($sqlRows AS $row)
        {
            if($row['closed_date'] != "0000-00-00")
            {
                $closedDates[] .= "'".$row['closed_date']."'";
            }
        }
        $dateRange = implode(",",$closedDates);

        return $dateRange;
    }

    /**
     * @param $type - SHORT OR LONG
     * @return string
     */
    public function getPeriodWord($type = "SHORT")
    {
        $periodWord = "";

        if ($this->getConfig('price_calculation_type') == 1)
        {
            // Count by days only
            $periodWord = $type == "LONG" ? $this->lang->getText('NRS_PER_DAY_TEXT') : $this->lang->getText('NRS_PER_DAY_SHORT_TEXT');
        } else if ($this->getConfig('price_calculation_type') == 2)
        {
            // Count by hours only
            $periodWord = $type == "LONG" ? $this->lang->getText('NRS_PER_HOUR_TEXT') : $this->lang->getText('NRS_PER_HOUR_SHORT_TEXT');
        } else if ($this->getConfig('price_calculation_type') == 3)
        {
            // Combined count - days+hours
            $periodWord = $type == "LONG" ? $this->lang->getText('NRS_PER_DAY_TEXT') : $this->lang->getText('NRS_PER_DAY_SHORT_TEXT');
        }
        return $periodWord;
    }

    /**************************************************************************************/
    /**************************************************************************************/
    /**************************************************************************************/
    /**************************************************************************************/

    public function getAdminPeriodFromByPriceType($paramDays, $paramHours)
    {
        return $this->getAdminPeriodByPriceType($paramDays, $paramHours, "FROM");
    }

    public function getAdminPeriodTillByPriceType($paramDays, $paramHours)
    {
        return $this->getAdminPeriodByPriceType($paramDays, $paramHours, "TILL");
    }

    /**
     * Based on price calculation type from duration days / hours it will return period (in seconds)
     * @param $paramDays
     * @param $paramHours
     * @param string $paramType
     * @return int
     */
    private function getAdminPeriodByPriceType($paramDays, $paramHours, $paramType = "FROM")
    {
        $validDays =  NRSValidator::getValidInteger($paramDays, 0);
        $validHours = NRSValidator::getValidInteger($paramHours, 0);
        if($this->getConfig('price_calculation_type') == "1")
        {
            // Days only
            $retPeriod = $paramType == "FROM" ? $validDays*86400 : $validDays*86400+86400-1;
        } else if($this->getConfig('price_calculation_type') == "1")
        {
            // Hours only
            $retPeriod = $paramType == "FROM" ? $validHours*3600 : $validHours*3600+3600-1;
        } else
        {
            // Mixed - Days & Hours
            $retPeriod = $paramType == "FROM" ? $validDays*86400 + $validHours*3600 : ($validDays*86400+86400-1) + ($validHours*3600+3600-1);
        }

        return $retPeriod;
    }


    public function getAdminDaysByPriceTypeFromPeriod($paramPeriod)
    {
        return $this->getAdminDurationByPriceTypeFromPeriod($paramPeriod, "DAYS");
    }

    public function getAdminHoursByPriceTypeFromPeriod($paramPeriod)
    {
        return $this->getAdminDurationByPriceTypeFromPeriod($paramPeriod, "HOURS");
    }

    /**
     * Based on price calculation type from period it will return duration in days / hours
     * @param $paramPeriod
     * @param string $paramType
     * @return int
     */
    private function getAdminDurationByPriceTypeFromPeriod($paramPeriod, $paramType = "DAYS")
    {
        $validPeriod =  NRSValidator::getValidInteger($paramPeriod, 0);
        if($this->getConfig('price_calculation_type') == "1")
        {
            // Days only
            $retDuration = $paramType == "DAYS" ? NRSValidator::getFloorDaysFromSeconds($validPeriod) : 0;
        } else if($this->getConfig('price_calculation_type') == "1")
        {
            // Hours only
            $retDuration = $paramType == "DAYS" ? 0 : NRSValidator::getFloorHoursFromSeconds($validPeriod);
        } else
        {
            // Mixed - Days & Hours
            $retDuration = $paramType == "DAYS" ? NRSValidator::getFloorDaysFromSeconds($validPeriod) : NRSValidator::getFloorHoursOnLastDayFromSeconds($validPeriod);
        }

        return $retDuration;
    }


    /*****************************************************************************/
    /***************************** SETTINGS SECTION ******************************/
    /*****************************************************************************/

    public function getGlobalSettings()
    {
        $globalSettingSelectDropDowns = array();

        $selectStyle = "";
        $currentStyle = $this->getConfig('system_style');
        foreach($GLOBALS['NRS_SUPPORTED_STYLESHEET'] AS $styleSheetId => $styleSheetDetails)
        {
            $selected = $styleSheetId == $currentStyle ? ' selected="selected"' : '';
            $selectStyle .= '<option value="'.$styleSheetId.'"'.$selected.'>'.$styleSheetDetails['title'].'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_system_style'] = $selectStyle;


        $sdfSelected1 = $this->getConfig('short_date_format') == "Y-m-d" ? ' selected="selected"' : '';
        $sdfSelected2 = $this->getConfig('short_date_format') == "d/m/Y" ? ' selected="selected"' : '';
        $sdfSelected3 = $this->getConfig('short_date_format') == "m/d/Y" ? ' selected="selected"' : '';
        $selectShortDateFormat  = '<option value="Y-m-d"'.$sdfSelected1.'>YYYY-MM-DD. Today - '.date_i18n('Y-m-d').'</option>'."\n";
        $selectShortDateFormat .= '<option value="d/m/Y"'.$sdfSelected2.'>DD/MM/YYYY. Today - '.date_i18n('d/m/Y').'</option>'."\n";
        $selectShortDateFormat .= '<option value="m/d/Y"'.$sdfSelected3.'>MM/DD/YYYY. Today - '.date_i18n('m/d/Y').'</option>'."\n";

        $globalSettingSelectDropDowns['select_short_date_format'] = $selectShortDateFormat;


        // MINIMUM PERIOD BETWEEN TWO BOOKINGS (BOOKED ITEM BLOCK TIME)
        $itemBlockList = array(
            '1' => '0 '.$this->lang->getText('NRS_MINUTES2_TEXT'),
            '899' => '15 '.$this->lang->getText('NRS_MINUTES_TEXT'),
            '1799' => '30 '.$this->lang->getText('NRS_MINUTES2_TEXT'),
            '2599' => '45 '.$this->lang->getText('NRS_MINUTES_TEXT'),
            '3599' => '1 '.$this->lang->getText('NRS_HOUR_TEXT'),
            '7199' => '2 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '10799' => '3 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '14399' => '4 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '17999' => '5 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '21599' => '6 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '25199' => '7 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '28799' => '8 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '32399' => '9 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '35999' => '10 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '39599' => '11 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '43199' => '12 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '46799' => '13 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '50399' => '14 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '53999' => '15 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '57599' => '16 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '61199' => '17 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '64799' => '18 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '68399' => '19 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '71999' => '20 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '75599' => '21 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '79199' => '22 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '82799' => '23 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '86399' => '24 '.$this->lang->getText('NRS_HOURS_TEXT'),
        );

        $select_list = "";
        foreach($itemBlockList as $key => $value)
        {
            if($key == $this->getConfig('minimum_block_period_between_bookings'))
            {
                $select_list .= '<option value="'.$key.'" selected="selected">'.$value.'</option>'."\n";
            } else
            {
                $select_list .= '<option value="'.$key.'">'.$value.'</option>'."\n";
            }
        }
        $globalSettingSelectDropDowns['select_minimum_block_period_between_bookings'] = $select_list;


        // MIN PERIOD BEFORE BOOKING START
        $selectMinimumPeriodBeforeBooking = "";
        for($minutes = 0; $minutes < 60; $minutes = $minutes+15)
        {
            $inSeconds = $minutes*60;
            if($inSeconds == $this->getConfig('minimum_period_until_pickup'))
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'" selected="selected">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            } else
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            }
        }
        for($hours = 1; $hours <= 23; $hours++)
        {
            $inSeconds = $hours*3600;
            if($inSeconds == $this->getConfig('minimum_period_until_pickup'))
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'" selected="selected">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>';
            } else
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>';
            }
        }
        for($days = 1; $days <= 30; $days++)
        {
            $inSeconds = $days*86400;
            if($inSeconds == $this->getConfig('minimum_period_until_pickup'))
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'" selected="selected">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>';
            } else
            {
                $selectMinimumPeriodBeforeBooking .= '<option value="'.$inSeconds.'">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>';
            }
        }
        $globalSettingSelectDropDowns['select_minimum_period_until_pickup'] = $selectMinimumPeriodBeforeBooking;


        if($this->getConfig('paypal_sandbox_mode') == 1)
        {
            $select_paypal_sandbox_mode  = '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_paypal_sandbox_mode .= '<option value="0">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        } else
        {
            $select_paypal_sandbox_mode  = '<option value="1">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_paypal_sandbox_mode .= '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        }
        $globalSettingSelectDropDowns['select_paypal_sandbox_mode'] = $select_paypal_sandbox_mode;


        if($this->getConfig('send_emails') == 1)
        {
            $select_send_emails  = '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_send_emails .= '<option value="0">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        } else
        {
            $select_send_emails  = '<option value="1">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_send_emails .= '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        }
        $globalSettingSelectDropDowns['select_send_emails'] = $select_send_emails;


        if($this->getConfig('recaptcha_enabled') == 1)
        {
            $select_recaptcha_enabled  = '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>' . "\n";
            $select_recaptcha_enabled .= '<option value="0">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>' . "\n";
        } else
        {
            $select_recaptcha_enabled  = '<option value="1">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>' . "\n";
            $select_recaptcha_enabled .= '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>' . "\n";
        }
        $globalSettingSelectDropDowns['select_recaptcha_enabled'] = $select_recaptcha_enabled;

        $selectAPIMaxRequestsPerPeriod = '';
        $selected = $this->getConfig('api_max_requests_per_period') == -1 ? ' selected="selected"' : '';
        $selectAPIMaxRequestsPerPeriod .= '<option value="-1"'.$selected.'>'.$this->lang->getText('NRS_UNLIMITED_TEXT').'</option>';
        for($requests = 10; $requests <= 200; $requests = $requests + 10)
        {
            $selected = $requests == $this->getConfig('api_max_requests_per_period') ? ' selected="selected"' : '';
            $selectAPIMaxRequestsPerPeriod .= '<option value="'.$requests.'"'.$selected.'>'.$requests.' ';
            $selectAPIMaxRequestsPerPeriod .= mb_strtolower($this->lang->getText('NRS_ADMIN_REQUESTS_TEXT'), 'UTF-8').' / ';
            $selectAPIMaxRequestsPerPeriod .= $this->lang->getText('NRS_PER_HOUR_TEXT').' / ';
            $selectAPIMaxRequestsPerPeriod .= $this->lang->getText('NRS_ADMIN_IP_TEXT').'</option>';
        }
        $globalSettingSelectDropDowns['select_api_max_requests_per_period'] = $selectAPIMaxRequestsPerPeriod;

        $selectAPIMaxFailedRequestsPerPeriod = '';
        $selected = $this->getConfig('api_max_failed_requests_per_period') == -1 ? ' selected="selected"' : '';
        $selectAPIMaxFailedRequestsPerPeriod .= '<option value="-1"'.$selected.'>'.$this->lang->getText('NRS_UNLIMITED_TEXT').'</option>';
        for($failedRequests = 1; $failedRequests <= 20; $failedRequests++)
        {
            $selected = $failedRequests == $this->getConfig('api_max_failed_requests_per_period') ? ' selected="selected"' : '';
            $selectAPIMaxFailedRequestsPerPeriod .= '<option value="'.$failedRequests.'"'.$selected.'>'.$failedRequests.' ';
            $selectAPIMaxFailedRequestsPerPeriod .= mb_strtolower($this->lang->getText($failedRequests == 1 ? 'NRS_ADMIN_REQUEST_TEXT' : 'NRS_ADMIN_REQUESTS_TEXT'), 'UTF-8').' / ';
            $selectAPIMaxFailedRequestsPerPeriod .= $this->lang->getText('NRS_PER_HOUR_TEXT').' / ';
            $selectAPIMaxFailedRequestsPerPeriod .=  $this->lang->getText('NRS_ADMIN_IP_TEXT').'</option>';
        }
        $globalSettingSelectDropDowns['select_api_max_failed_requests_per_period'] = $selectAPIMaxFailedRequestsPerPeriod;


        // Show prices with VAT
        if($this->getConfig('show_prices_with_vat') == 1)
        {
            $select_show_prices_with_vat  = '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_show_prices_with_vat .= '<option value="0">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        } else
        {
            $select_show_prices_with_vat  = '<option value="1">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>' . "\n";
            $select_show_prices_with_vat .= '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>' . "\n";
        }
        $globalSettingSelectDropDowns['select_show_prices_with_vat'] = $select_show_prices_with_vat;


        // MIN BOOKING PERIOD
        $select_minimum_booking_period = "";
        for($minutes = 0; $minutes < 60; $minutes = $minutes+15)
        {
            $inSeconds = $minutes*60;
            if($this->getConfig('minimum_booking_period') == $inSeconds)
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            } else
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            }
        }
        for($hours = 1; $hours <= 23; $hours++)
        {
            $inSeconds = $hours*3600;
            if($this->getConfig('minimum_booking_period') == $inSeconds)
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>'."\n";
            } else
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>'."\n";
            }
        }
        for($days = 1; $days <= 90; $days++)
        {
            $inSeconds = $days*86400;
            if($this->getConfig('minimum_booking_period') == $inSeconds)
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>'."\n";
            } else
            {
                $select_minimum_booking_period .= '<option value="'.$inSeconds.'">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>'."\n";
            }
        }
        $globalSettingSelectDropDowns['select_minimum_booking_period'] = $select_minimum_booking_period;


        // MAX BOOKING PERIOD
        $select_maximum_booking_period = "";
        for($minutes = 0; $minutes < 60; $minutes = $minutes+15)
        {
            $inSeconds = $minutes*60;
            if($this->getConfig('maximum_booking_period') == $inSeconds)
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            } else
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'">'.$minutes.' '.$this->lang->getText('NRS_MINUTES_TEXT').'</option>';
            }
        }
        for($hours = 1; $hours <= 23; $hours++)
        {
            $inSeconds = $hours*3600;
            if($this->getConfig('maximum_booking_period') == $inSeconds)
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>'."\n";
            } else
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'">'.$hours.' '.$this->lang->getText('NRS_HOURS_TEXT').'</option>'."\n";
            }
        }
        for($days = 1; $days <= 365; $days++)
        {
            $inSeconds = $days*86400;
            if($this->getConfig('maximum_booking_period') == $inSeconds)
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'" selected="selected">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>'."\n";
            } else
            {
                $select_maximum_booking_period .= '<option value="'.$inSeconds.'">'.$days.' '.$this->lang->getText('NRS_DAYS_TEXT').'</option>'."\n";
            }
        }
        $globalSettingSelectDropDowns['select_maximum_booking_period'] = $select_maximum_booking_period;


        // NOON TIME
        $select_noon_time = '';
        for($hour = 10; $hour < 17; $hour++)
        {
            for($min = 0; $min < 60; $min = $min+30)
            {
                $currentHour = sprintf("%02d", $hour);
                $currentMin = sprintf("%02d", $min);

                $currentTime = $currentHour.':'.$currentMin.':00';
                $selected = ($currentTime == $this->getConfig('noon_time') ? ' selected="selected"' : '');

                $UTCUnixTime = strtotime(date("Y-m-d")." ".$currentTime);
                $printCurrentTime = date_i18n(get_option('time_format'), $UTCUnixTime, TRUE);

                $select_noon_time .= '<option value="'.$currentTime.'"'.$selected.'>'.$printCurrentTime.'</option>';
            }
        }
        $globalSettingSelectDropDowns['select_noon_time'] = $select_noon_time;


        if($this->getConfig('classify_items') == 0)
        {
            $selectClassifyItems  = '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>'."\n";
            $selectClassifyItems .= '<option value="1">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>'."\n";
        } else
        {
            $selectClassifyItems  = '<option value="0">'.$this->lang->getText('NRS_ADMIN_NO_TEXT').'</option>'."\n";
            $selectClassifyItems .= '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_YES_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_classify_items'] = $selectClassifyItems;


        if($this->getConfig('search_enabled') == 1)
        {
            $selectSearchEnabled  = '<option value="0">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectSearchEnabled .= '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        } else
        {
            $selectSearchEnabled  = '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectSearchEnabled .= '<option value="1">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_search_enabled'] = $selectSearchEnabled;


        if($this->getConfig('booking_model') == 1)
        {
            $selectBookingModel   = '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectBookingModel  .= '<option value="2">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        } else
        {
            $selectBookingModel  = '<option value="1">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectBookingModel .= '<option value="2" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_booking_model'] = $selectBookingModel;


        if($this->getConfig('deposit_enabled') == 0)
        {
            $selectDepositEnabled  = '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectDepositEnabled .= '<option value="1">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        } else
        {
            $selectDepositEnabled  = '<option value="0">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectDepositEnabled .= '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_deposit_enabled'] = $selectDepositEnabled;


        if($this->getConfig('discount_enabled') == 0)
        {
            $selectDiscountEnabled  = '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectDiscountEnabled .= '<option value="1">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        } else
        {
            $selectDiscountEnabled  = '<option value="0">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectDiscountEnabled .= '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_discount_enabled'] = $selectDiscountEnabled;


        if($this->getConfig('prepayment_enabled') == 0)
        {
            $selectPrepaymentEnabled  = '<option value="0" selected="selected">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectPrepaymentEnabled .= '<option value="1">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        } else
        {
            $selectPrepaymentEnabled  = '<option value="0">'.$this->lang->getText('NRS_ADMIN_DISABLED_TEXT').'</option>'."\n";
            $selectPrepaymentEnabled .= '<option value="1" selected="selected">'.$this->lang->getText('NRS_ADMIN_ENABLED_TEXT').'</option>'."\n";
        }
        $globalSettingSelectDropDowns['select_prepayment_enabled'] = $selectPrepaymentEnabled;


        return $globalSettingSelectDropDowns;
    }


    public function updateGlobalSettings()
    {
        $this->updateSetting('conf_portal_name', sanitize_text_field($_POST['conf_portal_name']));
        $this->updateSetting('conf_portal_street_address', sanitize_text_field($_POST['conf_portal_street_address']));
        $this->updateSetting('conf_portal_city', sanitize_text_field($_POST['conf_portal_city']));
        $this->updateSetting('conf_portal_state', sanitize_text_field($_POST['conf_portal_state']));
        $this->updateSetting('conf_portal_country', sanitize_text_field($_POST['conf_portal_country']));
        $this->updateSetting('conf_portal_zipcode', sanitize_text_field($_POST['conf_portal_zipcode']));
        $this->updateSetting('conf_portal_phone', sanitize_text_field($_POST['conf_portal_phone']));
        $this->updateSetting('conf_portal_email', sanitize_email($_POST['conf_portal_email']));
        $this->updateSetting('conf_send_emails', $_POST['conf_send_emails'] == 1 ? 1 : 0);
        $this->updateSetting('conf_recaptcha_site_key', sanitize_text_field($_POST['conf_recaptcha_site_key']));
        $this->updateSetting('conf_recaptcha_secret_key', sanitize_text_field($_POST['conf_recaptcha_secret_key']));
        $this->updateSetting('conf_recaptcha_enabled', $_POST['conf_recaptcha_enabled'] == 1 ? 1 : 0);
        $this->updateSetting('conf_api_max_requests_per_period', NRSValidator::getValidInteger($_POST['conf_api_max_requests_per_period'], 0));
        $this->updateSetting('conf_api_max_failed_requests_per_period', NRSValidator::getValidInteger($_POST['conf_api_max_failed_requests_per_period'], 0));
        $this->updateSetting('conf_cancelled_payment_page_id',  NRSValidator::getValidInteger($_POST['conf_cancelled_payment_page_id'], 0));
        $this->updateSetting('conf_confirmation_page_id',  NRSValidator::getValidInteger($_POST['conf_confirmation_page_id'], 0));
        $this->updateSetting('conf_tos_page_id',  NRSValidator::getValidInteger($_POST['conf_tos_page_id'], 0));
        $this->updateSetting('conf_system_style',  NRSValidator::getValidInteger($_POST['conf_system_style'], 1));
        $this->updateSetting('conf_short_date_format', sanitize_text_field($_POST['conf_short_date_format']));
        $this->updateSetting('conf_currency_symbol',  htmlentities(sanitize_text_field($_POST['conf_currency_symbol']),ENT_COMPAT,'utf-8'));
        $this->updateSetting('conf_currency_code', sanitize_text_field($_POST['conf_currency_code']));
        $this->updateSetting('conf_measurement_unit', sanitize_text_field($_POST['conf_measurement_unit']));
        $this->updateSetting('conf_vat_percentage', floatval($_POST['conf_vat_percentage']));
        $this->updateSetting('conf_show_prices_with_vat', $_POST['conf_show_prices_with_vat'] == 1 ? 1 : 0);
        $this->updateSetting('conf_minimum_booking_period', NRSValidator::getValidInteger($_POST['conf_minimum_booking_period'], 0));
        $this->updateSetting('conf_maximum_booking_period', NRSValidator::getValidInteger($_POST['conf_maximum_booking_period'], 0));
        $this->updateSetting('conf_paypal_sandbox_mode', $_POST['conf_paypal_sandbox_mode'] == 1 ? 1 : 0);
        $this->updateSetting('conf_search_enabled', $_POST['conf_search_enabled'] == 1 ? 1 : 0);
        $this->updateSetting('conf_minimum_block_period_between_bookings', NRSValidator::getValidInteger($_POST['conf_minimum_block_period_between_bookings'], 0));
        $this->updateSetting('conf_minimum_period_until_pickup', NRSValidator::getValidInteger($_POST['conf_minimum_period_until_pickup'], 0));
        $this->updateSetting('conf_noon_time', NRSValidator::getValidISOTime($_POST['conf_noon_time'], 'H:i:s'));
        $this->updateSetting('conf_booking_model', $_POST['conf_booking_model'] == 2 ? 2 : 1);
        $this->updateSetting('conf_classify_items', $_POST['conf_classify_items'] == 1 ? 1 : 0);
        $this->updateSetting('conf_deposit_enabled', $_POST['conf_deposit_enabled'] == 1 ? 1 : 0);
        $this->updateSetting('conf_discount_enabled', $_POST['conf_discount_enabled'] == 1 ? 1 : 0);
        $this->updateSetting('conf_prepayment_enabled', $_POST['conf_prepayment_enabled'] == 1 ? 1 : 0);

        $_SESSION['ok_msg'] = 'Completed: Global settings updated successfully!';
    }

    public function updateCustomerSettings()
    {
        $this->updateSetting('conf_customer_title_visible', isset($_POST['conf_customer_title_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_title_required', isset($_POST['conf_customer_title_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_first_name_visible', isset($_POST['conf_customer_first_name_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_first_name_required', isset($_POST['conf_customer_first_name_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_last_name_visible', isset($_POST['conf_customer_last_name_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_last_name_required', isset($_POST['conf_customer_last_name_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_birthdate_visible', isset($_POST['conf_customer_birthdate_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_birthdate_required', isset($_POST['conf_customer_birthdate_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_street_address_visible', isset($_POST['conf_customer_street_address_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_street_address_required', isset($_POST['conf_customer_street_address_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_city_visible', isset($_POST['conf_customer_city_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_city_required', isset($_POST['conf_customer_city_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_state_visible', isset($_POST['conf_customer_state_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_state_required', isset($_POST['conf_customer_state_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_zipcode_visible', isset($_POST['conf_customer_zipcode_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_zipcode_required', isset($_POST['conf_customer_zipcode_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_country_visible', isset($_POST['conf_customer_country_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_country_required', isset($_POST['conf_customer_country_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_phone_visible', isset($_POST['conf_customer_phone_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_phone_required', isset($_POST['conf_customer_phone_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_email_visible', isset($_POST['conf_customer_email_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_email_required', isset($_POST['conf_customer_email_required']) ? 1 : 0);
        $this->updateSetting('conf_customer_additional_comments_visible', isset($_POST['conf_customer_additional_comments_visible']) ? 1 : 0);
        $this->updateSetting('conf_customer_additional_comments_required', isset($_POST['conf_customer_additional_comments_required']) ? 1 : 0);

        $_SESSION['ok_msg'] = 'Completed: Customer settings updated successfully!';
    }

    public function updateSearchSettings()
    {
        $this->updateSetting('conf_search_pickup_location_visible', isset($_POST['conf_search_pickup_location_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_pickup_location_required', isset($_POST['conf_search_pickup_location_required']) ? 1 : 0);
        $this->updateSetting('conf_search_pickup_date_visible', isset($_POST['conf_search_pickup_date_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_pickup_date_required', isset($_POST['conf_search_pickup_date_required']) ? 1 : 0);
        $this->updateSetting('conf_search_dropoff_location_visible', isset($_POST['conf_search_dropoff_location_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_dropoff_location_required', isset($_POST['conf_search_dropoff_location_required']) ? 1 : 0);
        $this->updateSetting('conf_search_dropoff_date_visible', isset($_POST['conf_search_dropoff_date_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_dropoff_date_required', isset($_POST['conf_search_dropoff_date_required']) ? 1 : 0);
        $this->updateSetting('conf_search_body_type_visible', isset($_POST['conf_search_body_type_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_body_type_required', isset($_POST['conf_search_body_type_required']) ? 1 : 0);
        $this->updateSetting('conf_search_transmission_type_visible', isset($_POST['conf_search_transmission_type_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_transmission_type_required', isset($_POST['conf_search_transmission_type_required']) ? 1 : 0);
        $this->updateSetting('conf_search_fuel_type_visible', isset($_POST['conf_search_fuel_type_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_fuel_type_required', isset($_POST['conf_search_fuel_type_required']) ? 1 : 0);
        $this->updateSetting('conf_search_booking_code_visible', isset($_POST['conf_search_booking_code_visible']) ? 1 : 0);
        $this->updateSetting('conf_search_booking_code_required', isset($_POST['conf_search_booking_code_required']) ? 1 : 0);

        $_SESSION['ok_msg'] = 'Completed: Search settings updated successfully!';
    }

    public function updatePriceCalculationSettings()
    {
        $this->updateSetting('conf_price_calculation_type', intval($_POST['price_calculation_model']));

        $_SESSION['ok_msg'] = 'Completed: Price settings updated successfully!';
    }

    private function updateSetting($key, $value)
    {
        $this->internalWPDB->query("
            UPDATE {$this->nrsPrefix}settings
            SET conf_value='".sanitize_text_field($value)."'
            WHERE conf_key='".sanitize_text_field($key)."'
        ");
    }

    /**
     * @param int $val
     * @param string $type - "YES/NO" (DEFAULT), "SHOW/HIDE", "ENABLED/DISABLED"
     * @return string
     */
    public function generateOption($val = 0, $type = "YES/NO")
    {
        $htmlOption = '';
        if($type == "SHOW/HIDE")
        {
            $arr = array(
                1 => $this->lang->getText('NRS_ADMIN_VISIBLE_TEXT'),
                0 => $this->lang->getText('NRS_ADMIN_HIDDEN_TEXT'),
            );
        } else if($type == "ENABLED/DISABLED")
        {
            $arr = array(
                1 => $this->lang->getText('NRS_ADMIN_ENABLED_TEXT'),
                0 => $this->lang->getText('NRS_ADMIN_DISABLED_TEXT'),
            );
        } else
        {
            $arr = array(
                1 => $this->lang->getText('NRS_ADMIN_YES_TEXT'),
                0 => $this->lang->getText('NRS_ADMIN_NO_TEXT'),
            );
        }

        foreach($arr as $key => $value)
        {
            if($val == $key)
            {
                $htmlOption .= '<option value="'.$key.'" selected="selected">'.$value.'</option>';
            } else
            {
                $htmlOption .= '<option value="'.$key.'">'.$value.'</option>';
            }
        }
        return $htmlOption;
    }

    public function getBookingExpirationTimeDropDown($fieldName, $fieldId, $selectedPeriod, $minPeriod, $maxPeriod)
    {
        $sanitizedFieldName = sanitize_text_field($fieldName);
        $sanitizedFieldId = sanitize_text_field($fieldId);
        // LIVE BOOKING TIMEOUT
        $expirationPeriods = array(
            '0' => $this->lang->getText('NRS_ADMIN_NEVER_TEXT'),
            '120' => '2 '.$this->lang->getText('NRS_MINUTES_TEXT'),
            '300' => '5 '.$this->lang->getText('NRS_MINUTES_TEXT'),
            '600' => '10 '.$this->lang->getText('NRS_MINUTES2_TEXT'),
            '1200' => '20 '.$this->lang->getText('NRS_MINUTES2_TEXT'),
            '1800' => '30 '.$this->lang->getText('NRS_MINUTES2_TEXT'),
            '2700' => '45 '.$this->lang->getText('NRS_MINUTES_TEXT'),
            '3600' => '1 '.$this->lang->getText('NRS_HOUR_TEXT'),
            '7200' => '2 '.$this->lang->getText('NRS_HOUR_TEXT'),
            '10800' => '3 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '14400' => '4 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '18000' => '5 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '21600' => '5 '.$this->lang->getText('NRS_HOURS_TEXT'),
            '43200' => '12 '.$this->lang->getText('NRS_HOURS2_TEXT'),
            '86400' => '1 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '172800' => '2 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '259200' => '3 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '345600' => '4 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '432000' => '5 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '518400' => '6 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '604800' => '7 '.$this->lang->getText('NRS_DAYS_TEXT'),
            '2592000' => '30 '.$this->lang->getText('NRS_DAYS2_TEXT'),
            '5184000' => '60 '.$this->lang->getText('NRS_DAYS2_TEXT'),
            '7776000' => '90 '.$this->lang->getText('NRS_DAYS2_TEXT'),
        );
        $html = '';

        $html .= '<select name="'.$sanitizedFieldName.'" id="'.$sanitizedFieldId.'">';
        foreach($expirationPeriods as $periodInSeconds => $periodTitle)
        {
            if(
                ($periodInSeconds >= $minPeriod || $periodInSeconds == 0)
                && $periodInSeconds <= $maxPeriod
            ){
                $selected = $periodInSeconds == $selectedPeriod ? ' selected="selected"' : '';
                $html .= '<option value="'.$periodInSeconds.'"'.$selected.'>'.$periodTitle.'</option>'."\n";
            }
        }
        $html .= '</select>';

        return $html;
    }

    public function getPaymentSettings()
    {
        $paymentSettings = array();

        $payPalPaymentData = $this->internalWPDB->get_row("
            SELECT * FROM {$this->nrsPrefix}payment_methods WHERE payment_method_code='pp'
        ", ARRAY_A);
        $bankPaymentData = $this->internalWPDB->get_row("
            SELECT * FROM {$this->nrsPrefix}payment_methods WHERE payment_method_code='bank'
        ", ARRAY_A);
        $phonePaymentData = $this->internalWPDB->get_row("
            SELECT * FROM {$this->nrsPrefix}payment_methods WHERE payment_method_code='phone'
        ", ARRAY_A);
        $payOnArrivalPaymentData = $this->internalWPDB->get_row("
            SELECT * FROM {$this->nrsPrefix}payment_methods WHERE payment_method_code='poa'
        ", ARRAY_A);

        $paymentSettings['paypal_payment_method_enabled'] = $payPalPaymentData['method_enabled'];
        $paymentSettings['bank_payment_method_enabled'] = $bankPaymentData['method_enabled'];
        $paymentSettings['pay_over_the_phone_payment_method_enabled'] = $phonePaymentData['method_enabled'];
        $paymentSettings['pay_on_arrival_payment_method_enabled'] = $payOnArrivalPaymentData['method_enabled'];

        $paymentSettings['paypal_payment_title'] = $payPalPaymentData['method_name'];
        $paymentSettings['bank_payment_title'] = $bankPaymentData['method_name'];
        $paymentSettings['pay_over_the_phone_payment_title'] = $phonePaymentData['method_name'];
        $paymentSettings['pay_on_arrival_payment_title'] = $payOnArrivalPaymentData['method_name'];

        $paymentSettings['paypal_email'] = $payPalPaymentData['method_details'];
        $paymentSettings['bank_details'] = $bankPaymentData['method_details'];
        $paymentSettings['phone_number'] = $phonePaymentData['method_details'];
        $paymentSettings['local_payment_details'] = $payOnArrivalPaymentData['method_details'];

        $paymentSettings['paypal_unpaid_booking_expiration_time'] = $payPalPaymentData['unpaid_booking_expiration_time'];
        $paymentSettings['bank_unpaid_booking_expiration_time'] = $bankPaymentData['unpaid_booking_expiration_time'];
        $paymentSettings['phone_unpaid_booking_expiration_time'] = $phonePaymentData['unpaid_booking_expiration_time'];
        $paymentSettings['pay_on_arrival_unpaid_booking_expiration_time'] = $payOnArrivalPaymentData['unpaid_booking_expiration_time'];

        return $paymentSettings;
    }

    public function updatePaymentSettings()
    {
        $validPayPalPaymentMethodEnabled = isset($_POST['paypal_payment_method_enabled']) ? 1 : 0;
        $validBankPaymentMethodEnabled = isset($_POST['bank_payment_method_enabled']) ? 1 : 0;
        $validPayOverThePhonePaymentMethodEnabled = isset($_POST['pay_over_the_phone_payment_method_enabled']) ? 1 : 0;
        $validPayOnArrivalPaymentMethodEnabled = isset($_POST['pay_on_arrival_payment_method_enabled']) ? 1 : 0;

        $sanitizedPayPalPaymentTitle = sanitize_text_field($_POST['paypal_payment_title']);
        $sanitizedBankPaymentTitle = sanitize_text_field($_POST['bank_payment_title']);
        $sanitizedPayOverThePhonePaymentTitle = sanitize_text_field($_POST['pay_over_the_phone_payment_title']);
        $sanitizedPayOnArrivalPaymentTitle = sanitize_text_field($_POST['pay_on_arrival_payment_title']);

        $sanitizedPayPalEmail = sanitize_text_field($_POST['paypal_email']);
        $sanitizedBankDetails = implode( "\n", array_map( 'sanitize_text_field', explode( "\n", $_POST['bank_details'] ) ) );
        $sanitizedPhoneNumber = sanitize_text_field($_POST['phone_number']);
        $sanitizedLocalPaymentDetails = implode( "\n", array_map( 'sanitize_text_field', explode( "\n", $_POST['local_payment_details'] ) ) );

        $validPayPalUnpaidBookingExpirationTime = NRSValidator::getValidInteger($_POST['paypal_unpaid_booking_expiration_time']);
        $validBankUnpaidBookingExpirationTime = NRSValidator::getValidInteger($_POST['bank_unpaid_booking_expiration_time']);
        $validPayOverThePhoneUnpaidBookingExpirationTime = NRSValidator::getValidInteger($_POST['phone_unpaid_booking_expiration_time']);
        $validPayOnArrivalUnpaidBookingExpirationTime = NRSValidator::getValidInteger($_POST['pay_on_arrival_unpaid_booking_expiration_time']);

        $this->internalWPDB->query("
            UPDATE {$this->nrsPrefix}payment_methods SET
            method_name='{$sanitizedPayPalPaymentTitle}', method_details='{$sanitizedPayPalEmail}',
            method_enabled='{$validPayPalPaymentMethodEnabled}',
            unpaid_booking_expiration_time='{$validPayPalUnpaidBookingExpirationTime}'
            WHERE payment_method_code='pp'
        ");
        $this->internalWPDB->query("
            UPDATE {$this->nrsPrefix}payment_methods SET
            method_name='{$sanitizedBankPaymentTitle}', method_details='{$sanitizedBankDetails}',
            method_enabled='{$validBankPaymentMethodEnabled}',
            unpaid_booking_expiration_time='{$validBankUnpaidBookingExpirationTime}'
            WHERE payment_method_code='bank'
        ");
        $this->internalWPDB->query("
            UPDATE {$this->nrsPrefix}payment_methods SET
            method_name='{$sanitizedPayOverThePhonePaymentTitle}', method_details='{$sanitizedPhoneNumber}',
            method_enabled='{$validPayOverThePhonePaymentMethodEnabled}',
            unpaid_booking_expiration_time='{$validPayOverThePhoneUnpaidBookingExpirationTime}'
            WHERE payment_method_code='phone'
        ");
        $this->internalWPDB->query("
            UPDATE {$this->nrsPrefix}payment_methods SET
            method_name='{$sanitizedPayOnArrivalPaymentTitle}', method_details='{$sanitizedLocalPaymentDetails}',
            method_enabled='{$validPayOnArrivalPaymentMethodEnabled}',
            unpaid_booking_expiration_time='{$validPayOnArrivalUnpaidBookingExpirationTime}'
            WHERE payment_method_code='poa'
        ");

        $_SESSION['ok_msg'] = 'Completed: Payment settings updated successfully!';
    }

    public function getPriceCalculationOptionsHTML()
    {
        //date format start
        $price_array = array(
            1 => $this->lang->getText('NRS_ADMIN_DAILY_TEXT'),
            2 => $this->lang->getText('NRS_ADMIN_HOURLY_TEXT'),
            3 => $this->lang->getText('NRS_ADMIN_COMBINED_TEXT'),
        );

        $priceCalculationOptionsHTML = "";
        foreach($price_array as $key => $value)
        {
            if($key == $this->getConfig('price_calculation_type'))
            {
                $priceCalculationOptionsHTML .= '<option value="'.$key.'" selected="selected">'.$value.'</option>';
            } else
            {
                $priceCalculationOptionsHTML .= '<option value="'.$key.'" >'.$value.'</option>';
            }
        }
        return $priceCalculationOptionsHTML;
    }
}