<?php
/**
 * NRS API logger
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSAPILog {
    // For security reasons we do not allow to change these values from admin
    const LOG_EXPIRATION_PERIOD = 2592000; // 30 Days
    const REQUEST_EXPIRATION_PERIOD = 3600; // 1 hour

    protected $internalWPDB = NULL;
    protected $nrsPrefix 	= "";
    protected $lang 		= NULL;
    protected $debugMode    = 0;

    protected $maxRequestsPerPeriod = 50;
    protected $maxFailedRequestsPerPeriod = 3;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set max requests per period
        $this->setMaxRequestsPerPeriod();

        // Set max failed requests per period
        $this->setMaxFailedRequestsPerPeriod();

        // Delete expired logs
        $this->deleteExpiredLogs();
    }

    /**
     * Set max requests per period - called in class constructor
     */
    private function setMaxRequestsPerPeriod()
    {
        $sql = "
				SELECT conf_value AS request_limit
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_api_max_requests_per_period'
			";
        $maxRequestsPerPeriod = $this->internalWPDB->get_var($sql);

        if(!is_null($maxRequestsPerPeriod))
        {
            $this->maxRequestsPerPeriod = NRSValidator::getValidInteger($maxRequestsPerPeriod, 50);
        } else
        {
            $this->maxRequestsPerPeriod = 50;
        }
    }

    /**
     * Set max failed requests per period - called in class constructor
     */
    private function setMaxFailedRequestsPerPeriod()
    {
        $sql = "
				SELECT conf_value AS request_limit
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_api_max_failed_requests_per_period'
			";
        $maxFailedRequestsPerPeriod = $this->internalWPDB->get_var($sql);

        if(!is_null($maxFailedRequestsPerPeriod))
        {
            $this->maxFailedRequestsPerPeriod = NRSValidator::getValidInteger($maxFailedRequestsPerPeriod, 3);
        } else
        {
            $this->maxFailedRequestsPerPeriod = 3;
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getTotalRequestsLeft()
    {
        $totalRequestsLeft = -1;
        if($this->maxRequestsPerPeriod > -1)
        {
            $sanitizedIP = sanitize_text_field($_SERVER['REMOTE_ADDR']);
            $sanitizedRealIP = sanitize_text_field(NRSSecurity::getRealIP());
            $timestampLimit = time() - self::REQUEST_EXPIRATION_PERIOD;

            $totalRequests = $this->internalWPDB->get_var("
                SELECT COUNT(*) AS total_requests
                FROM {$this->nrsPrefix}api_log
                WHERE log_timestamp >= '{$timestampLimit}'
                AND (ip='{$sanitizedIP}' OR ip='{$sanitizedRealIP}' OR real_ip='{$sanitizedIP}')
            ");

            $totalRequestsLeft = !is_null($totalRequests) ? max($this->maxRequestsPerPeriod - $totalRequests, 0) : $this->maxRequestsPerPeriod;
        }

        return $totalRequestsLeft;
    }

    public function getFailedRequestsLeft()
    {
        $failedRequestsLeft = -1;
        if($this->maxFailedRequestsPerPeriod > -1)
        {
            $sanitizedIP = sanitize_text_field($_SERVER['REMOTE_ADDR']);
            $sanitizedRealIP = sanitize_text_field(NRSSecurity::getRealIP());
            $timestampLimit = time() - self::REQUEST_EXPIRATION_PERIOD;

            $failedRequests = $this->internalWPDB->get_var("
                SELECT COUNT(*) AS total_requests
                FROM {$this->nrsPrefix}api_log
                WHERE status = '1' AND log_timestamp >= '{$timestampLimit}'
                AND (ip='{$sanitizedIP}' OR ip='{$sanitizedRealIP}' OR real_ip='{$sanitizedIP}')
            ");

            $failedRequestsLeft = !is_null($failedRequests) ? max($this->maxFailedRequestsPerPeriod - $failedRequests, 0) : $this->maxFailedRequestsPerPeriod;
        }

        return $failedRequestsLeft;
    }

    public function getFailedEmailAttemptsLeft($paramEmail)
    {
        $failedAttemptsLeft = -1;
        if($this->maxFailedRequestsPerPeriod > -1)
        {
            $validEmail = sanitize_email($paramEmail);
            $timestampLimit = time() - self::REQUEST_EXPIRATION_PERIOD;

            $failedAttempts = $this->internalWPDB->get_var("
                SELECT COUNT(*) AS total_attemps
                FROM {$this->nrsPrefix}api_log
                WHERE status = '1' AND log_timestamp >= '{$timestampLimit}'
                AND email='{$validEmail}'
            ");

            $failedAttemptsLeft = !is_null($failedAttempts) ? max($this->maxFailedRequestsPerPeriod - $failedAttempts, 0) : $this->maxFailedRequestsPerPeriod;
        }

        return $failedAttemptsLeft;
    }

    public function saveLog($paramEmail, $paramYear = "0000", $paramYearRequired = FALSE, $paramFailed = FALSE)
    {
        $sanitizedEmail = sanitize_email($paramEmail);
        $validYear = NRSValidator::getValidInteger($paramYear);
        $validYearRequired = $paramYearRequired ? 1 : 0;

        $sanitizedIP = sanitize_text_field($_SERVER['REMOTE_ADDR']);
        $sanitizedRealIP = sanitize_text_field(NRSSecurity::getRealIP());
        $sanitizedHost = sanitize_text_field(gethostbyaddr($_SERVER['REMOTE_ADDR']));
        $sanitizedAgent = sanitize_text_field(NRSSecurity::getAgent());
        $sanitizedBrowser = sanitize_text_field(NRSSecurity::getBrowser($sanitizedAgent));
        $sanitizedOS = sanitize_text_field( NRSSecurity::getOS($sanitizedAgent));
        $validIsRobot = NRSSecurity::isRobot($sanitizedAgent) ? 1 : 0;

        $totalRequestsLeft = $this->getTotalRequestsLeft();
        $failedRequestsLeft = $this->getFailedRequestsLeft();
        $emailAttemptsLeft = $this->getFailedEmailAttemptsLeft($sanitizedEmail);

        $validStatus = 0; // 0 - BLOCKED, 1 - FAILED, 2 - PASSED
        if($failedRequestsLeft > 0 && $totalRequestsLeft > 0 && $emailAttemptsLeft > 0)
        {
            $validStatus = $paramFailed ? 1 : 2;
        }

        /* insert the invoice data in {$this->nrsPrefix}invoice table */
        $insertSQL = "
            INSERT INTO {$this->nrsPrefix}api_log
            (
                email, year, year_required,
                ip, real_ip, host,
                agent, browser, os,
                total_requests_left, failed_requests_left, email_attempts_left,
                is_robot, status, log_timestamp
            ) VALUES
            (
                '{$sanitizedEmail}', '{$validYear}', '{$validYearRequired}',
                '{$sanitizedIP}', '{$sanitizedRealIP}', '{$sanitizedHost}',
                '{$sanitizedAgent}', '{$sanitizedBrowser}', '{$sanitizedOS}',
                '{$totalRequestsLeft}', '{$failedRequestsLeft}', '{$emailAttemptsLeft}',
                '{$validIsRobot}', '{$validStatus}', '".time()."'
            )
        ";

        // Debug
        //echo esc_html($insertSQL); die();
        $this->internalWPDB->query($insertSQL);
    }

    public function getAdminLogList()
    {
        $html = '';

        // Show logs not older than 30 days
        $validTimestampLimit = time() - self::LOG_EXPIRATION_PERIOD;

        $rows = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}api_log
            WHERE log_timestamp >= '{$validTimestampLimit}'
            ORDER BY log_timestamp DESC
        ", ARRAY_A);

        foreach ($rows AS $row)
        {
            $logTMZStamp = $row['log_timestamp'] + get_option( 'gmt_offset' ) * 3600;
            $logDateTime = date_i18n("Y-m-d H:i", $logTMZStamp, TRUE);

            $printYear = $row['year'] == 0 ? '0000' : $row['year'];

            $printYearRequired = "";
            if($row['year_required'])
            {
                $printYearRequired = '<br />('.$this->lang->getText('NRS_REQUIRED_TEXT').')';
            }

            if($row['is_robot'] == 1)
            {
                $printIsRobot = $this->lang->getText('NRS_ADMIN_YES_TEXT');
            } else
            {
                $printIsRobot = $this->lang->getText('NRS_ADMIN_NO_TEXT');
            }

            if($row['status'] == 1)
            {
                $printStatus = '<span style="color: black;">'.$this->lang->getText('NRS_ADMIN_FAILED_TEXT').'</span>';
            } else if($row['status'] == 2)
            {
                $printStatus = '<span style="color: green;">'.$this->lang->getText('NRS_ADMIN_ALLOWED_TEXT').'</span>';
            } else
            {
                $printStatus = '<span style="color: red;">'.$this->lang->getText('NRS_ADMIN_BLOCKED_TEXT').'</span>';
            }

            $html .= '<tr>';
            $html .= '<td>'.$logDateTime.'</td>';
            $html .= '<td nowrap="nowrap">'.$row['email'].'</td>';
            $html .= '<td>'.$printYear.$printYearRequired.'</td>';
            $html .= '<td><span title="'.$row['host'].'">'.$row['ip'].'<br />('.$row['real_ip'].')</span></td>';
            $html .= '<td>'.$row['browser'].'</td>';
            $html .= '<td>'.$row['os'].'</td>';
            $html .= '<td>'.$printIsRobot.'</td>';
            $html .= '<td style="text-align: right">';
            $html .= '<span title="Total requests left: '.$row['total_requests_left'];
            $html .= ', failed requests left: '.$row['failed_requests_left'].', e-mail attempts left: '.$row['email_attempts_left'].'">';
            $html .= $printStatus.'</span></td>';
            $html .= '</tr>';
        }
        return $html;
    }

    private function deleteExpiredLogs()
    {
        // Delete logs older than 30 days
        $deleteOlderThan = time() - self::LOG_EXPIRATION_PERIOD;

        $deleteSQL = "
            DELETE FROM {$this->nrsPrefix}api_log
            WHERE log_timestamp < '{$deleteOlderThan}'
        ";

        // Debug
        //echo esc_html($deleteSQL); die();
        $this->internalWPDB->query($deleteSQL);
    }
}