<?php
/**
 * Booking processor
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSBooking
{
    protected $internalWPDB	 	= NULL;
    protected $nrsPrefix 		= "";
    protected $lang 			= NULL;
    protected $debugMode 	 	= 0;
    protected $bookingId        = 0;
    protected $nrsMail           = NULL;

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - LANGUAGE
     * @param NRSMail $paramNRSMail - NRS Mailer, default is NULL (if not used)
     * @param $paramBookingCode
     * @param $paramBookingId = optional to set the id instantly
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, NRSMail $paramNRSMail = NULL, $paramBookingCode = "", $paramBookingId = 0)
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set the mailer
        $this->nrsMail = $paramNRSMail;

        // Set booking id
        if($paramBookingCode != "")
        {
            $this->setBookingIdByCode($paramBookingCode);
        } else
        {
            $this->bookingId = NRSValidator::getValidInteger($paramBookingId, 0);
        }
    }

    /**
     * Sets the booking id for class. Used only from class constructor
     * @param string $paramBookingCode
     * @return int
     */
    private function setBookingIdByCode($paramBookingCode)
    {
        $sanitizedBookingCode = sanitize_text_field($paramBookingCode);
        $bookingId = 0;
        if(strlen($sanitizedBookingCode) > 0)
        {
            $bookingData = $this->internalWPDB->get_row("
				SELECT booking_id
				FROM {$this->nrsPrefix}bookings
				WHERE booking_code='{$sanitizedBookingCode}'
			", ARRAY_A);
            if(!is_null($bookingData))
            {
                $this->bookingId = $bookingData['booking_id'];
            }
        }

        return $bookingId;
    }



    /**
     * If will get next insert id of next booking id by running SQL, not via param
     * @return string
     */
    private function generateBookingCode()
    {
        $validNextMySQLInsertId = 1;
        $row = $this->internalWPDB->get_row("SHOW TABLE STATUS LIKE '{$this->nrsPrefix}bookings'", ARRAY_A);
        if(!is_null($row))
        {
            $validNextMySQLInsertId = $row['Auto_increment']; // This is current max it+1, capital first letter is ok - it is there like that
        }
        $newBookingCode = "R".$validNextMySQLInsertId."A".$this->getIncrementalHash(5);

        return $newBookingCode;
    }

    private function getIncrementalHash($length = 5)
    {
        //$charset = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
        $charset = "ABCDEFGHIJKLMNPRSTUVYZ"; // fits LT & EN, O is skipped to similarity to Zero
        $charsetLength = strlen($charset);
        $result = '';

        $microTimesList = explode(' ', microtime());
        $now = $microTimesList[1];
        while ($now >= $charsetLength)
        {
            $i = $now % $charsetLength;
            $result = $charset[$i] . $result;
            $now /= $charsetLength;
        }
        return substr($result, -$length);
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }


    public function getBookingId()
    {
        return $this->bookingId;
    }

    public function getBookingCode()
    {
        $bookingCode = "";
        $bookingDetails = $this->getBookingDetails();
        if(!is_null($bookingDetails))
        {
            $bookingCode = $bookingDetails['booking_code'];
        }

        return $bookingCode;
    }

    public function isValidBooking()
    {
        $validBookingId = intval($this->bookingId);
        $bookingDataRow = $this->internalWPDB->get_row("
          SELECT booking_id
          FROM {$this->nrsPrefix}bookings
          WHERE booking_id='{$validBookingId}'
          AND pickup_timestamp > ".time()." AND is_cancelled='0'
          AND (payment_success='0' OR payment_success='1')
          ", ARRAY_A);

        if(!is_null($bookingDataRow))
        {
            return TRUE;
        } else
        {
            return FALSE;
        }
    }

    public function isDepartedBooking()
    {
        $validBookingId = intval($this->bookingId);
        $bookingDataRow = $this->internalWPDB->get_row("
          SELECT booking_id
          FROM {$this->nrsPrefix}bookings
          WHERE booking_id='{$validBookingId}' AND pickup_timestamp < ".time()."
          ", ARRAY_A);

        if(!is_null($bookingDataRow))
        {
            return TRUE;
        } else
        {
            return FALSE;
        }
    }

    public function isCancelledBooking()
    {
        $validBookingId = intval($this->bookingId);
        $bookingDataRow = $this->internalWPDB->get_row("
          SELECT booking_id
          FROM {$this->nrsPrefix}bookings
          WHERE booking_id='{$validBookingId}' AND is_cancelled='1'
          ", ARRAY_A);

        if(!is_null($bookingDataRow))
        {
            return TRUE;
        } else
        {
            return FALSE;
        }
    }

    public function isPaidBooking()
    {
        $validBookingId = intval($this->bookingId);
        $bookingDataRow = $this->internalWPDB->get_row("
          SELECT booking_id
          FROM {$this->nrsPrefix}bookings
          WHERE booking_id='{$validBookingId}' AND payment_success='1'
          ", ARRAY_A);

        if(!is_null($bookingDataRow))
        {
            return TRUE;
        } else
        {
            return FALSE;
        }
    }

    public function isRefundedBooking()
    {
        $validBookingId = intval($this->bookingId);
        $bookingDataRow = $this->internalWPDB->get_row("
          SELECT booking_id
          FROM {$this->nrsPrefix}bookings
          WHERE booking_id='{$validBookingId}' AND payment_success='2'
          ", ARRAY_A);

        if(!is_null($bookingDataRow))
        {
            return TRUE;
        } else
        {
            return FALSE;
        }
    }

    /**
     * Take the customer Id data from booking data
     * @return int
     */
    public function getCustomerId()
    {
        $bookingDetails = $this->getBookingDetails();
        $customerId = 0;
        if(!is_null($bookingDetails))
        {
            $customerId = $bookingDetails['customer_id'];
        }
        return $customerId;
    }

    /**
     * Used as a initializer and data puller of existing booking BEFORE search engine functions
     * @return mixed
     */
    public function getBookingDetails()
    {
        $bookingDetails = NULL;
        $validBookingId = intval($this->bookingId);
        if($validBookingId > 0)
        {
            // NOTE: Left join is a must here, because customer might be deleted.
            $bookingSql = "
				SELECT bb.*, cust.email
				FROM {$this->nrsPrefix}bookings bb
				LEFT JOIN {$this->nrsPrefix}customers cust ON cust.customer_id=bb.customer_id
				WHERE booking_id='{$validBookingId}'
			";
            $bookingDetails = $this->internalWPDB->get_row($bookingSql, ARRAY_A);

            if(!is_null($bookingDetails))
            {
                $bookedItemsSQL = "
					SELECT bop.item_id, bop.option_id, bop.units_booked,
					it.body_type_id, it.transmission_type_id, it.fuel_type_id
					FROM {$this->nrsPrefix}booking_options bop
					JOIN {$this->nrsPrefix}items it ON it.item_id=bop.item_id
					WHERE booking_id='{$validBookingId}'
				";
                $bookedExtrasSQL = "
					SELECT bop.extra_id, bop.option_id, bop.units_booked
					FROM {$this->nrsPrefix}booking_options bop
					JOIN {$this->nrsPrefix}extras extr ON extr.extra_id=bop.extra_id
					WHERE booking_id='{$validBookingId}'
				";
                $reservedItems = $this->internalWPDB->get_results($bookedItemsSQL, ARRAY_A);
                $reservedExtras = $this->internalWPDB->get_results($bookedExtrasSQL, ARRAY_A);

                // Cars and Car Units
                $bookingDetails['item_ids'] = array();
                $bookingDetails['item_units'] = array();
                $bookingDetails['item_options'] = array();
                $bookingDetails['items'] = array();
                foreach($reservedItems AS $reservedItem)
                {
                    $bookingDetails['item_ids'][] = $reservedItem['item_id'];
                    $bookingDetails['item_units'][$reservedItem['item_id']] = $reservedItem['units_booked'];
                    $bookingDetails['item_options'][$reservedItem['item_id']] = $reservedItem['option_id'];
                    $bookingDetails['items'][] = array(
                        "item_id" => $reservedItem['item_id'],
                        "body_type_id" => $reservedItem['body_type_id'],
                        "transmission_type_id" => $reservedItem['transmission_type_id'],
                        "fuel_type_id" => $reservedItem['fuel_type_id'],
                        "option_id" => $reservedItem['option_id'],
                        "units_booked" => $reservedItem['units_booked'],
                    );
                }

                // Extras and Extra Units
                $bookingDetails['extra_ids'] = array();
                $bookingDetails['extra_options'] = array();
                $bookingDetails['extra_units'] = array();
                $bookingDetails['extras'] = array();
                foreach($reservedExtras AS $reservedExtra)
                {
                    $bookingDetails['extra_ids'][] = $reservedExtra['extra_id'];
                    $bookingDetails['extra_options'][$reservedExtra['extra_id']] = $reservedExtra['option_id'];
                    $bookingDetails['extra_units'][$reservedExtra['extra_id']] = $reservedExtra['units_booked'];
                    $bookingDetails['extras'][] = array(
                        "extra_id"      => $reservedExtra['extra_id'],
                        "option_id"     => $reservedExtra['option_id'],
                        "units_booked"  => $reservedExtra['units_booked'],
                    );
                }

                // Get payment status text and color
                $bookingDetails['payment_status'] = "";
                $bookingDetails['payment_status_color'] = "#FF0000";
                if($bookingDetails['payment_success'] == 0)
                {
                    $bookingDetails['payment_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_UNPAID_TEXT');
                    $bookingDetails['payment_status_color'] = "#FF0000";
                } else if($bookingDetails['payment_success'] == 1)
                {
                    $bookingDetails['payment_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_PAID_TEXT');
                    $bookingDetails['payment_status_color'] = "black";
                } else if($bookingDetails['payment_success'] == 2)
                {
                    $bookingDetails['payment_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_REFUNDED_TEXT');
                    $bookingDetails['payment_status_color'] = "navy";
                }

                // Get booking status text and color
                $bookingDetails['booking_status'] = "";
                $bookingDetails['booking_status_color'] = "black";
                if($bookingDetails['is_cancelled'] == 0 && $bookingDetails['dropoff_timestamp'] >= time())
                {
                    $bookingDetails['booking_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_ACTIVE_TEXT');
                    $bookingDetails['booking_status_color'] = "green";
                } else if($bookingDetails['is_cancelled'] == 0 && $bookingDetails['dropoff_timestamp'] < time())
                {
                    $bookingDetails['booking_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_DEPARTED_TEXT');
                    $bookingDetails['booking_status_color'] = "blue";
                } else if($bookingDetails['is_cancelled'] == 1)
                {
                    $bookingDetails['booking_status'] = $this->lang->getText('NRS_ADMIN_BOOKING_STATUS_CANCELLED_TEXT');
                    $bookingDetails['booking_status_color'] = "red";
                }
            }
        }

        return $bookingDetails;
    }

    /**
     * Save booking data
     * @param int $paramCustomerId
     * @param array $paramSearchDataArray
     */
    public function saveBookingData($paramCustomerId = 0, $paramSearchDataArray = array())
    {
        $validBookingId = NRSValidator::getValidInteger($this->bookingId, 0);
        $validCustomerId = NRSValidator::getValidInteger($paramCustomerId, 0);

        if($this->debugMode == 1)
        {
            echo "<br /><strong>Reservation Id:</strong> {$validBookingId}, ";
            echo "<br /><strong>Customer Id:</strong> {$validCustomerId} ";
            echo "<br /><strong>Search params array:</strong> "; echo nl2br(print_r($paramSearchDataArray, TRUE));
        }

        $validPaymentMethodCode = isset($paramSearchDataArray['payment_method_code']) ? sanitize_key($paramSearchDataArray['payment_method_code']) : "";
        $validPickupTimestamp = isset($paramSearchDataArray['pickup_timestamp']) ? intval($paramSearchDataArray['pickup_timestamp']) : 0;
        $validDropoffTimestamp = isset($paramSearchDataArray['dropoff_timestamp']) ? intval($paramSearchDataArray['dropoff_timestamp']) : 0;
        $validPickupLocationId = isset($paramSearchDataArray['pickup_location_id']) ? intval($paramSearchDataArray['pickup_location_id']) : 0;
        $validDropoffLocationId = isset($paramSearchDataArray['dropoff_location_id']) ? intval($paramSearchDataArray['dropoff_location_id']) : 0;
        $validBodyTypeId = isset($paramSearchDataArray['body_type_id']) ? intval($paramSearchDataArray['body_type_id']) : 0;
        $validTransmissionTypeId = isset($paramSearchDataArray['transmission_type_id']) ? intval($paramSearchDataArray['transmission_type_id']) : 0;
        $validFuelTypeId = isset($paramSearchDataArray['fuel_type_id']) ? intval($paramSearchDataArray['fuel_type_id']) : 0;

        $validItemIds = isset($paramSearchDataArray['item_ids']) ? NRSValidator::getValidArray($paramSearchDataArray['item_ids'], "intval", array()) : array();
        $validItemUnits = isset($paramSearchDataArray['item_units']) ? NRSValidator::getValidArray($paramSearchDataArray['item_units'], "intval", array()) : array();
        $validItemOptions = isset($paramSearchDataArray['item_options']) ? NRSValidator::getValidArray($paramSearchDataArray['item_options'], "intval", array()) : array();

        $validExtraIds = isset($paramSearchDataArray['extra_ids']) ? NRSValidator::getValidArray($paramSearchDataArray['extra_ids'], "intval", array()) : array();
        $validExtraUnits = isset($paramSearchDataArray['extra_units']) ? NRSValidator::getValidArray($paramSearchDataArray['extra_units'], "intval", array()) : array();
        $validExtraOptions = isset($paramSearchDataArray['extra_options']) ? NRSValidator::getValidArray($paramSearchDataArray['extra_options'], "intval", array()) : array();

        // If this is an existing booking and payment method is not PayPal
        if ($validBookingId > 0 && $validPaymentMethodCode != "pp")
        {
            if($this->debugMode == 1)
            {
                echo "<br /><strong>FINALIZE BOOKING EDIT:</strong> UPDATE EVERYTHING EXCEPT BOOKING TIME. USE EDIT_TIME INSTEAD OF THAT";
            }
            // UPDATE EVERYTHING EXCEPT BOOKING TIMESTAMP. UPDATES EDIT_TIMESTAMP INSTEAD
            $this->internalWPDB->query("
				UPDATE {$this->nrsPrefix}bookings SET
				last_edit_timestamp='".time()."',
				pickup_timestamp='{$validPickupTimestamp}',
				dropoff_timestamp='{$validDropoffTimestamp}',
				pickup_location_id='{$validPickupLocationId}',
				dropoff_location_id='{$validDropoffLocationId}',
				body_type_id='{$validBodyTypeId}',
				transmission_type_id='{$validTransmissionTypeId}',
				fuel_type_id='{$validFuelTypeId}',
				customer_id='{$validCustomerId}',
				payment_method_code='{$validPaymentMethodCode}',
				WHERE booking_id='{$validBookingId}'
			");

            // OLD BOOKING DATA STILL EXIST IN RES DATA TABLE, SO WE NEED TO DELETE THESE ROWS THAT WE NEW DATA ROWS (ITEM IDs) COULD BE INSERTED THERE THERE
            $this->internalWPDB->query("DELETE FROM {$this->nrsPrefix}booking_options WHERE booking_id='{$validBookingId}'");
        } else
        {
            if ($validBookingId > 0 && $validPaymentMethodCode == "pp")
            {
                // Cancel previous PayPal booking
                $this->cancelBooking(TRUE);
                if($this->debugMode == 1)
                {
                    echo "<br />Previous PayPal Booking Id. {$validBookingId} were cancelled.";
                }
            }
            if($this->debugMode == 1)
            {
                echo "<br /><strong>FINALIZE NEW BOOKING:</strong> INSERT NEW DATA TO DATABASE.";
            }
            $newBookingCode = $this->generateBookingCode();
            // NOTE: We can't use sanitize_text_field function for $pickupLocation and $dropoffLocation,
            // because it has <br /> tags inside. So we must use 'wp_kses_post'.
            // Still, we sure that all data used for this field is generated from internal db content, not from user input
            $insertSQL = "
				INSERT INTO {$this->nrsPrefix}bookings
				(
					booking_code, booking_timestamp,
					pickup_timestamp, dropoff_timestamp,
					pickup_location_id,	dropoff_location_id,
					body_type_id, transmission_type_id, fuel_type_id,
					customer_id,
					payment_method_code
				) VALUES
				(
					'{$newBookingCode}', '".time()."',
					'{$validPickupTimestamp}', '{$validDropoffTimestamp}',
					'{$validPickupLocationId}', '{$validDropoffLocationId}',
					'{$validBodyTypeId}', '{$validTransmissionTypeId}', '{$validFuelTypeId}',
					'{$validCustomerId}',
					'{$validPaymentMethodCode}'
				)
			";

            //echo "<br />INSERT BOOKING: ".nl2br($insertSQL); die();
            $insertResult = $this->internalWPDB->query($insertSQL);

            if($insertResult === FALSE)
            {
                // Error inserting new booking
                $validBookingId = 0;
            } else
            {
                $validBookingId = intval($this->internalWPDB->insert_id);
                // Assign new booking id to class, that it would work with new function calls
                $this->bookingId = $validBookingId;
            }

        }

        foreach ($validItemIds AS $itemId)
        {
            $validItemId = NRSValidator::getValidInteger($itemId, 0);
            $validQuantity = isset($validItemUnits[$validItemId]) ? NRSValidator::getValidInteger($validItemUnits[$validItemId], 0) : 0;
            $validOptionId = isset($validItemOptions[$validItemId]) ? NRSValidator::getValidInteger($validItemOptions[$validItemId], 0) : 0;
            if($validQuantity > 0 && $validItemId > 0 && $validBookingId > 0)
            {
                $this->internalWPDB->query("
					INSERT INTO {$this->nrsPrefix}booking_options
					(
						booking_id, item_id, extra_id, option_id, units_booked
					) VALUES
					(
						'{$validBookingId}', '{$validItemId}', '0', '{$validOptionId}', '{$validQuantity}'
					)
				");
            }
        }

        foreach ($validExtraIds AS $extraId)
        {
            $validExtraId = NRSValidator::getValidInteger($extraId, 0);
            $validQuantity = isset($validExtraUnits[$validExtraId]) ? NRSValidator::getValidInteger($validExtraUnits[$validExtraId], 0) : 0;
            $validOptionId = isset($validExtraOptions[$validExtraId]) ? NRSValidator::getValidInteger($validExtraOptions[$validExtraId], 0) : 0;
            if($validQuantity > 0 && $validExtraId > 0 && $validBookingId > 0)
            {
                $this->internalWPDB->query("
					INSERT INTO {$this->nrsPrefix}booking_options
					(
						booking_id, item_id, extra_id, option_id, units_booked
					) VALUES
					(
						'{$validBookingId}', '0', '{$validExtraId}', '{$validOptionId}', '{$validQuantity}'
					)
				");
            }
        }
    }

    public function unblockBookingItem($paramItemId = 0)
    {
        $validBookingId  = NRSValidator::getValidInteger($this->bookingId, 0);
        $validItemId  = NRSValidator::getValidInteger($paramItemId, 0);

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}booking_options
            WHERE booking_id='{$validBookingId}' AND item_id='{$validItemId}'
        ");

        $relatedRows = $this->internalWPDB->get_var("
            SELECT booking_id
            FROM {$this->nrsPrefix}booking_options
            WHERE booking_id='{$validBookingId}'
        ");
        // If no related elements found to this booking
        if(is_null($relatedRows))
        {
            // Then also delete the booking itself as well
            $this->internalWPDB->query("
                DELETE FROM {$this->nrsPrefix}bookings
                WHERE booking_id='{$validBookingId}'
            ");
        }

        return true;
    }

    public function unblockBookingExtra($paramExtraId = 0)
    {
        $validBookingId  = NRSValidator::getValidInteger($this->bookingId, 0);
        $validExtraId  = NRSValidator::getValidInteger($paramExtraId, 0);

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}booking_options
            WHERE booking_id='{$validBookingId}' AND extra_id='{$validExtraId}'
        ");

        $relatedRows = $this->internalWPDB->get_var("
            SELECT booking_id
            FROM {$this->nrsPrefix}booking_options
            WHERE booking_id='{$validBookingId}'
        ");
        // If no related elements found to this booking
        if(is_null($relatedRows))
        {
            // Then also delete the booking itself as well
            $this->internalWPDB->query("
                DELETE FROM {$this->nrsPrefix}bookings
                WHERE booking_id='{$validBookingId}'
            ");
        }


        return true;
    }

    public function unblockBooking()
    {
        $validBookingId  = intval($this->bookingId);

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}bookings
            WHERE booking_id='{$validBookingId}'
        ");
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}booking_options
            WHERE booking_id='{$validBookingId}'
        ");

        return true;
    }

    public function cancelBooking($sendNotificationEmailToAdmin = TRUE)
    {
        $bookingCancelled = FALSE;
        $emailProcessed = TRUE;
        $validBookingId = intval($this->bookingId);

        if($validBookingId > 0)
        {
            // If there exists unpaid booking under this booking id
            $sql = "
                  SELECT booking_id, customer_id
                  FROM {$this->nrsPrefix}bookings
                  WHERE booking_id='{$validBookingId}' AND is_cancelled='0'
            ";

            $bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

            if(!is_null($bookingData))
            {
                $rowsAffected = $this->internalWPDB->query("
                      UPDATE {$this->nrsPrefix}bookings SET
                      is_cancelled='1'
                      WHERE booking_id='{$validBookingId}'
                ");
                $bookingCancelled = $rowsAffected > 0 ? TRUE : FALSE;

                if(!is_null($this->nrsMail))
                {
                    $emailProcessed = $this->nrsMail->sendBookingCancellationEmail($validBookingId, $sendNotificationEmailToAdmin);
                }
            }
        }

        if($bookingCancelled && $emailProcessed)
        {
            $_SESSION['ok_msg'] = 'Completed: Reservation was cancelled successfully!';
        } else if($bookingCancelled && $emailProcessed === FALSE)
        {
            $_SESSION['msg'] = 'Failed: Reservation was cancelled, but system was unable to send the cancellation email!';
        } else
        {
            $_SESSION['msg'] = 'Failed: Reservation was not cancelled!';
        }

        return $bookingCancelled;
    }

    public function deleteBooking($sendNotificationEmailToAdmin = TRUE)
    {
        $ret = false;

        $validBookingId = intval($this->bookingId);
        if($validBookingId > 0)
        {
            // If there exists booking under this booking id
            $sql = "
                  SELECT booking_id, is_cancelled
                  FROM {$this->nrsPrefix}bookings
                  WHERE booking_id='{$validBookingId}'
            ";
            $bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

            if(!is_null($bookingData))
            {
                if($bookingData['is_cancelled'] == 0)
                {
                    // First - cancel
                    // And send e-mails to disappointed customers if needed
                    $this->cancelBooking($sendNotificationEmailToAdmin);
                }
                $ret = $this->internalWPDB->query("
                    DELETE FROM {$this->nrsPrefix}bookings
                    WHERE booking_id='{$validBookingId}'
                ");
                $this->internalWPDB->query("
                    DELETE FROM {$this->nrsPrefix}booking_options
                    WHERE booking_id='{$validBookingId}'
                ");
            }
        }

        if($ret)
        {
            $_SESSION['ok_msg'] = 'Completed: Reservation was deleted successfully!';
        } else
        {
            $_SESSION['msg'] = 'Failed: Reservation was not deleted!';
        }

        return $ret;
    }

    public function markPaidBooking($sendNotificationEmailToAdmin = TRUE, $paramPaymentTxnId = "", $paramPayPalEmail = "")
    {
        $markedAsPaid = FALSE;
        $emailProcessed = TRUE;
        $validBookingId = intval($this->bookingId);
        $validPaymentTxnId = sanitize_text_field($paramPaymentTxnId); // Optional
        $validPayPalEmail = sanitize_email($paramPayPalEmail); // Optional

        if($validBookingId > 0)
        {
            // If there exists unpaid booking under this booking id
            $sql = "
                  SELECT booking_id, customer_id
                  FROM {$this->nrsPrefix}bookings
                  WHERE booking_id='{$validBookingId}' AND payment_success='0'
            ";

            $bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

            if(!is_null($bookingData))
            {
                $markedAsPaid = $this->internalWPDB->query("
                      UPDATE {$this->nrsPrefix}bookings SET
                      payment_success='1', payment_txnid='{$validPaymentTxnId}',
                      paypal_email='{$validPayPalEmail}'
                      WHERE booking_id='{$validBookingId}'
                ");
                $this->internalWPDB->query("
                    UPDATE {$this->nrsPrefix}customers SET
                    existing_customer='1'
                    WHERE customer_id='{$bookingData['customer_id']}'
                ");

                if(!is_null($this->nrsMail))
                {
                    $emailProcessed = $this->nrsMail->sendBookingConfirmationEmail($validBookingId, $sendNotificationEmailToAdmin);
                }
            }
        }

        if($markedAsPaid && $emailProcessed)
        {
            $_SESSION['ok_msg'] = 'Completed: Reservation was marked as paid!';
        } else if($markedAsPaid && $emailProcessed === FALSE)
        {
            $_SESSION['msg'] = 'Failed: Reservation was marked as paid, but system was unable to send the confirmation email!';
        } else
        {
            $_SESSION['msg'] = 'Failed: Reservation was not marked as paid!';
        }


        return $markedAsPaid;
    }

    /**
     * Money were sent back to the customer
     * @param $paramSendNotificationEmailToAdmin
     * @return bool
     */
    public function refundBooking($paramSendNotificationEmailToAdmin = TRUE)
    {
        $ret = false;
        $validBookingId = intval($this->bookingId);

        if($validBookingId > 0)
        {
            // If there exists unpaid booking under this booking id
            $sql = "
                  SELECT booking_id, customer_id
                  FROM {$this->nrsPrefix}bookings
                  WHERE booking_id='{$validBookingId}' AND payment_success='1'
            ";

            $bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

            if(!is_null($bookingData))
            {
                $this->internalWPDB->query("
                      UPDATE {$this->nrsPrefix}bookings SET
                      payment_success='2', is_cancelled='1'
                      WHERE booking_id='{$validBookingId}'
                ");

                if(!is_null($this->nrsMail))
                {
                    $ret = $this->nrsMail->sendBookingCancellationEmail($validBookingId, $paramSendNotificationEmailToAdmin);
                }
            }
        }

        return $ret;
    }
}