<?php
/**
 * NRS Bookings Observer (no setup for single booking)
 * Abstract class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSBookingsObserver
{
    protected $internalWPDB = NULL;
    protected $nrsPrefix = "";
    protected $lang = NULL;
    protected $debugMode = 0;
    protected $minBookingPeriod = 0;
    protected $maxBookingPeriod = 0;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;
        // set minimum booking period
        $this->setMinBookingPeriod();
        // Set maximum booking period
        $this->setMaxBookingPeriod();
    }

    /**
     * Set minimum booking period - called in class constructor
     */
    private function setMinBookingPeriod()
    {
        $sql = "
				SELECT conf_value AS minimum_booking_period
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_minimum_booking_period'
			";
        $minBookingPeriod = $this->internalWPDB->get_var($sql);

        if(!is_null($minBookingPeriod))
        {
            $this->minBookingPeriod = NRSValidator::getValidInteger($minBookingPeriod, 0);
        } else
        {
            $this->minBookingPeriod = 0;
        }
    }

    /**
     * Set maximum booking period - called in class constructor
     */
    private function setMaxBookingPeriod()
    {
        $sql = "
				SELECT conf_value AS maximum_booking_period
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_maximum_booking_period'
			";
        $maxBookingPeriod = $this->internalWPDB->get_var($sql);

        if(!is_null($maxBookingPeriod))
        {
            $this->maxBookingPeriod = NRSValidator::getValidInteger($maxBookingPeriod, 0);
        } else
        {
            $this->maxBookingPeriod = 0;
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getMinBookingPeriod()
    {
        return $this->minBookingPeriod;
    }

    public function getMaxBookingPeriod()
    {
        return $this->maxBookingPeriod;
    }

    /**
     * Get reservation period list, if not date and time used
     * @param string $fieldID
     * @param string $fieldName
     * @param string $fieldText
     * @param int $selectedReservationPeriod
     * @param string $emptyItemValue
     * @return string
     */
    public function getBookingPeriodDropDown($fieldID = "booking_period", $fieldName = "booking_period", $fieldText = "", $selectedReservationPeriod = 7200, $emptyItemValue = "0")
    {
        $selectPeriods = '<select name="'.$fieldName.'" id="'.$fieldID.'" class="home-select">';

        // For up to 1 week
        $hourSpeedUpLimit1 = 1; // hours
        $hourSpeedUpLimit2 = 3; // hours
        $hourSpeedUpLimit3 = 24; // hours
        for($hour = 0; $hour <= 720; $hour++)
        {
            if($hour > $hourSpeedUpLimit3)
            {
                // For more than two days, we have to advance by 24 hours
                $hour = $hour+23;
            }
            for($minute = 0; $minute < 60; $minute = $minute+15)
            {
                //echo "hour: $hour, MIN: $minute<br />";
                if($hour == 0 && $minute == 0)
                {
                    // Special case for first item
                    if($emptyItemValue == "0")
                    {
                        $selectedPeriodZero = $selectedReservationPeriod == 0 ? ' selected="selected"' : '';
                        $selectPeriods .= '<option value="0"'.$selectedPeriodZero.'>'.$fieldText.'</option>';
                    } else
                    {
                        $selectedPeriodEmpty = $selectedReservationPeriod == "" ? ' selected="selected"' : '';
                        $selectPeriods .= '<option value=""'.$selectedPeriodEmpty.'>'.$fieldText.'</option>';
                    }
                } else
                {
                    if($hour >= $hourSpeedUpLimit2 && $minute > 0)
                    {
                        // For more than a day, we have to advance not by 30 minutes, but by the whole hour
                        break;
                    }
                    if($hour < $hourSpeedUpLimit1 || ($hour >= $hourSpeedUpLimit1 && ($minute == 0 || $minute == 30)))
                    {
                        // For more than 60 minutes, we advance by 30 more minutes

                        // All other items
                        $currentDay = $hour > 23 ? floor($hour/24) : 0;
                        $currentHour = $hour > 23 ? $hour - $currentDay*24 : $hour;
                        $currentMinute = $minute;
                        $currentFullTimeInMinutes = $hour*60 + $minute;

                        $daysText = $this->lang->getTimeTextByPeriod($currentDay, $this->lang->getText('NRS_DAY_TEXT'), $this->lang->getText('NRS_DAYS_TEXT'), $this->lang->getText('NRS_DAYS2_TEXT'));
                        $hoursText = $this->lang->getTimeTextByPeriod($currentHour, $this->lang->getText('NRS_HOUR_TEXT'), $this->lang->getText('NRS_HOURS_TEXT'), $this->lang->getText('NRS_HOURS2_TEXT'));
                        $minutesText = $this->lang->getTimeTextByPeriod($currentMinute, $this->lang->getText('NRS_MINUTE_TEXT'), $this->lang->getText('NRS_MINUTES_TEXT'), $this->lang->getText('NRS_MINUTES2_TEXT'));

                        $optionTitle = "";
                        if($hour >= $hourSpeedUpLimit3)
                        {
                            $optionTitle .= $currentDay." ".$daysText;
                        } else if($hour >= $hourSpeedUpLimit2)
                        {
                            $optionTitle .= $hour." ".$hoursText;
                        } else
                        {
                            $optionTitle .= $currentFullTimeInMinutes." ".$minutesText;
                        }
                        $optionValueInSeconds = $currentDay*86400 + $currentHour*3600 + $currentMinute*60;
                        $selected = ($optionValueInSeconds == $selectedReservationPeriod ? ' selected="selected"' : '');

                        if(
                            $optionValueInSeconds >= $this->minBookingPeriod &&
                            $optionValueInSeconds <= $this->maxBookingPeriod
                        ){
                            // Output
                            $selectPeriods .= '<option value="'.$optionValueInSeconds.'"'.$selected.'>'.$optionTitle.'</option>';
                        }
                    }
                }
            }
        }

        $selectPeriods .= '</select>';

        return $selectPeriods;
    }

    public function getAdminTodayBookingList($currencySymbol, $paramUrlPath = "")
    {
        $todayStartTimestamp = NRSValidator::getStartOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        $todayEndTimestamp = NRSValidator::getLocalEndOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        return $this->getAdminBookingList("BOOKING_LIST", $todayStartTimestamp, $todayEndTimestamp, $currencySymbol, 0, $paramUrlPath);
    }

    public function getAdminTodayPickupList($currencySymbol, $paramUrlPath = "")
    {
        $todayStartTimestamp = NRSValidator::getStartOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        $todayEndTimestamp = NRSValidator::getLocalEndOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        return $this->getAdminBookingList("PICKUP_LIST", $todayStartTimestamp, $todayEndTimestamp, $currencySymbol, 0, $paramUrlPath);
    }

    public function getAdminTodayDropoffList($currencySymbol, $paramUrlPath = "")
    {
        $todayStartTimestamp = NRSValidator::getStartOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        $todayEndTimestamp = NRSValidator::getLocalEndOfDayTimestampInWebsiteTimezone(date("Y-m-d"));
        return $this->getAdminBookingList("DROPOFF_LIST", $todayStartTimestamp, $todayEndTimestamp, $currencySymbol, 0, $paramUrlPath);
    }

    public function getAdminCustomerBookingList($paramCustomerId, $currencySymbol = "", $paramUrlPath = "")
    {
        return $this->getAdminBookingList("ALL_BOOKING_LIST", 0, 0, $currencySymbol, $paramCustomerId, $paramUrlPath);
    }

    /**
     * @param $paramList
     * @param $timestampFrom
     * @param $timestampTo
     * @param $currencySymbol
     * @param int $paramCustomerId - Optional customer Id
     * @param string $paramUrlPath - Optional ulr path, i.e. &from_dropoff_date=1234567890&to_dropoff_date=2234567890
     * @return string
     */
    public function getAdminBookingList($paramList, $timestampFrom = 0, $timestampTo = 0, $currencySymbol = "", $paramCustomerId = 0, $paramUrlPath = "")
    {
        $validCustomerId = NRSValidator::getValidInteger($paramCustomerId, 0);
        $validTimestampFrom = NRSValidator::getValidInteger($timestampFrom, 0);
        $validTimestampTo = NRSValidator::getValidInteger($timestampTo, 0);
        //echo "PATH: $paramUrlPath";
        $validUrlPath = sanitize_text_field($paramUrlPath);

        // If not filled - return all bookings
        $sqlCondition = "";

        // If there is a customer id set
        if($validCustomerId > 0)
        {
            $sqlCondition .= " AND customer_id='".intval($validCustomerId)."'";
        }

        // OPTIMIZED - we don't use UNIX_TIMESTAMP() here is SQL WHERE, to save resource and not to recalculate time every time
        // so we use PHP's time() function instead, which does the work pretty well,
        // and do not needs to be calculated on every SQL query
        $sqlOrderBy = "booking_timestamp ASC";
        // If there is a listing property set
        if($paramList == "ALL_BOOKING_LIST")
        {
            // Do nothing, get all
            $sqlOrderBy = "booking_timestamp ASC";
        } else if($paramList == "ALL_PICKUP_LIST")
        {
            // Do nothing, get all
            $sqlOrderBy = "pickup_timestamp ASC";
        } else if($paramList == "ALL_DROPOFF_LIST")
        {
            // Do nothing, get all
            $sqlOrderBy = "dropoff_timestamp ASC";


        } else if ($paramList == "ACTIVE_BOOKING_LIST")
        {
            $sqlCondition .= " AND pickup_timestamp >= ".time()." AND dropoff_timestamp >= ".time();
            $sqlOrderBy = "booking_timestamp ASC";
        } else if ($paramList == "ACTIVE_PICKUP_LIST")
        {
            $sqlCondition .= " AND pickup_timestamp >= ".time();
            $sqlOrderBy = "pickup_timestamp ASC";
        } else if ($paramList == "ACTIVE_DROPOFF_LIST")
        {
            $sqlCondition .= " AND dropoff_timestamp >= ".time();
            $sqlOrderBy = "dropoff_timestamp ASC";


        } else if($paramList == "ARCHIVE_BOOKING_LIST")
        {
            $sqlCondition .= " AND pickup_timestamp < ".time()." AND dropoff_timestamp < ".time();
            $sqlOrderBy = "booking_timestamp DESC";
        } else if($paramList == "ARCHIVE_PICKUP_LIST")
        {
            $sqlCondition .= " AND pickup_timestamp < ".time();
            $sqlOrderBy = "pickup_timestamp DESC";
        } else if($paramList == "ARCHIVE_DROPOFF_LIST")
        {
            $sqlCondition .= " AND dropoff_timestamp < ".time();
            $sqlOrderBy = "dropoff_timestamp DESC";


        } else if ($paramList == "BOOKING_LIST")
        {
            $sqlCondition .= " AND booking_timestamp BETWEEN $validTimestampFrom AND $validTimestampTo";
            $sqlOrderBy = "booking_timestamp ASC";
        } else if($paramList == "PICKUP_LIST")
        {
            $sqlCondition .= " AND pickup_timestamp BETWEEN $validTimestampFrom AND $validTimestampTo";
            $sqlOrderBy = "pickup_timestamp ASC";
        } else if($paramList == "DROPOFF_LIST")
        {
            $sqlCondition .= " AND dropoff_timestamp BETWEEN $validTimestampFrom AND $validTimestampTo";
            $sqlOrderBy = "dropoff_timestamp ASC";
        }

        $sql = "
            SELECT booking_id
            FROM {$this->nrsPrefix}bookings b
            WHERE is_block='0'
            {$sqlCondition}
            ORDER BY {$sqlOrderBy}
        ";

        // DEBUG
        //echo nl2br($sql);

        $html = '';

        $bookingIds = $this->internalWPDB->get_col($sql);

        $i = 0;
        //$totalBookingsFound = sizeof($bookingIds);
        foreach($bookingIds AS $bookingId)
        {
            $i++;
            //$prefix = sprintf('%0'.strlen($totalBookingsFound).'d', $i);
            $objBooking = new NRSBooking($this->internalWPDB, $this->nrsPrefix, $this->lang, NULL, "", $bookingId);
            $objCustomer = new NRSCustomer($this->internalWPDB, $this->nrsPrefix, $this->lang, "", $objBooking->getCustomerId());
            $objInvoice = new NRSInvoice($this->internalWPDB, $this->nrsPrefix, $this->lang, $bookingId);

            $customerDetails = $objCustomer->getCustomerDetails();
            $bookingDetails = $objBooking->getBookingDetails();

            $invoiceDetails = $objInvoice->getInvoiceDetails();

            $arrBookedItemNames = array();
            $counter = 0;
            foreach($bookingDetails['items'] AS $bookedItems)
            {
                $counter++;
                $objItem = new NRSItem($GLOBALS['wpdb'], $GLOBALS['NRS_PREFIX'], $GLOBALS['NRS_LANG'], $bookedItems['item_id']);

                // 1 - Process item details & prices
                $itemDetails = $objItem->getFullDetails();
                $objSelectedOption = new NRSOption($this->internalWPDB, $this->nrsPrefix, $this->lang, $bookedItems['option_id']);
                $selectedOptionDetails = $objSelectedOption->getDetails();
                $selectedOptionName = isset($selectedOptionDetails['option_name']) ? $selectedOptionDetails['option_name'] : "";

                $printItemNameWithSelectedOption = $counter.'. '.$itemDetails['manufacturer_title'].' '.$itemDetails['model_name'];
				$printItemNameWithSelectedOption .= $itemDetails['body_type_title'] ? ', '.$itemDetails['body_type_title'] : '';
				$printItemNameWithSelectedOption .= $selectedOptionName ? ', '.$selectedOptionName.' '.$itemDetails['measurement_unit'] : '';

                // Add item name to booked items list
                $arrBookedItemNames[] = $printItemNameWithSelectedOption;
            }

            // Calculations
            $pickupTMZStamp = $bookingDetails['pickup_timestamp'] + get_option( 'gmt_offset' ) * 3600;
            $dropoffTMZStamp = $bookingDetails['dropoff_timestamp'] + get_option( 'gmt_offset' ) * 3600;
            $bookingTMZStamp = $bookingDetails['booking_timestamp'] + get_option( 'gmt_offset' ) * 3600;

            // Prints
            $printBookedItems = implode("<br />", $arrBookedItemNames);
            $printPickupDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $pickupTMZStamp, TRUE);
            $printDropoffDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $dropoffTMZStamp, TRUE);
            $printBookingDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $bookingTMZStamp, TRUE);
            $printGrandTotal = $currencySymbol.' '.number_format_i18n($invoiceDetails['grand_total'], 2);
            $printFixedDepositAmount = $currencySymbol.' '.number_format_i18n($invoiceDetails['fixed_deposit_amount'], 2);
            $printTotalPayNow = $currencySymbol.' '.number_format_i18n($invoiceDetails['total_pay_now'], 2);
            $printPayOnReturn = $currencySymbol.' '.number_format_i18n($invoiceDetails['grand_total']-$invoiceDetails['total_pay_now'], 2);

            if($bookingDetails['payment_method_code'] == "pp")
            {
                $printPayNowText = $this->lang->getText('NRS_STEP5_PAID_ONLINE_TEXT');
            } else
            {
                $printPayNowText = $this->lang->getText('NRS_STEP5_PAY_AT_PICKUP_TEXT');
            }

            $actions = "";
            if($bookingDetails['dropoff_timestamp'] >= time() && $bookingDetails['is_cancelled'] == 0)
            {
                if($bookingDetails['payment_success'] == 1)
                {
                    $actions .= '<a href="javascript:;" onclick="javascript:refundBooking('.$bookingId.',\''.$validUrlPath.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_REFUND_TEXT').'</a><br />';
                } else if($bookingDetails['payment_success'] == 0)
                {
                    $actions .= '<a href="javascript:;" onclick="javascript:markBookingAsPaid('.$bookingId.',\''.$validUrlPath.'\',\''.$validCustomerId.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_MARK_PAID_TEXT').'</a><br />';
                }
                $actions .= '<a href="javascript:;" onClick="javascript:cancelBooking('.$bookingId.',\''.$validUrlPath.'\');">'.$this->lang->getText('NRS_ADMIN_CANCEL_TEXT').'</a>';
            } else if($bookingDetails['dropoff_timestamp'] < time() && $bookingDetails['is_cancelled'] == 0)
            {
                if($bookingDetails['payment_success'] == 1)
                {
                    $actions .= '<a href="javascript:;" onclick="javascript:refundBooking('.$bookingId.',\''.$validUrlPath.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_REFUND_TEXT').'</a><br />';
                } else if($bookingDetails['payment_success'] == 0)
                {
                    $actions .= '<a href="javascript:;" onclick="javascript:markBookingAsPaid('.$bookingId.',\''.$validUrlPath.'\',\''.$validCustomerId.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_MARK_PAID_TEXT').'</a><br />';
                }
                $actions .= '<a href="javascript:;" onclick="javascript:deleteBooking('.$bookingId.',\''.$validUrlPath.'\',\''.$validCustomerId.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a>';
            } else
            {
                // Cancelled (is_cancelled = 1)
                $actions .= '<a href="javascript:;" onclick="javascript:deleteBooking('.$bookingId.',\''.$validUrlPath.'\',\''.$validCustomerId.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a>';
            }

            $html .= '<tr>
                <td>'.$i.'</td>
                <td>'.$bookingDetails['booking_code'].'<br /><hr />'.$customerDetails['print_full_name'].'<br />'.$printBookedItems.'</td>
                <td>'.$printPickupDate.'<br /><hr />'.$invoiceDetails['pickup_location'].'</td>
                <td>'.$printDropoffDate.'<br /><hr />'.$invoiceDetails['dropoff_location'].'</td>
                <td>'.$printBookingDate.'<br /><hr />
                    <span style="font-weight: bold;color:'.$bookingDetails['payment_status_color'].';">'.$bookingDetails['payment_status'].'</span>,
                    <span style="font-weight: bold;color:'.$bookingDetails['booking_status_color'].';">'.$bookingDetails['booking_status'].'</span>
                </td>
                <td>
                    '.$this->lang->getText('NRS_TOTAL_TEXT').': '.$printGrandTotal.'<br /><hr />
                    '.$this->lang->getText('NRS_DEPOSIT_TEXT').': '.$printFixedDepositAmount.'<br />
                    '.$printPayNowText.': '.$printTotalPayNow.'<br />
                    '.$this->lang->getText('NRS_PAY_ON_RETURN_TEXT').': '.$printPayOnReturn.'<br />
                </td>
                <td align="right">
                    <a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-view-booking-details&amp;booking_id='.$bookingId).'" class="bodytext">'.$this->lang->getText('NRS_ADMIN_VIEW_DETAILS_TEXT').'</a><br />
                    <a href="javascript:;" onclick="printInvoicePopup(\''.$GLOBALS['NRS_ADMIN_URL'].'booking-manager.print-invoice.php\',\''.$GLOBALS['NRS_EXTENSION'].'\',\''.$bookingId.'\');" class="bodytext">'.$this->lang->getText('NRS_ADMIN_PRINT_INVOICE_TEXT').'</a><br />
                    '.$actions.'
                </td>
             </tr>';
        }

        return $html;
    }


    /**
     * @param $paramLocationId
     */
    public function deleteBookingsByLocationId($paramLocationId)
    {
        $validLocationId = NRSValidator::getValidInteger($paramLocationId, 0);
        // OPTIMIZED SQL: We used Php function time() instead of UNIX_TIMESTAMP() to perform more faster search
        // If there exists booking with this pickup or dropoff location id
        $sql = "
              SELECT booking_id, is_cancelled, pickup_timestamp
              FROM {$this->nrsPrefix}bookings
              WHERE pickup_timestamp>='".time()."' AND
              pickup_location_id='{$validLocationId}' OR dropoff_location_id='{$validLocationId}'
        ";

        $bookingResults = $this->internalWPDB->get_results($sql, ARRAY_A);

        foreach($bookingResults AS $bookingData)
        {
            $objMail = new NRSMail($this->internalWPDB, $this->nrsPrefix, $this->lang);
            $objBooking = new NRSBooking($this->internalWPDB, $this->nrsPrefix, $this->lang, $objMail, "", $bookingData['booking_id']);
            $objInvoice = new NRSInvoice($this->internalWPDB, $this->nrsPrefix, $this->lang, $bookingData['booking_id']);
            $objBooking->deleteBooking(FALSE);
            $objInvoice->deleteInvoice();
        }
    }

    public function deleteBookingsByCustomerId($paramCustomerId)
    {
        $validCustomerId = NRSValidator::getValidInteger($paramCustomerId, 0);
        $bookingDataRows = $this->internalWPDB->get_results("
            SELECT booking_id, is_cancelled, pickup_timestamp
            FROM {$this->nrsPrefix}bookings
            WHERE customer_id='{$validCustomerId}' AND pickup_timestamp>='".time()."'
        ", ARRAY_A);

        foreach($bookingDataRows AS $bookingData)
        {
            $objMail = new NRSMail($this->internalWPDB, $this->nrsPrefix, $this->lang);
            $objBooking = new NRSBooking($this->internalWPDB, $this->nrsPrefix, $this->lang, $objMail, "", $bookingData['booking_id']);
            $objInvoice = new NRSInvoice($this->internalWPDB, $this->nrsPrefix, $this->lang, $bookingData['booking_id']);
            $objBooking->deleteBooking(FALSE);
            $objInvoice->deleteInvoice();
        }
        if(!isset($_SESSION['ok_msg']))
        {
            $_SESSION['ok_msg'] = "Completed:";
        }
        $_SESSION['ok_msg'] .= " All active reservations made by this customer where deleted successfully.";
        $_SESSION['ok_msg'] .= " All reserved cars by this customer are now unblocked.";
    }

    /*************************************************************************/
    /****************** MORE FUNCTIONS PART: START ***************************/
    /*************************************************************************/

    /**
     * @param $paramSearchBetween
     * @return string
     */
    public function getBackButtonHTML($paramSearchBetween)
    {
        switch($paramSearchBetween)
        {
            case "booking_date":
                $backToButton = '&nbsp;&nbsp; <input class="back-to" type="button" value="Back to Today Reservation List"
		   onClick="window.location.href=\'admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-booking-manager&tab=bookings\'"
		/>';
                break;

            case "pickup_date":
                $backToButton = '&nbsp;&nbsp; <input class="back-to" type="button" value="Back to Today Pickup List"
		   onClick="window.location.href=\'admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-booking-manager&tab=pickups\'"
		/>';
                break;

            case "dropoff_date":
                $backToButton = '&nbsp;&nbsp; <input class="back-to" type="button" value="Back to Today Dropoff List"
		   onClick="window.location.href=\'admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-booking-manager&tab=dropoffs\'"
		/>';
                break;

            case "customer_bookings":
                $backToButton = '&nbsp;&nbsp; <input class="back-to" type="button" value="Back to Customer List"
		   onClick="window.location.href=\'admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-booking-manager&tab=customers\'"
		/>';
                break;

            default:
                // Button is empty
                $backToButton = "";
        }

        return $backToButton;
    }
}