<?php
/**
 * Customer manager
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */
final class NRSCustomer
{
    protected $internalWPDB	 	= NULL;
    protected $nrsPrefix 		= "";
    protected $lang 			= NULL;
    protected $debugMode 	 	= 0;
    protected $customerId       = 0;

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - LANGUAGE
     * @param string $paramCustomerEmail
     * @param int $paramCustomerId
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramCustomerEmail = "", $paramCustomerId = 0)
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set customer id
        if($paramCustomerEmail != "")
        {
            $this->setCustomerIdByEmail($paramCustomerEmail);
        } else
        {
            $this->customerId = NRSValidator::getValidInteger($paramCustomerId);
        }
    }


    /**
     * Sets the customer id for class. Used only from class constructor
     * @param string $paramCustomerEmail
     * @return int
     */
    private function setCustomerIdByEmail($paramCustomerEmail)
    {
        $sanitizedCustomerEmail = sanitize_email($paramCustomerEmail);
        $customerId = 0;
        if(strlen($sanitizedCustomerEmail) > 0)
        {
            $customerData = $this->internalWPDB->get_row("
				SELECT customer_id
				FROM {$this->nrsPrefix}customers
				WHERE email='{$sanitizedCustomerEmail}'
			", ARRAY_A);
            if(!is_null($customerData))
            {
                $this->customerId = $customerData['customer_id'];
            }
        }

        return $customerId;
    }


    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }


    public function getCustomerId()
    {
        return $this->customerId;
    }


    public function getTitleDropDown($paramSelectedTitle, $paramRequired = FALSE)
    {
        $required = $paramRequired ? ' class="required"' : '';
        $html  = '<select name="title" id="title"'.$required.'>';
        $titleValueLabel = array(
            "Mr." 	=> $this->lang->getText('NRS_MR_TEXT'),
            "Ms." 	=> $this->lang->getText('NRS_MS_TEXT'),
            "Mrs." 	=> $this->lang->getText('NRS_MRS_TEXT'),
            "Miss." => $this->lang->getText('NRS_MISS_TEXT'),
            "Dr." 	=> $this->lang->getText('NRS_DR_TEXT'),
            "Prof." => $this->lang->getText('NRS_PROF_TEXT'),
        );

        $selected = $paramSelectedTitle == "" ? ' selected="selected"' : '';
        $html .= '<option value=""'.$selected.'>'.$this->lang->getText('NRS_SELECT_DROPDOWN_TEXT').'</option>';
        foreach($titleValueLabel as $key => $value)
        {
            if($paramSelectedTitle == $key)
            {
                $html .= '<option value="'.$key.'" selected="selected">'.$value.'</option>';
            } else
            {
                $html .= '<option value="'.$key.'">'.$value.'</option>';
            }
        }
        $html .= '</select>';
        return $html;
    }

    /**
     * Also declared in NRSCore
     * @return mixed
     */
    public function getCustomerDetails()
    {
        $validCustomerId = NRSValidator::getValidInteger($this->customerId, 0);
        $customerData = $this->internalWPDB->get_row("
           SELECT *
           FROM {$this->nrsPrefix}customers
           WHERE customer_id='{$validCustomerId}'
        ", ARRAY_A);

        if(!is_null($customerData))
        {

            $titleValueLabel = array(
                "Mr." 	=> $this->lang->getText('NRS_MR_TEXT'),
                "Ms." 	=> $this->lang->getText('NRS_MS_TEXT'),
                "Mrs." 	=> $this->lang->getText('NRS_MRS_TEXT'),
                "Miss." => $this->lang->getText('NRS_MISS_TEXT'),
                "Dr." 	=> $this->lang->getText('NRS_DR_TEXT'),
                "Prof." => $this->lang->getText('NRS_PROF_TEXT'),
            );
            $customerData['translated_title'] = "";
            $customerData['print_full_name'] = $this->lang->getText('NRS_CUSTOMER_TEXT');
            // Override if exist
            if($customerData['title'] != "" && isset($titleValueLabel[$customerData['title']]))
            {
                $customerData['translated_title'] = $titleValueLabel[$customerData['title']];
            }
            if($customerData['first_name'] != "" || $customerData['last_name'] != "")
            {
                $customerData['print_full_name'] = $customerData['translated_title']." ".$customerData['first_name']." ".$customerData['last_name'];
            }
        }

        return $customerData;
    }

    /**
     * @param array $paramCustomerDataArray
     * @param bool $paramUpdateLastVisit - do we want to update last visit time
     */
    public function saveCustomerData($paramCustomerDataArray = array(), $paramUpdateLastVisit = TRUE)
    {
        $validCustomerId = NRSValidator::getValidInteger($this->customerId, 0);
        $validTitle = isset($paramCustomerDataArray['title']) ? sanitize_text_field($paramCustomerDataArray['title']) : "SKIP";
        $validFirstName = isset($paramCustomerDataArray['first_name']) ? sanitize_text_field($paramCustomerDataArray['first_name']) : "SKIP";
        $validLastName = isset($paramCustomerDataArray['last_name']) ? sanitize_text_field($paramCustomerDataArray['last_name']) : "SKIP";
        $validBirthdate = isset($paramCustomerDataArray['birthdate']) ? sanitize_text_field($paramCustomerDataArray['birthdate']) : "SKIP";
        $validStreetAddress = isset($paramCustomerDataArray['street_address']) ? sanitize_text_field($paramCustomerDataArray['street_address']) : "SKIP";
        $validCity = isset($paramCustomerDataArray['city']) ? sanitize_text_field($paramCustomerDataArray['city']) : "SKIP";
        $validState = isset($paramCustomerDataArray['state']) ? sanitize_text_field($paramCustomerDataArray['state']) : "SKIP";
        $validZIPCode = isset($paramCustomerDataArray['zipcode']) ? sanitize_text_field($paramCustomerDataArray['zipcode']) : "SKIP";
        $validCountry = isset($paramCustomerDataArray['country']) ? sanitize_text_field($paramCustomerDataArray['country']) : "SKIP";
        $validPhone = isset($paramCustomerDataArray['phone']) ? sanitize_text_field($paramCustomerDataArray['phone']) : "SKIP";
        $validEmail = isset($paramCustomerDataArray['email']) ? sanitize_email($paramCustomerDataArray['email']) : "SKIP";
        $validAdditionalComments = isset($paramCustomerDataArray['additional_comments']) ? implode("\n", array_map('sanitize_text_field', explode("\n", $paramCustomerDataArray['additional_comments']))) : "SKIP";
        $validIP = isset($paramCustomerDataArray['ip']) ? sanitize_text_field($paramCustomerDataArray['ip']) : "0.0.0.0";

        if($validCustomerId > 0)
        {
            $existingCustomerDetails = $this->getCustomerDetails();
            $validLastVisit = 0;
            if(!is_null($existingCustomerDetails))
            {
                $validTitle                 = $validTitle == "SKIP" ? $existingCustomerDetails['title'] : $validTitle;
                $validFirstName             = $validFirstName == "SKIP" ? $existingCustomerDetails['first_name'] : $validFirstName;
                $validLastName              = $validLastName == "SKIP" ? $existingCustomerDetails['last_name'] : $validLastName;
                $validBirthdate             = $validBirthdate == "SKIP" ? $existingCustomerDetails['birthdate'] : $validBirthdate;
                $validStreetAddress         = $validStreetAddress == "SKIP" ? $existingCustomerDetails['street_address'] : $validStreetAddress;
                $validCity                  = $validCity == "SKIP" ? $existingCustomerDetails['city'] : $validCity;
                $validState                 = $validState == "SKIP" ? $existingCustomerDetails['state'] : $validState;
                $validZIPCode               = $validZIPCode == "SKIP" ? $existingCustomerDetails['zipcode'] : $validZIPCode;
                $validCountry               = $validCountry == "SKIP" ? $existingCustomerDetails['country'] : $validCountry;
                $validPhone                 = $validPhone == "SKIP" ? $existingCustomerDetails['phone'] : $validPhone;
                $validEmail                 = $validEmail == "SKIP" ? $existingCustomerDetails['email'] : $validEmail;
                $validAdditionalComments    = $validAdditionalComments == "SKIP" ? $existingCustomerDetails['additional_comments'] : $validAdditionalComments;
                $validLastVisit             = $existingCustomerDetails['last_visit_timestamp'];
            }
            $validLastVisit  = $paramUpdateLastVisit ? time() : $validLastVisit;

            // We expect that input data is valid
            $_SESSION['ok_msg'] = 'Completed: Customer details updated successfully!';
            $this->internalWPDB->query("
				UPDATE {$this->nrsPrefix}customers SET
				title = '{$validTitle}',
				first_name = '{$validFirstName}',
				last_name = '{$validLastName}',
				birthdate = '{$validBirthdate}',
				street_address = '{$validStreetAddress}',
				city = '{$validCity}',
				state = '{$validState}',
				zipcode = '{$validZIPCode}',
				country = '{$validCountry}',
				phone = '{$validPhone}',
				email = '{$validEmail}',
				additional_comments = '{$validAdditionalComments}',
				ip = '{$validIP}',
				last_visit_timestamp = '{$validLastVisit}'
				WHERE customer_id = '{$validCustomerId}'"
            );

            // Assign customer Id
            $this->customerId = $validCustomerId;
        } else
        {
            $validLastVisit  = $paramUpdateLastVisit ? time() : 0;

            // We expect that input data is valid
            $_SESSION['ok_msg'] = 'Completed: Customer added successfully!';
            $this->internalWPDB->query("
				INSERT INTO {$this->nrsPrefix}customers
				(
					title, first_name, last_name,
					birthdate,
					street_address, city, state,
					zipcode, country, phone,
					email, additional_comments, ip,
					existing_customer, registration_timestamp, last_visit_timestamp
				) values
				(
					'{$validTitle}', '{$validFirstName}', '{$validLastName}',
					'{$validBirthdate}',
					'{$validStreetAddress}', '{$validCity}' , '{$validState}',
					'{$validZIPCode}', '{$validCountry}', '{$validPhone}',
					'{$validEmail}', '{$validAdditionalComments}', '{$validIP}',
					'0', '".time()."', '{$validLastVisit}'
				)
			");
            $this->customerId = $this->internalWPDB->insert_id;
        }
    }

    public function deleteCustomer()
    {
        $validCustomerId = NRSValidator::getValidInteger($this->customerId, 0);
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}customers
            WHERE customer_id='{$validCustomerId}'
        ");
        $_SESSION['ok_msg'] = "Completed: Customer deleted successfully!";
    }
}