<?php
/**
 * NRS Extras Observer (no setup for single item)
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

class NRSExtrasObserver
{
    protected $internalWPDB = NULL;
    protected $nrsPrefix = "";
    protected $lang = NULL;
    protected $debugMode = 0;

    protected $currencySymbol		= '$';
    protected $currencyCode			= 'USD';
    protected $vatPercentage		= 0.00;
    protected $showPricesWithVAT	= 0;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        $this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
        $this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
        $this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
        $this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    /**
     * Get an extra which has amount > 0 of units in stock
     * @return mixed
     */
    public function getAvailableIds()
    {
        return $this->getIds("AVAILABLE");
    }

    public function getAllIds()
    {
        return $this->getIds("ALL");
    }

    /**
     * @param $displayMode - "ALL" or "AVAILABLE"
     * @return mixed one row (array-based) or null if not found
     */
    public function getIds($displayMode = "ALL")
    {
        $SQLWhere = $displayMode == "AVAILABLE" ? "WHERE units_in_stock > 0" : "WHERE 1";
        $searchSQL = "
            SELECT extra_id
            FROM {$this->nrsPrefix}extras
            {$SQLWhere}
            ORDER BY extra_name ASC
		";

        //DEBUG
        //echo nl2br($searchSQL)."<br /><br />";

        $searchResult = $this->internalWPDB->get_col($searchSQL);

        return $searchResult;
    }

    /**
     * @param int $paramExtraId
     * @param bool $paramShowExtraId
     * @return string
     */
    public function getExtrasDropDown($paramExtraId = 0, $paramShowExtraId = TRUE)
    {
        $extraHTML = '<option value="">'.$this->lang->getText('NRS_SELECT_DROPDOWN_TEXT').'</option>';

        $extraResult = $this->internalWPDB->get_results("
            SELECT extra_id, extra_name
            FROM {$this->nrsPrefix}extras
            ORDER BY extra_name ASC
        ", ARRAY_A);

        foreach ($extraResult AS $extraData)
        {
            $title = $extraData['extra_name'];
            if($paramShowExtraId)
            {
                $title .= " (ID=".$extraData['extra_id'].")";
            }
            if($paramExtraId == $extraData['extra_id'])
            {
                $extraHTML .= '<option value='.$extraData['extra_id'].' selected="selected">'.$title.'</option>';
            } else
            {
                $extraHTML .= '<option value='.$extraData['extra_id'].'>'.$title.'</option>';
            }
        }

        return $extraHTML;
    }

    /**
     * @param $paramDepositEnabled
     * @param $paramPrepaymentEnabled
     * @return string
     */
    public function getAdminList($paramDepositEnabled, $paramPrepaymentEnabled)
    {

        $internalConfig = array(
            "conf_currency_symbol" => $this->currencySymbol,
            "conf_currency_code" => $this->currencyCode,
            "conf_vat_percentage" => $this->vatPercentage,
            "conf_show_prices_with_vat" => $this->showPricesWithVAT,
        );

        $itemExtraList = '';
        $extraIds = $this->getAllIds();

        foreach($extraIds AS $extraId)
        {
            $objExtra = new NRSExtra($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId);
            $objPriceManager = new NRSExtraPriceManager(
                $this->internalWPDB, $this->nrsPrefix, $this->lang,
                $internalConfig, $extraId
            );
            $extraDetails = $objExtra->getFullDetails();
            $extraPriceDetails = $objPriceManager->getFullWeekCheapestDayMinimalPriceDetails();
            $extra = array_merge($extraDetails, $extraPriceDetails);

            $printUnitsRange  = '<span style="cursor:pointer;" title="Minimum - maximum extra units per booking">'.$extra['min_units_per_booking'].'-'.$extra['max_units_per_booking'].'</span> / ';
            $printUnitsRange .= '<span style="cursor:pointer;font-weight:bold" title="Total extra units in stock">'.$extra['units_in_stock'].'</span> ';
            $itemExtraList .= '<tr>';
            $itemExtraList .= '<td>'.$extra['extra_id'].'</td>';
            $itemExtraList .= '<td>'.$extra['extra_name'].'</td>';
            $itemExtraList .= '<td>'.$printUnitsRange.'</td>';
            $itemExtraList .= '<td>'.$extra['unit_print']['subtotal_price']." / ".$extra['time_extension_long_print'].'</td>';
            if($paramPrepaymentEnabled)
            {
                $itemExtraList .= '<td>'.$extra['percentage_print']['prepayment'].'</td>';
            }
            if($paramDepositEnabled)
            {
                $itemExtraList .= '<td>'.$extra['unit_print']['fixed_deposit_amount'].'</td>';
            }
            $itemExtraList .= '<td align="right">';
            $itemExtraList .= '<a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-extra&amp;extra_id='.$extraId).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
            $itemExtraList .= '<a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-extras-manager&amp;noheader=true&amp;delete_extra='.$extraId).'">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
            $itemExtraList .= '</tr>';

            //echo "<br /><br /><br />====================================================================================================";
            //echo "<br />EXTRA ID: {$extraId}<br />"; echo nl2br(print_r($extraPriceDetails, TRUE));
        }

        return  $itemExtraList;
    }

    public function getAdminBlockedList()
    {
        $getHtml = "";

        $rows = $this->internalWPDB->get_results("
            SELECT bb.booking_id, bopt.extra_id, bb.block_name,
            bb.pickup_timestamp AS start_timestamp,
            bb.dropoff_timestamp AS end_timestamp
            FROM {$this->nrsPrefix}bookings bb
            JOIN {$this->nrsPrefix}booking_options bopt ON bb.booking_id=bopt.booking_id AND bopt.extra_id!='0'
            WHERE bb.is_block='1'
        ", ARRAY_A);

        if (sizeof($rows) > 0)
        {
            $getHtml .='<tbody>';
            foreach ($rows AS $row)
            {
                $objExtra = new NRSExtra($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['extra_id']);
                $extraDetails = $objExtra->getFullDetails();
                $startTMZStamp = $row['start_timestamp'] + get_option( 'gmt_offset' ) * 3600;
                $endTMZStamp = $row['end_timestamp'] + get_option( 'gmt_offset' ) * 3600;
                $printStartDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $startTMZStamp, TRUE);
                $printEndDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $endTMZStamp, TRUE);

                $getHtml .= '<tr>
                    <td align="left">'.$row['block_name'].'</td>
                    <td>'.$row['extra_id'].'</td>
                    <td>'.$extraDetails['extra_name'].'</td>
                    <td>'.$this->lang->getText('NRS_FROM_TEXT').': '.$printStartDate.'<br />'.$this->lang->getText('NRS_TO_TEXT').': '.$printEndDate.'</td>
                    <td align="right">
                        <a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-extras-manager&amp;noheader=true&amp;action=unblock&amp;block_id='.$row['booking_id'].'&amp;extra_id='.$row['extra_id']).'">'.$this->lang->getText('NRS_ADMIN_UNBLOCK_TEXT').'</a>
                    </td>
                </tr>';
            }
            $getHtml .= '</tbody>';
        }
        return $getHtml;
    }


    /**
     * @param bool $showRowNumbers
     * @return string
     */
    public function getAdminListWithOptions($showRowNumbers = FALSE)
    {
        $extraList = '';
        $extraIds = $this->getAllIds();

        $i = 0;
        foreach($extraIds AS $extraId)
        {
            $i++;
            $objExtra = new NRSExtra($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId);
            $objOptionsManager = new NRSExtraOptionManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId);
            $optionsList = $objOptionsManager->getAdminList($showRowNumbers, sprintf('%02d', $i).".");
            $extraDetails = $objExtra->getFullDetails();
            $extraName = "";
            if(!is_null($extraDetails))
            {
                $extraName = $extraDetails['extra_name'];
            }

            if($optionsList != "")
            {
                // HTML OUTPUT
                $extraList .= '<tr>';
                $extraList .= $showRowNumbers ? '<td>'.sprintf('%02d', $i).'</td>' : '';
                $extraList .= '<td>'.$extraName.'</td>';
                $extraList .= '<td>ID: '.$extraId.'</td>';
                $extraList .= '<td>&nbsp;</td>';
                $extraList .= '</tr>';
                $extraList .= $optionsList;
            }
        }

        return  $extraList;
    }
}