<?php
/**
 * NRS Extra Price Table
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSExtrasPriceTable extends NRSPriceTable implements iNRSPriceTable
{
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
	{
		parent::__construct($paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig);
	}

	/**
	 * Get the price table
	 * Return example: priceTable = array("got_search_result" => TRUE, "extras" => array());
	 * Return example: priceTable['extras'][0]['extra_name'] = "GPS";
	 * Return example: priceTable['extras'][0]['period_list'][0]['period_from'] = "1234567890";
	 * Return example: priceTable['extras'][0]['period_list'][0]['print_dynamic_period_label'] = "10-20 Hours";
	 * @return mixed
	 */
	public function getPriceTable()
	{
		$objExtrasObserver = new NRSExtrasObserver($this->internalWPDB, $this->nrsPrefix, $this->lang, $this->savedConfig);
		$objDiscountsObserver = new NRSExtraDiscountsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$allDiscountPeriods = $objDiscountsObserver->getDiscountPeriodsOnBookingDuration();
		if(sizeof($allDiscountPeriods) == 0)
		{
			$allDiscountPeriods = array($objDiscountsObserver->getDefaultDiscountPeriod());
		}
		$extraIds = $objExtrasObserver->getAvailableIds();
		$gotSearchResult = false;
		$extras = array();
		foreach ($extraIds AS $extraId)
		{
			$objExtra = new NRSExtra($this->internalWPDB, $this->nrsPrefix, $this->lang, $extraId, $this->savedConfig);
			$objDepositManager = new NRSExtraDepositManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $this->savedConfig, $extraId);
			$extra = array_merge($objExtra->getBasicDetails(), $objDepositManager->getMinimalFixedDepositDetails());
			// Add period data to extra row
			$extra['period_list'] = $this->getPeriodsWithPrices($extraId);
			$extras[] = $extra;
			$gotSearchResult = true;
		}

        $printDynamicPeriodLabel = $this->lang->getText($this->priceCalculationType == 2 ? 'NRS_HOUR_PRICE_TEXT' : 'NRS_DAY_PRICE_TEXT');

        $priceTable = array(
			"print_periods" => $allDiscountPeriods,
			"total_periods" => sizeof($allDiscountPeriods),
            "print_dynamic_period_label" => $printDynamicPeriodLabel,
			"extras" => $extras,
			"got_search_result" => $gotSearchResult,
		);

		return $priceTable;
	}

	public function getPeriodsWithPrices($paramExtraId)
	{
		$periods = array();

		$objDiscountsObserver = new NRSExtraDiscountsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$allDiscountPeriods = $objDiscountsObserver->getDiscountPeriodsOnBookingDuration();
		if(sizeof($allDiscountPeriods) == 0)
		{
			$allDiscountPeriods = array($objDiscountsObserver->getDefaultDiscountPeriod());
		}
		foreach($allDiscountPeriods as $discountPeriod)
		{
			$objPriceManager = new NRSExtraPriceManager(
				$this->internalWPDB, $this->nrsPrefix, $this->lang,
				$this->savedConfig, $paramExtraId
			);
            // Extras has only one price for all days of week
			$minimalPriceDetails = $objPriceManager->getBasicWeekCheapestDayMinimalPriceDetails();
			$objDiscountManager = new NRSExtraDiscountManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $paramExtraId, $this->savedConfig);
			$discountPeriodFromPercentage = $objDiscountManager->getDiscountPercentageForBookingDurationByPeriod($discountPeriod['period_from']);
			$discountPeriodTillPercentage =  $objDiscountManager->getDiscountPercentageForBookingDurationByPeriod($discountPeriod['period_till']);
			$discountPercentageForBookingDuration = max($discountPeriodFromPercentage, $discountPeriodTillPercentage);
			$maxDiscountPercentageForBookingInAdvance = $objDiscountManager->getMaxDiscountPercentageForBookingInAdvance();
			$totalDiscountPercentage = $discountPercentageForBookingDuration + $maxDiscountPercentageForBookingInAdvance;
			// Protect system from max total discounts bigger than 100%
			if($totalDiscountPercentage > 100)
			{
				$totalDiscountPercentage = 100;
			}

			$finalPriceDynamic = $minimalPriceDetails['unit']['subtotal_price_dynamic'] / 100 * (100 - $discountPercentageForBookingDuration);
			$minFinalPriceDynamic = $minimalPriceDetails['unit']['subtotal_price_dynamic'] / 100 * (100 - $totalDiscountPercentage);

            // Text
            $textTotalDiscountPercentage = number_format_i18n(round($totalDiscountPercentage, 1), 1);
            $textMinFinalPriceDynamicLong = number_format_i18n($minFinalPriceDynamic, 2);
            $textMinFinalPriceDynamic = number_format_i18n($minFinalPriceDynamic, 0);

			// Durations discounts only
			$textFinalPriceDynamicLong = number_format_i18n($finalPriceDynamic, 2);
			$textFinalPriceDynamic = number_format_i18n($finalPriceDynamic, 0);
			$textDiscountPercentageForBookingDuration = number_format_i18n(round($discountPercentageForBookingDuration, 1), 1);

			$printPriceDescription = "";
			$printPriceDescription .= ($this->priceCalculationType == 2 ? $this->lang->getText('NRS_PRICE_FOR_HOUR_FROM_TEXT') : $this->lang->getText('NRS_PRICE_FOR_DAY_FROM_TEXT')).' ';
			if($maxDiscountPercentageForBookingInAdvance > 0)
			{
				$printPriceDescription .= $this->currencySymbol.' '.$textMinFinalPriceDynamicLong.'-'.$textFinalPriceDynamicLong.' ';
				$printPriceDescription .= $this->lang->getText('NRS_PRICE_WITH_APPLIED_TEXT').' ';
				$printPriceDescription .= $textDiscountPercentageForBookingDuration.'-'.$textTotalDiscountPercentage.' % ';
                $printPriceDescription .= $this->lang->getText('NRS_WITH_APPLIED_DISCOUNT_TEXT');
				$printPrice = $textMinFinalPriceDynamic.'-'.$textFinalPriceDynamic.' '.$this->currencyCode;
			} else
			{
				$printPriceDescription = $this->currencySymbol.' '.$textFinalPriceDynamicLong.', ';
                $printPriceDescription .= $this->lang->getText('NRS_PRICE_WITH_APPLIED_TEXT').' '.$textDiscountPercentageForBookingDuration.' % ';
                $printPriceDescription .= $this->lang->getText('NRS_WITH_APPLIED_DISCOUNT_TEXT');
				$printPrice = $textFinalPriceDynamic.' '.$this->currencyCode;
				// Durations discounts only
			}
			$discountPeriod['print_price_description'] = $printPriceDescription;
			$discountPeriod['print_price'] = $printPrice;
			$periods[] = $discountPeriod;

			if($this->debugMode)
			{
				echo "-&gt;  Extra Id: {$paramExtraId} , ";
				echo "Dynamic Price (full): {$minimalPriceDetails['unit']['subtotal_price_dynamic']}, ";
				echo "Dynamic Price (disc.): {$minFinalPriceDynamic}, ";
				echo "Discount (dur.+adv.): {$discountPercentageForBookingDuration} + {$maxDiscountPercentageForBookingInAdvance} = {$totalDiscountPercentage}, ";
				echo "Period (from/to): {$discountPeriod['period_from']} - {$discountPeriod['period_till']}<br />";
				//echo nl2br(print_r($minimalPriceDetails, TRUE))."<br />";
				//echo "=================================================================================================<br /><br />";
			}
		}

		return $periods;
	}
}