<?php
/**
 * NRS Initializer class to setup, update, install and load templates for plugin
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final Class NRSInitializer
{
	protected $internalWPDB = NULL;
	protected $nrsPrefix 	= "";
	protected $lang 		= NULL;
	protected $debugMode 	= 0;

	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage $paramNRSLang)
	{
		// Set class settings
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;
	}

	/**
	 * Is the NRS database version is newer or same as code version. If no - launch updater
	 * @param float $paramCodeVersion
	 * @return bool
	 */
	public function isDatabaseVersionUpToDate($paramCodeVersion)
	{
		$sql = "
				SELECT conf_value AS plugin_version
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_plugin_version'
			";
		$pluginVersionFieldValue = $this->internalWPDB->get_var($sql);

		$nrsDBVersion = 3.2;
		if(!is_null($pluginVersionFieldValue))
		{
			$nrsDBVersion = floatval($pluginVersionFieldValue);
		}
		// DEBUG
		//echo "DB VERSION: {$nrsDBVersion}<br />";
		//echo "CODE VERSION: {$codeVersion}<br />";

		return $nrsDBVersion >= $paramCodeVersion ? TRUE : FALSE;
	}

	public function install()
	{
		$WPError = false;

		// Language file already loaded, so we can use translated text
		// Create terms of use page post object
		$arrCancellationPage = array(
			'post_title'    => $this->lang->getText('NRS_CANCELLED_PAYMENT_PAGE_TITLE_TEXT'),
			'post_content'  => $this->lang->getText('NRS_CANCELLED_PAYMENT_PAGE_CONTENT_TEXT'),
			'post_status'   => 'publish',
			'post_type'     => $GLOBALS['NRS_EXTENSION'],
			/*'post_author'   => 1,*/ /*auto assign current user*/
			/*'post_category' => array(8,39)*/ /*no categories needed here*/
		);

		// Create terms of use page post object
		$arrConfirmationPage = array(
			'post_title'    => $this->lang->getText('NRS_CONFIRMATION_PAGE_TITLE_TEXT'),
			'post_content'  => $this->lang->getText('NRS_CONFIRMATION_PAGE_CONTENT_TEXT'),
			'post_status'   => 'publish',
			'post_type'     => $GLOBALS['NRS_EXTENSION'],
			/*'post_author'   => 1,*/ /*auto assign current user*/
			/*'post_category' => array(8,39)*/ /*no categories needed here*/
		);

		// Create terms of use page post object
		$arrTermsOfUsePage = array(
			'post_title'    => $this->lang->getText('NRS_TOU_PAGE_TITLE_TEXT'),
			'post_content'  => $this->lang->getText('NRS_TOU_PAGE_CONTENT_TEXT'),
			'post_status'   => 'publish',
			'post_type'     => $GLOBALS['NRS_EXTENSION'],
			/*'post_author'   => 1,*/ /*auto assign current user*/
			/*'post_category' => array(8,39)*/ /*no categories needed here*/
		);

		// Insert corresponding post as post type 'post'
		$newCancellationPageId = wp_insert_post( $arrCancellationPage, $WPError );
		$newConfirmationPageId = wp_insert_post( $arrConfirmationPage, $WPError );
		$newTermsOfUsePageId = wp_insert_post( $arrTermsOfUsePage, $WPError );

		// First - drop all tables if exists any
		// NOTE: This helps us to have a clean install as expected
		$this->dropAllTables();

		$sqlList = $this->getAllTablesSQL($newCancellationPageId, $newConfirmationPageId, $newTermsOfUsePageId);
		// Insert the install sql
		foreach($sqlList AS $sql)
		{
			$this->internalWPDB->query($sql);
		}

		// Flush the urls
		$this->installRewriteFlush();

	}

	public function uninstall()
	{
		/* ********* WORDPRESS PAGE DELETE: START *********** */
		$args = array(
			'posts_per_page' => -1,
			'post_type' => $GLOBALS['NRS_EXTENSION'],
		);
		$posts = get_posts( $args );
		$wpNRSForcePageDelete = true;
		if (is_array($posts))
		{
			foreach ($posts AS $post)
			{
				// delete posts
				wp_delete_post( $post->ID, $wpNRSForcePageDelete);
				//echo "Deleted Post: ".$post->title."\r\n";
			}
		}
		/* ********* WORDPRESS PAGE DELETE: END ************ */

		// Drop all plugin tables
		$this->dropAllTables();
	}

	private function getAllTablesSQL($cancellationPageId, $confirmationPageId, $termsOfUsePageId)
	{
		$pluginVersion = number_format(floatval($GLOBALS['NRS_VERSION']), 1,'.','');
		$extTitle = ucfirst($GLOBALS['NRS_EXTENSION']);
		// We don't want to call NRSValidator here, because it might not have been initialized yet
		$validCancellationPageId = intval($cancellationPageId);
		$validConfirmationPageId = intval($confirmationPageId);
		$validTermsOfUsePageId = intval($termsOfUsePageId);
		$arrSQL = array();

		// Plugin-related tables SQL's
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}api_log` (
			  `log_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `email` varchar(128) NOT NULL,
			  `year` smallint(4) unsigned NOT NULL DEFAULT '0',
			  `year_required` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `ip` varchar(32) NOT NULL DEFAULT '0.0.0.0',
			  `real_ip` varchar(32) NOT NULL DEFAULT '0.0.0.0',
			  `host` varchar(255) NOT NULL,
			  `agent` varchar(255) NOT NULL,
			  `browser` varchar(50) NOT NULL,
			  `os` varchar(50) NOT NULL,
			  `total_requests_left` int(11) NOT NULL DEFAULT '1',
			  `failed_requests_left` int(11) NOT NULL DEFAULT '1',
			  `email_attempts_left` int(11) NOT NULL DEFAULT '1',
			  `is_robot` tinyint(1) unsigned NOT NULL,
			  `status` tinyint(1) unsigned NOT NULL DEFAULT '2',
			  `log_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  PRIMARY KEY (`log_id`),
			  KEY `email` (`email`),
			  KEY `year` (`year`),
			  KEY `year_required` (`year_required`),
			  KEY `ip` (`ip`),
			  KEY `real_ip` (`real_ip`),
			  KEY `is_robot` (`is_robot`),
			  KEY `status` (`status`),
			  KEY `log_timestamp` (`log_timestamp`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}body_types` (
			  `body_type_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `body_type_title` varchar(255) NOT NULL,
			  PRIMARY KEY (`body_type_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";


		// The NULL value, in databases and programming, is the equivalent of saying that the field has no value (or it is unknown).
		// NOTE: 'DEFAULT NULL' allows for us to have unique columns with NULL value
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}bookings` (
			  `booking_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `booking_code` varchar(25) DEFAULT NULL,
			  `booking_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `last_edit_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `pickup_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `dropoff_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `pickup_location_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `dropoff_location_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `body_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `transmission_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `fuel_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `customer_id` int(11)unsigned NOT NULL DEFAULT '0',
			  `payment_method_code` varchar(25) DEFAULT NULL,
			  `payment_success` tinyint(1) NOT NULL DEFAULT '0',
			  `payment_txnid` varchar(100) DEFAULT NULL,
			  `paypal_email` varchar(255) DEFAULT NULL,
			  `is_block` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `is_cancelled` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `block_name` varchar(255) DEFAULT NULL,
			  PRIMARY KEY (`booking_id`),
			  UNIQUE KEY `booking_code` (`booking_code`),
			  KEY `customer_id` (`customer_id`),
			  KEY `booking_timestamp` (`booking_timestamp`),
			  KEY `last_edit_timestamp` (`last_edit_timestamp`),
			  KEY `pickup_timestamp` (`pickup_timestamp`),
			  KEY `dropoff_timestamp` (`dropoff_timestamp`),
			  KEY `is_cancelled` (`is_cancelled`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		// Make sure we allow units booked to be SIGNED and UNSIGNED, because of possible value -1, which stands for ALL
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}booking_options` (
			  `booking_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `extra_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `option_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `units_booked` int(11) NOT NULL DEFAULT '1',
			  KEY `booking_id` (`booking_id`),
			  KEY `item_id` (`item_id`),
			  KEY `extra_id` (`extra_id`),
			  KEY `option_id` (`option_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}closed_dates` (
			  `closed_date` date NOT NULL DEFAULT '0000-00-00',
			  KEY `closed_date` (`closed_date`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}customers` (
			  `customer_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `title` varchar(16) NOT NULL,
			  `first_name` varchar(64) NOT NULL,
			  `last_name` varchar(64) NOT NULL,
			  `birthdate` date NOT NULL DEFAULT '0000-00-00',
			  `street_address` varchar(255) NOT NULL,
			  `city` varchar(64) NOT NULL,
			  `state` varchar(128) NOT NULL,
			  `zipcode` varchar(64) NOT NULL,
			  `country` varchar(64) NOT NULL,
			  `phone` varchar(64) NOT NULL,
			  `email` varchar(128) NOT NULL,
			  `additional_comments` text NOT NULL,
			  `ip` varchar(32) NOT NULL,
			  `existing_customer` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `registration_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `last_visit_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  PRIMARY KEY (`customer_id`),
			  KEY `email` (`email`),
			  KEY `registration_timestamp` (`registration_timestamp`),
			  KEY `last_visit_timestamp` (`last_visit_timestamp`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}deposits` (
			  `deposit_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_from` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_till` int(11) unsigned NOT NULL DEFAULT '0',
			  `fixed_rental_deposit` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  PRIMARY KEY (`deposit_id`),
			  KEY `item_id` (`item_id`),
			  KEY `period` (`period_from`, `period_till`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}discounts` (
			  `discount_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `discount_type` int(11) unsigned NOT NULL DEFAULT '1',
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `extra_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_from` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_till` int(11) unsigned NOT NULL DEFAULT '0',
			  `discount_percentage` decimal(10,3) unsigned NOT NULL DEFAULT '0.000',
			  PRIMARY KEY (`discount_id`),
			  KEY `discount_type` (`discount_type`),
			  KEY `item_id` (`item_id`),
			  KEY `extra_id` (`extra_id`),
			  KEY `period` (`period_from`, `period_till`),
			  KEY `discount_percentage` (`discount_percentage`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}email_contents` (
			  `email_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `email_subject` varchar(255) NOT NULL,
			  `email_body` longtext NOT NULL,
			  PRIMARY KEY (`email_id`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "INSERT INTO `{$this->nrsPrefix}email_contents` (`email_id`, `email_subject`, `email_body`) VALUES
				(1, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_DETAILS_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n\n[INVOICE]\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
				(2, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_CONFIRMED_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_PAYMENT_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n\n[INVOICE]\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
				(3, '".$this->lang->getText('NRS_EMAIL_TITLE_RESERVATION_CANCELLED_TEXT')."', '".$this->lang->getText('NRS_DEAR_TEXT').", [CUSTOMER_NAME],\n\n".$this->lang->getText('NRS_EMAIL_BODY_YOUR_RESERVATION_DETAILS_TEXT')."\n\n".$this->lang->getText('NRS_REGARDS_TEXT').",\n[PORTAL_NAME],\n[PORTAL_PHONE],\n[PORTAL_EMAIL]'),
				(4, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_DETAILS_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_RECEIVED_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n[INVOICE]'),
				(5, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_CONFIRMED_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_PAID_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_DETAILS_TEXT')."\n[INVOICE]'),
				(6, '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_TITLE_RESERVATION_CANCELLED_TEXT')."', '".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_RESERVATION_CANCELLED_TEXT')."\n\n".$this->lang->getText('NRS_NOTIFICATION_EMAIL_BODY_CANCELLED_RESERVATION_DETAILS_TEXT')."\n[INVOICE]')";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}extras` (
			  `extra_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `extra_name` varchar(255) NOT NULL,
			  `price` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `price_type` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `prepayment_percentage` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `fixed_rental_deposit` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `units_in_stock` int(11) unsigned NOT NULL DEFAULT '1',
			  `min_units_per_booking` int(11) unsigned NOT NULL DEFAULT '0',
			  `max_units_per_booking` int(11) unsigned NOT NULL DEFAULT '1',
			  `options_display_mode` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `measurement_unit` varchar(25) NOT NULL,
			  PRIMARY KEY (`extra_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}features` (
			  `feature_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `feature_title` varchar(255) NOT NULL,
			  `display_in_item_list` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  PRIMARY KEY (`feature_id`),
			  KEY `display_in_item_list` (`display_in_item_list`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}fuel_types` (
			  `fuel_type_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `fuel_type_title` varchar(255) NOT NULL,
			  PRIMARY KEY (`fuel_type_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}invoices` (
			  `booking_id` int(11) unsigned NOT NULL,
			  `customer_name` varchar(255) NOT NULL,
			  `customer_email` varchar(255) NOT NULL,
			  `grand_total` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `fixed_deposit_amount` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `total_pay_now` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `pickup_location` varchar(255) NOT NULL,
			  `dropoff_location` varchar(255) NOT NULL,
			  `invoice` longtext NOT NULL,
			  PRIMARY KEY (`booking_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = 'INSERT INTO `'.$this->nrsPrefix.'invoices` (`booking_id`, `customer_name`, `customer_email`, `grand_total`, `fixed_deposit_amount`, `total_pay_now`, `pickup_location`, `dropoff_location`, `invoice`) VALUES
(0, \'Mr. John Smith\', \'john.smith@gmail.com\', 189.24, 230.00, 189.24, \'2. San Francisco Intl. Airport (SFO) - Hwy 101, San Francisco, CA 94128\', \'3. Oakland Intl. Airport (OAK) - 1 Airport Dr, Oakland, CA 94621\', \'<table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background-color:#eeeeee;width:840px;border:none" cellpadding="5" cellspacing="1">
    <tbody>
    <tr>
        <td align="left" style="font-weight:bold;background-color:#eeeeee;padding-left:5px" colspan="2">Customer Details</td>
    </tr>
    <tr>
        <td align="left" style="width:160px;background-color:#ffffff;padding-left:5px">Reservation code</td>
        <td align="left" style="background-color:#ffffff;padding-left:5px">R1AHKECN</td>
    </tr>
            <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Customer</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Mr. John Smith</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Date of Birth</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">1980-06-09</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Address</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">625 2nd Street</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">City</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">San Francisco</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">State</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">CA</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Postal Code</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">94107</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Country</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">United States</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Phone</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">+14506004790</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Email</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">john.smith@gmail.com</td>
        </tr>
                <tr>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Additional Comments</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Please leave car keys by the front door upon delivery.</td>
        </tr>
        </tbody>
</table>
<br />
<table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background:#999999;width:840px;border:none" cellpadding="5" cellspacing="1">
<tbody>
<tr>
    <td align="left" style="font-weight:bold;background-color:#eeeeee;padding-left:5px" colspan="3">Rental Details</td>
</tr>
            <tr style="background-color:#343434;color: white" class="location-headers">
            <td align="left" class="col1" style="padding-left:5px"><strong>Pick-up Location</strong></td>
            <td align="left" class="col2" style="padding-left:5px" colspan="2"><strong>Drop-Off Location</strong></td>
        </tr>

    <tr style="background-color:#FFFFFF" class="location-details">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1">
		2. San Francisco Intl. Airport (SFO) - Hwy 101, San Francisco, CA 94128            </td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="2">
		3. Oakland Intl. Airport (OAK) - 1 Airport Dr, Oakland, CA 94621            </td>
            </tr>


    <tr style="background-color:#343434;color: white" class="duration-headers">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1"><strong>Pick-up Date &amp; Time</strong></td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="1"><strong>Drop-off Date &amp; Time</strong></td>
            <td align="right" class="col3" style="padding-right:5px"><strong>Period</strong></td>
            </tr>

    <tr style="background-color:#FFFFFF" class="duration-details">
                    <td align="left" class="col1" style="padding-left:5px" colspan="1">
		October 16, 2015 &nbsp;&nbsp; 11:00 AM            </td>
                            <td align="left" class="col2" style="padding-left:5px" colspan="1">
		October 19, 2015 &nbsp;&nbsp; 10:00 PM            </td>
            <td align="right" class="col3" style="padding-right:5px">
		4 days            </td>
            </tr>

<!-- ITEMS -->
    <tr class="item-headers" style="background-color:#343434;color: white">
        <td align="left" class="col1" style="padding-left:5px"><strong>Selected Cars</strong></td>
        <td align="left" class="col2" style="padding-left:5px"><strong>Price</strong></td>
        <td align="right" class="col3" style="padding-right:5px"><strong>Total</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="items">
        <td align="left" class="col1" style="padding-left:5px">
		Mazda 6, Intermediate        </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 115.72 w/o Tax + $ 10.41 Tax = 1 x $ 126.14" style="cursor:pointer">
		$ 115.72            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 115.72 w/o Tax + $ 10.41 Tax = $ 126.14" style="cursor:pointer">
		$ 115.72            </span>
        </td>
    </tr>

<!-- PICKUP FEES -->
    <tr style="background-color:#343434;color: white" class="office-fee-headers">
        <td align="left" class="col1" style="padding-left:5px" colspan="3"><strong>Location Fees</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="office-fees">
        <td align="left" class="col1" style="padding-left:5px">Pick-up fee                    </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 7.44 w/o Tax + $ 0.67 Tax = 1 x $ 8.11" style="cursor:pointer">
		$ 7.44            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 7.44 w/o Tax + $ 0.67 Tax = $ 8.11" style="cursor:pointer">
		$ 7.44            </span>
        </td>
    </tr>



<!-- DROPOFF FEES -->
    <tr style="background-color:#FFFFFF" class="office-fees">
        <td align="left" class="col1" style="padding-left:5px">Drop-off fee           (Nightly rate applied)        </td>
        <td align="left" class="col2" style="padding-left:5px">
            <span title="1 vehicle(s) x $ 23.97 w/o Tax + $ 2.16 Tax = 1 x $ 26.13" style="cursor:pointer">
		$ 23.97            </span>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="$ 23.97 w/o Tax + $ 2.16 Tax = $ 26.13" style="cursor:pointer">
		$ 23.97            </span>
        </td>
    </tr>

<!-- EXTRAS -->
    <tr class="extra-headers" style="background-color:#343434;color: white">
       <td align="left" class="col1" colspan="3"><strong>Rental Options</strong></td>
    </tr>
    <tr style="background-color:#FFFFFF" class="extras">
         <td align="left" class="col1" style="padding-left:5px">Baby Seat</td>
         <td align="left" class="col2" style="padding-left:5px">
             <span title="1 extra(s) x $ 13.24 w/o Tax + $ 1.19 Tax = 1 x $ 14.43" style="cursor:pointer">
		$ 13.24             </span>
         </td>
         <td align="right" class="col3" style="padding-right:5px">
             <span title="$ 13.24 w/o Tax + $ 1.19 Tax = $ 14.43" style="cursor:pointer">
		$ 13.24            </span>
         </td>
    </tr>
    <tr style="background-color:#FFFFFF" class="extras">
         <td align="left" class="col1" style="padding-left:5px">GPS</td>
         <td align="left" class="col2" style="padding-left:5px">
             <span title="1 extra(s) x $ 13.24 w/o Tax + $ 1.19 Tax = 1 x $ 14.43" style="cursor:pointer">
		$ 13.24             </span>
         </td>
         <td align="right" class="col3" style="padding-right:5px">
             <span title="$ 13.24 w/o Tax + $ 1.19 Tax = $ 14.43" style="cursor:pointer">
		$ 13.24            </span>
         </td>
    </tr>


<!-- TOTAL -->
<tr style="background-color:#343434;color: white" class="total-headers">
    <td align="left" class="col1" colspan="3" style="padding-left:5px"><strong>Total</strong></td>
</tr>
    <tr style="background-color:#FFFFFF">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
            <strong>Sub Total:</strong>
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <strong>$ 173.61</strong>
        </td>
    </tr>
    <tr style="background-color:#f2f2f2">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
		Tax (9.00 %):
        </td>
        <td align="right" class="col3" style="padding-right:5px">
		$ 15.62        </td>
    </tr>
<tr style="background-color:#FFFFFF">
    <td align="right" class="col1" style="padding-right:5px" colspan="2">
        <strong>Grand Total:</strong>
    </td>
    <td align="right" class="col3" style="padding-right:5px">
        <strong>$ 189.24</strong>
    </td>
</tr>
    <tr style="background-color:#f2f2f2">
        <td align="right" class="col1" style="padding-right:5px" colspan="2">
		Deposit:
        </td>
        <td align="right" class="col3" style="padding-right:5px">
            <span title="Cars Deposit ($ 230.00) + Extras Deposit ($ 0.00) = $ 230.00" style="cursor:pointer">
		$ 230.00            </span>
        </td>
    </tr>
</tbody>
</table>

<!-- PAYMENT METHOD DETAILS -->
    <br />
    <table style="font-family:Verdana, Geneva, sans-serif;font-size: 12px;background:#999999;width:840px;border:none" cellpadding="4" cellspacing="1">
        <tr>
            <td align="left" colspan="2" style="font-weight:bold;background-color:#eeeeee;padding-left:5px">Payment Details</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Pay By</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Bank Transfer</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Payment Details</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">Receiver: NATIVE RENTAL, LTD<br />
Account no.: US27 7300 0204 2870 6432<br />
Bank: Bank of America, Inc.</td>
        </tr>
        <tr>
            <td align="left" width="30%" style="font-weight:bold;background-color:#ffffff;padding-left:5px">Transaction ID</td>
            <td align="left" style="background-color:#ffffff;padding-left:5px">N/A</td>
        </tr>
    </table>
\')';

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}items` (
			  `item_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `item_description_page_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `manufacturer_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `body_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `transmission_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `fuel_type_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `model_name` varchar(255) NOT NULL,
			  `item_image_1` varchar(255) NOT NULL,
			  `item_image_2` varchar(255) NOT NULL,
			  `item_image_3` varchar(255) NOT NULL,
			  `demo_image_1` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `demo_image_2` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `demo_image_3` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `mileage` varchar(50) NOT NULL,
			  `fuel_consumption` varchar(255) NOT NULL,
			  `engine_capacity` varchar(255) NOT NULL,
			  `max_passengers` int(11) unsigned NOT NULL DEFAULT '5',
			  `max_luggage` int(11) unsigned NOT NULL DEFAULT '2',
			  `item_doors` int(11) unsigned NOT NULL DEFAULT '5',
			  `min_driver_age` int(11) unsigned NOT NULL DEFAULT '18',
			  `units_in_stock` int(11) unsigned NOT NULL DEFAULT '1',
			  `max_units_per_booking` int(11) unsigned NOT NULL DEFAULT '1',
			  `enabled` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `display_in_slider` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `display_in_item_list` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `display_in_price_table` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `display_in_calendar` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `options_display_mode` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `measurement_unit` varchar(25) NOT NULL,
			  PRIMARY KEY (`item_id`),
			  KEY `item_description_page_id` (`item_description_page_id`),
			  KEY `manufacturer_id` (`manufacturer_id`),
			  KEY `body_type_id` (`body_type_id`),
			  KEY `fuel_type_id` (`fuel_type_id`),
			  KEY `transmission_type_id` (`transmission_type_id`),
			  KEY `units_in_stock` (`units_in_stock`),
			  KEY `max_units_per_booking` (`max_units_per_booking`),
			  KEY `enabled` (`enabled`),
			  KEY `display_in_slider` (`display_in_slider`),
			  KEY `display_in_item_list` (`display_in_item_list`),
			  KEY `display_in_price_table` (`display_in_price_table`),
			  KEY `display_in_calendar` (`display_in_calendar`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}item_features` (
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `feature_id` int(11) unsigned NOT NULL DEFAULT '0',
			  KEY `item_id` (`item_id`),
			  KEY `feature_id` (`feature_id`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		// Note for indexes
		// As per review we either search for:
		// a) ITEM_ID+(LOCATION_ID & LOCATION_TYPE)
		// b) (LOCATION_ID & LOCATION_TYPE) - that's why we need joined index for 2 columns
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}item_locations` (
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `location_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `location_type` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  KEY `item_id` (`item_id`),
			  KEY `location` (`location_id`, `location_type`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		// NOTE: We must allow to be SIGN for afterhours_pickup_location_id and afterhours_dropoff_location_id
		// to support '-1' value, for 'in same branch' location
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}locations` (
			  `location_id` int(11) NOT NULL AUTO_INCREMENT,
			  `location_title` varchar(255) NOT NULL,
			  `location_address` varchar(255) NOT NULL,
			  `pickup_fee` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `dropoff_fee` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `open_mondays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_tuesdays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_wednesdays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_thursdays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_fridays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_saturdays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_sundays` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `open_time_mon` time NOT NULL DEFAULT '08:00:00',
			  `open_time_tue` time NOT NULL DEFAULT '08:00:00',
			  `open_time_wed` time NOT NULL DEFAULT '08:00:00',
			  `open_time_thu` time NOT NULL DEFAULT '08:00:00',
			  `open_time_fri` time NOT NULL DEFAULT '08:00:00',
			  `open_time_sat` time NOT NULL DEFAULT '08:00:00',
			  `open_time_sun` time NOT NULL DEFAULT '08:00:00',
			  `close_time_mon` time NOT NULL DEFAULT '19:00:00',
			  `close_time_tue` time NOT NULL DEFAULT '19:00:00',
			  `close_time_wed` time NOT NULL DEFAULT '19:00:00',
			  `close_time_thu` time NOT NULL DEFAULT '19:00:00',
			  `close_time_fri` time NOT NULL DEFAULT '19:00:00',
			  `close_time_sat` time NOT NULL DEFAULT '19:00:00',
			  `close_time_sun` time NOT NULL DEFAULT '19:00:00',
			  `afterhours_pickup_location_id` int(11) NOT NULL DEFAULT '0',
			  `afterhours_pickup_fee` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `afterhours_dropoff_location_id` int(11) NOT NULL DEFAULT '0',
			  `afterhours_dropoff_fee` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  PRIMARY KEY (`location_id`),
			  KEY `afterhours_pickup_location_id` (`afterhours_pickup_location_id`),
			  KEY `afterhours_dropoff_location_id` (`afterhours_dropoff_location_id`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}manufacturers` (
			  `manufacturer_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `manufacturer_title` varchar(255) NOT NULL,
			  PRIMARY KEY (`manufacturer_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}options` (
			  `option_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `extra_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `option_name` varchar(255) NOT NULL,
			  PRIMARY KEY (`option_id`),
			  KEY `item_id` (`item_id`),
			  KEY `extra_id` (`extra_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		// We should allow unpaid_booking_expiration_time to be SIGNED, for potential '-1' value for 'never expires' situation
		// NOTE: payment_method_code is an unique identifier
		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}payment_methods` (
			  `payment_method_code` varchar(25) NOT NULL,
			  `method_name` varchar(255) NOT NULL,
			  `method_details` varchar(255) NOT NULL,
			  `method_enabled` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  `method_order` int(11) unsigned NOT NULL DEFAULT '0',
			  `unpaid_booking_expiration_time` int(11) NOT NULL DEFAULT '0',
			  UNIQUE KEY `payment_method_code` (`payment_method_code`),
			  KEY `method_enabled` (`method_enabled`),
			  KEY `method_order` (`method_order`),
			  KEY `unpaid_booking_expiration_time` (`unpaid_booking_expiration_time`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "INSERT INTO `{$this->nrsPrefix}payment_methods`
				(`payment_method_code`, `method_name`, `method_details`, `method_enabled`, `method_order`, `unpaid_booking_expiration_time`)
				VALUES
				('pp', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_PAYPAL_TEXT')."', 'yourpaypal@email.com', 1, 1, 600),
				('bank', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_BANK_TEXT')."', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_BANK_DETAILS_TEXT')."', 1, 2, 604800),
				('phone', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_PAY_OVER_THE_PHONE_TEXT')."', '+1 415 000 0000', 1, 3, 604800),
				('poa', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_PAY_ON_ARRIVAL_TEXT')."', '".$this->lang->getText('NRS_PAYMENT_GATEWAY_PAY_ON_ARRIVAL_DETAILS_TEXT')."', 1, 4, 0);";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}prepayments` (
			  `prepayment_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_from` int(11) unsigned NOT NULL DEFAULT '0',
			  `period_till` int(11) unsigned NOT NULL DEFAULT '0',
			  `prepayment_percentage` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  PRIMARY KEY (`prepayment_id`),
			  KEY `item_id` (`item_id`),
			  KEY `period` (`period_from`, `period_till`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "INSERT INTO `{$this->nrsPrefix}prepayments` (`item_id`, `period_from`, `period_till`, `prepayment_percentage`) VALUES ('0', '0', '31622399', '100.00')";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}price_plans` (
			  `plan_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `item_id` int(11) unsigned NOT NULL DEFAULT '0',
			  `start_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `end_timestamp` int(11) unsigned NOT NULL DEFAULT '0',
			  `price_type` tinyint(1) unsigned NOT NULL DEFAULT '1',
			  `mon` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `tue` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `wed` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `thu` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `fri` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `sat` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `sun` decimal(10,2) unsigned NOT NULL DEFAULT '0.00',
			  `seasonal_price` tinyint(1) unsigned NOT NULL DEFAULT '0',
			  PRIMARY KEY  (`plan_id`),
			  KEY `item_id` (`item_id`),
			  KEY `period` (`start_timestamp`, `end_timestamp`),
			  KEY `price_type` (`price_type`),
			  KEY `seasonal_price` (`seasonal_price`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}settings` (
			  `conf_key` varchar(100) NOT NULL,
			  `conf_value` varchar(255) NOT NULL,
			  UNIQUE KEY `conf_key` (`conf_key`)
			) ENGINE=InnoDB  DEFAULT CHARSET=utf8;";

		$arrSQL[] = "INSERT INTO `{$this->nrsPrefix}settings` (`conf_key`, `conf_value`) VALUES
			('conf_portal_name', '{$extTitle} Rental System'),
			('conf_portal_street_address', '99th Street Road'),
			('conf_portal_city', 'Your City'),
			('conf_portal_state', 'Your State'),
			('conf_portal_country', 'Your Country'),
			('conf_portal_zipcode', '00000'),
			('conf_portal_phone', '+1 415 000 0000'),
			('conf_portal_email', 'info@yourdomain.com'),
			('conf_short_date_format', 'm/d/Y'),
			('conf_currency_symbol', '$'),
			('conf_currency_code', 'USD'),
			('conf_vat_percentage', '9'),
			('conf_show_prices_with_vat', '0'),
			('conf_minimum_booking_period', '28800'),
			('conf_maximum_booking_period', '31536000'),
			('conf_send_emails', '1'),
			('conf_paypal_sandbox_mode', '0'),
			('conf_minimum_block_period_between_bookings', '7199'),
			('conf_price_calculation_type', '1'),
			('conf_minimum_period_until_pickup', '86400'),
			('conf_noon_time', '12:00:00'),
			('conf_recaptcha_enabled', '0'),
			('conf_recaptcha_site_key', ''),
			('conf_recaptcha_secret_key', ''),
			('conf_api_max_requests_per_period', '50'),
			('conf_api_max_failed_requests_per_period', '3'),
			('conf_cancelled_payment_page_id', '{$validCancellationPageId}'),
			('conf_confirmation_page_id', '{$validConfirmationPageId}'),
			('conf_tos_page_id', '{$validTermsOfUsePageId}'),
			('conf_system_style', '1'),
			('conf_measurement_unit', 'Mi'),
			('conf_plugin_version', '{$pluginVersion}'),
			('conf_updated', '0'),
			('conf_classify_items', '1'),
			('conf_booking_model', '1'),
			('conf_search_enabled', '1'),
			('conf_deposit_enabled', '1'),
			('conf_discount_enabled', '1'),
			('conf_prepayment_enabled', '1'),

			('conf_customer_title_visible', '1'),
			('conf_customer_first_name_visible', '1'),
			('conf_customer_last_name_visible', '1'),
			('conf_customer_birthdate_visible', '1'),
			('conf_customer_street_address_visible', '1'),
			('conf_customer_city_visible', '1'),
			('conf_customer_state_visible', '1'),
			('conf_customer_zipcode_visible', '1'),
			('conf_customer_country_visible', '1'),
			('conf_customer_phone_visible', '1'),
			('conf_customer_email_visible', '1'),
			('conf_customer_additional_comments_visible', '1'),

			('conf_customer_title_required', '1'),
			('conf_customer_first_name_required', '1'),
			('conf_customer_last_name_required', '1'),
			('conf_customer_birthdate_required', '1'),
			('conf_customer_street_address_required', '1'),
			('conf_customer_city_required', '1'),
			('conf_customer_state_required', '0'),
			('conf_customer_zipcode_required', '0'),
			('conf_customer_country_required', '0'),
			('conf_customer_phone_required', '1'),
			('conf_customer_email_required', '1'),
			('conf_customer_additional_comments_required', '0'),

			('conf_search_pickup_location_visible', '1'),
			('conf_search_pickup_date_visible', '1'),
			('conf_search_dropoff_location_visible', '1'),
			('conf_search_dropoff_date_visible', '1'),
			('conf_search_body_type_visible', '1'),
			('conf_search_transmission_type_visible', '0'),
			('conf_search_fuel_type_visible', '1'),
			('conf_search_booking_code_visible', '0'),

			('conf_search_pickup_location_required', '1'),
			('conf_search_pickup_date_required', '1'),
			('conf_search_dropoff_location_required', '1'),
			('conf_search_dropoff_date_required', '1'),
			('conf_search_body_type_required', '0'),
			('conf_search_transmission_type_required', '0'),
			('conf_search_fuel_type_required', '0'),
			('conf_search_booking_code_required', '0')";

		$arrSQL[] = "CREATE TABLE `{$this->nrsPrefix}transmission_types` (
			  `transmission_type_id` int(11) unsigned NOT NULL AUTO_INCREMENT,
			  `transmission_type_title` varchar(255) NOT NULL,
			  PRIMARY KEY (`transmission_type_id`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8;";

		return $arrSQL;
	}

	/**
	 * Drop all tables if exists
	 */
	private function dropAllTables()
	{
		$sql = "
			DROP TABLE IF EXISTS
				`{$this->nrsPrefix}api_log`,
				`{$this->nrsPrefix}body_types`,
				`{$this->nrsPrefix}bookings`,
				`{$this->nrsPrefix}booking_options`,
				`{$this->nrsPrefix}closed_dates`,
				`{$this->nrsPrefix}customers`,
				`{$this->nrsPrefix}deposits`,
				`{$this->nrsPrefix}discounts`,
				`{$this->nrsPrefix}email_contents`,
				`{$this->nrsPrefix}extras`,
				`{$this->nrsPrefix}features`,
				`{$this->nrsPrefix}fuel_types`,
				`{$this->nrsPrefix}invoices`,
				`{$this->nrsPrefix}items`,
				`{$this->nrsPrefix}item_features`,
				`{$this->nrsPrefix}item_locations`,
				`{$this->nrsPrefix}locations`,
				`{$this->nrsPrefix}manufacturers`,
				`{$this->nrsPrefix}options`,
				`{$this->nrsPrefix}payment_methods`,
				`{$this->nrsPrefix}prepayments`,
				`{$this->nrsPrefix}price_plans`,
				`{$this->nrsPrefix}settings`,
				`{$this->nrsPrefix}transmission_types`;
		";
		$this->internalWPDB->query($sql);
	}


	// Call this function only when we activate plugin. Don't call it afterwards
	private function installRewriteFlush()
	{
		// First, we "add" the custom post type via the above written function.
		// Note: "add" is written with quotes, as CPTs don't get added to the DB,
		// They are only referenced in the post_type column with a post entry,
		// when you add a post of this CPT.
		$this->customPostTypeInit();

		// ATTENTION: This is *only* done during plugin activation hook!
		// You should *NEVER EVER* do this on every page load!!
		// And this function is important, because otherwise it was really not working (new url)
		flush_rewrite_rules();
	}

	/*
	* Creating a function to create our CPT
	*/
	public function customPostTypeInit()
	{
		$menu_position1 = 95;
		$icon_url = $GLOBALS['NRS_ADMIN_URL'].'images/nrs-plugin.png';
		$extension = $GLOBALS['NRS_EXTENSION'];
		$extTitle = ucfirst($extension);

		// Set UI labels for Custom Post Type
		$labels = array(
			'name'                => _x( ''.$extTitle.' Descriptions', 'Post Type General Name', $GLOBALS['NRS_NAME'] ),
			'singular_name'       => _x( ''.$extTitle.' Description', 'Post Type Singular Name', $GLOBALS['NRS_NAME'] ),
			'menu_name'           => __( ''.$extTitle.' Descriptions', $GLOBALS['NRS_NAME'] ),
			'parent_item_colon'   => __( 'Parent '.$extTitle.'', $GLOBALS['NRS_NAME'] ),
			'all_items'           => __( 'All '.$extTitle.' Descriptions', $GLOBALS['NRS_NAME'] ),
			'view_item'           => __( 'View '.$extTitle.' Description', $GLOBALS['NRS_NAME'] ),
			'add_new_item'        => __( 'Add New '.$extTitle.' Description', $GLOBALS['NRS_NAME'] ),
			'add_new'             => __( 'Add New Description', $GLOBALS['NRS_NAME'] ),
			'edit_item'           => __( 'Edit '.$extTitle.' Description', $GLOBALS['NRS_NAME'] ),
			'update_item'         => __( 'Update '.$extTitle.' Description', $GLOBALS['NRS_NAME'] ),
			'search_items'        => __( 'Search '.$extTitle.' Description', $GLOBALS['NRS_NAME'] ),
			'not_found'           => __( 'Not Found', $GLOBALS['NRS_NAME'] ),
			'not_found_in_trash'  => __( 'Not found in Trash', $GLOBALS['NRS_NAME'] ),
		);

		// Set other options for Custom Post Type
		$args = array(
			'label'               => __( $extension.'s', $GLOBALS['NRS_NAME'] ),
			'description'         => __( 'List of '.$extension.'s descriptions', $GLOBALS['NRS_NAME'] ),
			'labels'              => $labels,
			// Features this CPT supports in Post Editor
			/*'supports'            => array( 'title', 'editor', 'excerpt', 'author', 'thumbnail', 'comments', 'revisions', 'custom-fields', ),*/
			'supports'            => array( 'title', 'editor', 'author', 'thumbnail', ),
			// You can associate this CPT with a taxonomy or custom taxonomy.
			/*'taxonomies'          => array( $extension.'s' ),*/
			/* A hierarchical CPT is like Pages and can have
			* Parent and child items. A non-hierarchical CPT
			* is like Posts. MUST BE hierarchical=TRUE, to have dropdowns
			*/
			'hierarchical'        => true,
			'public'              => true,
			'show_ui'             => true,
			'show_in_menu'        => true,
			'show_in_nav_menus'   => true,
			'show_in_admin_bar'   => true,
			'menu_position'       => $menu_position1,
			'menu_icon'			  => $icon_url,
			'can_export'          => true,
			'has_archive'         => false,
			'rewrite'			  => array(
				// TRANSLATION IS A MUST
				'slug' => $this->lang->getText('NRS_URL_SLUG'), // loaded from language file
				'with_front' => false,
				'pages' => false,
			),
			'exclude_from_search' => false,
			'publicly_queryable'  => true,
			/* To manage specific rights to edit only items*/
			//'capability_type'     => $extension,// cant be car, because we need to
			'capability_type'     => 'post',
			'map_meta_cap'        => true, // Set to false, if users are not allowed to edit/delete existing posts
			/*'capabilities' => array(
				'read'
			),*/
			/*'capabilities' => array(
				'create_posts' => false, // Removes support for the "Add New" function - DOESN'T WORK
				'delete_post' => false,
				'publish_posts' => false,
			),*/
		);

		// Registering your Custom Post Type
		register_post_type( $extension, $args );
		// flush_rewrite_rules();
	}

	/**
	 * Make sure this function is called in the very root before any headers were sent
	 * To make sure that session was set successfully
	 */
	public function registerVisitorSession()
	{
		// Start the session, because:
		// 1. Booking search uses session to save booking process
		// 2. NRS admin has ok/error messages saved in sessions
		if(!session_id())
		{
			session_start();
		}
	}

	public function addPluginAdminMenu()
	{
		$menu_position2 = 96;
		$menu_position3 = 97;
		$menu_position4 = 98;
		//$menu_position5 = 99;
		$icon_url = $GLOBALS['NRS_ADMIN_URL'].'images/nrs-plugin.png';
		$extension = $GLOBALS['NRS_EXTENSION'];
		$extTitle = ucfirst($extension);

		// Include core files
		require_once('static.class.NRSValidator.php');
		require_once('static.class.NRSFormatter.php');
		require_once('static.class.NRSSecurity.php');


		require_once('interface.iNRSElement.php');
		require_once('interface.iNRSOptionManager.php');
		require_once('interface.iNRSPriceManager.php');
		require_once('interface.iNRSDepositManager.php');
		require_once('interface.iNRSDiscountManager.php');
		require_once('interface.iNRSPrepaymentManager.php');

		require_once('interface.iNRSDepositsObserver.php');
		require_once('interface.iNRSDiscountsObserver.php');
		require_once('interface.iNRSPrepaymentsObserver.php');


		require_once('class.NRSCore.php');
		require_once('abstract.class.NRSElement.php');
		require_once('abstract.class.NRSOptionManager.php');
		require_once('abstract.class.NRSPriceManager.php');
		require_once('abstract.class.NRSDepositManager.php');
		require_once('abstract.class.NRSDiscountManager.php');
		require_once('abstract.class.NRSPrepaymentManager.php');

		require_once('abstract.class.NRSDepositsObserver.php');
		require_once('abstract.class.NRSDiscountsObserver.php');
		require_once('abstract.class.NRSPrepaymentsObserver.php');


		require_once('final.class.NRSItem.php');
		require_once('final.class.NRSExtra.php');
		require_once('final.class.NRSOption.php');
		require_once('final.class.NRSLocation.php');
		require_once('final.class.NRSCustomer.php');
		require_once('final.class.NRSBooking.php');
		require_once('final.class.NRSInvoice.php');
		require_once('final.class.NRSMail.php');
		require_once('final.class.NRSPricePlan.php');
		require_once('final.class.NRSDeposit.php');
		require_once('final.class.NRSDiscount.php');
		require_once('final.class.NRSPrepayment.php');

		require_once('final.class.NRSItemOptionManager.php');
		require_once('final.class.NRSExtraOptionManager.php');
		require_once('final.class.NRSItemPriceManager.php');
		require_once('final.class.NRSExtraPriceManager.php');
		require_once('final.class.NRSItemDiscountManager.php');
		require_once('final.class.NRSExtraDiscountManager.php');
		require_once('final.class.NRSItemDepositManager.php');
		require_once('final.class.NRSExtraDepositManager.php');
		require_once('final.class.NRSItemPrepaymentManager.php');
		require_once('final.class.NRSExtraPrepaymentManager.php');

		require_once('final.class.NRSItemsObserver.php');
		require_once('final.class.NRSExtrasObserver.php');
		require_once('final.class.NRSLocationsObserver.php');
		require_once('final.class.NRSCustomersObserver.php');
		require_once('final.class.NRSBookingsObserver.php');
		require_once('final.class.NRSItemDepositsObserver.php');
		require_once('final.class.NRSExtraDepositsObserver.php');
		require_once('final.class.NRSItemDiscountsObserver.php');
		require_once('final.class.NRSExtraDiscountsObserver.php');
		require_once('final.class.NRSItemPrepaymentsObserver.php');
		require_once('final.class.NRSExtraPrepaymentsObserver.php');

		$objDepositsObserver = new NRSItemDepositsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$objPrepaymentsObserver = new NRSItemPrepaymentsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$objDiscountsObserver = new NRSItemDiscountsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);

		if($this->isDatabaseVersionUpToDate($GLOBALS['NRS_VERSION']))
		{
			add_menu_page("{$extTitle} Rental System", "{$extTitle} Rental", "manage_options", "{$extension}-manager", array(&$this, "getItemManager"), $icon_url, $menu_position2 );

			add_submenu_page("{$extension}-manager", "{$extTitle} Manager","{$extTitle} Manager","manage_options","{$extension}-manager", array(&$this, "getItemManager"));
				add_submenu_page("Add / Edit {$extTitle}", "Add / Edit {$extTitle}", "Add / Edit {$extTitle}", "manage_options", "{$extension}-add-item", array(&$this, "getItemAddEdit"));
				add_submenu_page("Add / Edit Manufacturer", "Add / Edit Manufacturer", "Add / Edit Manufacturer", "manage_options", "{$extension}-add-manufacturer", array(&$this, "getManufacturerAddEdit"));
				add_submenu_page("Add / Edit Body Type", "Add / Edit Body Type", "Add / Edit Body Type","manage_options", "{$extension}-add-body-type", array(&$this, "getBodyTypeAddEdit"));
				add_submenu_page("Add / Edit Fuel Type", "Add / Edit Fuel Type", "Add / Edit Fuel Type","manage_options", "{$extension}-add-fuel-type", array(&$this, "getFuelTypeAddEdit"));
				add_submenu_page("Add / Edit Transmission Type", "Add / Edit Transmission Type", "Add / Edit Transmission Type", "manage_options", "{$extension}-add-transmission-type", array(&$this, "getTransmissionTypeAddEdit"));
				add_submenu_page("Add / Edit Feature", "Add / Edit Feature", "Add / Edit Feature", "manage_options", "{$extension}-add-feature", array(&$this, "getFeatureAddEdit"));
				add_submenu_page("Add / Edit {$extTitle} Option", "Add / Edit {$extTitle} Option", "Add / Edit {$extTitle} Option", "manage_options", "{$extension}-add-item-option", array(&$this, "getItemOptionAddEdit"));
				add_submenu_page("Block {$extTitle}", "Block {$extTitle}", "Block {$extTitle}", "manage_options", "{$extension}-block-item", array(&$this, "getBlockItem"));

			add_submenu_page("{$extension}-manager", "Car Prices", "Car Prices", "manage_options", "{$extension}-prices", array(&$this, "getItemPrices"));
				add_submenu_page("Add / Edit Price Plan", "Add / Edit Car Price Plan", "Add / Edit Car Price Plan", "manage_options", "{$extension}-add-edit-price-plan", array(&$this, "getPricePlanAddEdit"));
				if($objDepositsObserver->areDepositsEnabled())
				{
					add_submenu_page("Add / Edit Deposit", "Add / Edit Deposit", "Add / Edit Deposit", "manage_options", "{$extension}-add-deposit", array(&$this, "getDepositAddEdit"));
				}
				if($objPrepaymentsObserver->arePrepaymentsEnabled())
				{
					add_submenu_page("Add / Edit Prepayment", "Add / Edit Prepayment", "Add / Edit Prepayment", "manage_options", "{$extension}-add-prepayment", array(&$this, "getPrepaymentAddEdit"));
				}

			add_submenu_page("{$extension}-manager", "Extras Manager", "Extras Manager", "manage_options", "{$extension}-extras-manager", array(&$this, "getExtrasManager"));
				add_submenu_page("Add / Edit Extra", "Add / Edit Extra", "Add / Edit Extra", "manage_options", "{$extension}-add-extra", array(&$this, "getExtraAddEdit"));
				add_submenu_page("Add / Edit Extra Option", "Add / Edit Extra Option", "Add / Edit Extra Option", "manage_options", "{$extension}-add-extra-option", array(&$this, "getExtraOptionAddEdit"));
				add_submenu_page("Block Extra", "Block Extra", "Block Extra", "manage_options", "{$extension}-block-extra", array(&$this, "getBlockExtra"));

			add_submenu_page("{$extension}-manager", "Location Manager", "Location Manager", "manage_options", "{$extension}-location-manager", array(&$this, "getLocationManager"));
				add_submenu_page("Add / Edit Location", "Add / Edit Location", "Add / Edit Location", "manage_options", "{$extension}-add-edit-location", array(&$this, "getLocationAddEdit"));

			if($objDiscountsObserver->areDiscountsEnabled())
			{
				add_submenu_page("{$extension}-manager", "Discount Manager", "Discount Manager", "manage_options", "{$extension}-discount-manager", array(&$this, "getDiscountManager"));
					add_submenu_page("Add / Edit Discount", "Add / Edit Discount", "Add / Edit Discount", "manage_options", "{$extension}-add-discount", array(&$this, "getDiscountAddEdit"));
			}

			add_submenu_page("{$extension}-manager", "Reservation Manager", "Reservation Manager", "manage_options", "{$extension}-booking-manager", array(&$this, "getBookingManager"));
				add_submenu_page("Reservation Search Results", "Reservation Search Results", "Reservation Search Results", "manage_options", "{$extension}-booking-search-results", array(&$this, "getBookingSearchResults"));
				add_submenu_page("{$extTitle} Calendar Search Results", "{$extTitle} Calendar Search Results", "{$extTitle} Calendar Search Results", "manage_options", "{$extension}-item-calendar-search-results", array(&$this, "getItemCalendarSearchResults"));
				add_submenu_page("Extras Calendar Search Results", "Extras Calendar Search Results", "Extras Calendar Search Results", "manage_options", "{$extension}-extras-calendar-search-results", array(&$this, "getExtrasCalendarSearchResults"));
				add_submenu_page("Customer Search Results", "Customer Search Results", "Customer Search Results", "manage_options", "{$extension}-customer-search-results", array(&$this, "getCustomerSearchResults"));
				add_submenu_page("Add/Edit Customer", "Add/Edit Customer", "Add/Edit Customer", "manage_options", "{$extension}-add-customer", array(&$this, "getCustomerAddEdit"));
				add_submenu_page("View Booking Details", "View Booking Details", "View Booking Details", "manage_options","{$extension}-view-booking-details", array(&$this, "getViewBookingDetails"));

			add_submenu_page("{$extension}-manager", "Settings", "Settings", "manage_options","{$extension}-settings", array(&$this, "getSettings"));
				add_submenu_page("Content Preview", "Content Preview", "Content Preview", "manage_options","{$extension}-preview", array(&$this, "getPreview"));

			add_submenu_page("{$extension}-manager", "Instructions", "Instructions", "manage_options","{$extension}-user-manual", array(&$this, "getUserManual"));
		} else
		{
			add_menu_page("{$extTitle} Rental System", "{$extTitle} Rental", "manage_options", "{$extension}-settings", array(&$this, "getUpdater"), $icon_url, $menu_position4 );
				add_submenu_page("{$extension}-settings", "System Update", "System Update", "manage_options", "{$extension}-updater", array(&$this, "getUpdater"));
			remove_submenu_page("{$extension}-settings", "{$extension}-settings");
		}
	}

	public function loadAdminScripts()
	{
		// JS ADDED: START
		wp_enqueue_script('jquery');
		wp_enqueue_script('jquery-ui-core');
		wp_enqueue_script('jquery-ui-tabs');
		wp_enqueue_script('jquery-ui-accordion');
		wp_enqueue_script('nrs_admin_functions', $GLOBALS['NRS_ADMIN_URL'].'js/nrs-admin.js');
		// END

		wp_enqueue_script('jquery-ui-datepicker', array('jquery','jquery-ui-core'));
		wp_enqueue_style('jquery-style', $GLOBALS['NRS_ADMIN_URL'].'css/datepicker.css');
		wp_enqueue_style('custom-style7', $GLOBALS['NRS_ADMIN_URL'].'css/style.css');
		wp_enqueue_style('custom-style8', $GLOBALS['NRS_ADMIN_URL'].'css/jquery.validate.css');
		wp_enqueue_script('jquery-validate', $GLOBALS['NRS_ADMIN_URL'].'js/jquery.validate.js');
	}

	private function registerFrontEndScripts()
	{
		//echo $date; die;
		wp_register_script('jquery-ui-datepicker', array('jquery','jquery-ui-core'));
		wp_enqueue_script('jquery-ui-datepicker');

		// Register scripts for further use - in file_exists we must use PATH, and in register_script we must use URL
		if(file_exists($GLOBALS['NRS_EXTENSION_LANG_PATH'].'datepicker/'.get_locale().'.js'))
		{
			wp_register_script(
				'nrs-jquery-datepicker', $GLOBALS['NRS_EXTENSION_LANG_URL'].'datepicker/'.get_locale().'.js',
				array('jquery', 'jquery-ui-datepicker')
			);
		} else
		{
			wp_register_script(
				'nrs-jquery-datepicker', $GLOBALS['NRS_EXTENSION_LANG_URL'].'datepicker/en_US.js',
				array('jquery', 'jquery-ui-datepicker')
			);
		}

		// to check browser capabilities
		// wp_deregister_script( 'modernizr' ); // we may leave it
		wp_register_script( 'modernizr', $GLOBALS['NRS_FRONT_URL'].'js/modernizr.js');

		// wp_deregister_script( 'jquery.easing' ); // we may leave it
		wp_register_script(
			'jquery.easing', $GLOBALS['NRS_FRONT_URL'].'js/jquery.easing.js',
			array('jquery')
		);

		// wp_deregister_script( 'jquery.fitvid' ); // we may leave it
		wp_register_script(
			'jquery.fitvid', $GLOBALS['NRS_FRONT_URL'].'js/jquery.fitvid.js',
			array('jquery')
		);

		// wp_deregister_script( 'jquery.mousewheel' ); // we may leave it
		wp_register_script(
			'jquery.mousewheel', $GLOBALS['NRS_FRONT_URL'].'js/jquery.mousewheel.js',
			array('jquery')
		);

		// wp_deregister_script( 'jquery.flexslider' ); // we may leave it
		wp_register_script(
			'jquery.flexslider', $GLOBALS['NRS_FRONT_URL'].'js/jquery.flexslider-min.js',
			array('jquery')
		);

		$placeScriptInFooter = TRUE;
		wp_register_script(
			'nrs-flexslider-js', $GLOBALS['NRS_FRONT_URL'].'js/nrs-flexslider-settings.js',
			array('jquery', 'jquery.flexslider'), '2.0', $placeScriptInFooter
		);

		wp_register_script(
			'nrs-frontend-js', $GLOBALS['NRS_FRONT_URL'].'js/nrs-frontend.js',
			array('jquery'), '1.0', $placeScriptInFooter
		);
	}

	private function registerFrontEndStyles()
	{
		$nrsStyleSql = "SELECT conf_value AS conf_system_style
			FROM {$this->nrsPrefix}settings
			WHERE conf_key='conf_system_style'
		";

		$nrsRTLSql = "SELECT conf_value AS conf_rtl_enabled
			FROM {$this->nrsPrefix}settings
			WHERE conf_key='conf_rtl_enabled'
		";

		$nrsStyleSetting = $this->internalWPDB->get_row($nrsStyleSql, ARRAY_A);
		$nrsRTLSetting = $this->internalWPDB->get_row($nrsRTLSql, ARRAY_A);

		$nrsStyle = 1;
		if(!is_null($nrsStyleSetting) && isset($nrsStyleSetting['conf_system_style']))
		{
			$nrsStyle = intval($nrsStyleSetting['conf_system_style']);
		}

		// RTL SUPPORT
		$RTLEnabled = false;
		if(!is_null($nrsRTLSetting) && isset($nrsRTLSetting['conf_rtl_enabled']))
		{
			$RTLEnabled = $nrsRTLSetting['conf_rtl_enabled'] == "1" ? true : false;
		}

		// Set stylesheet
		$dir = $GLOBALS['NRS_FRONT_PATH'].'css/';
		$file = "style-crimson-red.css";
		if(isset($GLOBALS['NRS_SUPPORTED_STYLESHEET'][$nrsStyle]['file_name']))
		{
			$file = sanitize_text_field($GLOBALS['NRS_SUPPORTED_STYLESHEET'][$nrsStyle]['file_name']);
		}

		// Set stylesheet only it exist
		if(file_exists($dir.$file))
		{
			// "Steel Blue" style
			$nrsCSSFilePath = $GLOBALS['NRS_FRONT_URL']."css/".$file;
		} else
		{
			// "Crimson Red" style
			$nrsCSSFilePath = $GLOBALS['NRS_FRONT_URL']."css/style-crimson-red.css";
		}

		// Register Styles
		wp_register_style('jquery-style2', $nrsCSSFilePath);
		wp_register_style('jquery-style', $GLOBALS['NRS_FRONT_URL'].'css/datepicker.css');
		if($RTLEnabled)
		{
			wp_register_style('nrs-rtl', $GLOBALS['NRS_FRONT_URL'].'css/rtl.css');
		}

		// Enqueue Styles
		wp_enqueue_style('jquery-style');
		wp_enqueue_style('jquery-style2');
		if($RTLEnabled)
		{
			wp_enqueue_style('nrs-rtl');
		}

		// Register styles for further use
		wp_register_style( 'nrs-flexslider-css', $GLOBALS['NRS_FRONT_URL'].'css/nrs-flexslider.css');
	}

	public function registerFrontEndPages($attributes)
	{
		// Include Validator & formatter
		require_once('static.class.NRSValidator.php');
		require_once('static.class.NRSFormatter.php');
		require_once('static.class.NRSSecurity.php');

		// Include core files
		require_once('interface.iNRSElement.php');
		require_once('interface.iNRSPriceManager.php');
		require_once('interface.iNRSUnitsManager.php');
		require_once('interface.iNRSDepositManager.php');
		require_once('interface.iNRSDiscountManager.php');
		require_once('interface.iNRSPrepaymentManager.php');
		require_once('interface.iNRSDepositsObserver.php');
		require_once('interface.iNRSDiscountsObserver.php');
		require_once('interface.iNRSPrepaymentsObserver.php');


		require_once('class.NRSCore.php');
		require_once('abstract.class.NRSElement.php');
		require_once('abstract.class.NRSPriceManager.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('abstract.class.NRSDepositManager.php');
		require_once('abstract.class.NRSDiscountManager.php');
		require_once('abstract.class.NRSPrepaymentManager.php');

		require_once('abstract.class.NRSDepositsObserver.php');
		require_once('abstract.class.NRSDiscountsObserver.php');
		require_once('abstract.class.NRSPrepaymentsObserver.php');


		require_once('final.class.NRSItem.php');
		require_once('final.class.NRSExtra.php');
		require_once('final.class.NRSItemPriceManager.php');
		require_once('final.class.NRSExtraPriceManager.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('final.class.NRSItemDiscountManager.php');
		require_once('final.class.NRSExtraDiscountManager.php');
		require_once('final.class.NRSItemDepositManager.php');
		require_once('final.class.NRSExtraDepositManager.php');
		require_once('final.class.NRSItemPrepaymentManager.php');
		require_once('final.class.NRSExtraPrepaymentManager.php');

		require_once('final.class.NRSItemsObserver.php');
		require_once('final.class.NRSExtrasObserver.php');
		require_once('final.class.NRSItemDepositsObserver.php');
		require_once('final.class.NRSExtraDepositsObserver.php');
		require_once('final.class.NRSItemDiscountsObserver.php');
		require_once('final.class.NRSExtraDiscountsObserver.php');
		require_once('final.class.NRSItemPrepaymentsObserver.php');
		require_once('final.class.NRSExtraPrepaymentsObserver.php');

		//print_r($attributes);
		$validItemId = isset($attributes['item']) ? NRSValidator::getValidInteger($attributes['item'], 0) : 0;
		$sanitizedDisplay = isset($attributes['display']) ? sanitize_key($attributes['display']) : "search";

		// register front-end scripts
		$this->registerFrontEndScripts();
		// register front-end styles
		$this->registerFrontEndStyles();

		// Start the output buffer
		ob_start();

		// Render the page HTML to output buffer cache
		switch($sanitizedDisplay)
		{
			case "search":
				// render search
				$this->displayItemSearch($validItemId, FALSE);
				break;

			case "slider":
				// render item slider
				$this->displayItemsSlider();
				break;

			case "item":
				// render single item page
				$this->displaySingleItem($validItemId);
				break;

			case "list":
				// render item list
				$this->displayItemsList();
				break;

			case "price_table":
				// render item price table
				$this->displayItemsPriceTable();
				break;

			case "extras_price_table":
				// render item price table
				$this->displayExtrasPriceTable();
				break;

			case "calendar":
				// render item calendar
				$this->displayItemCalendar();
				break;

			case "extras_calendar":
				// render item calendar
				$this->displayExtraCalendar();
				break;

			case "edit":
				// render item edit
				$this->displayItemSearch($validItemId, TRUE);
				break;

			default:
				// render search
				$this->displayItemSearch($validItemId, FALSE);
		}

		// Get the output buffer cache content to variable
		$retContent = ob_get_contents();
		// Then clean and disabled the output buffer
		ob_end_clean();

		// Return page content to shortcode
		return $retContent;
	}
	
	private function displayItemsSlider()
	{
		include($GLOBALS['NRS_FRONT_PATH'].'display-items-slider.php');
	}

	private function displayItemsList()
	{
		include($GLOBALS['NRS_FRONT_PATH'].'display-items-list.php');
	}

	/**
	 * @note - PARAM $paramItemId IS USED IN THE INCLUDED FILE
	 * @param $paramItemId
	 */
	private function displaySingleItem($paramItemId)
	{
		include($GLOBALS['NRS_FRONT_PATH'].'display-single-item.php');
	}

	private function displayItemsPriceTable()
	{
		require_once('interface.iNRSPriceTable.php');
		require_once('abstract.class.NRSPriceTable.php');
		require_once('final.class.NRSItemsPriceTable.php');
		include($GLOBALS['NRS_FRONT_PATH'].'display-items-price-table.php');
	}

	private function displayExtrasPriceTable()
	{
		require_once('interface.iNRSPriceTable.php');
		require_once('abstract.class.NRSPriceTable.php');
		require_once('final.class.NRSExtrasPriceTable.php');
		include($GLOBALS['NRS_FRONT_PATH'].'display-extras-price-table.php');
	}

	private function displayItemCalendar()
	{
		require_once('interface.iNRSCalendar.php');
		require_once('abstract.class.NRSCalendar.php');
		require_once('final.class.NRSItemsCalendar.php');
		include($GLOBALS['NRS_FRONT_PATH'].'display-items-calendar.php');
	}

	private function displayExtraCalendar()
	{
		require_once('interface.iNRSCalendar.php');
		require_once('abstract.class.NRSCalendar.php');
		require_once('final.class.NRSExtrasCalendar.php');
		include($GLOBALS['NRS_FRONT_PATH'].'display-extras-calendar.php');
	}

	/**
	 * @note - PARAM $paramItemId IS USED IN SOME INCLUDED FILES
	 * @param int $paramItemId
	 * @param bool $paramStartFromEditPage
	 * @return bool
	 */
	private function displayItemSearch($paramItemId = 0, $paramStartFromEditPage = false)
	{
		// Main includes, used in search root
		require_once('interface.iNRSSearch.php');
		require_once('interface.iNRSOptionManager.php');
		require_once('abstract.class.NRSSearch.php');
		require_once('abstract.class.NRSOptionManager.php');
		require_once('final.class.NRSOption.php');
		require_once('final.class.NRSLocation.php');
		require_once('final.class.NRSCustomer.php');
		require_once('final.class.NRSBooking.php');
		require_once('final.class.NRSInvoice.php');
		require_once('class.NRSPayPalIpnListener.php');
		require_once('final.class.NRSPayPal.php');
		require_once('final.class.NRSMail.php');

		require_once('final.class.NRSItemOptionManager.php');
		require_once('final.class.NRSExtraOptionManager.php');
		require_once('final.class.NRSLocationsObserver.php');
		require_once('final.class.NRSCustomersObserver.php');
		require_once('final.class.NRSBookingsObserver.php');
		// Classes used only in front-end
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/interface.iNRSSearchManager.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/abstract.class.NRSFrontEndSearch.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/abstract.class.NRSSearchManager.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/final.class.NRSItemSearch.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/final.class.NRSSearchResults.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/final.class.NRSSearchManagerForItems.php');
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/final.class.NRSSearchManagerForExtras.php');

		// Include ReCaptcha. Used in step 5
		require_once($GLOBALS['NRS_FRONT_PATH'].'includes/function.ReCaptchaAutoLoad.php');


		// Separate steps 1 to 8
		if(
			!isset($_REQUEST['nrs_do_search0']) && !isset($_REQUEST['nrs_do_search']) && !isset($_REQUEST['nrs_do_search2'])
			&& !isset($_REQUEST['nrs_do_search3']) && !isset($_REQUEST['nrs_do_search4'])
			&& !$paramStartFromEditPage
		) {
			// If no requests are passed - start from edit page
			// Booking step no. 1
			include($GLOBALS['NRS_FRONT_PATH'].'display-step1-item-search.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search0']))
		{
			// If there is a call back to step 1 from step 2 / 3 / 4
			// Booking step no. 1
			include($GLOBALS['NRS_FRONT_PATH'].'display-step1-item-search-edit.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search']) && (
				!isset($_POST['booking_code']) || isset($_POST['booking_code'])
				&& (strlen($_POST['booking_code']) == 0 || $_POST['booking_code'] == $this->lang->getText('NRS_I_HAVE_BOOKING_NUMBER_TEXT'))
			)
		) {
			// Regular call from step 1 to step 2
			// Booking step no. 2
			include($GLOBALS['NRS_FRONT_PATH'].'display-step2-search-results.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search']) && isset($_POST['booking_code']) && strlen($_POST['booking_code']) > 0)
		{
			// Call from step 1 with booking code entered to step 3
			// Booking step no. 3
			include($GLOBALS['NRS_FRONT_PATH'].'display-step3-booking-options.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search2']))
		{
			// Call from step 1b to step 3
			// Booking step no. 3
			include($GLOBALS['NRS_FRONT_PATH'].'display-step3-booking-options.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search3']))
		{
			// Booking step no. 4
			include($GLOBALS['NRS_FRONT_PATH'].'display-step4-booking-details.php');
			return true;
		} else if(isset($_REQUEST['nrs_do_search4']))
		{
			// Booking step no. 5
			include($GLOBALS['NRS_FRONT_PATH'].'display-step5-booking-process.php');
			return true;
		} else if(isset($_REQUEST['btn_cancel_reservation']))
		{
			// Cancel Booking
			include($GLOBALS['NRS_FRONT_PATH'].'display-step7-cancel-booking.php');
			return true;
		} else if($paramStartFromEditPage == TRUE)
		{
			// Load the edit page
			// Booking step no. 8 - edit
			include($GLOBALS['NRS_FRONT_PATH'].'display-step8-edit-booking.php');
			return true;
		}

		return false;
	}

	// Item Manager
	public function getItemManager()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.php');
	}
	public function getItemAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-item.php');
	}
	public function getManufacturerAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-manufacturer.php');
	}
	public function getBodyTypeAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-body-type.php');
	}
	public function getFuelTypeAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-fuel-type.php');
	}
	public function getTransmissionTypeAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-transmission-type.php');
	}
	public function getFeatureAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-feature.php');
	}
	public function getItemOptionAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.add-edit-option.php');
	}
	public function getBlockItem()
	{
		require_once('interface.iNRSSearch.php');
		require_once('interface.iNRSUnitsManager.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('abstract.class.NRSSearch.php');
		require_once($GLOBALS['NRS_ADMIN_PATH'].'includes/final.class.NRSItemBlockSearch.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'item-manager.block-item.php');
	}


	// Item Price Manager
	public function getItemPrices()
	{
		require_once('interface.iNRSPriceTable.php');
		require_once('abstract.class.NRSPriceTable.php');
		require_once('final.class.NRSItemsPriceTable.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'item-prices.php');
	}
	public function getPricePlanAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-prices.add-edit-price-plan.php');
	}
	public function getDepositAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-prices.add-edit-deposit.php');
	}
	public function getPrepaymentAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'item-prices.add-edit-prepayment.php');
	}


	// Extras Manager
	public function getExtrasManager()
	{
		require_once('interface.iNRSPriceTable.php');
		require_once('abstract.class.NRSPriceTable.php');
		require_once('final.class.NRSExtrasPriceTable.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'extras-manager.php');
	}
	public function getExtraAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'extras-manager.add-edit-extra.php');
	}
	public function getExtraOptionAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'extras-manager.add-edit-option.php');
	}
	public function getBlockExtra()
	{
		require_once('interface.iNRSSearch.php');
		require_once('interface.iNRSUnitsManager.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('abstract.class.NRSSearch.php');
		require_once($GLOBALS['NRS_ADMIN_PATH'].'includes/final.class.NRSExtraBlockSearch.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'extras-manager.block-extra.php');
	}


	// Location Manager
	public function getLocationManager()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'location-manager.php');
	}
	public function getLocationAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'location-manager.add-edit-location.php');
	}
	public function getClosedDaysOptions()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'location-manager.closed-days.php');
	}


	// Item & Extras Discount Manager
	public function getDiscountManager()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'discount-manager.php');
	}
	public function getDiscountAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'discount-manager.add-edit-discount.php');
	}


	// Booking Manager
	public function getBookingManager()
	{
		// Booking manager uses calendar item
		require_once('interface.iNRSUnitsManager.php');
		require_once('interface.iNRSCalendar.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('abstract.class.NRSCalendar.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('final.class.NRSItemsCalendar.php');
		require_once('final.class.NRSExtrasCalendar.php');
		require_once('final.class.NRSApiLog.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.php');
	}
	public function getBookingSearchResults()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.booking-search-results.php');
	}
	public function getItemCalendarSearchResults()
	{
		//Calendar search uses calendar item
		require_once('interface.iNRSUnitsManager.php');
		require_once('interface.iNRSCalendar.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('abstract.class.NRSCalendar.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('final.class.NRSItemsCalendar.php');
		require_once('final.class.NRSExtrasCalendar.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.item-calendar-search-results.php');
	}
	public function getExtrasCalendarSearchResults()
	{
		//Calendar search uses calendar item
		require_once('interface.iNRSUnitsManager.php');
		require_once('interface.iNRSCalendar.php');
		require_once('abstract.class.NRSUnitManager.php');
		require_once('abstract.class.NRSCalendar.php');
		require_once('final.class.NRSItemUnitManager.php');
		require_once('final.class.NRSExtraUnitManager.php');
		require_once('final.class.NRSItemsCalendar.php');
		require_once('final.class.NRSExtrasCalendar.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.extras-calendar-search-results.php');
	}
	public function getCustomerSearchResults()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.customer-search-results.php');
	}
	public function getCustomerAddEdit()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.add-edit-customer.php');
	}
	public function getViewBookingDetails()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'booking-manager.booking-details.php');
	}


	// NRS settings
	public function getSettings()
	{
		// For settings tab only
		require_once($GLOBALS['NRS_EXTENSION_ADMIN_PATH'].'includes/final.class.NRSAdminDemoImporter.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'settings.php');
	}
	public function getPriceCalculationSettings()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'settings.price-calculation-settings.php');
	}
	public function getPaymentSettings()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'settings.payment-settings.php');
	}
	public function getEmailSettings()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'settings.email-settings.php');
	}


	public function getPreview()
	{
		// For settings tab only
		include($GLOBALS['NRS_ADMIN_PATH'].'preview.php');
	}

	// User Manual
	public function getUserManual()
	{
		include($GLOBALS['NRS_ADMIN_PATH'].'user-manual.php');
	}


	// Updater (available only if there is newer plugin code uploaded than set in db version)
	public function getUpdater()
	{
		require_once($GLOBALS['NRS_ADMIN_PATH'].'includes/final.class.NRSAdminUpdater.php');
		include($GLOBALS['NRS_ADMIN_PATH'].'updater.php');
	}
}