<?php
/**
 * Invoice processor
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSInvoice
{
    protected $internalWPDB	 	= NULL;
    protected $nrsPrefix 		= "";
    protected $lang 			= NULL;
    protected $debugMode 	 	= 0;
    protected $bookingId        = 0;

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - LANGUAGE
     * @param int $paramBookingId = unique element identifier, mandatory, for managing invoices
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramBookingId)
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set booking id
        $this->bookingId = NRSValidator::getValidInteger($paramBookingId, 0);
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getBookingId()
    {
        return $this->bookingId;
    }

    /**
     * Insert invoice HTML to database
     * @param string $paramName
     * @param string $paramEmail
     * @param string $paramPickupLocationHTML
     * @param string $paramDropoffLocationHTML
     * @param float $paramGrandTotal
     * @param float $paramFixedDepositAmount
     * @param float $paramTotalPayNow
     * @param string $paramInvoiceHTML - invoice HTML
     */
    public function saveInvoiceHTML(
        $paramName, $paramEmail, $paramGrandTotal, $paramFixedDepositAmount, $paramTotalPayNow,
        $paramPickupLocationHTML, $paramDropoffLocationHTML, $paramInvoiceHTML
    ) {
        $validBookingId = intval($this->bookingId);
        $sanitizedName = sanitize_text_field($paramName);
        $sanitizedEmail = sanitize_email($paramEmail);
        $validGrandTotal = floatval($paramGrandTotal);
        $validFixedDepositAmount = floatval($paramFixedDepositAmount);
        $validTotalPayNow = floatval($paramTotalPayNow);
        $ksesedPickupLocation = wp_kses_post($paramPickupLocationHTML);
        $ksesedDropoffLocation = wp_kses_post($paramDropoffLocationHTML);
        $ksesedInvoiceHTML = wp_kses_post($paramInvoiceHTML);

        $invoiceExist = $this->internalWPDB->get_row("
            SELECT booking_id
            FROM {$this->nrsPrefix}invoices
            WHERE booking_id='{$validBookingId}'
        ");

        if (!is_null($invoiceExist))
        {
            /* update the invoice data in {$this->nrsPrefix}invoice table */
            $this->internalWPDB->query("
				UPDATE {$this->nrsPrefix}invoices SET
				customer_name='{$sanitizedName}', customer_email='{$sanitizedEmail}',
				grand_total='{$validGrandTotal}', fixed_deposit_amount='{$validFixedDepositAmount}',
				total_pay_now='{$validTotalPayNow}',
				pickup_location='{$ksesedPickupLocation}', dropoff_location='{$ksesedDropoffLocation}',
				invoice='{$ksesedInvoiceHTML}'
				WHERE booking_id='{$validBookingId}'
			");
        } else
        {
            /* insert the invoice data in {$this->nrsPrefix}invoice table */
            $insertSQL = "
				INSERT INTO {$this->nrsPrefix}invoices
				(
					booking_id, customer_name, customer_email,
					grand_total, fixed_deposit_amount,
					total_pay_now,
					pickup_location, dropoff_location,
					invoice
				) VALUES
				(
					'{$validBookingId}', '{$sanitizedName}', '{$sanitizedEmail}',
					'{$validGrandTotal}', '{$validFixedDepositAmount}',
					'{$validTotalPayNow}',
					'{$ksesedPickupLocation}', '{$ksesedDropoffLocation}',
					'{$ksesedInvoiceHTML}'
				)
			";
            // Debug
            //echo esc_html($insertSQL); die();
            $this->internalWPDB->query($insertSQL);
        }
    }

    /**
     * Append invoice HTML in Database
     * @param string $paramHTMLToAppendInvoice - HTML to append
     * @return bool|false|int
     *
     */
    public function appendInvoiceHTML($paramHTMLToAppendInvoice)
    {
        $validBookingId = intval($this->bookingId);
        $ksesedHTMLtoAppendInvoice = wp_kses_post($paramHTMLToAppendInvoice);

        $sqlExist = "
            SELECT booking_id
            FROM {$this->nrsPrefix}invoices
            WHERE booking_id='{$validBookingId}'
        ";

        // DEBUG
        // echo nl2br($sqlExist);

        $invoiceExist = $this->internalWPDB->get_row($sqlExist, ARRAY_A);

        if (!is_null($invoiceExist))
        {
            /* update the invoice data in {$this->nrsPrefix}invoice table */
            $completed = $this->internalWPDB->query("
				UPDATE {$this->nrsPrefix}invoices SET
				invoice = CONCAT(invoice, '{$ksesedHTMLtoAppendInvoice}')
				WHERE booking_id='{$validBookingId}'
			");
        } else
        {
            $completed = FALSE;
        }

        return $completed;
    }

    private function getInvoiceDataFromDatabaseById($paramBookingId)
    {
        $validBookingId =  NRSValidator::getValidInteger($paramBookingId, 0);
        $invoiceData = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}invoices
            WHERE booking_id='{$validBookingId}'
        ", ARRAY_A);

        return $invoiceData;
    }

    public function getInvoiceDetails()
    {
        $invoiceDetails = $this->getInvoiceDataFromDatabaseById($this->bookingId);

        return $invoiceDetails;
    }

    /**
     * Return Invoice HTML
     * @return string
     */
    public function getInvoiceHTML()
    {
        $ret = "";
        $invoiceData = $this->getInvoiceDataFromDatabaseById($this->bookingId);

        if(!is_null($invoiceData))
        {
            $ret = $invoiceData['invoice'];
        }

        return $ret;
    }

    public function deleteInvoice()
    {
        $ret = false;

        $validBookingId = intval($this->bookingId);
        if($validBookingId > 0)
        {
            $ret = $this->internalWPDB->query("
                DELETE FROM {$this->nrsPrefix}invoices
                WHERE booking_id='{$validBookingId}'
            ");
        }

        return $ret;
    }
}