<?php
/**
 * Item Deposit Manager
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSItemDepositManager extends NRSDepositManager implements iNRSDepositManager
{
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig, $paramElementId)
	{
		parent::__construct($paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig, $paramElementId);
	}

	/**
	 * @param int $paramPeriodInSeconds
	 * @return float
	 */
	public function getFixedDepositAmountByPeriod($paramPeriodInSeconds)
	{
		$fixedRentalDeposit = 0;

		if($this->depositEnabled)
		{
			$validBookingDurationInSeconds = NRSValidator::getValidInteger($paramPeriodInSeconds, 0);
			// We use float here, because we want to have duration of hours and minutes, not just days. So day will be divided by 24 / 60

			$validItemId = NRSValidator::getValidInteger($this->elementId, 0);

			// SPEED OPTIMIZATION NOTE: it is IMPORTANT to keep the WHERE columns in period_from, period_till order to speed up SQL search,
			// because we have an INDEX in database called 'period (period_from, period_to)'.
			$sqlForZeroOrBiggerValue = "
			  	SELECT fixed_rental_deposit
			  	FROM {$this->nrsPrefix}deposits
			  	WHERE item_id='{$validItemId}' AND '{$validBookingDurationInSeconds}' BETWEEN period_from AND period_till
			";
			$sqlForZeroValueOnly = "
			  	SELECT fixed_rental_deposit
			  	FROM {$this->nrsPrefix}deposits
			  	WHERE item_id='0' AND '{$validBookingDurationInSeconds}' BETWEEN period_from AND period_till
			";

            if($this->debugMode == 2)
            {
                // Deep debug:
                echo "<br />";
                echo "<br /><strong>SQL</strong> Item deposit for ZERO or bigger item id value: ".nl2br($sqlForZeroOrBiggerValue);
                echo "<br /><strong>SQL</strong> Item deposit for ZERO item id ONLY: ".nl2br($sqlForZeroValueOnly);
            }

			$rowZeroOrBigger = $this->internalWPDB->get_row($sqlForZeroOrBiggerValue, ARRAY_A);
			if(!is_null($rowZeroOrBigger))
			{
				// Use different deposits for each item (unless 0 was passed, than it will return default here)
				$fixedRentalDeposit = $rowZeroOrBigger['fixed_rental_deposit'];
			} else if($validItemId > 0)
			{
				// SPEED OPTIMIZATION NOTE: For better system speed - do not move the get_row line out of THIS ELSE SCOPE!
				$rowZeroOnly = $this->internalWPDB->get_row($sqlForZeroValueOnly, ARRAY_A);
				if(!is_null($rowZeroOnly))
				{
					// Use same deposits for all items
					$fixedRentalDeposit = $rowZeroOnly['fixed_rental_deposit'];
				}
			}
		}

		return $fixedRentalDeposit;
	}

	/**
	 * @return int
	 */
	public function getMinimalFixedDepositAmount()
	{
		$fixedRentalDeposit = 0;

		if($this->depositEnabled)
		{
			$validItemId = NRSValidator::getValidInteger($this->elementId, 0);

			$sqlForZeroOrBiggerValue = "
			  	SELECT fixed_rental_deposit
			  	FROM {$this->nrsPrefix}deposits
			  	WHERE item_id='{$validItemId}'
			  	ORDER BY fixed_rental_deposit ASC
			";
			$sqlForZeroValueOnly = "
			  	SELECT fixed_rental_deposit
			  	FROM {$this->nrsPrefix}deposits
			  	WHERE item_id='0'
			  	ORDER BY fixed_rental_deposit ASC
			";

			$rowZeroOrBigger = $this->internalWPDB->get_row($sqlForZeroOrBiggerValue, ARRAY_A);
			if(!is_null($rowZeroOrBigger))
			{
				// Use different deposits for each item (unless 0 was passed, than it will return default here)
				$fixedRentalDeposit = $rowZeroOrBigger['fixed_rental_deposit'];
			} else if($validItemId > 0)
			{
				$rowZeroOnly = $this->internalWPDB->get_row($sqlForZeroValueOnly, ARRAY_A);
				if(!is_null($rowZeroOnly))
				{
					// Use same deposits for all items
					$fixedRentalDeposit = $rowZeroOnly['fixed_rental_deposit'];
				}
			}
		}

		return $fixedRentalDeposit;
	}

	public function getMinimalFixedDepositDetails()
    {
        $retDeposit = array();
        $unitFixedDepositAmount = $this->getMinimalFixedDepositAmount();
        $roundedDeposit = round($unitFixedDepositAmount, 2);

        $retDeposit['unit']['fixed_deposit_amount'] = $unitFixedDepositAmount;
        // Unit prints
        $retDeposit['unit_tiny_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "tiny", $this->currencySymbol, $this->currencyCode);
        $retDeposit['unit_tiny_without_fraction_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "tiny_without_fraction", $this->currencySymbol, $this->currencyCode);
        $retDeposit['unit_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "regular", $this->currencySymbol, $this->currencyCode);
        $retDeposit['unit_without_fraction_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "regular_without_fraction", $this->currencySymbol, $this->currencyCode);
        $retDeposit['unit_long_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "long", $this->currencySymbol, $this->currencyCode);
        $retDeposit['unit_long_without_fraction_print']['fixed_deposit_amount'] = NRSFormatter::getFormattedPrice($roundedDeposit, "long_without_fraction", $this->currencySymbol, $this->currencyCode);
        // Word prints
        $retDeposit['word_print']['fixed_deposit_amount'] = $roundedDeposit == 0.00 ? $this->lang->getText('NRS_NOT_REQUIRED_TEXT') : $this->lang->getText('NRS_REQUIRED_TEXT');

        return $retDeposit;
    }
}
