<?php
/**
 * NRS Item Calendar
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSItemsCalendar extends NRSCalendar implements iNRSCalendar
{
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
	{
		parent::__construct($paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig);
	}

	/**
	 * Get the calendar
	 * Return example: calendar = array("got_search_result" => TRUE, "body_types" => array());
	 * Return example: calendar['body_types'][0] = array("items" => array());
	 * Return example: calendar['body_types'][0]['items'][0]['transmission_type_title'] = "Manual";
	 * Return example: calendar['body_types'][0]['items'][0]['day_list'][0]['print_day'] = "1";
	 * Return example: calendar['body_types'][0]['items'][0]['day_list'][0]['units_in_stock'] = "5";
	 * @param string $paramYear
	 * @param string $paramMonth
	 * @return mixed
	 */
	public function getCalendar($paramYear="current", $paramMonth="current")
	{
		$internalConfig = array(
			"conf_currency_symbol" => $this->currencySymbol,
			"conf_currency_code" => $this->currencyCode,
			"conf_vat_percentage" => $this->vatPercentage,
			"conf_show_prices_with_vat" => $this->showPricesWithVAT,
		);
		$objItemsObserver = new NRSItemsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang, $internalConfig);
		$validYear = $paramYear == "current" ? date("Y") : NRSValidator::getValidInteger($paramYear, 0);
		$validMonth = $paramMonth == "current" ? date("m") : NRSValidator::getValidInteger($paramMonth, 0);
		// Last True means that we convert to GMT for time, because it's a strict date provided
		$nameOfMonth = date_i18n("F", strtotime("{$validYear}-{$validMonth}-01 00:00:00"), TRUE);
		// No SQL executed inside
		$arrDaysOfMonth = $this->getAllDaysOfTheMonth($validYear, $validMonth);

		$gotSearchResult = false;
		$itemTypesWithItems = array();
		// Includes items with no type
		if($objItemsObserver->areItemsClassified())
		{
            // Fast SQL
            $allBodyTypes = $objItemsObserver->getAllBodyTypes();
			foreach($allBodyTypes AS $type)
			{
				$itemIds = $objItemsObserver->getAvailableIdsForCalendar($type['body_type_id'], TRUE);
				$type['items'] = array();
				$type['got_search_result'] = false;
				foreach($itemIds AS $itemId)
				{
					$objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId, $this->savedConfig);
					$item = $objItem->getBasicDetails();
					// Add days data to item row
					$item['day_list'] = $this->getMonthDaysWithQuantity($itemId, $validYear, $validMonth);
					$type['items'][] = $item;
					$type['got_search_result'] = true;
					$gotSearchResult = true;
				}
				// Add to stack
				$itemTypesWithItems[] = $type;
			}
		} else
		{
			// Same, just everything added to param zero
			$itemIds = $objItemsObserver->getAvailableIdsForCalendar(0, FALSE);
			$type['items'] = array();
			$type['got_search_result'] = false;
			foreach($itemIds AS $itemId)
			{
				$objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId, $this->savedConfig);
				$item = $objItem->getBasicDetails();
				// Add days data to item row
				$item['day_list'] = $this->getMonthDaysWithQuantity($itemId, $validYear, $validMonth);
				$type['items'][] = $item;
				$type['got_search_result'] = true;
			}
			// Add to stack
			$itemTypesWithItems[0] = $type;
		}

		$calendar = array(
			"print_year" => $validYear,
			"print_month" => $validMonth,
			"print_month_name" => $nameOfMonth,
			"print_days" => $arrDaysOfMonth,
			"total_days" => sizeof($arrDaysOfMonth),
			"body_types" => $itemTypesWithItems,
			"got_search_result" => $gotSearchResult,
		);

		if($this->debugMode)
		{
			echo "Year: {$validYear}, Month: {$validMonth}, Name of Month: {$nameOfMonth}<br />";
		}

		return $calendar;
	}

	public function getMonthDaysWithQuantity($paramItemId, $paramYear="current", $paramMonth="current")
	{
        if($this->debugMode)
        {
            echo "<strong>[START] getMonthDaysWithQuantity for Item Id: {$paramItemId}</strong><br />";
            echo "-----------------------------------------------------------------------<br />";
        }

		$validYear = $paramYear == "current" ? date("Y") : NRSValidator::getValidInteger($paramYear, 0);
		$validMonth = $paramMonth == "current" ? date("m") : NRSValidator::getValidInteger($paramMonth, 0);

		$days = array();
        // No SQL queries inside
		$allDaysOfSelectedMonth = $this->getAllDaysOfTheMonth($validYear, $validMonth);
		foreach($allDaysOfSelectedMonth AS $selectedDay)
		{
			$localStartOfDayTimestamp = NRSValidator::getStartOfDayTimestampInWebsiteTimezone("{$validYear}-{$validMonth}-{$selectedDay}");
			$localNoonOfDayTimestamp = NRSValidator::getLocalNoonOfDayTimestampInWebsiteTimezone("{$validYear}-{$validMonth}-{$selectedDay}", $this->noonTime);
			$localEndOfDayTimestamp = NRSValidator::getLocalEndOfDayTimestampInWebsiteTimezone("{$validYear}-{$validMonth}-{$selectedDay}");
			$itemUnitsManager = new NRSItemUnitManager(
				$this->internalWPDB,
				$this->nrsPrefix,
				$this->lang,
				$paramItemId,
				$localStartOfDayTimestamp,
				$localEndOfDayTimestamp,
                $this->savedConfig
			);
            $partialItemUnitsManager = new NRSItemUnitManager(
                $this->internalWPDB,
                $this->nrsPrefix,
                $this->lang,
                $paramItemId,
                $localNoonOfDayTimestamp,
                $localEndOfDayTimestamp,
                $this->savedConfig
            );

            // How many units of one item we have (in stock/available/booked)
            $arrTotalUnits = $itemUnitsManager->getTotalUnits();

            // How many units of one item is in stock
            $totalUnitsInStock = $arrTotalUnits['units_in_stock'];

            // How many units of one item is available for a full day (00:00:00 [LOCAL TIME] - 23:59:59 [LOCAL TIME])
            $unitsAvailable = $arrTotalUnits['units_available'];

            // How many units of one item is available in 2nd half of the day (CONFIG 'noon_time' (DEFAULT - 12:00:00 [LOCAL TIME]) - 23:59:59 [LOCAL TIME])
            $partialUnitsAvailable = $partialItemUnitsManager->getTotalUnitsAvailable();

            $quantityClass = $unitsAvailable == 0 ? "all-taken" : "has-available";
			$printUnitsAvailable = NRSValidator::getDashInsteadOfNumberIfTimestampIsPast($unitsAvailable, $localEndOfDayTimestamp);
            $printPartialUnitsAvailable = NRSValidator::getDashInsteadOfNumberIfTimestampIsPast($partialUnitsAvailable, $localEndOfDayTimestamp);

			$printOrderExtension = $this->lang->getOrderExtensionByPosition(
                (int) $selectedDay,
                $this->lang->getText('NRS_ON_ST_TEXT'),
                $this->lang->getText('NRS_ON_ND_TEXT'),
                $this->lang->getText('NRS_ON_RD_TEXT'),
                $this->lang->getText('NRS_ON_TH_TEXT')
            );

            $printSelectedDay = ((int) $selectedDay).$printOrderExtension;

            $days[] = array(
				"units_in_stock" 				=> $totalUnitsInStock,
				"units_available" 				=> $unitsAvailable,
                "partial_units_available" 		=> $partialUnitsAvailable,
                "print_day" 					=> $printSelectedDay,
				"print_quantity_class"  		=> $quantityClass,
				"print_units_available"  		=> $printUnitsAvailable,
				"print_partial_units_available"	=> $printPartialUnitsAvailable,
			);

			if($this->debugMode)
			{
				echo "-&gt;  Year: {$validYear} Month: {$validMonth}, Day: {$selectedDay}, ";
                echo "Units (avail./part. avail./in stock): {$unitsAvailable}/{$partialUnitsAvailable}/{$totalUnitsInStock}<br />";
				echo "-&gt;  Timestamps in Local TMZ (start/noon/end): {$localStartOfDayTimestamp} - {$localNoonOfDayTimestamp} - {$localEndOfDayTimestamp}<br />";
                echo "<br />";
			}
		}

		return $days;
	}
}