<?php
/**
 * NRS Items Observer (no setup for single item)
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSItemsObserver
{
    protected $internalWPDB = NULL;
    protected $nrsPrefix = "";
    protected $lang = NULL;
    protected $debugMode = 0;
    protected $classifyItems = FALSE;

    protected $currencySymbol		= '$';
    protected $currencyCode			= 'USD';
    protected $vatPercentage		= 0.00;
    protected $showPricesWithVAT	= 0;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        $this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
        $this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
        $this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
        $this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));

        // Set classified status
        $this->setClassifiedStatus();
    }

    /**
     * Set classified status - called in class constructor
     */
    private function setClassifiedStatus()
    {
        $sql = "
				SELECT conf_value AS classify_items
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_classify_items'
			";
        $classifyItems = $this->internalWPDB->get_var($sql);

        if(!is_null($classifyItems))
        {
            $this->classifyItems = $classifyItems == "1" ? TRUE : FALSE;
        } else
        {
            $this->classifyItems = FALSE;
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getAvailableIdsForSlider($paramTypeID = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("IN_SLIDER", $paramTypeID, $paramExactTypeOnly);
    }

    public function getAvailableIdsForList($paramTypeID = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("IN_ITEM_LIST", $paramTypeID, $paramExactTypeOnly);
    }

    public function getAvailableIdsForPriceTable($paramTypeID = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("IN_PRICE_TABLE", $paramTypeID, $paramExactTypeOnly);
    }

    public function getAvailableIdsForCalendar($paramTypeID = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("IN_AVAILABILITY_CALENDAR", $paramTypeID, $paramExactTypeOnly);
    }

    /**
     * Get a car which has amount > 0 of items and status != hidden
     * @param int $paramTypeId
     * @param bool $paramExactTypeOnly
     * @return array
     */
    public function getAvailableIds($paramTypeId = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("AVAILABLE", $paramTypeId, $paramExactTypeOnly);
    }

    public function getAllIds($paramTypeId = 0, $paramExactTypeOnly = FALSE)
    {
        return $this->getIds("ALL", $paramTypeId, $paramExactTypeOnly);
    }

    /**
     * @param string $displayMode - one of display modes: "ALL", "AVAILABLE", "IN_SLIDER", "IN_ITEM_LIST", "IN_PRICE_TABLE", "IN_AVAILABILITY_CALENDAR"
     * @param int $paramTypeId
     * @param bool $paramExactTypeOnly
     * @return array
     */
    private function getIds($displayMode = "ALL", $paramTypeId = 0, $paramExactTypeOnly = FALSE)
    {
        $validTypeID = NRSValidator::getValidInteger($paramTypeId, 0);

        switch($displayMode)
        {
            case "IN_SLIDER":
                $sqlWhere = "WHERE it.units_in_stock > 0 AND enabled = '1' AND it.display_in_slider='1'";
                break;

            case "IN_ITEM_LIST":
                $sqlWhere = "WHERE it.units_in_stock > 0 AND enabled = '1' AND it.display_in_item_list='1'";
                break;

            case "IN_PRICE_TABLE":
                $sqlWhere = "WHERE it.units_in_stock > 0 AND enabled = '1' AND it.display_in_price_table='1'";
                break;

            case "IN_AVAILABILITY_CALENDAR":
                $sqlWhere = "WHERE it.units_in_stock > 0 AND enabled = '1' AND it.display_in_calendar='1'";
                break;

            case "AVAILABLE":
                $sqlWhere = "WHERE it.units_in_stock > 0 AND enabled = '1'";
                break;

            default:
                $sqlWhere = "WHERE 1";
        }
        if(($paramExactTypeOnly == TRUE && $validTypeID >= 0) || ($paramExactTypeOnly == FALSE && $validTypeID > 0))
        {
            $sqlWhere .= " AND it.body_type_id='{$validTypeID}'";
        }

        $searchSQL = "
            SELECT it.item_id
            FROM {$this->nrsPrefix}items it
            LEFT JOIN {$this->nrsPrefix}manufacturers mf ON it.manufacturer_id=mf.manufacturer_id
            {$sqlWhere}
            ORDER BY manufacturer_title ASC, model_name ASC
		";

        //DEBUG
        //echo nl2br($searchSQL)."<br /><br />";

        $searchResult = $this->internalWPDB->get_col($searchSQL);

        return $searchResult;
    }

    /**
     * Do items are classified?
     * @return bool
     */
    public function areItemsClassified()
    {
        return $this->classifyItems;
    }

    /**
     * @param int $paramItemId
     * @param bool $paramShowItemId
     * @return string
     */
    public function getItemDropDown($paramItemId = 0, $paramShowItemId = TRUE)
    {
        $itemHTML = '<option value="">'.$this->lang->getText('NRS_SELECT_DROPDOWN_TEXT').'</option>';

        $itemResult = $this->internalWPDB->get_results("
            SELECT it.item_id, it.manufacturer_id, it.model_name, mf.manufacturer_title
            FROM {$this->nrsPrefix}items it
            LEFT JOIN {$this->nrsPrefix}manufacturers mf ON it.manufacturer_id=mf.manufacturer_id
            ORDER BY manufacturer_title ASC, model_name ASC
        ", ARRAY_A);

        foreach ($itemResult AS $itemData)
        {
            $title = $itemData['manufacturer_title'].' '.$itemData['model_name'];
            if($paramShowItemId)
            {
                $title .= " (ID=".$itemData['item_id'].")";
            }
            if($paramItemId == $itemData['item_id'])
            {
                $itemHTML .= '<option value='.$itemData['item_id'].' selected="selected">'.$title.'</option>';
            } else
            {
                $itemHTML .= '<option value='.$itemData['item_id'].'>'.$title.'</option>';
            }
        }

        return $itemHTML;
    }

    /**
     * @param int $paramManufacturerId
     * @param string $selectLabel
     * @return string
     */
    public function getManufacturersDropDown($paramManufacturerId = 0, $selectLabel = "")
    {
        $manufacturerHTML = '<select name="manufacturer_id" id="manufacturer_id">';
        $selectedManufacturerZero = $paramManufacturerId == 0 ? ' selected="selected"' : '';
        $manufacturerHTML .= '<option value="0"'.$selectedManufacturerZero.'>'.$selectLabel.'</option>';
        $manufacturerData = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}manufacturers", ARRAY_A);
        foreach($manufacturerData AS $manufacturerRow)
        {
            if($manufacturerRow['manufacturer_id'] == $paramManufacturerId)
            {
                $manufacturerHTML .= '<option value="'.$manufacturerRow['manufacturer_id'].'" selected="selected">'.$manufacturerRow['manufacturer_title'].'</option>';
            } else
            {
                $manufacturerHTML .= '<option value="'.$manufacturerRow['manufacturer_id'].'">'.$manufacturerRow['manufacturer_title'].'</option>';
            }
        }
        $manufacturerHTML .= '</select>';
        return $manufacturerHTML;
    }

    /**
     * Get car body type - sedan, compact, jeep etc.
     * @param int $paramSelectedBodyTypeId - for edit mode
     * @param string $selectLabel
     * @return string type drop-down html
     */
    public function getBodyTypesDropDown($paramSelectedBodyTypeId = 0, $selectLabel = "")
    {
        $itemBodyTypeHTML = '<select name="body_type_id" id="body_type_id">';
        $selectedTypeZero = $paramSelectedBodyTypeId == 0 ? ' selected="selected"' : '';
        $itemBodyTypeHTML .= '<option value="0"'.$selectedTypeZero.'>'.$selectLabel.'</option>';
        $itemBodyTypes = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}body_types", ARRAY_A);
        foreach($itemBodyTypes AS $itemTypeRow)
        {
            if($itemTypeRow['body_type_id'] == $paramSelectedBodyTypeId)
            {
                $itemBodyTypeHTML .= '<option value="'.$itemTypeRow['body_type_id'].'" selected="selected">'.$itemTypeRow['body_type_title'].'</option>';
            } else
            {
                $itemBodyTypeHTML .= '<option value="'.$itemTypeRow['body_type_id'].'">'.$itemTypeRow['body_type_title'].'</option>';
            }
        }
        $itemBodyTypeHTML .= '</select>';

        return $itemBodyTypeHTML;
    }

    /**
     * Get Car transmission
     * @param int $paramSelectedTransmissionId - for edit mode
     * @param string $selectLabel
     * @return string transmissions drop-down html
     */
    public function getTransmissionTypesDropDown($paramSelectedTransmissionId = 0, $selectLabel = "")
    {
        $itemTransmissionsHTML = '<select name="transmission_type_id" id="transmission_type_id">';
        $selectedTransmissionZero = $paramSelectedTransmissionId == 0 ? ' selected="selected"' : '';
        $itemTransmissionsHTML .= '<option value="0"'.$selectedTransmissionZero.'>'.$selectLabel.'</option>';
        $arrTransmissionTypes = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}transmission_types", ARRAY_A);
        foreach($arrTransmissionTypes AS $transmissionTypeRow)
        {
            if($transmissionTypeRow['transmission_type_id'] == $paramSelectedTransmissionId)
            {
                $itemTransmissionsHTML .= '<option value="'.$transmissionTypeRow['transmission_type_id'].'" selected="selected">'.$transmissionTypeRow['transmission_type_title'].'</option>';
            } else
            {
                $itemTransmissionsHTML .= '<option value="'.$transmissionTypeRow['transmission_type_id'].'">'.$transmissionTypeRow['transmission_type_title'].'</option>';
            }
        }
        $itemTransmissionsHTML .= '</select>';

        return $itemTransmissionsHTML;
    }

    /**
     * Get car fuel type - petrol, diesel etc.
     * @param int $selectedTypeID - for edit mode
     * @param string $selectLabel
     * @return string type drop-down html
     */
    public function getFuelTypesDropDown($selectedTypeID = 0, $selectLabel = "")
    {
        $itemFuelTypeHTML = '<select name="fuel_type_id" id="fuel_type_id">';
        $selectedTypeZero = $selectedTypeID == 0 ? ' selected="selected"' : '';
        $itemFuelTypeHTML .= '<option value="0"'.$selectedTypeZero.'>'.$selectLabel.'</option>';
        $itemFuelTypes = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}fuel_types", ARRAY_A);
        foreach($itemFuelTypes AS $itemTypeRow)
        {
            if($itemTypeRow['fuel_type_id'] == $selectedTypeID)
            {
                $itemFuelTypeHTML .= '<option value="'.$itemTypeRow['fuel_type_id'].'" selected="selected">'.$itemTypeRow['fuel_type_title'].'</option>';
            } else
            {
                $itemFuelTypeHTML .= '<option value="'.$itemTypeRow['fuel_type_id'].'">'.$itemTypeRow['fuel_type_title'].'</option>';
            }
        }
        $itemFuelTypeHTML .= '</select>';

        return $itemFuelTypeHTML;
    }

    public function getAllBodyTypes($includeUnclassified = TRUE)
    {
        $itemTypes = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}body_types
            ORDER BY body_type_title ASC
        ", ARRAY_A);
        if($itemTypes == null)
        {
            $itemTypes = array();
        }

        // Add unclassified types results
        if($includeUnclassified)
        {
            $itemTypes[] = array(
                "body_type_id" => "0",
                "body_type_title" => $this->lang->getText('NRS_UNCLASSIFIED_ITEM_TYPE_TEXT'),
            );
        }

        return $itemTypes;
    }

    /*********************************************************************************************/
    /*********************************************************************************************/
    /*********************************************************************************************/
    /**
     * ----------------------------------------------------------
     * ------------------ FEATURES SECTION ----------------------
     * ----------------------------------------------------------
     */

    /**
     * If add_to_all_items is checked - this function will also add it to all items for new item
     * @param $paramFeatureId
     */
    public function updateFeature($paramFeatureId)
    {
        $validFeatureId = NRSValidator::getValidInteger($paramFeatureId);
        $sanitizedFeatureTitle = sanitize_text_field($_POST['feature_title']);
        $validDisplayInItemList = (sanitize_text_field($_POST['display_in_item_list']) == "on" ? 1 : 0);
        if($validFeatureId == 0)
        {
            $addToAllItems = (sanitize_text_field($_POST['add_to_all_items']) == "on" ? TRUE : FALSE);
        } else
        {
            $addToAllItems = FALSE;
        }


        if($validFeatureId != 0)
        {
            $exist1 = $this->internalWPDB->get_row("
                SELECT feature_id
                FROM {$this->nrsPrefix}features
                WHERE feature_title='{$sanitizedFeatureTitle}' AND feature_id!='{$validFeatureId}'
            ", ARRAY_A);

            if(is_null($exist1))
            {
                $this->internalWPDB->query("
                     UPDATE {$this->nrsPrefix}features SET
                     feature_title='{$sanitizedFeatureTitle}', display_in_item_list='{$validDisplayInItemList}'
                     WHERE feature_id='{$validFeatureId}'
                ");
                $_SESSION['ok_msg'] = "Completed: Car feature updated successfully!";
            } else
            {
                $_SESSION['msg'] = "Error: This car feature already exists!";
            }
        } else
        {
            $exist1 = $this->internalWPDB->get_row("
                SELECT *
                FROM {$this->nrsPrefix}features
                WHERE feature_title='{$sanitizedFeatureTitle}'
            ", ARRAY_A);

            if(is_null($exist1))
            {
                $featureInsertedSuccessfully = $this->internalWPDB->query("
                      INSERT INTO {$this->nrsPrefix}features (
                        feature_title, display_in_item_list
                      ) VALUES
                      (
                        '{$sanitizedFeatureTitle}', '{$validDisplayInItemList}'
                      )
                ");
                $insertedNewFeatureId = $this->internalWPDB->insert_id;
                if($featureInsertedSuccessfully === FALSE)
                {
                    // TODO: MOVE TO NRS ADMIN LANG FILE
                    $_SESSION['msg'] = "Error: New car data was not allowed to insert into database!";

                } else
                {
                    $items = $this->internalWPDB->get_results("
                        SELECT item_id
                        FROM {$this->nrsPrefix}items
                    ", ARRAY_A);

                    if(!is_null($items) && $addToAllItems == TRUE)
                    {
                        foreach($items as $item)
                        {
                            $this->internalWPDB->query("
                                  INSERT INTO {$this->nrsPrefix}item_features
                                  (
                                    item_id, feature_id
                                  ) VALUES
                                  (
                                    '{$item['item_id']}', '{$insertedNewFeatureId}'
                                  )
                            ");
                        }
                    }

                    $_SESSION['ok_msg'] = "Completed: Car feature added successfully!";
                }
            } else
            {
                $_SESSION['msg'] = "Error: This car feature already exists!";
            }
        }
    }

    public function getFeatureData($paramFeatureId)
    {
        $featuresRow = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}features
            WHERE feature_id='".intval($paramFeatureId)."'
        ", ARRAY_A);
        return $featuresRow;
    }

    public function deleteFeature($paramFeatureId)
    {
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}features
            WHERE feature_id='".intval($paramFeatureId)."'
        ");
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}item_features
            WHERE feature_id='".intval($paramFeatureId)."'
        ");
        $_SESSION['ok_msg'] = "Completed: Car feature deleted successfully!";
    }

    /**
     * ----------------------------------------------------------
     * ------------------ BODY TYPES SECTION --------------------
     * ----------------------------------------------------------
     */

    /**
     * @param $paramBodyTypeId
     */
    public function updateBodyType($paramBodyTypeId)
    {
        $validBodyTypeId = NRSValidator::getValidInteger($paramBodyTypeId,0);
        $sanitizedBodyTypeTitle = sanitize_text_field($_POST['body_type_title']);

        if($validBodyTypeId != 0)
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT body_type_id
                FROM {$this->nrsPrefix}body_types
                WHERE body_type_title='{$sanitizedBodyTypeTitle}' AND body_type_id!='{$validBodyTypeId}'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                      UPDATE {$this->nrsPrefix}body_types SET
                      body_type_title='{$sanitizedBodyTypeTitle}'
                      WHERE body_type_id='".intval($validBodyTypeId)."'
                ");
                $_SESSION['ok_msg'] = "Completed: Car body type updated successfully!";
                //die("ITEM TYPE UPDATED");
            } else
            {
                $_SESSION['msg'] = "Error: This car type already exists!";
            }
        } else
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT body_type_id
                FROM {$this->nrsPrefix}body_types
                WHERE body_type_title='{$sanitizedBodyTypeTitle}'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}body_types
                    (body_type_title)
                    VALUES
                    ('{$sanitizedBodyTypeTitle}')
                ");
                $_SESSION['ok_msg'] = "Completed: Car body type added successfully!";
                //die("NEW ITEM TYPE INSERTED");
            } else
            {
                $_SESSION['msg'] = "Error: This car type already exists!";
            }
        }
    }

    public function getBodyTypeData($paramBodyTypeId)
    {
        $typesRow = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}body_types
            WHERE body_type_id='".intval($paramBodyTypeId)."'
        ", ARRAY_A);
        return $typesRow;

    }

    public function deleteBodyType($paramBodyTypeId)
    {
        $rows = $this->internalWPDB->get_results("
            SELECT item_id
            FROM {$this->nrsPrefix}items
            WHERE body_type_id='".intval($paramBodyTypeId)."'
        ", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['item_id']);
                $objItem->delete();
            }
        }//die;

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}body_types
            WHERE body_type_id='".intval($paramBodyTypeId)."'
        ");
        $_SESSION['ok_msg'] = "Completed: Car body type deleted successfully!";
    }


    /**
     * ----------------------------------------------------------
     * ------------ ITEM FUEL TYPES SECTION ----------------------
     * ----------------------------------------------------------
     */

    /**
     * @param $paramFuelTypeId
     */
    public function updateFuelType($paramFuelTypeId)
    {
        $validFuelTypeId = NRSValidator::getValidInteger($paramFuelTypeId,0);
        $sanitizedFuelTypeTitle = sanitize_text_field($_POST['fuel_type_title']);

        if($validFuelTypeId != 0)
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT fuel_type_id
                FROM {$this->nrsPrefix}fuel_types
                WHERE fuel_type_title='{$sanitizedFuelTypeTitle}' AND fuel_type_id!='".intval($validFuelTypeId)."'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                      UPDATE {$this->nrsPrefix}fuel_types SET
                      fuel_type_title='{$sanitizedFuelTypeTitle}'
                      WHERE fuel_type_id='{$validFuelTypeId}'
                ");
                $_SESSION['ok_msg'] = "Completed: Car fuel type updated successfully!";
                //die("ITEM TYPE UPDATED");
            } else
            {
                $_SESSION['msg'] = "Error: This car fuel type already exists!";
            }
        } else
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT fuel_type_id
                FROM {$this->nrsPrefix}fuel_types
                WHERE fuel_type_title='{$sanitizedFuelTypeTitle}'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}fuel_types
                    (fuel_type_title)
                    VALUES
                    ('{$sanitizedFuelTypeTitle}')
                ");
                $_SESSION['ok_msg'] = "Completed: Car fuel type added successfully!";
                //die("NEW ITEM TYPE INSERTED");
            } else
            {
                $_SESSION['msg'] = "Error: This car fuel type already exists!";
            }
        }
    }

    public function getFuelTypeData($paramFuelTypeId)
    {
        $fuelTypeRow = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}fuel_types
            WHERE fuel_type_id='".intval($paramFuelTypeId)."'
        ", ARRAY_A);
        return $fuelTypeRow;

    }

    public function deleteFuelType($paramFuelTypeId)
    {
        $rows = $this->internalWPDB->get_results("
            SELECT item_id
            FROM {$this->nrsPrefix}items
            WHERE fuel_type_id='".intval($paramFuelTypeId)."'
        ", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['item_id']);
                $objItem->delete();
            }
        }//die;

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}fuel_types
            WHERE fuel_type_id='".intval($paramFuelTypeId)."'
        ");
        $_SESSION['ok_msg'] = "Completed: Car fuel type deleted successfully!";
    }



    /**
     * ----------------------------------------------------------
     * ------------ ITEM TRANSMISSIONS SECTION -------------------
     * ----------------------------------------------------------
     */

    /**
     * @param $paramTransmissionTypeId
     */
    public function updateTransmissionType($paramTransmissionTypeId)
    {
        $validTransmissionTypeId = NRSValidator::getValidInteger($paramTransmissionTypeId,0);
        $sanitizedTransmissionTypeTitle = sanitize_text_field($_POST['transmission_type_title']);

        if($validTransmissionTypeId != 0)
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT transmission_type_id
                FROM {$this->nrsPrefix}transmission_types
                WHERE transmission_type_title='{$sanitizedTransmissionTypeTitle}' AND transmission_type_id!='{$validTransmissionTypeId}'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                    UPDATE {$this->nrsPrefix}transmission_types SET
                    transmission_type_title='{$sanitizedTransmissionTypeTitle}'
                    WHERE transmission_type_id='{$validTransmissionTypeId}'
                ");
                $_SESSION['ok_msg'] = "Completed: Car transmission type updated successfully!";
                //die("ITEM TRANSMISSION UPDATED");
            } else
            {
                $_SESSION['msg'] = "Error: This car transmission type already exists!";
            }
        } else
        {
            $exist2 = $this->internalWPDB->get_row("
                SELECT transmission_type_id
                FROM {$this->nrsPrefix}transmission_types
                WHERE transmission_type_title='{$sanitizedTransmissionTypeTitle}'
            ", ARRAY_A);

            if(is_null($exist2))
            {
                $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}transmission_types
                    (transmission_type_title)
                    VALUES
                    ('{$sanitizedTransmissionTypeTitle}')
                ");
                $_SESSION['ok_msg'] = "Completed: Car transmission type added successfully!";
                //die("NEW ITEM TRANSMISSION INSERTED");
            } else
            {
                $_SESSION['msg'] = "Error: This car transmission type already exists!";
            }
        }
    }

    public function getTransmissionTypeData($paramTransmissionTypeId)
    {
        $transmissionRow = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}transmission_types
            WHERE transmission_type_id='".intval($paramTransmissionTypeId)."'
        ", ARRAY_A);
        return $transmissionRow;

    }

    public function deleteTransmissionType($paramTransmissionTypeId)
    {
        $rows = $this->internalWPDB->get_results("
            SELECT item_id
            FROM {$this->nrsPrefix}items
            WHERE transmission_type_id='".intval($paramTransmissionTypeId)."'
        ", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['item_id']);
                $objItem->delete();
            }
        }//die;

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}transmission_types
            WHERE transmission_type_id='".intval($paramTransmissionTypeId)."'
        ");
        $_SESSION['ok_msg'] = "Completed: Car transmission deleted successfully!";
    }


    /**
     * ----------------------------------------------------------
     * --------------- MANUFACTURERS SECTION --------------------
     * ----------------------------------------------------------
     */

    /**
     * @param $paramManufacturerId
     * @return mixed
     */
    public function getManufacturerData($paramManufacturerId)
    {
        $typesRow = $this->internalWPDB->get_row("
            SELECT * FROM {$this->nrsPrefix}manufacturers
            WHERE manufacturer_id='".intval($paramManufacturerId)."'
        ", ARRAY_A);

        return $typesRow;
    }

    public function updateManufacturer($paramManufacturerId)
    {
        $validManufacturerId = NRSValidator::getValidInteger($paramManufacturerId,0);
        $sanitizedManufacturerTitle = sanitize_text_field($_POST['manufacturer_title']);

        if($validManufacturerId != 0)
        {
            $exist = $this->internalWPDB->get_row("
                SELECT manufacturer_id
                FROM {$this->nrsPrefix}manufacturers
                WHERE manufacturer_title='{$sanitizedManufacturerTitle}' AND manufacturer_id!='{$validManufacturerId}'
             ", ARRAY_A);
            if(is_null($exist))
            {
                $this->internalWPDB->query("
                    UPDATE {$this->nrsPrefix}manufacturers SET
                    manufacturer_title='{$sanitizedManufacturerTitle}'
                    WHERE manufacturer_id='{$validManufacturerId}'
                ");
                $_SESSION['ok_msg'] = "Completed: Car manufacturer updated successfully!";
            } else
            {
                $_SESSION['msg'] = "Error: This car manufacturer already exists!";
            }
        } else
        {
            $exist = $this->internalWPDB->get_row("
                SELECT manufacturer_id
                FROM {$this->nrsPrefix}manufacturers
                WHERE manufacturer_title='{$sanitizedManufacturerTitle}'
            ", ARRAY_A);

            if(is_null($exist))
            {
                $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}manufacturers
                    (manufacturer_title)
                    VALUES
                    ('{$sanitizedManufacturerTitle}')
                ");
                $_SESSION['ok_msg'] = "Completed: Car manufacturer added successfully!";
            } else
            {
                $_SESSION['msg'] = "Error: This Car Manufacturer already exists!";
            }
        }
    }

    public function deleteManufacturer($paramManufacturerId)
    {
        $rows = $this->internalWPDB->get_results("
            SELECT item_id
            FROM {$this->nrsPrefix}items
            WHERE manufacturer_id='".intval($paramManufacturerId)."'
        ", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['item_id']);
                $objItem->delete();
            }
        }//die;

        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}manufacturers
            WHERE manufacturer_id='".intval($paramManufacturerId)."'
        ");
        $_SESSION['ok_msg'] = "Completed: Car manufacturer deleted successfully!";
    }

    /*******************************************************************************/
    /********************** METHODS FOR ADMIN ACCESS ONLY **************************/
    /*******************************************************************************/

    public function getAdminManufacturersList()
    {
        $manufacturersHTML = '';
        $rows = $this->internalWPDB->get_results("
            SELECT * FROM {$this->nrsPrefix}manufacturers
        ", ARRAY_A);

        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $manufacturersHTML .= '<tr>';
                $manufacturersHTML .= '<td style="width: 1%">'.$row['manufacturer_id'].'</td>';
                $manufacturersHTML .= '<td>'.$row['manufacturer_title'].'</td>';
                $manufacturersHTML .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-manufacturer&amp;manufacturer_id='.$row['manufacturer_id']).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
                $manufacturersHTML .= '<a href="javascript:;" onclick="javascript:deleteManufacturer(\''.$row['manufacturer_id'].'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
                $manufacturersHTML .= '</tr>';
            }
        }

        return  $manufacturersHTML;
    }

    public function getAdminBodyTypesList()
    {
        $bodyTypesHTML = '';
        $rows = $this->internalWPDB->get_results("
            SELECT body_type_id, body_type_title
            FROM {$this->nrsPrefix}body_types
            ORDER BY body_type_title ASC
        ", ARRAY_A);

        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $bodyTypesHTML .= '<tr>';
                $bodyTypesHTML .= '<td style="width: 1%">'.$row['body_type_id'].'</td>';
                $bodyTypesHTML .= '<td>'.$row['body_type_title'].'</td>';
                $bodyTypesHTML .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-body-type&amp;body_type_id='.$row['body_type_id']).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
                $bodyTypesHTML .= ' <a href="javascript:;" onclick="javascript:deleteBodyType(\''.$row['body_type_id'].'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a>';
                $bodyTypesHTML .= '</td>';
                $bodyTypesHTML .= '</tr>';
            }
        }

        return  $bodyTypesHTML;
    }

    public function getAdminTransmissionTypesList()
    {
        $transmissionTypesHtml = '';
        $rows = $this->internalWPDB->get_results("
            SELECT transmission_type_id, transmission_type_title
            FROM {$this->nrsPrefix}transmission_types
            ORDER BY transmission_type_title ASC
        ", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $transmissionTypesHtml .= '<tr>';
                $transmissionTypesHtml .= '<td style="width: 1%">'.$row['transmission_type_id'].'</td>';
                $transmissionTypesHtml .= '<td>'.$row['transmission_type_title'].'</td>';
                $transmissionTypesHtml .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-transmission-type&amp;transmission_type_id='.$row['transmission_type_id']).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
                $transmissionTypesHtml .= '<a href="javascript:;" onclick="javascript:deleteTransmissionType(\''.$row['transmission_type_id'].'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
                $transmissionTypesHtml .= '</tr>';
            }
        }
        return  $transmissionTypesHtml;
    }

    public function getAdminFuelTypesList()
    {
        $fuelTypesHTML = '';
        $rows = $this->internalWPDB->get_results("
            SELECT fuel_type_id, fuel_type_title
            FROM {$this->nrsPrefix}fuel_types
            ORDER BY fuel_type_title ASC
        ", ARRAY_A);

        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                $fuelTypesHTML .= '<tr>';
                $fuelTypesHTML .= '<td style="width: 1%">'.$row['fuel_type_id'].'</td>';
                $fuelTypesHTML .= '<td>'.$row['fuel_type_title'].'</td>';
                $fuelTypesHTML .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-fuel-type&amp;fuel_type_id='.$row['fuel_type_id']).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
                $fuelTypesHTML .= ' <a href="javascript:;" onclick="javascript:deleteFuelType(\''.$row['fuel_type_id'].'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a>';
                $fuelTypesHTML .= '</td>';
                $fuelTypesHTML .= '</tr>';
            }
        }

        return  $fuelTypesHTML;
    }

    public function getAdminFeaturesList()
    {
        $featuresHTML = '';
        $rows = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}features", ARRAY_A);
        if(sizeof($rows) > 0)
        {
            foreach ($rows AS $row)
            {
                // TO-DO: TRANSLATION NEEDED
                $textInItemList = ($row['display_in_item_list'] == 1 ? "Displayed" : "Hidden");
                $featuresHTML .= '<tr>';
                $featuresHTML .= '<td>'.$row['feature_title'].'</td>';
                $featuresHTML .= '<td>'.$textInItemList.'</td>';
                $featuresHTML .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-feature&amp;feature_id='.$row['feature_id']).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
                $featuresHTML .= '<a href="javascript:;" onclick="javascript:deleteFeature(\''.$row['feature_id'].'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
                $featuresHTML .= '</tr>';
            }
        }
        return  $featuresHTML;
    }

    /**
     * @return string
     */
    public function getAdminList()
    {
        $itemIds = $this->getAllIds();
        $itemList = '';

        foreach($itemIds AS $itemId)
        {
            $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId);
            $itemDetails = $objItem->getFullDetails();
            $enabled = $itemDetails['enabled'] == 1 ? 'Available' : 'Hidden';
            $displayInSlider = $itemDetails['display_in_slider'] == 1 ? 'Displayed' :  'Hidden';

            if($itemDetails['item_description_page_id'] != 0)
            {
                $itemDetailsPageURL = get_permalink( $itemDetails['item_description_page_id'] );
                $itemDetailsPageTitle = get_the_title( $itemDetails['item_description_page_id'] );
                $itemDetailsPage = '<a href="'.$itemDetailsPageURL.'" target="_blank" title="View page in new window">'.$itemDetailsPageTitle.'</a>';
            } else
            {
                $itemDetailsPage = '<span class="item-description-page-not-set">Not set</span>';
            }

            // TODO: MOVE TEXT TO ADMIN LANG FILE
            $itemList .= '<tr>';
            $itemList .= '<td>'.$itemId.'</td>';
            $itemList .= '<td>'.$itemDetails['body_type_title'].'</td>';
            $itemList .= '<td>'.$itemDetails['transmission_type_title'].'</td>';
            $itemList .= '<td>'.$itemDetails['manufacturer_title'].'</td>';
            $itemList .= '<td>'.$itemDetails['model_name'].'</td>';
            $itemList .= '<td>'.$itemDetailsPage.'</td>';
            $itemList .= '<td style="white-space: nowrap">';
            $itemList .= '<span style="cursor:pointer;" title="Maximum car units per booking">'.$itemDetails['max_units_per_booking'].'</span> / ';
            $itemList .= '<span style="cursor:pointer;font-weight:bold" title="Total car units in garage">'.$itemDetails['units_in_stock'].'</span> ';
            $itemList .= '</td>';
            $itemList .= '<td>'.$itemDetails['fuel_type_title'].'</td>';
            $itemList .= '<td>'.$enabled.'</td>';
            $itemList .= '<td>'.$displayInSlider.'</td>';
            $itemList .= '<td align="right"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-item&amp;item_id='.$itemId).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
            $itemList .= '<a href="javascript:;" onclick="javascript:deleteItem(\''.$itemId.'\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
            $itemList .= '</tr>';
        }

        return  $itemList;
    }

    public function getAdminBlockedList()
    {
        $getHtml = "";

        $internalConfig = array(
            "conf_currency_symbol" => $this->currencySymbol,
            "conf_currency_code" => $this->currencyCode,
            "conf_vat_percentage" => $this->vatPercentage,
            "conf_show_prices_with_vat" => $this->showPricesWithVAT,
        );

        $rows = $this->internalWPDB->get_results("
            SELECT bb.booking_id, bb.pickup_location_id AS blocked_location_id, bopt.item_id, bb.block_name,
            bb.pickup_timestamp AS start_timestamp,
            bb.dropoff_timestamp AS end_timestamp
            FROM {$this->nrsPrefix}bookings bb
            JOIN {$this->nrsPrefix}booking_options bopt ON bb.booking_id=bopt.booking_id AND bopt.item_id!='0'
            WHERE bb.is_block='1'
        ", ARRAY_A);

        if (sizeof($rows) > 0)
        {
            $getHtml .='<tbody>';
            foreach ($rows AS $row)
            {
                $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $row['item_id']);
                if($row['blocked_location_id'] > 0)
                {
                    $objBlockedLocationManager = new NRSLocation(
                        $this->internalWPDB, $this->nrsPrefix, $this->lang,
                        $internalConfig, $row['blocked_location_id']
                    );
                    $blockedLocationName = $objBlockedLocationManager->getLocationName();
                } else
                {
                    $blockedLocationName = $this->lang->getText('NRS_ADMIN_ANY_BRANCH_TEXT');
                }

                $itemDetails = $objItem->getFullDetails();

                $startTMZStamp = $row['start_timestamp'] + get_option( 'gmt_offset' ) * 3600;
                $endTMZStamp = $row['end_timestamp'] + get_option( 'gmt_offset' ) * 3600;
                $printStartDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $startTMZStamp, TRUE);
                $printEndDate = date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), $endTMZStamp, TRUE);


                $getHtml .= '<tr>
                    <td align="left">'.$row['block_name'].'</td>
                    <td>'.$row['item_id'].'</td>
                    <td>'.$itemDetails['body_type_title'].'</td>
                    <td>'.$itemDetails['manufacturer_title'].' '.$itemDetails['model_name'].'</td>
                    <td style="white-space: nowrap">'.$this->lang->getText('NRS_FROM_TEXT').': '.$printStartDate.'<br />'.$this->lang->getText('NRS_TO_TEXT').': '.$printEndDate.'</td>
                    <td>'.$blockedLocationName.'</td>
                    <td align="right">
                        <a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-manager&amp;noheader=true&amp;action=unblock&amp;block_id='.$row['booking_id'].'&amp;item_id='.$row['item_id']).'">'.$this->lang->getText('NRS_ADMIN_UNBLOCK_TEXT').'</a>
                    </td>
                </tr>';
            }
            $getHtml .= '</tbody>';
        }
        return $getHtml;
    }

    /**
     * @param bool $showRowNumbers
     * @return string
     */
    public function getAdminListWithOptions($showRowNumbers = FALSE)
    {
        $itemList = '';
        $itemIds = $this->getAllIds();

        $i = 0;
        foreach($itemIds AS $itemId)
        {
            $i++;
            $objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId);
            $objOptionsManager = new NRSItemOptionManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId);
            $optionsList = $objOptionsManager->getAdminList($showRowNumbers, sprintf('%02d', $i).".");
            $itemDetails = $objItem->getFullDetails();
            $itemModelName = "";
            $manufacturerTitle = "";
            $bodyTypeTitle = "";
            $transmissionTypeTitle = "";
            if(!is_null($itemDetails))
            {
                $itemModelName = $itemDetails['model_name'];
                $manufacturerTitle = $itemDetails['manufacturer_title'];
                $bodyTypeTitle = $itemDetails['body_type_title'];
                $transmissionTypeTitle = $itemDetails['transmission_type_title'];
            }

            if($optionsList != "")
            {
                // HTML OUTPUT
                $itemList .= '<tr>';
                $itemList .= $showRowNumbers ? '<td>'.sprintf('%02d', $i).'</td>' : '';
                $itemList .= '<td>'.$manufacturerTitle.' '.$itemModelName.'</td>';
                $itemList .= '<td>ID: '.$itemId.', '.$bodyTypeTitle.', '.$transmissionTypeTitle.'</td>';
                $itemList .= '<td>&nbsp;</td>';
                $itemList .= '</tr>';
                $itemList .= $optionsList;
            }
        }

        return  $itemList;
    }
}