<?php
/**
 * NRS Item Price Table
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSItemsPriceTable extends NRSPriceTable implements iNRSPriceTable
{
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
	{
		parent::__construct($paramWPDB, $paramNRSPrefix, $paramNRSLang, $paramConfig);
	}

	/**
	 * Get the price table
	 * Return example: priceTable = array("got_search_result" => TRUE, "body_types" => array());
	 * Return example: priceTable['body_types'][0] = array("items" => array());
	 * Return example: priceTable['body_types'][0]['items'][0]['transmission_type_title'] = "Manual";
	 * Return example: priceTable['body_types'][0]['items'][0]['period_list'][0]['period_from'] = "1234567890";
	 * Return example: priceTable['body_types'][0]['items'][0]['period_list'][0]['print_dynamic_period_label'] = "10-20 Hours";
	 * @return mixed
	 */
	public function getPriceTable()
	{
		$objItemsObserver = new NRSItemsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang, $this->savedConfig);
		$objDiscountsObserver = new NRSItemDiscountsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		// SQL is inside
		$allDiscountPeriods = $objDiscountsObserver->getDiscountPeriodsOnBookingDuration();
		if(sizeof($allDiscountPeriods) == 0)
		{
			$allDiscountPeriods = array($objDiscountsObserver->getDefaultDiscountPeriod());
		}

		$gotSearchResult = false;
		$itemTypesWithItems = array();
		// Includes items with no type
		if($objItemsObserver->areItemsClassified())
		{
			$allItemTypes = $objItemsObserver->getAllBodyTypes();

			foreach($allItemTypes AS $type)
			{
				$itemIds = $objItemsObserver->getAvailableIdsForPriceTable($type['body_type_id'], TRUE);
				$type['items'] = array();
				$type['got_search_result'] = false;
				foreach($itemIds AS $itemId)
				{
					$objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId, $this->savedConfig);
                    $objDepositManager = new NRSItemDepositManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $this->savedConfig, $itemId);
					$item = array_merge($objItem->getBasicDetails(), $objDepositManager->getMinimalFixedDepositDetails());
					// Add periods data to item row
					$item['period_list'] = $this->getPeriodsWithPrices($itemId);
					$type['items'][] = $item;
					$type['got_search_result'] = true;
					$gotSearchResult = true;
				}
				// Add to stack
				$itemTypesWithItems[] = $type;
			}
		} else
		{
			// Same, just everything added to param zero
			$itemIds = $objItemsObserver->getAvailableIdsForPriceTable(0, FALSE);
			$type['items'] = array();
			$type['got_search_result'] = false;
			foreach($itemIds AS $itemId)
			{
				$objItem = new NRSItem($this->internalWPDB, $this->nrsPrefix, $this->lang, $itemId, $this->savedConfig);
                $objDepositManager = new NRSItemDepositManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $this->savedConfig, $itemId);
                $item = array_merge($objItem->getBasicDetails(), $objDepositManager->getMinimalFixedDepositDetails());
				// Add periods data to item row
				$item['period_list'] = $this->getPeriodsWithPrices($itemId);
				$type['items'][] = $item;
				$type['got_search_result'] = true;
				$gotSearchResult = true;
			}
			// Add to stack
			$itemTypesWithItems[0] = $type;
		}

        $printDynamicPeriodLabel = $this->lang->getText($this->priceCalculationType == 2 ? 'NRS_HOUR_PRICE_TEXT' : 'NRS_DAY_PRICE_TEXT');

		$priceTable = array(
			"print_periods" => $allDiscountPeriods,
			"total_periods" => sizeof($allDiscountPeriods),
            "print_dynamic_period_label" => $printDynamicPeriodLabel,
			"body_types" => $itemTypesWithItems,
			"got_search_result" => $gotSearchResult,
		);

		return $priceTable;
	}

	public function getPeriodsWithPrices($paramItemId)
	{
		$periods = array();

		$objDiscountsObserver = new NRSItemDiscountsObserver($this->internalWPDB, $this->nrsPrefix, $this->lang);
		$allDiscountPeriods = $objDiscountsObserver->getDiscountPeriodsOnBookingDuration();
		if(sizeof($allDiscountPeriods) == 0)
		{
			$allDiscountPeriods = array($objDiscountsObserver->getDefaultDiscountPeriod());
		}
		foreach($allDiscountPeriods as $discountPeriod)
		{
			$objPriceManager = new NRSItemPriceManager(
				$this->internalWPDB, $this->nrsPrefix, $this->lang,
				$this->savedConfig, $paramItemId
			);
            // Items has different prices for each days of week
            $minimalWeekPriceDetails = $objPriceManager->getBasicWeekMinimalPriceDetails();
            $minimalWeekCheapestDayPriceDetails = $minimalWeekPriceDetails['cheapest_day'];
            $minimalWeekPriciestDayPriceDetails = $minimalWeekPriceDetails['priciest_day'];
			$objDiscountManager = new NRSItemDiscountManager($this->internalWPDB, $this->nrsPrefix, $this->lang, $paramItemId, $this->savedConfig);
			$discountPeriodFromPercentage = $objDiscountManager->getDiscountPercentageForBookingDurationByPeriod($discountPeriod['period_from']);
			$discountPeriodTillPercentage =  $objDiscountManager->getDiscountPercentageForBookingDurationByPeriod($discountPeriod['period_till']);
			$discountPercentageForBookingDuration = max($discountPeriodFromPercentage, $discountPeriodTillPercentage);
			$maxDiscountPercentageForBookingInAdvance = $objDiscountManager->getMaxDiscountPercentageForBookingInAdvance();
			$totalDiscountPercentage = $discountPercentageForBookingDuration + $maxDiscountPercentageForBookingInAdvance;
            // Protect system from max total discounts bigger than 100%
            if($totalDiscountPercentage > 100)
            {
                $totalDiscountPercentage = 100;
            }

			$cheapestDiscountedPriceDynamic = $minimalWeekCheapestDayPriceDetails['unit']['subtotal_price_dynamic'] / 100 * (100 - $totalDiscountPercentage);
			$priciestDiscountedPriceDynamic = $minimalWeekPriciestDayPriceDetails['unit']['subtotal_price_dynamic'] / 100 * (100 - $discountPercentageForBookingDuration);

            // Texts
            $textCheapestDiscountedPriceDynamicLong = number_format_i18n($cheapestDiscountedPriceDynamic, 2);
            $textCheapestDiscountedPriceDynamic = number_format_i18n($cheapestDiscountedPriceDynamic, 0);

			// Only duration discount applied
            $textPriciestDiscountedPriceDynamicLong = number_format_i18n($priciestDiscountedPriceDynamic, 2);
            $textPriciestDiscountedPriceDynamic = number_format_i18n($priciestDiscountedPriceDynamic, 0);

            $textTotalDiscountPercentage = number_format_i18n(round($totalDiscountPercentage, 1), 1);
			$textDiscountPercentageForBookingDuration = number_format_i18n(round($discountPercentageForBookingDuration, 1), 1);

			$printPriceDescription = "";
			$printPriceDescription .= ($this->priceCalculationType == 2 ? $this->lang->getText('NRS_PRICE_FOR_HOUR_FROM_TEXT') : $this->lang->getText('NRS_PRICE_FOR_DAY_FROM_TEXT')).' ';
			if($maxDiscountPercentageForBookingInAdvance > 0 || $cheapestDiscountedPriceDynamic < $priciestDiscountedPriceDynamic)
			{
				$printPriceDescription .= $this->currencySymbol.' '.$textCheapestDiscountedPriceDynamicLong.'-'.$textPriciestDiscountedPriceDynamicLong.' ';
				$printPriceDescription .= $this->lang->getText('NRS_PRICE_WITH_APPLIED_TEXT').' ';
				$printPriceDescription .= $textDiscountPercentageForBookingDuration;
                // Display only if there is different size discounts applied (booking duration and total discount)
                if($discountPercentageForBookingDuration < $totalDiscountPercentage)
                {
				    $printPriceDescription .= '-'.$textTotalDiscountPercentage;
                }
				$printPriceDescription .= ' % '.$this->lang->getText('NRS_WITH_APPLIED_DISCOUNT_TEXT');
				$printPrice = $textCheapestDiscountedPriceDynamic.'-'.$textPriciestDiscountedPriceDynamic.' '.$this->currencyCode;
			} else
			{
				$printPriceDescription = $this->currencySymbol.' '.$textCheapestDiscountedPriceDynamicLong.', ';
                $printPriceDescription .= $this->lang->getText('NRS_PRICE_WITH_APPLIED_TEXT').' '.$textDiscountPercentageForBookingDuration.' % ';
                $printPriceDescription .= $this->lang->getText('NRS_WITH_APPLIED_DISCOUNT_TEXT');
				$printPrice = $textCheapestDiscountedPriceDynamic.' '.$this->currencyCode;
			}
			$discountPeriod['print_price_description'] = $printPriceDescription;
			$discountPeriod['print_price'] = $printPrice;
			$periods[] = $discountPeriod;

			if($this->debugMode)
			{
				echo "-&gt;  Item Id: {$paramItemId} , ";
                echo "Discount (dur.+adv.): {$discountPercentageForBookingDuration} + {$maxDiscountPercentageForBookingInAdvance} = {$totalDiscountPercentage}, ";
                echo "Period (from/to): {$discountPeriod['period_from']} - {$discountPeriod['period_till']},<br />";
                echo "Week&#39;s Cheapest Day Dynamic Price (full): {$minimalWeekCheapestDayPriceDetails['unit']['subtotal_price_dynamic']}, ";
				echo "Week&#39;s Cheapest Day Dynamic Price (disc.): {$cheapestDiscountedPriceDynamic},<br />";
                echo "Week&#39;s Priciest Day Dynamic Price (full): {$minimalWeekPriciestDayPriceDetails['unit']['subtotal_price_dynamic']}, ";
                echo "Week&#39;s Priciest Day Dynamic Price (disc.): {$priciestDiscountedPriceDynamic}<br /><br />";
				//echo nl2br(print_r($minimalPriceDetails, TRUE))."<br />";
				//echo "=================================================================================================<br /><br />";
			}
		}

		return $periods;
	}
}