<?php
/**
 * Language Manager
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @note This class is made to work without any other external plugin classes,
 * and it should remain that for independent include support
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSLanguage implements iNRSLanguage
{
    private $lang = array();

    public function __construct()
    {
        $this->setLanguage();
    }

    // Load language file
    private function setLanguage()
    {
        // Singleton pattern to allow define NRS language only once
        $lang = array();
        $objLanguage = "en_US";
        if(file_exists($GLOBALS['NRS_EXTENSION_PATH'].'languages/'.get_locale().'.php'))
        {
            $objLanguage = get_locale();
        }
        // Include the language file
        require($GLOBALS['NRS_EXTENSION_PATH'].'languages/'.$objLanguage.'.php');

        // NOTE: This might be a system slowing-down process
        if(sizeof($lang) > 0)
        {
            foreach($lang AS $key => $value)
            {
                $sanitizedKey = sanitize_key($key);
                if(strlen($sanitizedKey) > 0)
                {
                    // Assign the language internally
                    $this->lang[$sanitizedKey] = sanitize_text_field($value);
                }
            }
        }
        //die("NRS LANG:".$objLanguage." WP LOCALE:". get_locale());

        return $objLanguage;
    }

    public function getText($paramKey)
    {
        $sanitizedKey = sanitize_key($paramKey);
        $ret = "";
        if(strlen($sanitizedKey) > 0)
        {
            if(isset($this->lang[$sanitizedKey]))
            {
                $ret = $this->lang[$sanitizedKey];
            }
        }
        return $ret;
    }

    /**
     * Is current language is right to left?
     * @return bool
     */
    public function isRTL()
    {
        return ((isset($this->lang['rtl']) && $this->lang['rtl'] == TRUE) ? TRUE : FALSE);
    }


    public function getItemQuantityText($quantity = 1)
    {
        $text = $this->getUnitsTextByQuantity($quantity, $this->getText('NRS_DAY_TEXT'), $this->getText('NRS_DAYS_TEXT'), $this->getText('NRS_DAYS2_TEXT'));
        $ret = $quantity. " ".$text;

        return $ret;
    }
    public function getExtraQuantityText($quantity = 1)
    {
        $text = $this->getUnitsTextByQuantity($quantity, $this->getText('NRS_DAY_TEXT'), $this->getText('NRS_DAYS_TEXT'), $this->getText('NRS_DAYS2_TEXT'));
        $ret = $quantity. " ".$text;

        return $ret;
    }

    /**
     * Used for items and extras in booking summary
     * Localize the amount text by quantity
     * @param $units
     * @param $singularText
     * @param $pluralText
     * @param $pluralText2
     * @return mixed
     */
    public function getUnitsTextByQuantity($units, $singularText, $pluralText, $pluralText2)
    {
        // Set default - plural text
        $unitsText = $pluralText;

        if($units == 1)
        {
            // Change to singular if it's 1
            $unitsText = $singularText;
        } else if($units == 0 || ($units % 10 == 0) || ($units >= 11 && $units <= 19))
        {
            // Change to plural 2, if it's 0, divides by 10 without fraction (10,20,30,40,..), or is between 11 to 19
            $unitsText = $pluralText2;
        }

        return $unitsText;
    }

    /**
     * Used for items and extras in booking summary
     * Localize the amount text by quantity
     * @param $position
     * @param $textST
     * @param $textND
     * @param $textRD
     * @param $textTH
     * @return string
     */
    public function getOrderExtensionByPosition($position, $textST, $textND, $textRD, $textTH)
    {
        // Set default - th
        $orderExtension = $textTH;

        if($position == 1 || ($position % 10 == 1 && $position >= 20))
        {
            //-st. Change to text 1, if it's 1, divides by 10 with fraction = 1 and is more than 20 (21,31,41,..)
            $orderExtension = $textST;
        } else if($position == 2 || ($position % 10 == 2 && $position >= 20))
        {
            //-nd. Change to text 1, if it's 2, divides by 10 with fraction = 2 and is more than 20 (22,32,42,..)
            $orderExtension = $textND;
        } else if($position == 3 || ($position % 10 == 3 && $position >= 20))
        {
            //-rd. Change to text 1, if it's 3, divides by 10 with fraction = 3 and is more than 20 (23,33,43,..)
            $orderExtension = $textRD;
        } else if($position == 0 || ($position % 10 == 0) || ($position >= 11 && $position <= 19))
        {
            //-th. Change to text 0, if it's 0, divides by 10 without fraction (10,20,30,40,..), or is between 11 to 19
            $orderExtension = $textTH;
        }

        return $orderExtension;
    }


    /**
     * Localize the time text by period
     * @param $period
     * @param $singularText
     * @param $pluralText
     * @param $pluralText2
     * @return mixed
     */
    public function getTimeTextByPeriod($period, $singularText, $pluralText, $pluralText2)
    {
        // Set default - plural text
        $timeText = $pluralText;

        if($period == 1)
        {
            // Change to singular if it's 1
            $timeText = $singularText;
        } else if($period == 0 || ($period % 10 == 0) || ($period >= 11 && $period <= 19))
        {
            // Change to plural 2, if it's 0, divides by 10 without fraction (10,20,30,40,..), or is between 11 to 19
            $timeText = $pluralText2;
        }

        return $timeText;
    }

    public function getPrintFloorDurationByPeriod($paramPriceCalculationType, $paramPeriodInSeconds)
    {
        return $this->getPrintDurationByPeriod("FLOOR", $paramPriceCalculationType, $paramPeriodInSeconds);
    }

    public function getPrintCeilDurationByPeriod($paramPriceCalculationType, $paramPeriodInSeconds)
    {
        return $this->getPrintDurationByPeriod("CEIL", $paramPriceCalculationType, $paramPeriodInSeconds);
    }

    /**
     * @note THIS FUNCTION IS PRIMARY DEFINED AS PUBLIC IN NRS CORE
     * @param $paramRoundingType - "FLOOR", "CEIL"
     * @param int $paramPriceCalculationType - 1 (daily), 2 (hourly) or 3 (mixed - daily & hourly)
     * @param int $paramPeriodInSeconds
     * @return string
     */
    private function getPrintDurationByPeriod($paramRoundingType = "CEIL", $paramPriceCalculationType, $paramPeriodInSeconds)
    {
        $textShowDuration = "";
        if($paramRoundingType == "FLOOR")
        {
            // FLOOR ROUNDING
            $durationInDaysOnly = floor($paramPeriodInSeconds / 86400);
            $durationInHoursOnly = floor($paramPeriodInSeconds / 3600);
            $secondsOnLastDay = $paramPeriodInSeconds-floor($paramPeriodInSeconds / 86400)*86400;
            $durationInDaysAndHours = array(
                "days" => floor($paramPeriodInSeconds / 86400),
                "hours" => floor($secondsOnLastDay / 3600),
            );
        } else
        {
            // CEIL ROUNDING
            $durationInDaysOnly = ceil($paramPeriodInSeconds / 86400);
            $durationInHoursOnly = ceil($paramPeriodInSeconds / 3600);
            $secondsOnLastDay = $paramPeriodInSeconds-floor($paramPeriodInSeconds / 86400)*86400;
            $durationInDaysAndHours = array(
                "days" => floor($paramPeriodInSeconds / 86400),
                "hours" => ceil($secondsOnLastDay / 3600),
            );
        }

        if ($paramPriceCalculationType == 1)
        {
            // Count by days only
            $daysText = $this->getTimeTextByPeriod($durationInDaysOnly, $this->getText('NRS_DAY_TEXT'), $this->getText('NRS_DAYS_TEXT'), $this->getText('NRS_DAYS2_TEXT'));
            $textShowDuration = $durationInDaysOnly.' '.$daysText;
        } else if ($paramPriceCalculationType == 2)
        {
            // Count by hours only
            $hoursText = $this->getTimeTextByPeriod($durationInHoursOnly, $this->getText('NRS_HOUR_TEXT'), $this->getText('NRS_HOURS_TEXT'), $this->getText('NRS_HOURS2_TEXT'));
            $textShowDuration = $durationInHoursOnly.' '.$hoursText;
        } else if ($paramPriceCalculationType == 3)
        {
            // Combined count - days+hours
            $daysText = $this->getTimeTextByPeriod($durationInDaysAndHours['days'], $this->getText('NRS_DAY_TEXT'), $this->getText('NRS_DAYS_TEXT'), $this->getText('NRS_DAYS2_TEXT'));
            $hoursText = $this->getTimeTextByPeriod($durationInDaysAndHours['hours'], $this->getText('NRS_HOUR_TEXT'), $this->getText('NRS_HOURS_TEXT'), $this->getText('NRS_HOURS2_TEXT'));

            if($durationInDaysAndHours['days'] > 0 && $durationInDaysAndHours['hours'] > 0)
            {
                $textShowDuration = $durationInDaysAndHours['days'].' '.$daysText.' '.$durationInDaysAndHours['hours'].' '.$hoursText;
            } else if($durationInDaysAndHours['days'] > 0 && $durationInDaysAndHours['hours'] == 0)
            {
                $textShowDuration = $durationInDaysAndHours['days'].' '.$daysText;
            } else
            {
                $textShowDuration = $durationInDaysAndHours['hours'].' '.$hoursText;
            }
        }
        return $textShowDuration;
    }
}