<?php
/**
 * Location Manager
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSLocation
{
    protected $internalWPDB	 	= NULL;
    protected $nrsPrefix 		= "";
    protected $lang 			= NULL;
    protected $debugMode 	 	= 0;

    protected $locationId           = 0;
    protected $currencySymbol		= '$';
    protected $currencyCode			= 'USD';
    protected $vatPercentage		= 0.00;
    protected $showPricesWithVAT	= 0;

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - Language
     * @param $paramConfig
     * @param $paramLocationId
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig, $paramLocationId)
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        $this->locationId = NRSValidator::getValidInteger($paramLocationId);
        $this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
        $this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
        $this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
        $this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getId()
    {
        return $this->locationId;
    }

    protected function getFormattedPrice($paramPrice, $paramFormatType)
    {
        return NRSFormatter::getFormattedPrice($paramPrice, $paramFormatType, $this->currencySymbol, $this->currencyCode);
    }
    protected function getFormattedPercentage($percentage, $paramFormatType)
    {
        return NRSFormatter::getFormattedPercentage($percentage, $paramFormatType);
    }

    /**
     * Get location data from MySQL database
     * @note - MUST BE PRIVATE. FOR INTERNAL USE ONLY
     * @param int $paramLocationId - primary it's this class unique id, with some exceptions when we call for afterhours id
     * @param string $dayOfWeek
     * @return mixed
     */
    private function getLocationDataFromDatabaseById($paramLocationId, $dayOfWeek = "mon")
    {
        $SQLOpenTimeField = $this->getOpenTimeFieldByDayOfWeek($dayOfWeek);
        $SQLCloseTimeField = $this->getCloseTimeFieldByDayOfWeek($dayOfWeek);
        $SQLOpenStatusField = $this->getOpenStatusFieldByDayOfWeek($dayOfWeek);

        // For all items reservation
        $validLocationId = NRSValidator::getValidInteger($paramLocationId);
        $sqlQuery = "
			SELECT
				location_id, location_title, location_address, pickup_fee, dropoff_fee,
				{$SQLOpenTimeField} AS open_time,
				{$SQLCloseTimeField} AS close_time,
				{$SQLOpenStatusField} AS open_today,
                      open_mondays,
                      open_tuesdays,
                      open_wednesdays,
                      open_thursdays,
                      open_fridays,
                      open_saturdays,
                      open_sundays,
                      open_time_mon,
                      open_time_tue,
                      open_time_wed,
                      open_time_thu,
                      open_time_fri,
                      open_time_sat,
                      open_time_sun,
                      close_time_mon,
                      close_time_tue,
                      close_time_wed,
                      close_time_thu,
                      close_time_fri,
                      close_time_sat,
                      close_time_sun,
				afterhours_pickup_location_id, afterhours_pickup_fee,
				afterhours_dropoff_location_id, afterhours_dropoff_fee
			FROM {$this->nrsPrefix}locations
			WHERE location_id='{$validLocationId}'
		";
        $locationData = $this->internalWPDB->get_row($sqlQuery, ARRAY_A);

        // Debug
        //echo nl2br($sqlQuery);

        return $locationData;
    }

    /**
     * Get open time field by day of week
     * @param string $paramDayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     */
    public function getOpenTimeFieldByDayOfWeek($paramDayOfWeek = "mon")
    {
        $SQLOpenTimeFields = array(
            "mon" => "open_time_mon",
            "tue" => "open_time_tue",
            "wed" => "open_time_wed",
            "thu" => "open_time_thu",
            "fri" => "open_time_fri",
            "sat" => "open_time_sat",
            "sun" => "open_time_sun",
        );

        // Main purpose of the action bellow is to lowercase the key
        $validDayOfWeek = sanitize_key($paramDayOfWeek);
        $SQLOpenTimeField = isset($SQLOpenTimeFields[$validDayOfWeek]) ? $SQLOpenTimeFields[$validDayOfWeek] : $SQLOpenTimeFields['mon'];

        return $SQLOpenTimeField;
    }

    /**
     * Get close time field by day of week
     * @param string $paramDayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     */
    public function getCloseTimeFieldByDayOfWeek($paramDayOfWeek = "mon")
    {
        $SQLCloseTimeFields = array(
            "mon" => "close_time_mon",
            "tue" => "close_time_tue",
            "wed" => "close_time_wed",
            "thu" => "close_time_thu",
            "fri" => "close_time_fri",
            "sat" => "close_time_sat",
            "sun" => "close_time_sun",
        );

        // Main purpose of the action bellow is to lowercase the key
        $validDayOfWeek = sanitize_key($paramDayOfWeek);
        $SQLCloseTimeField = isset($SQLCloseTimeFields[$validDayOfWeek]) ? $SQLCloseTimeFields[$validDayOfWeek] : $SQLCloseTimeFields['mon'];

        return $SQLCloseTimeField;
    }

    /**
     * Get open status field by day of week
     * @param string $paramDayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     */
    public function getOpenStatusFieldByDayOfWeek($paramDayOfWeek = "mon")
    {
        $SQLOpenStatusFields = array(
            "mon" => "open_mondays",
            "tue" => "open_tuesdays",
            "wed" => "open_wednesdays",
            "thu" => "open_thursdays",
            "fri" => "open_fridays",
            "sat" => "open_saturdays",
            "sun" => "open_sundays",
        );

        // Main purpose of the action bellow is to lowercase the key
        $validDayOfWeek = sanitize_key($paramDayOfWeek);
        $SQLOpenStatusField = isset($SQLOpenStatusFields[$validDayOfWeek]) ? $SQLOpenStatusFields[$validDayOfWeek] : $SQLOpenStatusFields['mon'];

        return $SQLOpenStatusField;
    }


    /**
     * @param $paramDayOfWeek
     * @param $paramTimeToCheck
     * @return bool
     */
    public function isAfterhoursTime($paramDayOfWeek, $paramTimeToCheck)
    {
        $sanitizedDayOfWeek = sanitize_text_field($paramDayOfWeek);
        $validTimeToCheck = NRSValidator::getValidISOTime($paramTimeToCheck, 'H:i:s');
        $nightlyPickupRateApplied = FALSE;

        $locationData = $this->getLocationDetails($sanitizedDayOfWeek);

        if(!is_null($locationData))
        {
            if($validTimeToCheck < $locationData['open_time'] || $validTimeToCheck > $locationData['close_time'])
            {
                $nightlyPickupRateApplied = TRUE;
            }
        }

        return $nightlyPickupRateApplied;
    }

    /**
     * Check if location is not blocked by admin for specified date
     * @param $paramISODate
     * @return bool
     */
    public function isLocationOpenAtDate($paramISODate)
    {
        $validDate = NRSValidator::getValidISODate($paramISODate, 'Y-m-d');
        $sql = "SELECT closed_date
            FROM {$this->nrsPrefix}closed_dates
            WHERE closed_date='{$validDate}'
        ";
        //echo nl2br($sql);
        $blockedDate = $this->internalWPDB->get_row($sql, ARRAY_A);
        if(!is_null($blockedDate))
        {
            // Location is closed at this date - is blocked for selected date
            return FALSE;
        } else
        {
            // Location is open at this date - is not blocked for selected date
            return TRUE;
        }
    }

    /**
     * @param $paramTime
     * @param $paramMode "PICKUP" / "DROPOFF"
     * @param string $dayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     * @return int 1 (same location + fee), 0 - not allowed, 2 - (other location + fee)
     */
    public function isLocationOpenAtTime($paramTime, $paramMode, $dayOfWeek = "mon")
    {
        //echo "<br />[isLocationOpenAtTime: \$time = $time, \$mode=$mode]";
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId, $dayOfWeek);

        if(is_null($locationData) || $locationData['open_today'] == 0)
        {
            return 0;
        }
        $afterHoursLocationId = $paramMode == "PICKUP" ? $locationData['afterhours_pickup_location_id'] : $locationData['afterhours_dropoff_location_id'];
        if($paramTime < $locationData['open_time'] || $paramTime > $locationData['close_time'])
        {
            // Location is not working as this time normally
            if($afterHoursLocationId == 0)
            {
                // Afterhours pickup/dropoff is not allowed
                return 0;
            }

            if($afterHoursLocationId == -1)
            {
                // Pickup/Dropoff allowed at this location after hours, but extra fee is applied
                return 1;
            }

            if($afterHoursLocationId >= 1)
            {
                $afterHoursLocationRow = $this->getLocationDataFromDatabaseById($afterHoursLocationId, $dayOfWeek);
                if(is_null($afterHoursLocationRow))
                {
                    // After hours location do not exist in DB anymore
                    return 0;
                }

                /***DEBUG***/ //echo "<br />CHECK: $time IS LESS THAN {$afterHoursLocationRow['open_time']} || $time IS MORE THAN {$afterHoursLocationRow['close_time']}";

                if($paramTime < $afterHoursLocationRow['open_time'] || $paramTime > $afterHoursLocationRow['close_time'])
                {
                    $anotherAfterHoursLocationId = $paramMode == "PICKUP" ? $afterHoursLocationRow['afterhours_pickup_location_id'] : $afterHoursLocationRow['afterhours_dropoff_location_id'];
                    if($anotherAfterHoursLocationId == -1)
                    {
                        // After hours location is different, and fees applied
                        // but after hours pickup/dropoff location is working on it's AFTER HOURS and pickup/dropoff is allowed
                        // (after hours location pickup/dropoff ID is -1, which means that pickup/dropoff is in that afterhours location),
                        return 2;
                    } else
                    {
                        // After hours location doesn't work that time
                        return 0;
                    }
                } else
                {
                    // After hours location is different, and fees applied, but after hours pickup/dropoff location is working on it's REGULAR HOURS and pickup/dropoff is allowed
                    return 2;
                }
            }
        }

        return 1;
    }

    public function isPickupWorkingInAfterhours($dayOfWeek = "mon")
    {
        return $this->isWorkingInAfterhours("PICKUP", $dayOfWeek);
    }

    public function isDropoffWorkingInAfterhours($dayOfWeek = "mon")
    {
        return $this->isWorkingInAfterhours("DROPOFF", $dayOfWeek);
    }

    /**
     * Result is TRUE/FALSE for after hour location id
     * @param string $paramMode - "PICKUP" or "DROPOFF"
     * @param string $dayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     * @return bool TRUE/FALSE after hour location ID
     */
    private function isWorkingInAfterhours($paramMode = "DROPOFF", $dayOfWeek = "mon")
    {
        // Get SQL DB fields
        $SQLOpenTimeField = $this->getOpenTimeFieldByDayOfWeek($dayOfWeek);
        $SQLCloseTimeField = $this->getCloseTimeFieldByDayOfWeek($dayOfWeek);
        $SQLOpenStatusField = $this->getOpenStatusFieldByDayOfWeek($dayOfWeek);

        $searchField = $paramMode == "DROPOFF" ? "afterhours_dropoff_location_id" : "afterhours_pickup_location_id";
        // It doesn't matter if the main location is closed at that day. So we do not include that check into first query
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);

        $afterhoursLocationId = $locationData[$searchField];

        $validAfterHoursLocation = FALSE;
        if(!is_null($locationData))
        {
            if($afterhoursLocationId == -1 && $locationData['open_today'] == 1)
            {
                $validAfterHoursLocation = TRUE;
            } else if($afterhoursLocationId == 0)
            {
                $validAfterHoursLocation = FALSE;
            } else if($afterhoursLocationId >= 1)
            {
                // We will only give results if after hours delivery location exist, and it's close hour is later than current location close hour or it's 24/7
                $sqlQuery = "
                    SELECT location_id
                    FROM {$this->nrsPrefix}locations
                    WHERE location_id='{$afterhoursLocationId}' AND (
                        {$SQLOpenTimeField} <= '{$locationData['open_time']}'
                        OR {$SQLCloseTimeField} >= '{$locationData['close_time']}'
                        OR {$searchField}='-1'
                    ) AND {$SQLOpenStatusField}='1'
                ";

                $afterHoursLocationRow = $this->internalWPDB->get_row($sqlQuery, ARRAY_A);
                if(!is_null($afterHoursLocationRow))
                {
                    $validAfterHoursLocation = TRUE;
                }
            }
        }

        return $validAfterHoursLocation;
    }

    /**
     * For afterhours pickup check only
     * @return bool
     */
    public function isComplexAfterhoursPickup()
    {
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);
        $afterhoursLocationId = 0;
        if(!is_null($locationData) && $locationData['afterhours_pickup_location_id'] > 0)
        {
            $afterhoursLocationId = $locationData['afterhours_pickup_location_id'];
            $isComplexAfterhoursLocation = $this->isComplexLocationById($afterhoursLocationId, "PICKUP");
        } else
        {
            $isComplexAfterhoursLocation = FALSE;
            /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID: {$this->locationId}, AFTERHOURS PICKUP LOCATION ID={$afterhoursLocationId}] IS NOT COMPLEX."; }
        }

        return $isComplexAfterhoursLocation;
    }

    /**
     * For afterhours dropoff check only
     * @return bool
     */
    public function isComplexAfterhoursDropoff()
    {
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);
        $afterhoursLocationId = 0;
        if(!is_null($locationData) && $locationData['afterhours_dropoff_location_id'] > 0)
        {
            $afterhoursLocationId = $locationData['afterhours_dropoff_location_id'];
            $isComplexAfterhoursLocation = $this->isComplexLocationById($afterhoursLocationId, "DROPOFF");
        } else
        {
            $isComplexAfterhoursLocation = FALSE;
            /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID: {$this->locationId}, AFTERHOURS DROPOFF LOCATION ID={$afterhoursLocationId}] IS NOT COMPLEX."; }
        }

        return $isComplexAfterhoursLocation;
    }

    public function isComplexLocation()
    {
        return $this->isComplexLocationById($this->locationId, "BOTH");
    }

    /**
     * Result is TRUE/FALSE to describe is location in complex or not. Id is used only internally
     * @param int $paramLocationId
     * @param string $paramMode - "PICKUP", "DROPOFF" or "BOTH"
     * @return bool
     */
    private function isComplexLocationById($paramLocationId, $paramMode = "BOTH")
    {
        $validLocationId = NRSValidator::getValidInteger($paramLocationId);
        $locationData = $this->getLocationDataFromDatabaseById($validLocationId);

        if(!is_null($locationData))
        {
            // Complex check
            if($locationData['open_time'] == "00:00:00" && $locationData['close_time'] == "23:59:59")
            {
                // Works all time (doesn't closes at all)
                // And location is not complex then
                $isComplexLocation = FALSE;
                /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID={$validLocationId}] IS NOT COMPLEX - works 24/7"; }
            } else if(
                ($paramMode == "BOTH" && $locationData['afterhours_pickup_location_id'] == -1 && $locationData['afterhours_dropoff_location_id'] == -1)
                || ($paramMode == "PICKUP" && $locationData['afterhours_pickup_location_id'] == -1)
                || ($paramMode == "DROPOFF" && $locationData['afterhours_dropoff_location_id'] == -1)
            ) {
                // Works all time (doesn't closes at all)
                // And location is not complex then
                $isComplexLocation = FALSE;
                /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID={$validLocationId}] IS NOT COMPLEX - afterhours (pickup /& dropoff) is in the same location"; }
            } else
            {
                $isComplexLocation = TRUE;
                /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID={$validLocationId}] IS COMPLEX - doesn&#39;t work 24/7, or afterhours (pickup /& dropoff) is different location"; }
            }
        } else
        {
            $isComplexLocation = FALSE;
            /***DEBUG***/ if($this->debugMode) { echo "<br />[LOCATION ID={$validLocationId}] IS NOT COMPLEX - it does not exist anymore in database (been deleted)."; }

        }

        return $isComplexLocation;
    }

    public function getLocationName()
    {
        $locationName = "";
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);
        if(!is_null($locationData))
        {
            $locationName = $locationData['location_title'].' - '.$locationData['location_address'];
        }

        return $locationName;
    }

    /**
     * Location data, plus extra field 'print_open_hours' (front end only)
     * @note 'print_open_hours' in front_end only, generated on the fly
     * @note 'print_location_name' in front_end only, generated on the fly
     * @param string $dayOfWeek - 'mon', 'tue', wed', 'fri', 'sat', 'sun'
     * @return mixed
     */
    public function getLocationDetails($dayOfWeek = "mon")
    {
        $locationRow = $this->getLocationDataFromDatabaseById($this->locationId, $dayOfWeek);
        if(!is_null($locationRow))
        {
            // Count pickup/dropoff/after-hours location prices
            $pickupFee 		    = $locationRow['pickup_fee'];
            $pickupFeeWithVAT   = $pickupFee + ($pickupFee / 100 * $this->vatPercentage);
            $pickupFeeDynamic   = $this->showPricesWithVAT == "1" ? $pickupFeeWithVAT : $pickupFee;
            $pickupVATAmount 	= $pickupFeeWithVAT-$pickupFee;

            $dropoffFee 		= $locationRow['dropoff_fee'];
            $dropoffFeeWithVAT  = $dropoffFee + ($dropoffFee / 100 * $this->vatPercentage);
            $dropoffFeeDynamic  = $this->showPricesWithVAT == "1" ? $dropoffFeeWithVAT : $dropoffFee;
            $dropoffVATAmount   = $dropoffFeeWithVAT-$dropoffFee;

            $afterHoursPickupFee 		= $locationRow['afterhours_pickup_fee'];
            $afterHoursPickupFeeWithVAT = $afterHoursPickupFee + ($afterHoursPickupFee / 100 * $this->vatPercentage);
            $afterHoursPickupFeeDynamic = $this->showPricesWithVAT == "1" ? $afterHoursPickupFeeWithVAT : $afterHoursPickupFee;
            $afterHoursPickupVATAmount 	= $afterHoursPickupFeeWithVAT-$afterHoursPickupFee;

            $afterHoursDropoffFee 		 = $locationRow['afterhours_dropoff_fee'];
            $afterHoursDropoffFeeWithVAT = $afterHoursDropoffFee + ($afterHoursDropoffFee / 100 * $this->vatPercentage);
            $afterHoursDropoffFeeDynamic = $this->showPricesWithVAT == "1" ? $afterHoursDropoffFeeWithVAT : $afterHoursDropoffFee;
            $afterHoursDropoffVATAmount  = $afterHoursDropoffFeeWithVAT-$afterHoursDropoffFee;

            // Prices output stack
            $locationRow['print_pickup_fee'] 		 	= $this->getFormattedPrice($pickupFee, "long");
            $locationRow['print_pickup_fee_with_vat'] 	= $this->getFormattedPrice($pickupFeeWithVAT, "long");
            $locationRow['print_pickup_fee_dynamic']  	= $this->getFormattedPrice($pickupFeeDynamic, "long");
            $locationRow['print_pickup_vat_amount']  	= $this->getFormattedPrice($pickupVATAmount, "long");

            $locationRow['print_dropoff_fee'] 		 	= $this->getFormattedPrice($dropoffFee, "long");
            $locationRow['print_dropoff_fee_with_vat'] 	= $this->getFormattedPrice($dropoffFeeWithVAT, "long");
            $locationRow['print_dropoff_fee_dynamic'] 	= $this->getFormattedPrice($dropoffFeeDynamic, "long");
            $locationRow['print_dropoff_vat_amount'] 	= $this->getFormattedPrice($dropoffVATAmount, "long");

            $locationRow['print_afterhours_pickup_fee'] 			    = $this->getFormattedPrice($afterHoursPickupFee, "long");
            $locationRow['print_afterhours_pickup_fee_with_vat'] 	    = $this->getFormattedPrice($afterHoursPickupFeeWithVAT, "long");
            $locationRow['print_afterhours_pickup_fee_dynamic'] 	    = $this->getFormattedPrice($afterHoursPickupFeeDynamic, "long");
            $locationRow['print_afterhours_pickup_vat_amount'] 		    = $this->getFormattedPrice($afterHoursPickupVATAmount, "long");

            $locationRow['print_afterhours_dropoff_fee'] 			    = $this->getFormattedPrice($afterHoursDropoffFee, "long");
            $locationRow['print_afterhours_dropoff_fee_with_vat'] 	    = $this->getFormattedPrice($afterHoursDropoffFeeWithVAT, "long");
            $locationRow['print_afterhours_dropoff_fee_dynamic'] 	    = $this->getFormattedPrice($afterHoursDropoffFeeDynamic, "long");
            $locationRow['print_afterhours_dropoff_vat_amount'] 	    = $this->getFormattedPrice($afterHoursDropoffVATAmount, "long");

            $locationRow['print_pickup_fee_short'] 			            = $this->getFormattedPrice($pickupFee, "regular");
            $locationRow['print_pickup_fee_with_vat_short'] 	        = $this->getFormattedPrice($pickupFeeWithVAT, "regular");
            $locationRow['print_pickup_fee_dynamic_short'] 	            = $this->getFormattedPrice($pickupFeeDynamic, "regular");
            $locationRow['print_pickup_vat_amount_short'] 		        = $this->getFormattedPrice($pickupVATAmount, "regular");

            $locationRow['print_dropoff_fee_short'] 			        = $this->getFormattedPrice($dropoffFee, "regular");
            $locationRow['print_dropoff_fee_with_vat_short']            = $this->getFormattedPrice($dropoffFeeWithVAT, "regular");
            $locationRow['print_dropoff_fee_dynamic_short'] 	        = $this->getFormattedPrice($dropoffFeeDynamic, "regular");
            $locationRow['print_dropoff_vat_amount_short'] 		        = $this->getFormattedPrice($dropoffVATAmount, "regular");

            $locationRow['print_afterhours_pickup_fee_short'] 			= $this->getFormattedPrice($afterHoursPickupFee, "regular");
            $locationRow['print_afterhours_pickup_fee_with_vat_short']  = $this->getFormattedPrice($afterHoursPickupFeeWithVAT, "regular");
            $locationRow['print_afterhours_pickup_fee_dynamic_short'] 	= $this->getFormattedPrice($afterHoursPickupFeeDynamic, "regular");
            $locationRow['print_afterhours_pickup_vat_amount_short'] 	= $this->getFormattedPrice($afterHoursPickupVATAmount, "regular");

            $locationRow['print_afterhours_dropoff_fee_short'] 			= $this->getFormattedPrice($afterHoursDropoffFee, "regular");
            $locationRow['print_afterhours_dropoff_fee_with_vat_short'] = $this->getFormattedPrice($afterHoursDropoffFeeWithVAT, "regular");
            $locationRow['print_afterhours_dropoff_fee_dynamic_short'] 	= $this->getFormattedPrice($afterHoursDropoffFeeDynamic, "regular");
            $locationRow['print_afterhours_dropoff_vat_amount_short'] 	= $this->getFormattedPrice($afterHoursDropoffVATAmount, "regular");

            // Location & it's hours
            if($locationRow['open_today'] == "1")
            {
                $locationRow['print_open_hours']         = substr($locationRow['open_time'],0,5).' - '.substr($locationRow['close_time'],0,5);
            } else
            {
                $locationRow['print_open_hours']         = $this->lang->getText('NRS_CLOSED_TEXT');
            }


            // PICKUP - Set defaults for afterhours location (in case if after hours delivery is not allowed)
            $locationRow['afterhours_pickup_location_title'] = "";
            $locationRow['afterhours_pickup_location_address'] = "";
            $locationRow['afterhours_pickup_open_time'] = "";
            $locationRow['afterhours_pickup_close_time'] = "";
            $locationRow['print_afterhours_pickup_open_hours'] = "";

            // DROPOFF - Set defaults for afterhours location (in case if after hours delivery is not allowed)
            $locationRow['afterhours_dropoff_location_title'] = "";
            $locationRow['afterhours_dropoff_location_address'] = "";
            $locationRow['afterhours_dropoff_open_time'] = "";
            $locationRow['afterhours_dropoff_close_time'] = "";
            $locationRow['print_afterhours_dropoff_open_hours'] = "";

            // PICKUP - Set defaults for hours before & after regular open hours location
            $locationRow['afterhours_pickup_before_start_time'] = "";
            $locationRow['afterhours_pickup_before_end_time'] = "";
            $locationRow['afterhours_pickup_after_start_time'] = "";
            $locationRow['afterhours_pickup_after_end_time'] = "";
            $locationRow['print_afterhours_pickup_before'] = "";
            $locationRow['print_afterhours_pickup_after'] = "";
            $locationRow['is_pickup_working_in_afterhours'] = $this->isPickupWorkingInAfterhours($dayOfWeek);

            // DROPOFF - Set defaults for hours before & after regular open hours location
            $locationRow['afterhours_dropoff_before_start_time'] = "";
            $locationRow['afterhours_dropoff_before_end_time'] = "";
            $locationRow['afterhours_dropoff_after_start_time'] = "";
            $locationRow['afterhours_dropoff_after_end_time'] = "";
            $locationRow['print_afterhours_dropoff_before'] = "";
            $locationRow['print_afterhours_dropoff_after'] = "";
            $locationRow['is_dropoff_working_in_afterhours'] = $this->isDropoffWorkingInAfterhours($dayOfWeek);

            // PICKUP - Include after hours location data
            if($locationRow['is_pickup_working_in_afterhours'] == TRUE && ($locationRow['afterhours_pickup_location_id'] == -1 || $locationRow['afterhours_pickup_location_id'] >= 1))
            {
                if($locationRow['afterhours_pickup_location_id'] == -1)
                {
                    // After hours pickup is in same branch
                    $locationRow['afterhours_pickup_location_title'] = $locationRow['location_title'];
                    $locationRow['afterhours_pickup_location_address'] = $locationRow['location_address'];
                    $locationRow['afterhours_pickup_open_time'] = "00:00:00";
                    $locationRow['afterhours_pickup_close_time'] = "23:59:59";
                    $locationRow['print_afterhours_pickup_open_hours'] = "00:00 - 24:00";
                } else
                {
                    // We already know that after hour location exist, and it's close hour is later than current location close hour or it's 24/7
                    $afterHoursPickupLocationData = $this->getLocationDataFromDatabaseById($locationRow['afterhours_pickup_location_id'], $dayOfWeek);

                    if(!is_null($afterHoursPickupLocationData) && $afterHoursPickupLocationData['open_today'] == 1)
                    {
                        $locationRow['afterhours_pickup_location_title'] = $afterHoursPickupLocationData['location_title'];
                        $locationRow['afterhours_pickup_location_address'] = $afterHoursPickupLocationData['location_address'];

                        // Get After hours location working hours
                        if($afterHoursPickupLocationData['afterhours_pickup_location_id'] == -1)
                        {
                            $locationRow['afterhours_pickup_open_time'] = "00:00:00";
                            $locationRow['afterhours_pickup_close_time'] = "23:59:59";
                            $locationRow['print_afterhours_pickup_open_hours'] = $this->lang->getText('NRS_ALL_DAY_TEXT');
                        } else
                        {
                            // IF == 0, or ID (>=1), then allow to return only during after hours location working hours
                            $locationRow['afterhours_pickup_open_time'] = $afterHoursPickupLocationData['open_time'];
                            $locationRow['afterhours_pickup_close_time'] = $afterHoursPickupLocationData['close_time'];
                            $locationRow['print_afterhours_pickup_open_hours'] =
                                substr($locationRow['afterhours_pickup_open_time'],0,5).' - '.substr($locationRow['afterhours_pickup_close_time'],0,5);
                        }
                    }
                }

                // Process both - if after hours location is in the same place or other place

                // Hours before branch opening
                if($locationRow['afterhours_pickup_open_time'] < $locationRow['open_time'])
                {
                    // Exist
                    $locationRow['afterhours_pickup_before_start_time'] = $locationRow['afterhours_pickup_open_time'];
                    $locationRow['afterhours_pickup_before_end_time'] = $locationRow['open_time'];
                }

                // Hours after branch closing
                if($locationRow['afterhours_pickup_close_time'] > $locationRow['close_time'])
                {
                    // Exist
                    $locationRow['afterhours_pickup_after_start_time'] = $locationRow['close_time'];
                    $locationRow['afterhours_pickup_after_end_time'] = $locationRow['afterhours_pickup_close_time'];
                }

                $locationRow['print_afterhours_pickup_before'] =
                    substr($locationRow['afterhours_pickup_before_start_time'],0,5).' - '.substr($locationRow['afterhours_pickup_before_end_time'],0,5);
                $locationRow['print_afterhours_pickup_after'] =
                    substr($locationRow['afterhours_pickup_after_start_time'],0,5).' - '.substr($locationRow['afterhours_pickup_after_end_time'],0,5);
            }



            // DROPOFF - Include after hours location data
            if($locationRow['is_dropoff_working_in_afterhours'] == TRUE && ($locationRow['afterhours_dropoff_location_id'] == -1 || $locationRow['afterhours_dropoff_location_id'] >= 1))
            {
                if($locationRow['afterhours_dropoff_location_id'] == -1)
                {
                    // After hours dropoff is in same branch
                    $locationRow['afterhours_dropoff_location_title'] = $locationRow['location_title'];
                    $locationRow['afterhours_dropoff_location_address'] = $locationRow['location_address'];
                    $locationRow['afterhours_dropoff_open_time'] = "00:00:00";
                    $locationRow['afterhours_dropoff_close_time'] = "23:59:59";
                    $locationRow['print_afterhours_dropoff_open_hours'] = "00:00 - 24:00";
                } else
                {
                    // We already know that after hour location exist, and it's close hour is later than current location close hour or it's 24/7
                    $afterHoursDropoffLocationData = $this->getLocationDataFromDatabaseById($locationRow['afterhours_dropoff_location_id'], $dayOfWeek);
                    if(!is_null($afterHoursDropoffLocationData) && $afterHoursDropoffLocationData['open_today'] == 1)
                    {
                        $locationRow['afterhours_dropoff_location_title'] = $afterHoursDropoffLocationData['location_title'];
                        $locationRow['afterhours_dropoff_location_address'] = $afterHoursDropoffLocationData['location_address'];

                        // Get After hours location working hours
                        if($afterHoursDropoffLocationData['afterhours_dropoff_location_id'] == -1)
                        {
                            // Works all time (dropoff in same place)
                            $locationRow['afterhours_dropoff_open_time'] = "00:00:00";
                            $locationRow['afterhours_dropoff_close_time'] = "23:59:59";
                            $locationRow['print_afterhours_dropoff_open_hours'] = $this->lang->getText('NRS_ALL_DAY_TEXT');
                        } else
                        {
                            // IF == 0, or ID (>=1), then allow to return only during after hours location working hours
                            $locationRow['afterhours_dropoff_open_time'] = $afterHoursDropoffLocationData['open_time'];
                            $locationRow['afterhours_dropoff_close_time'] = $afterHoursDropoffLocationData['close_time'];
                            $locationRow['print_afterhours_dropoff_open_hours'] =
                                substr($locationRow['afterhours_dropoff_open_time'],0,5).' - '.substr($locationRow['afterhours_dropoff_close_time'],0,5);
                        }
                    }
                }

                // Process both - if after hours location is in the same place or other place

                // Hours before branch opening
                if($locationRow['afterhours_dropoff_open_time'] < $locationRow['open_time'])
                {
                    // Exist
                    $locationRow['afterhours_dropoff_before_start_time'] = $locationRow['afterhours_dropoff_open_time'];
                    $locationRow['afterhours_dropoff_before_end_time'] = $locationRow['open_time'];
                }

                // Hours after branch closing
                if($locationRow['afterhours_dropoff_close_time'] > $locationRow['close_time'])
                {
                    // Exist
                    $locationRow['afterhours_dropoff_after_start_time'] = $locationRow['close_time'];
                    $locationRow['afterhours_dropoff_after_end_time'] = $locationRow['afterhours_dropoff_close_time'];
                }

                $locationRow['print_afterhours_dropoff_before'] =
                    substr($locationRow['afterhours_dropoff_before_start_time'],0,5).' - '.substr($locationRow['afterhours_dropoff_before_end_time'],0,5);
                $locationRow['print_afterhours_dropoff_after'] =
                    substr($locationRow['afterhours_dropoff_after_start_time'],0,5).' - '.substr($locationRow['afterhours_dropoff_after_end_time'],0,5);
            }
        }

        return $locationRow;
    }


    public function getAfterhoursPickupLocationName()
    {
        return $this->getAfterhoursLocationText("PICKUP", "NAME");
    }

    public function getAfterhoursPickupLocationAlias()
    {
        return $this->getAfterhoursLocationText("PICKUP", "ALIAS");
    }

    public function getAfterhoursDropoffLocationName()
    {
        return $this->getAfterhoursLocationText("DROPOFF", "NAME");
    }

    public function getAfterhoursDropoffLocationAlias()
    {
        return $this->getAfterhoursLocationText("DROPOFF", "ALIAS");
    }

    /**
     * @param string $paramMode - "PICKUP" or "DROPOFF"
     * @param string $returnTextType - "NAME" or "ALIAS"
     * @return string
     */
    private function getAfterhoursLocationText($paramMode="DROPOFF", $returnTextType = "NAME")
    {
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);

        $afterhoursLocation = "";
        if(!is_null($locationData))
        {
            $afterHoursLocationId = $paramMode == "DROPOFF" ? $locationData['afterhours_dropoff_location_id'] : $locationData['afterhours_pickup_location_id'];
            $locationName = $locationData['location_title'];
            if($locationData['location_address'] != "")
            {
                $locationName .= '<br />'.$locationData['location_address'];
            }

            $afterhoursLocation = "";
            if($afterHoursLocationId == 0 && $returnTextType == "NAME")
            {
                $afterhoursLocation = "";
            } else if($afterHoursLocationId == 0 && $returnTextType == "ALIAS")
            {
                if($paramMode == "DROPOFF")
                {
                    $afterhoursLocation = $this->lang->getText('NRS_AFTERHOURS_RETURN_IS_NOT_ALLOWED_TEXT');
                } else
                {
                    $afterhoursLocation = $this->lang->getText('NRS_AFTERHOURS_PICKUP_IS_NOT_ALLOWED_TEXT');
                }
            } else if($afterHoursLocationId == -1 && $returnTextType == "NAME")
            {
                $afterhoursLocation = $locationName;
            } else if($afterHoursLocationId == -1 && $returnTextType == "ALIAS")
            {
                $afterhoursLocation = $this->lang->getText('NRS_RETURN_IN_THIS_BRANCH_TEXT');
            } else
            {
                $searchField = $paramMode == "DROPOFF" ? "afterhours_dropoff_location_id" : "afterhours_pickup_location_id";

                // It must be -1, because we want to be sure that afterhours location is a final location in the tree
                $afterhoursLocationData = $this->internalWPDB->get_row("
                    SELECT location_title, location_address
                    FROM {$this->nrsPrefix}locations
                    WHERE {$searchField}='-1' AND location_id='{$afterHoursLocationId}'
                ", ARRAY_A);

                if(!is_null($afterhoursLocationData))
                {
                    // Return is same for both - NAME and ALIAS
                    $afterhoursLocation = $afterhoursLocationData['location_title'];
                    if($afterhoursLocationData['location_address'] != "")
                    {
                        $afterhoursLocation .= '<br />'.$afterhoursLocationData['location_address'];
                    }
                } else if($returnTextType == "NAME")
                {
                    $afterhoursLocation = "";
                } else if($returnTextType == "ALIAS")
                {
                    if($paramMode == "DROPOFF")
                    {
                        $afterhoursLocation = $this->lang->getText('NRS_AFTERHOURS_RETURN_IS_NOT_ALLOWED_TEXT');
                    } else
                    {
                        $afterhoursLocation = $this->lang->getText('NRS_AFTERHOURS_PICKUP_IS_NOT_ALLOWED_TEXT');
                    }
                }
            }
        }

        return $afterhoursLocation;
    }

    public function getPrintPickupLocation($dayOfWeek = "mon")
    {
        return $this->getPrintLocation("PICKUP", $dayOfWeek);
    }
    public function getPrintDropoffLocation($dayOfWeek = "mon")
    {
        return $this->getPrintLocation("DROPOFF", $dayOfWeek);
    }

    private function getPrintLocation($paramType = "DROPOFF", $dayOfWeek = "mon")
    {
        $word = $paramType == "PICKUP" ? "pickup" : "dropoff";
        $locationData = $this->getLocationDetails($dayOfWeek);
        $locationHTML = "";
        if($this->isComplexLocation())
        {
            if($locationData['is_'.$word.'_working_in_afterhours'] == TRUE)
            {
                $locationHTML .= '(' . $locationData['print_afterhours_'.$word.'_before'] . ') ';
                $locationHTML .= $paramType == "PICKUP" ? $this->getAfterhoursPickupLocationAlias() : $this->getAfterhoursDropoffLocationAlias();
                $locationHTML .= '<br />';
            }
            $locationHTML .= '('.$locationData['print_open_hours'].') '.$locationData['location_title'];
            if($locationData['location_address'] != "")
            {
                $locationHTML .= ' - '.$locationData['location_address'];
            }
            if($locationData['is_'.$word.'_working_in_afterhours'] == TRUE)
            {
                $locationHTML .= '<br />(' . $locationData['print_afterhours_'.$word.'_after'] . ') ';
                $locationHTML .= $paramType == "PICKUP" ? $this->getAfterhoursPickupLocationAlias() : $this->getAfterhoursDropoffLocationAlias();
            }
        } else
        {
            $locationHTML .= $locationData['location_title'];
            if($locationData['location_address'] != "")
            {
                $locationHTML .= ' - '.$locationData['location_address'];
            }
        }

        return $locationHTML;
    }

    public function getPrintWeekWorkingHours()
    {
        $weeklyWorkingHours = $this->getWeekWorkingHours();

        // Despite that array keys are text names of days of week, the implode function will glue final text only from each row values
        $retPrintWorkingHours = implode("<br />", $weeklyWorkingHours);

        return $retPrintWorkingHours;
    }

    public function getPrintWeekWorkingHoursWithDayName()
    {
        $weeklyWorkingHours = $this->getWeekWorkingHours();

        $retPrintWorkingHoursWithDayName = "";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_MONDAYS_TEXT').": ".$weeklyWorkingHours['mon']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_TUESDAYS_TEXT').": ".$weeklyWorkingHours['mon']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_WEDNESDAYS_TEXT').": ".$weeklyWorkingHours['mon']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_THURSDAYS_TEXT').": ".$weeklyWorkingHours['thu']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_FRIDAYS_TEXT').": ".$weeklyWorkingHours['fri']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_SATURDAYS_TEXT').": ".$weeklyWorkingHours['sat']."<br />";
        $retPrintWorkingHoursWithDayName .= $this->lang->getText('NRS_SUNDAYS_TEXT').": ".$weeklyWorkingHours['sun'];

        return $retPrintWorkingHoursWithDayName;
    }

    public function getPrintWorkingHoursForWeekday($paramDayOfWeek)
    {
        $weeklyWorkingHours = $this->getWeekWorkingHours();
        $validDayOfWeek = sanitize_key($paramDayOfWeek);
        $workingHours = isset($weeklyWorkingHours[$validDayOfWeek]) ? $weeklyWorkingHours[$validDayOfWeek] : "";

        return $workingHours;
    }

    public function getWeekWorkingHours()
    {
        $locationData = $this->getLocationDataFromDatabaseById($this->locationId);

        // Set defaults
        $printOpenHours = array(
            "mon" => "",
            "tue" => "",
            "wed" => "",
            "thu" => "",
            "fri" => "",
            "sat" => "",
            "sun" => "",
        );
        if(!is_null($locationData))
        {
            if($locationData['open_mondays'] == "1")
            {
                $printOpenHours['mon'] = substr($locationData['open_time_mon'],0,5).' - '.substr($locationData['close_time_mon'],0,5);
            } else
            {
                $printOpenHours['mon'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_tuesdays'] == "1")
            {
                $printOpenHours['tue'] = substr($locationData['open_time_tue'],0,5).' - '.substr($locationData['close_time_tue'],0,5);
            } else
            {
                $printOpenHours['tue'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_wednesdays'] == "1")
            {
                $printOpenHours['wed'] = substr($locationData['open_time_wed'],0,5).' - '.substr($locationData['close_time_wed'],0,5);
            } else
            {
                $printOpenHours['wed'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_thursdays'] == "1")
            {
                $printOpenHours['thu'] = substr($locationData['open_time_thu'],0,5).' - '.substr($locationData['close_time_thu'],0,5);
            } else
            {
                $printOpenHours['thu'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_fridays'] == "1")
            {
                $printOpenHours['fri'] = substr($locationData['open_time_fri'],0,5).' - '.substr($locationData['close_time_fri'],0,5);
            } else
            {
                $printOpenHours['fri'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_saturdays'] == "1")
            {
                $printOpenHours['sat'] = substr($locationData['open_time_sat'],0,5).' - '.substr($locationData['close_time_sat'],0,5);
            } else
            {
                $printOpenHours['sat'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }

            if($locationData['open_sundays'] == "1")
            {
                $printOpenHours['sun'] = substr($locationData['open_time_sun'],0,5).' - '.substr($locationData['close_time_sun'],0,5);
            } else
            {
                $printOpenHours['sun'] = $this->lang->getText('NRS_CLOSED_TEXT');
            }
        }

        return $printOpenHours;
    }

    /**
     * @param array $paramLocationDataArray
     */
    public function saveLocationData($paramLocationDataArray = array())
    {
        $validLocationId = NRSValidator::getValidInteger($this->locationId, 0);
        $sanitizedLocationTitle = sanitize_text_field($paramLocationDataArray['location_title']);
        $sanitizedLocationAddress = sanitize_text_field($paramLocationDataArray['location_address']);
        $validPickupFee = floatval($paramLocationDataArray['pickup_fee']);
        $validDropoffFee = floatval($paramLocationDataArray['dropoff_fee']);


        $validOpenMondays       = (sanitize_text_field($paramLocationDataArray['open_mondays']) == "on" ? 1 : 0);
        $validOpenTuesdays      = (sanitize_text_field($paramLocationDataArray['open_tuesdays']) == "on" ? 1 : 0);
        $validOpenWednesdays    = (sanitize_text_field($paramLocationDataArray['open_wednesdays']) == "on" ? 1 : 0);
        $validOpenThursdays     = (sanitize_text_field($paramLocationDataArray['open_thursdays']) == "on" ? 1 : 0);
        $validOpenFridays       = (sanitize_text_field($paramLocationDataArray['open_fridays']) == "on" ? 1 : 0);
        $validOpenSaturdays     = (sanitize_text_field($paramLocationDataArray['open_saturdays']) == "on" ? 1 : 0);
        $validOpenSundays       = (sanitize_text_field($paramLocationDataArray['open_sundays']) == "on" ? 1 : 0);

        $validOpenTimeMon       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_mon'], 'H:i:s');
        $validOpenTimeTue       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_tue'], 'H:i:s');
        $validOpenTimeWed       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_wed'], 'H:i:s');
        $validOpenTimeThu       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_thu'], 'H:i:s');
        $validOpenTimeFri       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_fri'], 'H:i:s');
        $validOpenTimeSat       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_sat'], 'H:i:s');
        $validOpenTimeSun       = NRSValidator::getValidISOTime($paramLocationDataArray['open_time_sun'], 'H:i:s');

        $validCloseTimeMon      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_mon'], 'H:i:s');
        $validCloseTimeTue      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_tue'], 'H:i:s');
        $validCloseTimeWed      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_wed'], 'H:i:s');
        $validCloseTimeThu      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_thu'], 'H:i:s');
        $validCloseTimeFri      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_fri'], 'H:i:s');
        $validCloseTimeSat      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_sat'], 'H:i:s');
        $validCloseTimeSun      = NRSValidator::getValidISOTime($paramLocationDataArray['close_time_sun'], 'H:i:s');

        // NOTE! We must use 'intval' here instead of 'NRSValidator::getValidInteger' because afterhours pickup location id can be equal to '-1'
        $validAfterHoursPickupLocationId = intval($paramLocationDataArray['afterhours_pickup_location_id']);
        $validAfterHoursPickupFee = floatval($paramLocationDataArray['afterhours_pickup_fee']);
        // NOTE! We must use 'intval' here instead of 'NRSValidator::getValidInteger' because afterhours dropoff location id can be equal to '-1'
        $validAfterHoursDropoffLocationId = intval($paramLocationDataArray['afterhours_dropoff_location_id']);
        $validAfterHoursDropoffFee = floatval($paramLocationDataArray['afterhours_dropoff_fee']);


        if($validLocationId != 0)
        {
            $exist = $this->internalWPDB->get_row("
              SELECT *
              FROM {$this->nrsPrefix}locations
              WHERE location_title='{$sanitizedLocationTitle}' AND location_address='{$sanitizedLocationAddress}'
              AND location_id!='{$validLocationId}'
            ", ARRAY_A);

            if(!is_null($exist))
            {
                // TODO: MOVE TO NRS ADMIN LANG FILE
                $_SESSION['ok_msg'] = "Error: Location with this name and address already exist!";
            } else
            {
                // TODO: MOVE TO NRS ADMIN LANG FILE
                $_SESSION['ok_msg'] = "Completed: Location data was updated successfully!";
                $this->internalWPDB->query("
                    UPDATE {$this->nrsPrefix}locations SET
                    location_title='{$sanitizedLocationTitle}', location_address='{$sanitizedLocationAddress}',
                    pickup_fee='{$validPickupFee}', dropoff_fee='{$validDropoffFee}',
                    open_mondays='{$validOpenMondays}',
                    open_tuesdays='{$validOpenTuesdays}',
                    open_wednesdays='{$validOpenWednesdays}',
                    open_thursdays='{$validOpenThursdays}',
                    open_fridays='{$validOpenFridays}',
                    open_saturdays='{$validOpenSaturdays}',
                    open_sundays='{$validOpenSundays}',
                    open_time_mon='{$validOpenTimeMon}',
                    open_time_tue='{$validOpenTimeTue}',
                    open_time_wed='{$validOpenTimeWed}',
                    open_time_thu='{$validOpenTimeThu}',
                    open_time_fri='{$validOpenTimeFri}',
                    open_time_sat='{$validOpenTimeSat}',
                    open_time_sun='{$validOpenTimeSun}',
                    close_time_mon='{$validCloseTimeMon}',
                    close_time_tue='{$validCloseTimeTue}',
                    close_time_wed='{$validCloseTimeWed}',
                    close_time_thu='{$validCloseTimeThu}',
                    close_time_fri='{$validCloseTimeFri}',
                    close_time_sat='{$validCloseTimeSat}',
                    close_time_sun='{$validCloseTimeSun}',
                    afterhours_pickup_location_id='{$validAfterHoursPickupLocationId}', afterhours_pickup_fee='{$validAfterHoursPickupFee}',
                    afterhours_dropoff_location_id='{$validAfterHoursDropoffLocationId}', afterhours_dropoff_fee='{$validAfterHoursDropoffFee}'
                    WHERE location_id='{$validLocationId}'
                ", ARRAY_A);
            }
        } else
        {
            $exist = $this->internalWPDB->get_row("
                SELECT *
                FROM {$this->nrsPrefix}locations
                WHERE location_title='{$sanitizedLocationTitle}'
            ", ARRAY_A);

            if(!is_null($exist))
            {
                // TODO: MOVE TO NRS ADMIN LANG FILE
                $_SESSION['ok_msg'] = "Error: Location with this name and address already exist!";
            } else
            {
                // TODO: MOVE TO NRS ADMIN LANG FILE
                $_SESSION['ok_msg'] = "Completed: New location data was inserted successfully!";
                $locationInsertedSuccessfully = $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}locations
                    (
                        location_title, location_address,
                        pickup_fee, dropoff_fee,
                        open_mondays,
                        open_tuesdays,
                        open_wednesdays,
                        open_thursdays,
                        open_fridays,
                        open_saturdays,
                        open_sundays,
                        open_time_mon,
                        open_time_tue,
                        open_time_wed,
                        open_time_thu,
                        open_time_fri,
                        open_time_sat,
                        open_time_sun,
                        close_time_mon,
                        close_time_tue,
                        close_time_wed,
                        close_time_thu,
                        close_time_fri,
                        close_time_sat,
                        close_time_sun,
                        afterhours_pickup_location_id, afterhours_pickup_fee,
                        afterhours_dropoff_location_id, afterhours_dropoff_fee
                    ) VALUES
                    (
                        '{$sanitizedLocationTitle}', '{$sanitizedLocationAddress}',
                        '{$validPickupFee}', '{$validDropoffFee}',
                        '{$validOpenMondays}',
                        '{$validOpenTuesdays}',
                        '{$validOpenWednesdays}',
                        '{$validOpenThursdays}',
                        '{$validOpenFridays}',
                        '{$validOpenSaturdays}',
                        '{$validOpenSundays}',
                        '{$validOpenTimeMon}',
                        '{$validOpenTimeTue}',
                        '{$validOpenTimeWed}',
                        '{$validOpenTimeThu}',
                        '{$validOpenTimeFri}',
                        '{$validOpenTimeSat}',
                        '{$validOpenTimeSun}',
                        '{$validCloseTimeMon}',
                        '{$validCloseTimeTue}',
                        '{$validCloseTimeWed}',
                        '{$validCloseTimeThu}',
                        '{$validCloseTimeFri}',
                        '{$validCloseTimeSat}',
                        '{$validCloseTimeSun}',
                        '{$validAfterHoursPickupLocationId}', '{$validAfterHoursPickupFee}',
                        '{$validAfterHoursDropoffLocationId}', '{$validAfterHoursDropoffFee}'
                    )
                ", ARRAY_A);
                if($locationInsertedSuccessfully === FALSE)
                {
                    // TODO: MOVE TO NRS ADMIN LANG FILE
                    $_SESSION['msg'] = "Error: New location data was not allowed to insert into database!";
                } else
                {
                    $insertedNewLocationId = $this->internalWPDB->insert_id;

                    if($validAfterHoursPickupLocationId == -1)
                    {
                        // Drop-Off to current location
                        $validAfterHoursDropoffLocationId = $insertedNewLocationId;

                        $this->internalWPDB->query("
                        UPDATE {$this->nrsPrefix}locations SET
                        afterhours_pickup_location_id='{$validAfterHoursPickupLocationId}'
                        WHERE location_id='{$insertedNewLocationId}'
                    ", ARRAY_A);
                    }
                    if($validAfterHoursDropoffLocationId == -1)
                    {
                        // Drop-Off to current location
                        $validAfterHoursDropoffLocationId = $insertedNewLocationId;

                        $this->internalWPDB->query("
                        UPDATE {$this->nrsPrefix}locations SET
                        afterhours_dropoff_location_id='".intval($validAfterHoursDropoffLocationId)."'
                        WHERE location_id='".intval($insertedNewLocationId)."'
                    ", ARRAY_A);
                    }
                }
            }
        }
    }

    public function deleteLocation()
    {
        $validLocationId = NRSValidator::getValidInteger($this->locationId);
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}locations
            WHERE location_id='{$validLocationId}'
        ");
        $this->internalWPDB->query("
            DELETE FROM {$this->nrsPrefix}item_locations
            WHERE location_id='{$validLocationId}'
        ");
    }
}