<?php
/**
 * NRS Locations Observer (no setup for single location)
 * Abstract class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSLocationsObserver
{
    protected $internalWPDB = NULL;
    protected $nrsPrefix = "";
    protected $lang = NULL;
    protected $debugMode = 0;

    protected $currencySymbol		= '$';
    protected $currencyCode			= 'USD';
    protected $vatPercentage		= 0.00;
    protected $showPricesWithVAT	= 0;

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramConfig = array())
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        $this->currencySymbol = NRSValidator::getValidSetting($paramConfig['conf_currency_symbol'], "textval", "$");
        $this->currencyCode = NRSValidator::getValidSetting($paramConfig['conf_currency_code'], "textval", "USD");
        $this->vatPercentage = NRSValidator::getValidSetting($paramConfig['conf_vat_percentage'], "floatval", 21.00);
        $this->showPricesWithVAT = NRSValidator::getValidSetting($paramConfig['conf_show_prices_with_vat'], "intval", 1, array(0, 1));
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getAllIds()
    {
        $locationIds = $this->internalWPDB->get_col("
            SELECT location_id
            FROM {$this->nrsPrefix}locations
        ");

        return $locationIds;
    }

    /**
     * Total amount of available item pickup locations
     * @param int $reserveItemId
     * @return string
     */
    public function getItemPickupLocationsCount($reserveItemId = 0)
    {
        $totalPickupLocations = sizeof($this->getItemLocationIds($reserveItemId, "PICKUP"));
        return $totalPickupLocations;
    }

    /**
     * Total amount of available item dropoff locations
     * @param int $reserveItemId
     * @return string
     */
    public function getItemDropoffLocationsCount($reserveItemId = 0)
    {
        $totalDropoffLocations = sizeof($this->getItemLocationIds($reserveItemId, "DROPOFF"));
        return $totalDropoffLocations;
    }

    /**
     * Gets all location ids for selected location
     * @param int $reserveItemId - optional car id
     * @param string $paramLocationType - "PICKUP" or "DROPOFF"
     * @return string
     */
    private function getItemLocationIds($reserveItemId = 0, $paramLocationType = "PICKUP")
    {
        $locationType = $paramLocationType == "PICKUP" ? 1 : 2;

        if ($reserveItemId > 0)
        {
            // For single car reservation
            $sqlQuery = "
				SELECT location_id
				FROM {$this->nrsPrefix}locations
				WHERE location_id in
				(
					SELECT location_id
					FROM {$this->nrsPrefix}item_locations
					WHERE item_id='{$reserveItemId}' and location_type='{$locationType}'
				)
				ORDER BY location_title
			";
        } else
        {
            // For all items reservation
            $sqlQuery = "
				SELECT location_id
				FROM {$this->nrsPrefix}locations
				ORDER BY location_title
			";
        }

        $locationIds = $this->internalWPDB->get_col($sqlQuery);

        return $locationIds;
    }

    public function getItemPickupLocationsDropDown($reserveItemId = 0, $fieldName = "pickup_location_id", $fieldID = "pickup_location_id", $selectTitle = "", $selectedLocation = 0, $emptyItemValue = "0")
    {
        $selectPickup = $this->getLocationDropDown($reserveItemId, "PICKUP", $fieldName, $fieldID, $selectTitle, $selectedLocation, $emptyItemValue);
        return $selectPickup;
    }

    public function getItemDropoffLocationsDropDown($reserveItemID = 0, $fieldName = "dropoff_location_id", $fieldID = "dropoff_location_id", $selectTitle = "", $selectedLocation = 0, $emptyItemValue = "0")
    {
        $selectDropoff = $this->getLocationDropDown($reserveItemID, "DROPOFF", $fieldName, $fieldID, $selectTitle, $selectedLocation, $emptyItemValue);
        return $selectDropoff;
    }

    public function getPickupLocationsDropDown($fieldName = "pickup_location_id", $fieldID = "pickup_location_id", $selectTitle = "", $selectedLocation = 0, $emptyItemValue = "0")
    {
        $selectPickup = $this->getLocationDropDown(0, "PICKUP", $fieldName, $fieldID, $selectTitle, $selectedLocation, $emptyItemValue);
        return $selectPickup;
    }

    public function getDropoffLocationsDropDown($fieldName = "dropoff_location_id", $fieldID = "dropoff_location_id", $selectTitle = "", $selectedLocation = 0, $emptyItemValue = "0")
    {
        $selectDropoff = $this->getLocationDropDown(0, "DROPOFF", $fieldName, $fieldID, $selectTitle, $selectedLocation, $emptyItemValue);
        return $selectDropoff;
    }

    /**
     * getLocationDropDown($reserveItemID, "PICKUP", "pickup_location_id", "pickup_location_id", $this->lang->getText('NRS_SELECT_PICKUP_LOCATION_TEXT'))
     * getLocationDropDown($reserveItemID, "DROPOFF", "dropoff_location_id", "dropoff_location_id", $this->lang->getText('NRS_SELECT_DROPOFF_LOCATION_TEXT'))
     * @param int $reserveItemId - optional car id
     * @param string $paramLocationType - "PICKUP" or "DROPOFF"
     * @param string $fieldName - i.e. dropoff
     * @param string $fieldID - i.e. dropoff
     * @param string $fieldText - i.e. select dropoff
     * @param int $selectedLocationID - Selected location ID or 0 as default
     * @param string $emptyItemValue - empty item value, 0 or ""
     * @return string
     */
    private function getLocationDropDown($reserveItemId = 0, $paramLocationType = "PICKUP", $fieldName = "pickup_location_id", $fieldID = "pickup_location_id", $fieldText = "", $selectedLocationID = 0, $emptyItemValue = "0")
    {
        $locationType = $paramLocationType == "PICKUP" ? 1 : 2;
        $selectLocation = '';

        if ($reserveItemId > 0)
        {
            // For single car reservation
            $sqlQuery = "
				SELECT *
				FROM {$this->nrsPrefix}locations
				WHERE location_id in
				(
					SELECT location_id
					FROM {$this->nrsPrefix}item_locations
					WHERE item_id='{$reserveItemId}' and location_type='{$locationType}'
				)
				ORDER BY location_title
			";
        } else
        {
            // For all items reservation
            $sqlQuery = "
				SELECT *
				FROM {$this->nrsPrefix}locations
				ORDER BY location_title
			";
        }

        $sqlRows = $this->internalWPDB->get_results($sqlQuery, ARRAY_A);

        if(sizeof($sqlRows) == 1)
        {
            $locationRow = $sqlRows[0];
            $selectLocation .= '<input type="hidden" name="'.$fieldName.'" id="'.$fieldID.'" value="'.$locationRow['location_id'].'" />'.$locationRow['location_title'];
        } else
        {
            if($emptyItemValue == "0")
            {
                $selectedLocationZero = $selectedLocationID == 0 ? ' selected="selected"' : '';
                $selectLocation .= '<select name="'.$fieldName.'" id="'.$fieldID.'" class="home-select">';
                $selectLocation .= '<option value="0"'.$selectedLocationZero.'>'.$fieldText.'</option>';
            } else
            {
                $selectedLocationEmpty = $selectedLocationID == "" ? ' selected="selected"' : '';
                $selectLocation .= '<select name="'.$fieldName.'" id="'.$fieldID.'" class="home-select">';
                $selectLocation .= '<option value=""'.$selectedLocationEmpty.'>'.$fieldText.'</option>';
            }


            foreach ($sqlRows AS $row)
            {
                $selectedCurrent = $row['location_id'] == $selectedLocationID ? ' selected="selected"' : '';
                $selectLocation .= '<option value="'.$row['location_id'].'"'.$selectedCurrent.'>'.$row['location_title'].'</option>';
            }
            $selectLocation .= '</select>';
        }
        return $selectLocation;
    }



    /**
     * @return string
     */
    public function getAdminLocationsList()
    {
        $internalConfig = array(
            "conf_currency_symbol" => $this->currencySymbol,
            "conf_currency_code" => $this->currencyCode,
            "conf_vat_percentage" => $this->vatPercentage,
            "conf_show_prices_with_vat" => $this->showPricesWithVAT,
        );

        $locationsHTML = '';
        $locationIds = $this->getAllIds();

        foreach ($locationIds AS $locationId)
        {
            $objLocation = new NRSLocation(
                $GLOBALS['wpdb'], $GLOBALS['NRS_PREFIX'], $GLOBALS['NRS_LANG'],
                $internalConfig, $locationId
            );
            $locationDetails = $objLocation->getLocationDetails();
            $printWorkingHours = $objLocation->getPrintWeekWorkingHours();
            $printAfterHoursPickupLocation = $objLocation->getAfterhoursPickupLocationAlias();
            $printAfterHoursDropoffLocation = $objLocation->getAfterhoursDropoffLocationAlias();

            $locationsHTML .= '<tr>';
            $locationsHTML .= '<td>'.$locationDetails['location_title'];
            if($locationDetails['location_address'] != "")
            {
                $locationsHTML .= '<br />'.$locationDetails['location_address'];
            }
            /**DEBUG***/ if($this->debugMode)
            /**DEBUG***/ {
            /**DEBUG***/     $locationsHTML .= '<br /><strong>ID:</strong> '.$locationId.', ';
            /**DEBUG***/     $locationsHTML .= '<br /><strong>PID:</strong> '.$locationDetails['afterhours_pickup_location_id'].', ';
            /**DEBUG***/     $locationsHTML .= '<br /><strong>DID:</strong> '. $locationDetails['afterhours_dropoff_location_id'];
            /**DEBUG***/ }
            $locationsHTML .= '</td>';
            $locationsHTML .= '<td>';
            $locationsHTML .= 'Pickup:&nbsp;&nbsp; '.$locationDetails['print_pickup_fee'].' ('.$this->lang->getText('NRS_WITHOUT_VAT_TEXT').')<br />';
            $locationsHTML .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; '.$locationDetails['print_pickup_fee_with_vat'].'<br />';
            $locationsHTML .= 'Dropoff: '.$locationDetails['print_dropoff_fee'].' ('.$this->lang->getText('NRS_WITHOUT_VAT_TEXT').')<br />';
            $locationsHTML .= '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp; '.$locationDetails['print_dropoff_fee_with_vat'];
            $locationsHTML .= '</td>';
            $locationsHTML .= '<td style="white-space: nowrap">'.$printWorkingHours.'</td>';
            $locationsHTML .= '<td>';
            $locationsHTML .= 'Pickup:<br />'.$printAfterHoursPickupLocation.'<br /><br />';
            $locationsHTML .= 'Dropoff:<br />'.$printAfterHoursDropoffLocation;
            $locationsHTML .= '</td>';
            $locationsHTML .= '<td>';
            $locationsHTML .= 'Pickup:&nbsp;&nbsp; '.$locationDetails['print_afterhours_pickup_fee'].'<br />';
            $locationsHTML .= 'Dropoff: '.$locationDetails['print_afterhours_dropoff_fee'];
            $locationsHTML .= '</td>';
            $locationsHTML .= '<td align="right" style="white-space: nowrap"><a href="'.admin_url('admin.php?page='.$GLOBALS['NRS_EXTENSION'].'-add-edit-location&amp;lid='.$locationId).'">'.$this->lang->getText('NRS_ADMIN_EDIT_TEXT').'</a> || ';
            $locationsHTML .= '<a href="javascript:;" onclick="javascript:deleteLocation(\''.$locationId.'\', \'?page='.$GLOBALS['NRS_EXTENSION'].'-location-manager\')">'.$this->lang->getText('NRS_ADMIN_DELETE_TEXT').'</a></td>';
            $locationsHTML .= '</tr>';
        }

        return  $locationsHTML;
    }

    public function getLocationsDropDown($selected="0")
    {
        $selectDropPickLocation = '';
        $sqlRows = $this->internalWPDB->get_results("
            SELECT *
            FROM {$this->nrsPrefix}locations
            ORDER BY location_title
        ", ARRAY_A);

        $selectDropPickLocation .= '<option value="0"'.($selected == 0 ? ' selected="selected"' : '').'>'.$this->lang->getText('NRS_ADMIN_SELECT_LOCATION_TEXT').'</option>';
        foreach($sqlRows AS $row)
        {
            $selectDropPickLocation .= '<option value="'.$row['location_id'].'"'.($selected == $row['location_id'] ? ' selected="selected"' : '').'>'.$row['location_title'].' - '.$row['location_address'].'</option>';
        }
        return $selectDropPickLocation;
    }

    public function getAfterHoursLocationsDropDown($mode="DROPOFF", $id="afterhours_dropoff_location_id", $name="afterhours_dropoff_location_id", $currentLocationIDtoSkip, $currentAfterHoursLocationID=0)
    {
        $searchField = $mode == "DROPOFF" ? "afterhours_dropoff_location_id" : "afterhours_pickup_location_id";
        $sqlRows = $this->internalWPDB->get_results("
          SELECT * FROM {$this->nrsPrefix}locations
          WHERE {$searchField}='-1' AND location_id!='$currentLocationIDtoSkip'
          ORDER BY location_title
        ", ARRAY_A);

        $selectedNotAllowed = ($currentAfterHoursLocationID == 0 ? ' selected="selected"' : '');
        $selectedCurrentLocation = ($currentAfterHoursLocationID == -1 ? ' selected="selected"' : '');

        $selectAfterHoursLocation = '<select name="'.$name.'" id="'.$id.'">';
        $selectAfterHoursLocation .= '<option value="0"'.$selectedNotAllowed.'>'.$this->lang->getText('NRS_AFTERHOURS_RETURN_IS_NOT_ALLOWED_TEXT').'</option>';
        $selectAfterHoursLocation .= '<option value="-1"'.$selectedCurrentLocation.'>'.$this->lang->getText('NRS_RETURN_IN_THIS_BRANCH_TEXT').'</option>';
        foreach($sqlRows AS $row)
        {
            $selected = ($row['location_id'] == $currentAfterHoursLocationID ? ' selected="selected"' : '');
            $selectAfterHoursLocation .= '<option value="'.$row['location_id'].'"'.$selected.'>'.$row['location_title'].'</option>';
        }
        $selectAfterHoursLocation .= '</select>';

        return $selectAfterHoursLocation;
    }
}