<?php
/**
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
*/
//error_reporting(0); - this should always be controlled by WordPress

final class NRSMail
{
	protected $internalWPDB = NULL;
	protected $nrsPrefix 	= "";
	protected $lang 		= NULL;
	protected $debugMode    = 0;
	private $portalName 	= "";
	private $portalEmail 	= "";
	private $portalPhone 	= "";
	private $sendEmails 	= true;

	/**
	 * @param $paramWPDB
	 * @param $paramNRSPrefix
	 * @param NRSLanguage &$paramNRSLang - Language
	 */
	public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
	{
		$this->internalWPDB = $paramWPDB;
		$this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
		// Already sanitized before in it's constructor. Too much sanitation will kill the system speed
		$this->lang = $paramNRSLang;
		// Set internal object settings
		$this->setSettings();

	}

	/**
	 * Set measurement unit - called in class constructor
	 */
	private function setSettings()
	{
		$sqlPortalName = "
				SELECT conf_value AS portal_name
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_portal_name'
			";
		$sqlPortalEmail = "
				SELECT conf_value AS portal_email
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_portal_email'
			";
		$sqlPortalPhone = "
				SELECT conf_value AS portal_phone
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_portal_phone'
			";
		$sqlSendEmails = "
				SELECT conf_value AS send_emails
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_send_emails'
			";
		$portalName = $this->internalWPDB->get_var($sqlPortalName);
		$portalEmail = $this->internalWPDB->get_var($sqlPortalEmail);
		$portalPhone = $this->internalWPDB->get_var($sqlPortalPhone);
		$sendEmails = $this->internalWPDB->get_var($sqlSendEmails);

		$this->portalName = !is_null($portalName) ? sanitize_text_field($portalName) : "";
		$this->portalEmail = !is_null($portalEmail) ? sanitize_email($portalEmail) : "";
		$this->portalPhone = !is_null($portalPhone) ? sanitize_text_field($portalPhone) : "";
		$this->sendEmails = !is_null($sendEmails) && $sendEmails == "1" ? TRUE : FALSE;
	}

	public function checkSendEmails()
	{
		return $this->sendEmails;
	}

	public function sendBookingDetailsEmail($paramBookingId, $paramSendNotificationToAdmin = TRUE, $paramEmailWidth = 840)
	{
		return $this->sendBookingEmail("DETAILS", $paramBookingId, $paramSendNotificationToAdmin, $paramEmailWidth);
	}

	public function sendBookingConfirmationEmail($paramBookingId, $paramSendNotificationToAdmin = TRUE, $paramEmailWidth = 840)
	{
		return $this->sendBookingEmail("CONFIRMED", $paramBookingId, $paramSendNotificationToAdmin, $paramEmailWidth);
	}

	public function sendBookingCancellationEmail($paramBookingId, $paramSendNotificationToAdmin = TRUE, $paramEmailWidth = 840)
	{
		return $this->sendBookingEmail("CANCELLED", $paramBookingId, $paramSendNotificationToAdmin, $paramEmailWidth);
	}

	/**
	 * Existing valid booking id MUST be provided to successfully process this method to the end
	 * @param $paramType - "DETAILS", "CONFIRMED", "CANCELLED"
	 * @param $paramBookingId
	 * @param $paramSendNotificationToAdmin
	 * @param $paramEmailWidth - email width in pixels
	 * @return bool
	 */
	private function sendBookingEmail($paramType, $paramBookingId, $paramSendNotificationToAdmin = TRUE, $paramEmailWidth = 840)
	{
		$ret = FALSE;
		$validBookingId = NRSValidator::getValidInteger($paramBookingId,0);
		$validEmailWidth = NRSValidator::getValidInteger($paramEmailWidth, 840);

		$emailContent = "";
		$notifyEmailContent = "";
		if($paramType == "DETAILS")
		{
			$emailContent     	= $this->getEmailContent(1);
			$notifyEmailContent	= $this->getEmailContent(4);
		} else if($paramType == "CONFIRMED")
		{
			$emailContent     	= $this->getEmailContent(2);
			$notifyEmailContent = $this->getEmailContent(5);
		} else if($paramType == "CANCELLED")
		{
			$emailContent     	= $this->getEmailContent(3);
			$notifyEmailContent = $this->getEmailContent(6);
		}

		if($validBookingId > 0)
		{
			// If there exists booking under this booking id
			// We should not check here if it's paid or not
			$sql = "
				  SELECT booking_id, customer_id, booking_code
				  FROM {$this->nrsPrefix}bookings
				  WHERE booking_id='{$validBookingId}'
			";

			$bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

			if(!is_null($bookingData))
			{
				$invoiceDetails   = $this->internalWPDB->get_row("
					SELECT * FROM {$this->nrsPrefix}invoices WHERE booking_id='{$validBookingId}'
				", ARRAY_A);

				// 1 - Replace shortcodes in email title
				$emailSubject = $this->replaceBBCodesInEmailSubject(
					$emailContent['email_subject'], $bookingData['booking_code'], $invoiceDetails['customer_name']
				);
				$notifyEmailSubject = $this->replaceBBCodesInEmailSubject(
					$notifyEmailContent['email_subject'], $bookingData['booking_code'], $invoiceDetails['customer_name']
				);

				// 2 - Replaces bb codes in email body
				$emailBody = $this->replaceBBCodesInEmailBody(
					$emailContent['email_body'], $bookingData['booking_code'], $bookingData['customer_id'],
					$invoiceDetails['customer_name'], $invoiceDetails['invoice']
				);
				$notifyEmailBody = $this->replaceBBCodesInEmailBody(
					$notifyEmailContent['email_body'], $bookingData['booking_code'], $bookingData['customer_id'],
					$invoiceDetails['customer_name'], $invoiceDetails['invoice']
				);

				// 2 - Put email data in a div, to all center and other tags looks well.
				$emailBody = '<div style="width:'.$validEmailWidth.'px;">'.$emailBody.'</div>';
				$notifyEmailBody = '<div style="width:'.$validEmailWidth.'px;">'.$notifyEmailBody.'</div>';

				if($this->checkSendEmails())
				{
					$ret = $this->sendEMail($invoiceDetails['customer_email'], $emailSubject, $emailBody);
					if ($ret == TRUE && $paramSendNotificationToAdmin == TRUE)
					{
						$ret = $this->sendEMail($this->portalEmail, $notifyEmailSubject, $notifyEmailBody);
					}
				} else
				{
					$ret = TRUE;
				}
			}
		}

		return $ret;
	}

	/**
	 * Admin email content preview
	 * @param $emailId
	 * @param int $paramEmailWidth
	 * @return array
	 */
	public function getEmailContentPreview($emailId, $paramEmailWidth = 840)
	{
		$validEmailWidth 	= NRSValidator::getValidInteger($paramEmailWidth, $paramEmailWidth);
		$emailContent   	= $this->getEmailContent($emailId);
		$bookingCode 		= "DEMO2015A0001";// Demo
		$customerId 		= 1001; // Demo

		// Select the newest invoice for testing. Invoice with booking id=0 will exist there by default as a demo
		$invoiceDetails   = $this->internalWPDB->get_row("
			SELECT * FROM {$this->nrsPrefix}invoices
			ORDER BY booking_id DESC LIMIT 1
		", ARRAY_A);

		// If there exists booking under this booking id
		$sql = "
				  SELECT booking_id, customer_id, booking_code
				  FROM {$this->nrsPrefix}bookings
				  WHERE booking_id='{$invoiceDetails['booking_id']}'
			";

		$bookingData = $this->internalWPDB->get_row($sql, ARRAY_A);

		if(!is_null($bookingData))
		{
			$bookingCode = $bookingData['booking_code'];
			$customerId = $bookingData['customer_id'];
		}

		// 1 - Replace shortcodes in email title
		$emailSubject = $this->replaceBBCodesInEmailSubject(
			$emailContent['email_subject'], $bookingCode, $invoiceDetails['customer_name']
		);

		// 2 - Replaces bb codes in email body
		$emailBody = $this->replaceBBCodesInEmailBody(
			$emailContent['email_body'], $bookingCode, $customerId,
			$invoiceDetails['customer_name'], $invoiceDetails['invoice']
		);

		// 2 - Put email data in a div, to all center and other tags looks well.
		$emailBody = '<div style="width:'.$validEmailWidth.'px;">'.$emailBody.'</div>';

		$preview = array(
			'email_subject' => $emailSubject,
			'email_body' => $emailBody,
		);

		return $preview;
	}

	/**
	 * Replace bb codes in email subject
	 * @param $trustedEmailSubject
	 * @param $bookingCode
	 * @param $customerName
	 * @return string
	 */
	private function replaceBBCodesInEmailSubject($trustedEmailSubject, $bookingCode, $customerName)
	{
		$from = array(
			"[BOOKING_CODE]", "[CUSTOMER_NAME]",
		);
		$to = array(
			$bookingCode, $customerName
		);
		$modifiedEmailSubject = str_replace($from, $to, $trustedEmailSubject);

		return $modifiedEmailSubject;
	}

	/**
	 * Replace bb codes in email body
	 * @param $trustedEmailBody
	 * @param $bookingCode
	 * @param $customerId
	 * @param $customerName
	 * @param $trustedInvoice
	 * @return string
	 */
	private function replaceBBCodesInEmailBody($trustedEmailBody, $bookingCode, $customerId, $customerName, $trustedInvoice)
	{

		// 3 - Replace Site url
		$modifiedEmailBody = str_replace('[PORTAL_URL]', site_url(), $trustedEmailBody);

		// 4 - Replace basic bb code
		$modifiedEmailBody = preg_replace('#\[S\](.*?)\[/S\]#si', '<strong>\1</strong>', $modifiedEmailBody);
		$modifiedEmailBody = preg_replace('#\[EM\](.*?)\[/EM\]#si', '<em>\1</em>', $modifiedEmailBody);
		$modifiedEmailBody = preg_replace('#\[CENTER\](.*?)\[/CENTER\]#si', '<center>\1</center>', $modifiedEmailBody);
		$modifiedEmailBody = preg_replace('#\[HR\]#si', '<hr />', $modifiedEmailBody);

		// 5 - Auto replace links
		$modifiedEmailBody = preg_replace_callback(
			'#(^|[\n ])([\w]+?://[ąčęėįšųūžĄČĘĖĮŠŲŪŽ\w\#$%&~/.\-;:=,?@\(?\)?\]\|+]*)#si',
			function ($matches)
			{
				return $matches[1].'<a href="'.$matches[2].'" target="_blank">'.$matches[2].'</a>';
			},
			$modifiedEmailBody
		);
		$modifiedEmailBody = preg_replace_callback(
			'#(^|[\n ])((www|ftp)\.[ąčęėįšųūžĄČĘĖĮŠŲŪŽ\w\#$%&~/.\-;:=,?@\(?\)?\]\|+]*)#si',
			function ($matches)
			{
				return $matches[1].'<a href="'.$matches[2].'" target="_blank">'.$matches[2].'</a>';
			},
			$modifiedEmailBody
		);

		// 6 - Auto replace links
		$modifiedEmailBody = preg_replace_callback(
			'#(^|[\n ])([a-z0-9&\-_.]+?)@([\w\-]+\.([\w\-\.]+\.)*[\w]+)#si',
			function ($matches)
			{
				return $matches[1].'<a href="mailto:'.$matches[2].'@'.$matches[23].'">'.$matches[2].'@'.$matches[3].'</a>';
			},
			$modifiedEmailBody
		);

		// 7 - Replace image shortcodes in email body - [IMG]URL[/IMG]
		$modifiedEmailBody = preg_replace_callback(
			'#\[IMG\]([\r\n]*)(http://|ftp://|https://|ftps://)([ąčęėįšųūžĄČĘĖĮŠŲŪŽ\w\#$%~/.\-;:=,&?@\(?\)?\[\]\|+]*)([\r\n]*)\[/IMG\]#si',
			function ($matches)
			{
				return '<img src="'.$matches[2].$matches[3].'" alt="'.$this->lang->getText('NRS_IMAGE_ALT_TEXT').'" />';
			},
			$modifiedEmailBody
		);
		// For more strict (image extension only) matches - $matches[2].$matches[3].$matches[4] use this code:
		// '#\[ZF\-LIKE\]([\r\n]*)(http://|ftp://|https://|ftps://)([ąčęėįšųūžĄČĘĖĮŠŲŪŽ\w\#$%~/.\-;:=,&?@\(?\)?\[\]\|+]*)(\.(jpg|jpeg|gif|png|JPG|JPEG|GIF|PNG))([\r\n]*)\[/ZF\-LIKE\]#si'

		// 8 - Convert \n to line breaks
		$modifiedEmailBody = nl2br($modifiedEmailBody);

		// 9 - Replace all other shortcodes in email body
		$from = array(
			"[BOOKING_CODE]", "[CUSTOMER_ID]", "[CUSTOMER_NAME]",
			"[INVOICE]",
			"[PORTAL_NAME]", "[PORTAL_PHONE]", "[PORTAL_EMAIL]",
		);
		$to = array(
			$bookingCode, $customerId, $customerName,
			$trustedInvoice,
			$this->portalName, $this->portalPhone, '<a href="mailto:'.$this->portalEmail.'">'.$this->portalEmail.'</a>',
		);
		$modifiedEmailBody = str_replace($from, $to, $modifiedEmailBody);

		return $modifiedEmailBody;
	}

	public function sendEMail($paramEmailTo, $paramEmailSubject, $paramEmailBody)
	{

		$emailSentSuccessfully = false;
		$sanitizedEmailTo = sanitize_email($paramEmailTo);
		$sanitizedEmailSubject = sanitize_text_field($paramEmailSubject);
		$emailHeaders = array();
		$emailHeaders[] = 'From: '.$this->portalName.' <'.$this->portalEmail.'>';
		$emailHeaders[] = 'Content-Type: text/html; charset=UTF-8';

		$weTrustThatItsClearEmailBody = $paramEmailBody;
		if($sanitizedEmailTo != "")
		{
			$emailSentSuccessfully = wp_mail( $sanitizedEmailTo, $sanitizedEmailSubject, $weTrustThatItsClearEmailBody, $emailHeaders );
		}

		if($this->debugMode)
		{
			echo "<br />Send email to: ".$sanitizedEmailTo;
			echo "<br />Email subject: ".$sanitizedEmailSubject;
			echo "<br />Email headers: ".nl2br(esc_html(print_r($emailHeaders, TRUE)));
			echo "<br />Email sent successfully: "; var_dump($emailSentSuccessfully);
		}

		return $emailSentSuccessfully;
	}

	public function getEmailContent($paramEmailId = 1)
	{
		$validEmailId = NRSValidator::isInteger($paramEmailId) ? intval($paramEmailId) : 1;
		$currentRow = $this->internalWPDB->get_row("
				SELECT *
				FROM {$this->nrsPrefix}email_contents
				WHERE email_id='{$validEmailId}'
			", ARRAY_A);

		$retEmailContent = array(
			'email_id' 		=> isset($currentRow['email_id']) ? $currentRow['email_id'] : 0,
			'email_subject' => isset($currentRow['email_subject']) ? $currentRow['email_subject'] : "",
			'email_body' 	=> isset($currentRow['email_body']) ? $currentRow['email_body'] : "",
		);

		return $retEmailContent;
	}

	/****************************************************************************************/
	public function updateEmailContent()
	{
		$validEmailId = NRSValidator::getValidInteger($_POST['email_id'], 1);
		$sanitizedEmailSubject = sanitize_text_field($_POST['email_subject']);
		$sanitizedEmailBody =  implode( "\n", array_map( 'sanitize_text_field', explode( "\n", $_POST['email_body'] ) ) );

		$check = $this->internalWPDB->get_row("
			SELECT email_id
			FROM {$this->nrsPrefix}email_contents
			WHERE email_subject='{$sanitizedEmailSubject}'
			AND email_id!='{$validEmailId}'
       ", ARRAY_A);

		if(is_null($check))
		{
			$_SESSION['ok_msg'] = 'Completed: Email settings updated successfully!';

			$this->internalWPDB->query("
				UPDATE {$this->nrsPrefix}email_contents SET
				email_subject='{$sanitizedEmailSubject}', email_body='{$sanitizedEmailBody}'
				WHERE email_id='{$validEmailId}'
		   ");
		} else
		{
			$_SESSION['msg'] = 'Failed: Other email type already exist with this subject!';
		}
	}

	public function getEmailContents($selectedEmailId = 0)
	{
		$selected = $selectedEmailId == 0 ? ' selected="selected"' : '';
		$dropList = '<option value="0"'.$selected.'>'.$this->lang->getText('NRS_ADMIN_SELECT_EMAIL_TYPE_TEXT').'</option>';
		$rows = $this->internalWPDB->get_results("SELECT * FROM {$this->nrsPrefix}email_contents", ARRAY_A);

		foreach ($rows AS $rowEmailInfo)
		{
			$selected = $selectedEmailId == $rowEmailInfo['email_id'] ? ' selected="selected"' : '';
			$dropList .= '<option value="'.$rowEmailInfo['email_id'].'"'.$selected.'>'.$rowEmailInfo['email_id'].'. '.$rowEmailInfo['email_subject'].'</option>';
		}

		return $dropList;
	}
}