<?php
/**
 * Prepayment processor. Used in administration side only
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSPrepayment
{
    protected $internalWPDB	 	= NULL;
    protected $nrsPrefix 		= "";
    protected $lang 			= NULL;
    protected $debugMode 	 	= 0;
    protected $prepaymentId        = 0;

    /**
     * @param $paramWPDB
     * @param $paramNRSPrefix
     * @param NRSLanguage &$paramNRSLang - LANGUAGE
     * @param $paramPrepaymentId
     */
    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang, $paramPrepaymentId)
    {
        // Set class settings
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);

        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;

        // Set prepayment id
        $this->prepaymentId = NRSValidator::getValidInteger($paramPrepaymentId, 0);
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function getPrepaymentId()
    {
        return $this->prepaymentId;
    }

    /**
     * For internal class use only
     * @param $paramPrepaymentId
     * @return mixed
     */
    private function getPrepaymentDataFromDatabaseById($paramPrepaymentId)
    {
        $validPrepaymentId = NRSValidator::getValidInteger($paramPrepaymentId, 0);
        $prepaymentData = $this->internalWPDB->get_row("
            SELECT *
            FROM {$this->nrsPrefix}prepayments
            WHERE prepayment_id='{$validPrepaymentId}'
        ", ARRAY_A);

        return $prepaymentData;
    }

    public function getPrepaymentDetails()
    {
        $prepaymentData = $this->getPrepaymentDataFromDatabaseById($this->prepaymentId);

        return $prepaymentData;
    }

    public function savePrepayment($paramPrepaymentId, $paramItemId, $paramPeriodFrom, $paramPeriodTill, $paramPrepaymentPercentage)
    {
        $validPrepaymentId = NRSValidator::getValidInteger($paramPrepaymentId,0);
        $validItemId = NRSValidator::getValidInteger($paramItemId,0);
        $validPeriodFrom = NRSValidator::getValidInteger($paramPeriodFrom, 0);
        $validPeriodTill = NRSValidator::getValidInteger($paramPeriodTill, 0);
        $validPrepaymentPercentage = floatval($paramPrepaymentPercentage);

        // Do not allow to have prepayments more than 100%
        if($validPrepaymentPercentage > 100)
        {
            $validPrepaymentPercentage = 100;
        }

        if($validPrepaymentId > 0)
        {
            $itemExists = $this->internalWPDB->get_row("
                SELECT prepayment_id
                FROM {$this->nrsPrefix}prepayments
                WHERE prepayment_id='{$validPrepaymentId}'
            ", ARRAY_A);

            // If expr is greater than or equal to min and expr is less than or equal to max, BETWEEN returns 1, otherwise it returns 0
            $minDaysValueCheck = $this->internalWPDB->get_results("
                SELECT prepayment_id
                FROM {$this->nrsPrefix}prepayments
                WHERE prepayment_id!='{$validPrepaymentId}' AND item_id='{$validItemId}' AND (
                  '{$validPeriodFrom}' BETWEEN period_from AND period_till
                  OR '{$validPeriodTill}' BETWEEN period_from AND period_till
                )
            ", ARRAY_A);

            if(sizeof($minDaysValueCheck) > 0)
            {
                $_SESSION['msg'] = 'Error: Current prepayment plan days range intersects other prepayment plan!';
            } else if(is_null($itemExists))
            {
                $_SESSION['msg'] = 'Error: Prepayment plan do not exist!';
            } else
            {
                $updated = $this->internalWPDB->query("
                    UPDATE {$this->nrsPrefix}prepayments SET
                    item_id='{$validItemId}', period_from='{$validPeriodFrom}', period_till='{$validPeriodTill}',
                    prepayment_percentage='{$validPrepaymentPercentage}'
                    WHERE prepayment_id='{$validPrepaymentId}'
                ");
                if($updated === FALSE)
                {
                    $_SESSION['msg'] = 'Error: MySQL update error for existing prepayment!';
                } else
                {
                    $_SESSION['ok_msg'] = 'Completed: Prepayment plan was updated successfully!';
                }
            }
        } else
        {
            //If expr is greater than or equal to min and expr is less than or equal to max, BETWEEN returns 1, otherwise it returns 0
            $minDaysValueCheck = $this->internalWPDB->get_results("
                SELECT *
                FROM {$this->nrsPrefix}prepayments
                WHERE item_id='{$validItemId}' AND (
                  '{$validPeriodFrom}' BETWEEN period_from AND period_till
                  OR '{$validPeriodTill}' BETWEEN period_from AND period_till
                )
            ", ARRAY_A);

            if(sizeof($minDaysValueCheck) > 0)
            {
                $_SESSION['msg'] = 'Error: Current prepayment plan days range intersects with other prepayment plan!';
            } else
            {
                $inserted = $this->internalWPDB->query("
                    INSERT INTO {$this->nrsPrefix}prepayments
                    (
                      item_id, period_from, period_till, prepayment_percentage
                    ) VALUES
                    (
                      '{$validItemId}', '{$validPeriodFrom}', '{$validPeriodTill}', '".floatval($validPrepaymentPercentage)."'
                    )
                 ");
                if($inserted === FALSE || $inserted === 0)
                {
                    $_SESSION['msg'] = 'Error: MySQL insert error for new prepayment!';
                } else
                {
                    $_SESSION['ok_msg'] = 'Completed: New prepayment plan was added successfully!';
                    // Update class prepayment id with newly inserted prepayment it for future work
                    $this->prepaymentId = $this->internalWPDB->insert_id;
                }
            }
        }
    }

    public function deletePrepayment()
    {
        $validPrepaymentId = NRSValidator::getValidInteger($this->prepaymentId);
        $deleted = $this->internalWPDB->query("
          DELETE FROM {$this->nrsPrefix}prepayments
          WHERE prepayment_id='{$validPrepaymentId}'
        ");
        if($deleted === FALSE || $deleted === 0)
        {
            $_SESSION['msg'] = 'Error: MySQL delete error for existing prepayment. No prepayment row were deleted from database!';
        } else
        {
            $_SESSION['ok_msg'] = "Completed: Prepayment plan deleted successfully!";
        }
    }
}