<?php
/**
 * Configuration core class implementation for Admin part of the website
 * Final class cannot be inherited anymore. We use them when creating new instances
 * @package NRS
 * @uses NRSCoreBackEnd, NRSMail, NRSLocationManager, NRSCustomerManager, NRSBookingManager, NRSInvoiceManager
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

final class NRSPricePlan
{
    protected $internalWPDB     = NULL;
    protected $nrsPrefix        = "";
    protected $lang             = NULL;
    protected $debugMode        = 0;
    protected $shortDateFormat  = "Y-m-d";

    public function __construct(wpdb &$paramWPDB, $paramNRSPrefix, NRSLanguage &$paramNRSLang)
    {
        $this->internalWPDB = $paramWPDB;
        $this->nrsPrefix = sanitize_text_field($paramNRSPrefix);
        // Already sanitized before in it's constructor. Too much sanitation will kill the system speed
        $this->lang = $paramNRSLang;


        // Set price calculation type
        $this->setShortDateFormat();
    }

    /**
     * Set short date format - called in class constructor
     */
    private function setShortDateFormat()
    {
        $sql = "
				SELECT conf_value AS short_date_format
				FROM {$this->nrsPrefix}settings
				WHERE conf_key='conf_short_date_format'
			";
        $shortDateFormat = $this->internalWPDB->get_var($sql);

        if(!is_null($shortDateFormat) && NRSValidator::isValidDateFormat($shortDateFormat))
        {
            $this->shortDateFormat = sanitize_text_field($shortDateFormat);
        } else
        {
            $this->shortDateFormat = "Y-m-d";
        }
    }

    public function inDebug()
    {
        return ($this->debugMode >= 1 ? TRUE : FALSE);
    }

    public function updatePricePlan()
    {
        $validItemId = NRSValidator::isInteger($_POST['item_id']) ? intval($_POST['item_id']) : 0;
        $validStartTimestamp = 0;
        if(isset($_POST['start_date']) && $_POST['start_date'] != "")
        {
            $validISOStartDate = NRSValidator::getValidISODate($_POST['start_date'], $this->shortDateFormat);
            $validStartTimestamp = NRSValidator::getUTCTimestampFromLocalISODateTime($validISOStartDate, '00:00:00');
        }
        $validEndTimestamp = 0;
        if($validStartTimestamp > 0 && isset($_POST['end_date']) && $_POST['end_date'] != "")
        {
            $validISOEndDate = NRSValidator::getValidISODate($_POST['end_date'], $this->shortDateFormat);
            $validEndTimestamp = NRSValidator::getUTCTimestampFromLocalISODateTime($validISOEndDate, '23:59:59');
        }
        $arrPricePlans = isset($_POST['price_plans']) && is_array($_POST['price_plans']) ? $_POST['price_plans'] : array();
        $seasonalPrice = ($validStartTimestamp > 0 && $validEndTimestamp > 0) ? 1 : 0;

        $pricePlanIds = array();
        foreach($arrPricePlans AS $pricePlan)
        {
            if(isset($pricePlan['plan_id']) && NRSValidator::isInteger($pricePlan['plan_id']) && $pricePlan['plan_id'] > 0)
            {
                // Add price plan to price plan ids list
                $pricePlanIds[] = NRSValidator::getValidInteger($pricePlan['plan_id']);
            }
        }

        $SQLExistConflictingDates = "
            SELECT *
            FROM {$this->nrsPrefix}price_plans
            WHERE
            (
                ('{$validStartTimestamp}' BETWEEN start_timestamp AND end_timestamp)
                OR ('{$validEndTimestamp}' BETWEEN start_timestamp AND end_timestamp)
                OR (start_timestamp BETWEEN '{$validStartTimestamp}' AND '{$validEndTimestamp}')
                OR (end_timestamp BETWEEN '{$validStartTimestamp}' AND '{$validEndTimestamp}')
            ) AND item_id='{$validItemId}'
        ";
        $SQLPaymentPlanCheckOnlyForEditMode = "";
        if(sizeof($pricePlanIds) > 0)
        {
            $SQLPaymentPlanCheckOnlyForEditMode = " AND plan_id NOT IN (".implode(',', $pricePlanIds).")";
        }

        if($validItemId > 0 && sizeof($pricePlanIds) > 0)
        {
            // DEBUG
            //echo "(\$validItemId=$validItemId, \$plan_ids=".implode(',', $pricePlanIds)."})<br />";
            //echo "<br />Query: {$SQLExistConflictingDates}{$SQLPaymentPlanCheckOnlyForEditMode}";
            //die("Update existing price plan");
            $existConflictingDates = $this->internalWPDB->get_row($SQLExistConflictingDates.$SQLPaymentPlanCheckOnlyForEditMode, ARRAY_A);
            if(is_null($existConflictingDates))
            {
                foreach($arrPricePlans AS $priceType => $pricePlan)
                {
                    $query = "UPDATE `{$this->nrsPrefix}price_plans` SET
                        `start_timestamp`='{$validStartTimestamp}',
                        `end_timestamp`='{$validEndTimestamp}',
                        `price_type`='".intval($priceType)."',
                        `mon`='".floatval($pricePlan['mon'])."',
                        `tue`='".floatval($pricePlan['tue'])."',
                        `wed`='".floatval($pricePlan['wed'])."',
                        `thu`='".floatval($pricePlan['thu'])."',
                        `fri`='".floatval($pricePlan['fri'])."',
                        `sat`='".floatval($pricePlan['sat'])."',
                        `sun`='".floatval($pricePlan['sun'])."'
                        WHERE plan_id='".intval($pricePlan['plan_id'])."'
                    ";

                    // DEBUG
                    //echo nl2br($query)."<br /><br />";
                    $this->internalWPDB->query($query);
                }
                $_SESSION['ok_msg'] = "Completed: Existing price plan updated successfully!";
            } else
            {
                $_SESSION['msg'] = "Error: Can&#39;t update a existing price plan - there is an existing price plan for this date range!";
            }
        } else
        {
            //echo "(\$validItemId=$validItemId, \$plan_ids=".implode(',', $pricePlanIds)."})<br />";
            //die('Add new price plan');
            $existConflictingDates = $this->internalWPDB->get_row($SQLExistConflictingDates, ARRAY_A);
            if(is_null($existConflictingDates))
            {
                foreach($arrPricePlans AS $priceType => $pricePlan)
                {
                    $this->internalWPDB->query("
                      INSERT INTO `{$this->nrsPrefix}price_plans`
                      (
                        `item_id`,
                        `price_type`,
                        `start_timestamp`,
                        `end_timestamp`,
                        `mon`,
                        `tue`,
                        `wed`,
                        `thu`,
                        `fri`,
                        `sat`,
                        `sun`,
                        `seasonal_price`
                      ) VALUES
                      (
                        '{$validItemId}',
                        '".intval($priceType)."',
                        '{$validStartTimestamp}',
                        '{$validEndTimestamp}',
                        '".floatval($pricePlan['mon'])."',
                        '".floatval($pricePlan['tue'])."',
                        '".floatval($pricePlan['wed'])."',
                        '".floatval($pricePlan['thu'])."',
                        '".floatval($pricePlan['fri'])."',
                        '".floatval($pricePlan['sat'])."',
                        '".floatval($pricePlan['sun'])."',
                        '{$seasonalPrice}'
                      );
                    ");
                }
                $_SESSION['ok_msg'] = "Completed: New price plan inserted successfully!";
            } else
            {
                $_SESSION['msg'] = "Error: Can&#39;t insert new price plan - there is an existing price plan for this date range!";
            }
        }
    }

    /**
     * Seasonal price plan delete method. All data securely validated
     * @param $paramDeletionDetails
     * @return mixed
     */
    public function deletePricePlan($paramDeletionDetails)
    {
        $detailRow = explode("|", $paramDeletionDetails);
        $validItemId = isset($detailRow[0]) ? NRSValidator::getValidInteger($detailRow[0], 0) : 0;
        $validPricePlanStartTimestamp = isset($detailRow[1]) ? NRSValidator::getValidInteger($detailRow[1]) : 0;
        $validPricePlanEndTimestamp = isset($detailRow[2]) ? NRSValidator::getValidInteger($detailRow[2]): 0;
        $this->internalWPDB->query("
            DELETE FROM `{$this->nrsPrefix}price_plans`
            WHERE item_id='{$validItemId}' AND start_timestamp='{$validPricePlanStartTimestamp}' AND end_timestamp='{$validPricePlanEndTimestamp}'
        ");

        return $validItemId;
    }
}