<?php
/**
 * NRS Modern data formatter
 * @package NRS
 * @author Kestutis Matuliauskas
 * @copyright Kestutis Matuliauskas
 * See Licensing/README_License.txt for copyright notices and details.
 */

class NRSFormatter
{
    const WORLD_TIMEZONES_MAX_DIFFERENCE_IN_SECONDS = 43200;

    /**
     * We do not apply validators here because of flexible data and speed
     * @param $array
     * @param $multiplier - how many times to multiply
     * @return array
     */
    public static function getMultipliedNumberArray($array, $multiplier)
    {
        $retArray = array();
        foreach($array AS $key => $number)
        {
            $retArray[$key] = $number * $multiplier;
        }

        return $retArray;
    }

    /**
     * We do not apply validators here because of flexible data and speed
     * @param array $array1
     * @param array $array2
     * @return array
     */
    public static function getSumOfTwoArrays($array1, $array2)
    {
        $sumArray = array();
        foreach($array1 AS $key => $number)
        {
            $sumArray[$key] = isset($array2[$key]) ? $array2[$key] + $number : $number;
        }

        return $sumArray;
    }

    /**
     * We do not apply validators here because of flexible data and speed
     * @param $array
     * @param $formatType
     * @param $currencySymbol
     * @param $currencyCode
     * @return array
     */
    public static function getFormattedPriceArray($array, $formatType, $currencySymbol, $currencyCode)
    {
        $retArray = array();
        foreach($array AS $key => $price)
        {
            $retArray[$key] = static::getFormattedPrice($price, $formatType, $currencySymbol, $currencyCode);
        }

        return $retArray;
    }

    /**
     * We do not apply validators here because of flexible data and speed
     * @param $array
     * @param $formatType
     * @return array
     */
    public static function getFormattedPercentageArray($array, $formatType)
    {
        $retArray = array();
        foreach($array AS $key => $percentage)
        {
            $retArray[$key] = static::getFormattedPercentage($percentage, $formatType);
        }

        return $retArray;
    }

    /**
     * We do not apply validators here because of flexible data and speed
     * @param float $price
     * @param string $formatType - tiny, tiny_without_fraction, regular, regular_without_fraction, long, long_without_fraction
     * @param string $currencySymbol
     * @param string $currencyCode
     * @return string
     */
    public static function getFormattedPrice($price, $formatType, $currencySymbol, $currencyCode)
    {
        switch($formatType)
        {
            case "tiny":
                $formattedNumber = $currencySymbol.number_format_i18n($price, 2);
                break;
            case "tiny_without_fraction":
                $formattedNumber = $currencySymbol.number_format_i18n($price, 0);
                break;
            case "regular":
                $formattedNumber = $currencySymbol.' '.number_format_i18n($price, 2);
                break;
            case "regular_without_fraction":
                $formattedNumber = $currencySymbol.' '.number_format_i18n($price, 0);
                break;
            case "long":
                $formattedNumber = number_format_i18n($price, 2)." ".$currencyCode;
                break;
            case "long_without_fraction":
                $formattedNumber = number_format_i18n($price, 0)." ".$currencyCode;
                break;
            default:
                $formattedNumber = $price;
        }

        return $formattedNumber;
    }

    /**
     * We do not apply validators here because of flexible data and speed
     * @param $percentage
     * @param $formatType
     * @return string
     */
    public static function getFormattedPercentage($percentage, $formatType)
    {
        switch($formatType)
        {
            case "tiny":
                $formattedPercentage = number_format_i18n($percentage, 2)."%";
                break;
            case "tiny_without_fraction":
                $formattedPercentage = number_format_i18n($percentage, 0)."%";
                break;
            case "regular":
                $formattedPercentage = number_format_i18n($percentage, 2)." %";
                break;
            case "regular_without_fraction":
                $formattedPercentage = number_format_i18n($percentage, 0)." %";
                break;
            default:
                $formattedPercentage = $percentage;
        }

        return $formattedPercentage;
    }

    public static function generateDropDown(
        $from, $to, $fieldName="selector", $fieldId="selector", $selectedValue = "", $defaultText = "",
        $required = FALSE, $prefixed = FALSE, $suffix = ""
    )
    {
        $ret = "";
        $suffix = $suffix != '' ? ' '.$suffix : '';

        $requiredClass = $required ? ' class="required"' : '';
        $ret .= '<select name="'.$fieldName.'" id="'.$fieldId.'"'.$requiredClass.'>';
        if($defaultText != "")
        {
            $selected = $selectedValue == "" ? ' selected="selected"' : "";
            $ret .= '<option value=""'.$selected.'>'.$defaultText.'</option>';
        }

        for($i = $from; $i <= $to; $i++)
        {
            $prefixedValue = $prefixed ? sprintf('%0'.strlen($to).'d', $i) : $i;
            $selected = $prefixedValue == $selectedValue ? ' selected="selected"' : "";
            $ret .= '<option value="'.$prefixedValue.'"'.$selected.'>'.$i.$suffix.'</option>';
        }
        $ret .= "</select>";

        return $ret;
    }

    public static function priceCompare($a, $b)
    {
        if ($a['unit']['discounted_total'] == $b['unit']['discounted_total'])
        {
            return 0;
        }

        return ($a['unit']['discounted_total'] < $b['unit']['discounted_total']) ? -1 : 1;
    }
}